<?php
add_action('wp_ajax_stms_save_email', 'stms_ajax_save_email');
function stms_ajax_save_email() {
	if ( isset($_REQUEST['type']) ) {
		$type = $_REQUEST['type'];
		$stms = get_option('stms_settings');
		
		// delete the cache first
		delete_transient('instamem_email_' . $type);
		
		if ( !is_array($stms) )
			$stms = array();
		
		$stms['email_templates'][$type]['fromname'] = trim(sanitize_text_field($_REQUEST['fromname']));
		$stms['email_templates'][$type]['frommail'] = trim(sanitize_email($_REQUEST['frommail']));
		$stms['email_templates'][$type]['subject'] = trim(strip_tags(sanitize_text_field($_REQUEST['subject'])));
		$stms['email_templates'][$type]['body'] = trim(strip_tags($_REQUEST['body']));
		
		update_option('stms_settings', $stms);
		
		echo "ok";
	}
	die();
}

add_action('wp_ajax_stms_email_template', 'stms_ajax_email_template');
function stms_ajax_email_template() {
	if ( isset($_REQUEST['type']) ) {
		$type = $_REQUEST['type'];
		$json = ''; //get_transient('instamem_email_' . $type);
		if ( empty($json) ) {
			$stms = get_option('stms_settings');
			$default_fromname = stms_sanitize($stms['emails']['admin_from'], get_option('blogname'));
			$default_frommail = stms_sanitize($stms['emails']['admin_email'], get_option('admin_email'));
			$temp = stms_default_emails();
			$from_name = ( stms_isset($stms['email_templates'][$type]['fromname']) != '' ) ? $stms['email_templates'][$type]['fromname'] : $default_fromname;
			$from_email = ( stms_isset($stms['email_templates'][$type]['frommail']) != '' ) ? $stms['email_templates'][$type]['frommail'] : $default_frommail;
			$subject = ( stms_isset($stms['email_templates'][$type]['subject']) != '' ) ? $stms['email_templates'][$type]['subject'] : stms_isset($temp[$type]['subject']);
			$body = ( stms_isset($stms['email_templates'][$type]['body']) != '' ) ? $stms['email_templates'][$type]['body'] : stms_isset($temp[$type]['body']);
			
			$json = array(
				'fromname' => stms_sanitize(esc_attr($from_name)),
				'frommail' => stms_sanitize(sanitize_email($from_email)),
				'subject' => stms_sanitize($subject),
				'body' => stms_sanitize($body)
			);
			set_transient('instamem_email_' . $type, $json, 3600);
		}
				
		$response = $_REQUEST['callback'] . "(" . json_encode($json) . ")"; 
		echo $response;
	}
	die();
}

add_action('wp_ajax_stms_meta_apply_protect', 'stms_ajax_meta_apply_protect');
function stms_ajax_meta_apply_protect() {
	if ( isset($_REQUEST['product_id']) ) {
		global $wpdb;
		
		$product_id = (int) $_REQUEST['product_id'];
		$post_id = (int) $_REQUEST['post_id'];
		
		// check if row exists
		$protect = $wpdb->get_row(
			$wpdb->prepare(
				"SELECT ID FROM `{$wpdb->prefix}stms_protects`
				WHERE `product_id` = %d AND `post_id` = %d",
				$product_id, $post_id
			)
		);
		$update = false;
		if ( isset($protect->ID) )
			$update = true;
			
		$data = array(
			'type' => 'content',
			'product_id' => $product_id,
			'post_id' => $post_id,
			'is_expired' => intval($_REQUEST['is_expired']),
			'drip_start' => trim($_REQUEST['start']),
			'drip_end' => trim($_REQUEST['end']),
			'drip_start_mode' => trim($_REQUEST['start_mode']),
			'drip_end_mode' => trim($_REQUEST['end_mode']),
			'date' => stms_time()
		);
		if ( $update )
			$wpdb->update("{$wpdb->prefix}stms_protects", $data, array('ID' => $protect->ID) );
		else
			$wpdb->insert("{$wpdb->prefix}stms_protects", $data );
		
		echo "ok";
	}
	die();
}

add_action('wp_ajax_stms_meta_protect', 'stms_ajax_meta_protect');
function stms_ajax_meta_protect() {
	if ( isset($_REQUEST['product_id']) ) {
		$product_id = $_REQUEST['product_id'];
		$post_id = $_REQUEST['post_id'];
		$protect = stms_get_contentdrip( $product_id, $post_id );
		$json = array(
			'start' => ( isset($protect->drip_start) ) ? $protect->drip_start : 0,
			'start_mode' => ( isset($protect->drip_start_mode) ) ? $protect->drip_start_mode : 'day',
			'is_expired' => ( isset($protect->is_expired) ) ? $protect->is_expired : 0,
			'end' => ( isset($protect->drip_end) ) ? $protect->drip_end : 30,
			'end_mode' => ( isset($protect->drip_end_mode) ) ? $protect->drip_end_mode : 'day',
		);
		$response = $_REQUEST['callback'] . "(" . json_encode($json) . ")"; 
		echo $response;
	}
	die();
}

add_action('wp_ajax_stms_apply_license', 'stms_ajax_apply_license');
function stms_ajax_apply_license() {
	if ( isset($_REQUEST['post']) ) {
		set_time_limit(0);
		global $wpdb;
		
		$vars = urldecode($_REQUEST['post']);
		parse_str($vars, $post);
		
		$product_id = $post['stms_license_product_id'];
		$data = $post['stms_lic'];
		
		stms_update_product_meta($product_id, 'license', $data);
		echo 'ok';
	}
	die();
}

add_action('wp_ajax_stms_apply_protect', 'stms_ajax_apply_protect');
function stms_ajax_apply_protect() {
	if ( isset($_REQUEST['post']) ) {
		set_time_limit(0);
		global $wpdb;
		
		$product_id = (int) $_REQUEST['product_id'];
		$vars = urldecode($_REQUEST['post']);
		parse_str($vars, $post);
		
		$_data = $post['stms_post_protect'];
		if ( isset($_data['drip_start']) ) : $i = 0;
			//foreach ( $_data['drip_start'] as $drip_start ) {
			for ( $i = 0; $i < count($_data['drip_start']); $i++ ) {
				$post_id = $_data['post_id'][$i];
				// check if row exists
				$protect = $wpdb->get_row(
					$wpdb->prepare(
						"SELECT ID FROM `{$wpdb->prefix}stms_protects`
						WHERE `product_id` = %d AND `post_id` = %d",
						$product_id, $post_id
					)
				);
				$update = false;
				if ( isset($protect->ID) )
					$update = true;
					
				if ( isset($_data['protect_id'][$i]) ) {
					$data = array(
						'type' => 'content',
						'product_id' => $product_id,
						'post_id' => $post_id,
						'is_expired' => ( isset($_data['drip_enable_exp'][$i]) ) ? 1 : 0,
						'drip_start' => stms_isset($_data['drip_start'][$i]),
						'drip_end' => stms_isset($_data['drip_exp'][$i]),
						'drip_start_mode' => stms_isset($_data['drip_start_mode'][$i]),
						'drip_end_mode' => stms_isset($_data['drip_end_mode'][$i]),
						'date' => stms_time()
					);
					if ( $update )
						$wpdb->update("{$wpdb->prefix}stms_protects", $data, array('ID' => $protect->ID));
					else
						$wpdb->insert("{$wpdb->prefix}stms_protects", $data);
				} else if ( $_data['is_removed'][$i] == 'yes' && $update ) {
					$wpdb->delete("{$wpdb->prefix}stms_protects", array('ID' => $protect->ID) );
				}
				wp_cache_delete($product_id, 'stms_protected_contents');
			}
		endif;
		echo 'ok';
	}
	die();
}

add_action('wp_ajax_stms_apply_categories_protect', 'stms_ajax_apply_categories_protect');
function stms_ajax_apply_categories_protect() {
	if ( isset($_REQUEST['post']) ) {
		set_time_limit(0);
		global $wpdb;
		
		$product_id = (int) $_REQUEST['product_id'];
		$vars = urldecode($_REQUEST['post']);
		parse_str($vars, $post);
		
		$_data = $post['stms_cat_protect'];
		if ( isset($_data['drip_start']) ) : $i = 0;
			//foreach ( $_data['drip_start'] as $drip_start ) {
			for ( $i = 0; $i < count($_data['drip_start']); $i++ ) {
				$cat_id = $_data['cat_id'][$i];
				// check if row exists
				$protect = $wpdb->get_row(
					$wpdb->prepare(
						"SELECT ID FROM `{$wpdb->prefix}stms_protects`
						WHERE `product_id` = %d AND `cat_id` = %s",
						$product_id, $cat_id
					)
				);
				$update = false;
				if ( isset($protect->ID) )
					$update = true;
					
				if ( isset($_data['protect_id'][$i]) ) {
					$data = array(
						'type' => 'categories',
						'product_id' => $product_id,
						'cat_id' => $cat_id,
						'is_expired' => ( isset($_data['drip_enable_exp'][$i]) ) ? 1 : 0,
						'drip_start' => stms_isset($_data['drip_start'][$i]),
						'drip_end' => stms_isset($_data['drip_exp'][$i]),
						'drip_start_mode' => stms_isset($_data['drip_start_mode'][$i]),
						'drip_end_mode' => stms_isset($_data['drip_end_mode'][$i]),
						'date' => stms_time()
					);
					if ( $update )
						$wpdb->update("{$wpdb->prefix}stms_protects", $data, array('ID' => $protect->ID) );
					else
						$wpdb->insert("{$wpdb->prefix}stms_protects", $data );
				} else if ( $_data['is_removed'][$i] == 'yes' && $update ) {
					$wpdb->delete("{$wpdb->prefix}stms_protects", array('ID' => $protect->ID) );
				}
			}
		endif;
		echo 'ok';
	}
	die();
}

add_action('wp_ajax_stms_apply_file_protect', 'stms_ajax_apply_file_protect');
function stms_ajax_apply_file_protect() {
	if ( isset($_REQUEST['post']) ) {
		set_time_limit(0);
		global $wpdb;
		
		$product_id = (int) $_REQUEST['product_id'];
		$vars = urldecode($_REQUEST['post']);
		parse_str($vars, $post);
		
		$_data = $post['stms_file_protect'];
		if ( isset($_data['drip_start']) ) : $i = 0;
			//foreach ( $_data['drip_start'] as $drip_start ) {
			for ( $i = 0; $i < count($_data['drip_start']); $i++ ) {
				$filename = $_data['filename'][$i];
				// check if row exists
				$protect = $wpdb->get_row(
					$wpdb->prepare(
						"SELECT ID FROM `{$wpdb->prefix}stms_protects`
						WHERE `product_id` = %d AND `filename` = %s",
						$product_id, $filename
					)
				);
				$update = false;
				if ( isset($protect->ID) )
					$update = true;
					
				if ( isset($_data['protect_id'][$i]) ) {
					$data = array(
						'type' => 'file',
						'product_id' => $product_id,
						'filename' => $filename,
						'is_expired' => ( isset($_data['drip_enable_exp'][$i]) ) ? 1 : 0,
						'drip_start' => stms_isset($_data['drip_start'][$i]),
						'drip_end' => stms_isset($_data['drip_exp'][$i]),
						'drip_start_mode' => stms_isset($_data['drip_start_mode'][$i]),
						'drip_end_mode' => stms_isset($_data['drip_end_mode'][$i]),
						'date' => stms_time()
					);
					if ( $update )
						$wpdb->update("{$wpdb->prefix}stms_protects", $data, array('ID' => $protect->ID) );
					else
						$wpdb->insert("{$wpdb->prefix}stms_protects", $data );
				} else if ( $_data['is_removed'][$i] == 'yes' && $update ) {
					$wpdb->delete("{$wpdb->prefix}stms_protects", array('ID' => $protect->ID) );
				}
			}
		endif;
		echo 'ok';
	}
	die();
}

add_action('wp_ajax_stms_apply_s3file_protect', 'stms_ajax_apply_s3file_protect');
function stms_ajax_apply_s3file_protect() {
	if ( isset($_REQUEST['post']) ) {
		set_time_limit(0);
		global $wpdb;
		
		$product_id = (int) $_REQUEST['product_id'];
		$vars = rawurldecode($_REQUEST['post']);
		parse_str($vars, $post);
		
		$_data = $post['stms_file_protect'];
		if ( isset($_data['drip_start']) ) : $i = 0;
			//foreach ( $_data['drip_start'] as $drip_start ) {
			for ( $i = 0; $i < count($_data['drip_start']); $i++ ) {
				$filename = $_data['filename'][$i];
				// check if row exists
				$protect = $wpdb->get_row(
					$wpdb->prepare(
						"SELECT `ID` FROM `{$wpdb->prefix}stms_protects`
						WHERE `type` = %s AND `product_id` = %d AND `filename` = %s",
						's3file', $product_id, $filename
					)
				);
				$update = false;
				if ( isset($protect->ID) )
					$update = true;
					
				if ( isset($_data['protect_id'][$i]) ) {
					$data = array(
						'type' => 's3file',
						'product_id' => $product_id,
						'filename' => $filename,
						'is_expired' => ( isset($_data['drip_enable_exp'][$i]) ) ? 1 : 0,
						'drip_start' => stms_isset($_data['drip_start'][$i]),
						'drip_end' => stms_isset($_data['drip_exp'][$i]),
						'drip_start_mode' => stms_isset($_data['drip_start_mode'][$i]),
						'drip_end_mode' => stms_isset($_data['drip_end_mode'][$i]),
						'date' => stms_time()
					);
					if ( $update )
						$wpdb->update("{$wpdb->prefix}stms_protects", $data, array('ID' => $protect->ID) );
					else
						$wpdb->insert("{$wpdb->prefix}stms_protects", $data );
					
					// Set the file ACP so that public users can't read
					stms_s3_set_file_acp($filename);
				} else if ( $_data['is_removed'][$i] == 'yes' && $update ) {
					stms_s3_set_file_acp($filename, 'unprotect');
					$wpdb->delete("{$wpdb->prefix}stms_protects", array('ID' => $protect->ID) );
				}
			}
		endif;
		echo 'ok';
	}
	die();
}

add_action('wp_ajax_stms_manual_payment', 'stms_ajax_manual_payment');
function stms_ajax_manual_payment() {
	if ( isset($_REQUEST['vars']) && isset($_REQUEST['member_id'])) {
		$member_id = (int) $_REQUEST['member_id'];
		$vars = urldecode($_REQUEST['vars']);
		parse_str($vars, $post);
	
		$tx_id = stms_isset($post['stms_payment_id'], 0);
		//$pr = stms_get_product(stms_isset($post['stms_pays_level']));
		$pay_info = array(
			'product_id' => stms_isset($post['stms_pays_level']),
			'paysys' => 'manual',
			'receipt' => 'manual',
			'amount' => stms_isset($post['stms_pays_amount']),
			'start' => stms_isset($post['stms_pays_start']),
			'expire' => stms_isset($post['stms_pays_expired']),
			'paid' => ( isset($post['stms_pays_completed']) ) ? 1 : 0
		);
		
		if ( $tx_id > 0 ) {
			$pay_info['status'] = $post['stms_pays_status'];
			$pay_info['paid'] = ( $post['stms_pays_status'] == 'pending' ) ? 0 : stms_isset($post['stms_is_paid']);
		}
		$payment = stms_manual_payment($member_id, $pay_info, $tx_id);
		if ( $payment )
			echo 'ok';
		else
			echo 'fail';
	}
	die();
}

add_action('wp_ajax_stms_merge_tags', 'stms_ajax_merge_tags_handler');
function stms_ajax_merge_tags_handler() {
	$type = $_REQUEST['type'];
	$pass = ( $_REQUEST['pass'] == 'yes' ) ? true : false;
	$tags = stms_mail_tags( $type, $pass );
	$i = 0;
	
	echo '<option value="">[ -- Insert Email Macro -- ]</option>';
	foreach ( $tags as $k => $v ) {
		if ( stristr($k, 'group') ) {
			echo "<optgroup label='$v'>";
		} else if ( stristr($k, 'end') ) {
			echo "</optgroup>";
		} else {
			echo "<option value='{$k}'>{$k} - {$v}</option>";
		}
		$i++;
	}
	die();
}
add_action('wp_ajax_stms_ajax_save', 'stms_ajax_save_handler');
function stms_ajax_save_handler() {
	if ( isset($_REQUEST['vars']) ) {
		$vars = rawurldecode($_REQUEST['vars']);
		parse_str($vars, $post);
		$stms = get_option('stms_settings');
		$config = $post['config_name'];
		if ( !is_array($stms) )
			$stms = array();
		
		$stms[$config] = $post['stms_sets'];
		update_option('stms_settings', $stms);
		
		if ( ($second_time = get_option('stms_second_time')) === FALSE )
			update_option('stms_second_time', 1);
		
		do_action('stms_settings_saved', $config, $post);
		echo 'ok';
	}
	die();
}

add_action('wp_ajax_stms_validate_username', 'stms_ajax_validate_username');
function stms_ajax_validate_username() {
	$username = trim(strtolower($_REQUEST['stms_member']['username']));
	$valid = 'true';
	if ( isset($_REQUEST['member_id']) && $_REQUEST['member_id'] > 0 ) {
		$member_id = (int) $_REQUEST['member_id'];
		$member = stms_get_memberdata($member_id);
		if ( $username == $member->username ) {
			echo 'true';
			die();
		}
	}
	if( stms_username_exists($username) )
		$valid = 'false';	
	echo $valid;
	die();
}

add_action('wp_ajax_stms_validate_email', 'stms_ajax_validate_email');
function stms_ajax_validate_email() {
	$email = sanitize_email(trim($_REQUEST['stms_member']['email']));
	$old_email = sanitize_email(trim($_REQUEST['old_email']));
	if ( $email == $old_email ) {
		echo 'true';
		die();
	}
	
	$valid = 'true';
	if ( isset($_REQUEST['member_id']) && $_REQUEST['member_id'] > 0 ) {
		$member_id = (int) $_REQUEST['member_id'];
		$member = stms_get_memberdata($member_id);
		if ( $email == $member->email ) {
			echo 'true';
			die();
		}
	}
	
	if( stms_email_exists($email) )
		$valid = 'false';
	
	echo $valid;
	die();
}

add_action('wp_ajax_stms_change_slug', 'stms_ajax_change_slug');
function stms_ajax_change_slug() {
	$stms = get_option('stms_settings');
	$slugs = get_option('stms_slugs');
	$handler_id = stms_isset($stms['global']['magic']);
	$type = $_REQUEST['type'];
	$post_name = stms_sanitize($_REQUEST['slug_name']);
	$home_id = stms_isset($stms['global']['home']);
	$the_post = get_post($home_id);
	
	if ( empty($post_name) || !preg_match('/^[a-zA-Z0-9_-]+$/', $post_name) ) {
		echo 'error';
		die();
	}
	
	$old_post_id = ( isset($slugs[$type]) && $slugs[$type] > 0 ) ? $slugs[$type] : 0;
	
	$post = array();
	$post['ID'] = ( $type == 'home' ) ? $home_id : $old_post_id;
	$post['post_title'] = ( $type == 'home' ) ? stms_sanitize($the_post->post_title) : ucwords(str_replace(array("-", "_"), " ", $post_name));
  	$post['post_content'] = ( $type == 'home' ) ? stms_sanitize($the_post->post_content) : '';
  	$post['post_name'] = $post_name;
  	$post['post_type'] = 'page';
  	$post['post_status'] = 'publish';
	if ( $type != 'home' ) {
		if ( $the_post->post_parent > 0 )
			$post['post_parent'] = $the_post->post_parent;
		$post['comment_status'] = 'closed';
		$post['ping_status'] = 'closed';
	}
	
	$post_id = wp_insert_post($post, true);
	if ( is_wp_error($post_id) ) {
		echo 'error';
		die();
	}
	
	if ( $type != 'home' ) {
		if ( !is_array($slugs) )
			$slugs = array();
		
		$slugs[$type] = $post_id;
		update_option('stms_slugs', $slugs);
	}
	$new_post = get_post($post_id);
	echo $new_post->post_name . '|' . get_permalink($post_id);
	die();
}

add_action('wp_ajax_stms_test_email', 'stms_ajax_test_email');
function stms_ajax_test_email() {
	$stms = get_option('stms_settings');
	$to = sanitize_email($_POST['target_email']);
	if ( empty($to) )
		die('invalid');
	
	$domain = get_bloginfo('url');
	$subject = 'InstaMember Test Email';
	$message = 'Hello,' . "\n\n" .
			'This is a test email sent from:' . "\n" .
			$domain . ' (via InstaMember)' . "\n\n";
			
	$send = stms_mail($to, $subject, $message);
	
	if ( !$send ) die('failed');
	die('ok');
}

/*
add_action('wp_ajax_stms_validate_productname', 'stms_ajax_validate_productname');
function stms_ajax_validate_productname() {
	$title = trim(strtolower($_REQUEST['stms_level']['name']));
	$valid = 'true';
	if ( isset($_REQUEST['product_id']) && $_REQUEST['product_id'] > 0 ) {
		$product_id = (int) $_REQUEST['product_id'];
		$product = stms_get_product($product_id);
		$title = stms_sanitize($title);
		$orig_title = stms_sanitize($product->title);
		if ( strtolower($username) == strtolower($orig_title) ) {
			echo 'true';
			die();
		}
	}
	
	if( $product = stms_get_product_bytitle($title) )
		$valid = 'false';
	echo $valid;
	die();
}
*/