<?php
if ( !class_exists('WP_List_Table') )
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
    
class stmsAccessContent extends WP_List_Table {
	function __construct() {
		global $wpdb;
		// Set parent defaults
        parent::__construct( array(
            'singular'  => 'content',     // singular name of the listed records
            'plural'    => 'contents',    // plural name of the listed records
            'ajax'      => true        // does this table support ajax?
        ));
	}
	
	function column_default($item, $column_name) {
		$value = '';
        switch ($column_name) {
            case 'product':
            case 'drip':
                $value = $item[$column_name];
				break;
            default:
                return stms_dump($item); // Show the whole array for troubleshooting purposes
        }
		
		return $value;
    }
	
	function column_title($item) {

        // Build row actions
        $actions = array();

        // Return the title contents
        return sprintf('%1$s %2$s',
            $item['title'],
            $this->row_actions($actions)
        );
    }
	
	function column_cb($item) {
        return sprintf(
            '<input type="checkbox" name="%1$s[]" value="%2$s" />',
            /*$1%s*/ $this->_args['singular'],  //Let's simply repurpose the table's singular label
            /*$2%s*/ $item['ID']                //The value of the checkbox should be the record's id
        );
    }
	
	function get_table_classes() {
		return array( 'widefat', 'fixed', 'stms-normal-table' );
	}
	
	function get_columns() {
        $columns = array(
            'cb' => '<input type="checkbox" />', // Render a checkbox instead of text
            'title' => 'Product Name',
            'product' => 'Product(s)',
            'drip'  => 'Drip-Feed',
        );
        return $columns;
    }
	
	/*
	function get_sortable_columns() {
        $sortable_columns = array(
            'title' => array('title', false),     // true means it's already sorted
            'type'    => array('type', false),
            'price'  => array('price', false),
            'dur'  => array('dur', false)
        );
        return $sortable_columns;
    }
	*/
	
	function get_bulk_actions() {
        $actions = array(
            'stms_bulk_product_delete' => 'Delete'
        );
        return $actions;
    }
	
	function process_bulk_action() {
		// do nothing
	}
	
	function no_items() {
		echo 'No content found.';
	}
	
	function prepare_items() {
        global $wpdb; // This is used only if making any database queries

        $per_page = 10;
        $columns = $this->get_columns();
        $hidden = array();
        $sortable = $this->get_sortable_columns();
        $this->_column_headers = array($columns, $hidden, $sortable);
        $this->process_bulk_action();
		
		$current_page = $this->get_pagenum();
		$offset = ( $current_page ) ? $current_page - 1 : 0;
		if ( $offset < 0 ) $offset = 0;
		$offset = $offset * $per_page;
		$args = array(
			'numberposts' => $per_page,
			'offset' => $offset,
			'post_status' => 'publish',
			'orderby' => 'date',
			'order' => 'DESC',
			'post_type' => 'page'
		);
	
	 	
		$posts = get_posts($args);
        $data = array();
       	if ( $posts && count($posts) > 0 ) {
			$i = 0;
			foreach ( $posts as $post ) {
				$drip = stms_dripfeed_dialog('Page', stms_sanitize($post->post_title), $i);
				$data[] = array(
					'ID' => $post->ID,
					'title' => stms_sanitize($post->post_title),
					'product' => stms_products_multiselect('content_access', $i),
					'drip' => $drip,
				);
				$i++;
			}
		}
		$count_posts = wp_count_posts('page');
        $total_items = $count_posts->publish;
        $this->items = $data;
        $this->set_pagination_args( array(
            'total_items' => $total_items,                  // We have to calculate the total number of items
            'per_page'    => $per_page,                     // We have to determine how many items to show on a page
            'total_pages' => ceil($total_items/$per_page)   // We have to calculate the total number of pages
        ));
    }
}