<?php
if ( !class_exists('WP_List_Table') )
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
    
class stmsAffReport extends WP_List_Table {
	function __construct() {
		global $wpdb;
		// Set parent defaults
        parent::__construct( array(
            'singular'  => 'commission',     // singular name of the listed records
            'plural'    => 'commissions',    // plural name of the listed records
            'ajax'      => true        // does this table support ajax?
        ));
	}
	
	function column_default($item, $column_name) {
		$value = '';
        switch ($column_name) {
			case 'uclick':
			case 'sales':
			case 'refund':
			case 'convert':
			case 'revenue':
			case 'refunded':
			case 'commissions':
			case 'corrections':
			case 'payout':
                $value = $item[$column_name];
				break;
            default:
                return stms_dump($item); // Show the whole array for troubleshooting purposes
        }
		
		return $value;
    }
	
	function column_title($item) {
        // Build row actions
        $actions = array();
        $actions['details'] = sprintf('<a href="#members|mode=affstats&object_ID=%s">View Details</a>', $item['ID']);
		
        // Return the title contents
        return sprintf('%1$s %2$s',
            '<a href="#members|mode=edit&object_ID=' . $item['ID'] . '"><strong>' . $item['title'] . '</strong></a>',
            $this->row_actions($actions)
        );
    }
	
	function column_cb($item) {
        return sprintf(
            '<input type="checkbox" name="%1$s[]" value="%2$s" />',
            /*$1%s*/ $this->_args['singular'],  //Let's simply repurpose the table's singular label
            /*$2%s*/ $item['ID']                //The value of the checkbox should be the record's id
        );
    }
	
	function get_columns() {
        $columns = array(
            'title' => 'Affiliate',
            'uclick'  => 'Clicks',
            'sales' => 'Sales#',
            'refund' => 'Refund#',
            'convert' => 'Conv%',
            'revenue' => 'Rev.',
            'refunded' => 'Rfnd.',
            'commissions'  => 'Credit',
            'corrections'  => 'Debit',
            'payout'  => 'Total'
        );
        return $columns;
    }
	
	function no_items() {
		echo 'No data found.';
	}
	
	function prepare_items() {
        global $wpdb; // This is used only if making any database queries

        $per_page = 10;
        $columns = $this->get_columns();
        $hidden = array();
        $sortable = $this->get_sortable_columns();
        $this->_column_headers = array($columns, $hidden, $sortable);

        $data = stms_tabledata_affiliates();
        $current_page = $this->get_pagenum();
        $total_items = count($data);
        $data = array_slice($data,(($current_page-1) * $per_page), $per_page);
        $this->items = $data;
        $this->set_pagination_args( array(
            'total_items' => $total_items,                  // We have to calculate the total number of items
            'per_page'    => $per_page,                     // We have to determine how many items to show on a page
            'total_pages' => ceil($total_items/$per_page)   // We have to calculate the total number of pages
        ));
    }
}