<?php
if ( !class_exists('WP_List_Table') )
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
    
class stmsMemberLevel extends WP_List_Table {
	function __construct() {
		global $wpdb;
		// Set parent defaults
        parent::__construct( array(
            'singular'  => 'level',     // singular name of the listed records
            'plural'    => 'levels',    // plural name of the listed records
            'ajax'      => true        // does this table support ajax?
        ));
	}
	
	function column_default($item, $column_name) {
		$value = '';
        switch ($column_name) {
        	case 'title':
				$value = '<a href="#level|mode=edit&object_ID=' . $item['ID'] . '"><strong>' . $item[$column_name] . '</strong></a>';
				break;
            case 'price':
            case 'type':
			case 'dur':
                $value = $item[$column_name];
				break;
            default:
                return stms_dump($item); // Show the whole array for troubleshooting purposes
        }
		
		return $value;
    }
	
	function column_title($item) {
		$start = date("m/d/Y", strtotime("30 days ago", stms_time()));
		$end = date("m/d/Y", stms_time());
        // Build row actions
        $actions = array();
        $actions['edit'] = sprintf('<a href="#level|mode=%s&object_ID=%d">Edit</a>', 'edit', $item['ID']);
		$actions['protect'] = sprintf('<a href="#level|mode=%s&object_ID=%d">Protect Contents</a>', 'protects', $item['ID']);
		$actions['license'] = sprintf('<a href="#level|mode=%s&object_ID=%d">Licensing</a>', 'license', $item['ID']);
		$actions['pay_button'] = sprintf('<a href="#level|mode=%s&object_ID=%d">Reg/Pay Button</a>', 'pay_button', $item['ID']);
		$actions['report'] = sprintf('<a href="#reports|start=%s&end=%s&product_id=%d">Report</a>', $start, $end, $item['ID']);
		$actions['clone'] = sprintf('<a href="%s&action=stms_productdata_clone&ID=%s">Clone</a>', admin_url('admin.php?page=' . $_REQUEST['page']), $item['ID']);
		$actions['delete'] = sprintf('<a href="%s&action=stms_productdata_delete&ID=%s" onclick="return confirm(\'Are you sure want to delete this product?\');">Delete</a>', admin_url('admin.php?page=' . $_REQUEST['page']), $item['ID']);
		
        // Return the title contents
        return sprintf('%1$s %2$s',
            '<a href="#level|mode=edit&object_ID=' . $item['ID'] . '"><strong>' . stms_sanitize($item['title']) . '</strong></a>',
            $this->row_actions($actions)
        );
    }
	
	function column_cb($item) {
        return sprintf(
            '<input type="checkbox" name="%1$s[]" value="%2$s" />',
            /*$1%s*/ $this->_args['singular'],  //Let's simply repurpose the table's singular label
            /*$2%s*/ $item['ID']                //The value of the checkbox should be the record's id
        );
    }
	
	function get_table_classes() {
		return array( 'widefat', 'fixed', 'stms-normal-table' );
	}
	
	function get_columns() {
        $columns = array(
            'cb' => '<input type="checkbox" />', // Render a checkbox instead of text
            'title' => 'Product Name',
            'type'    => 'Type',
            'price'  => 'Price',
            'dur'  => 'Duration',
            //'act'  => 'Actions'
        );
        return $columns;
    }
	
	/*
	function get_sortable_columns() {
        $sortable_columns = array(
            'title' => array('title', false),     // true means it's already sorted
            'type'    => array('type', false),
            'price'  => array('price', false),
            'dur'  => array('dur', false)
        );
        return $sortable_columns;
    }
	*/
	
	function get_bulk_actions() {
        $actions = array(
        	'stms_bulk_product_clone' => 'Clone',
            'stms_bulk_product_delete' => 'Delete'
        );
        return $actions;
    }
	
	function process_bulk_action() {
		// do nothing
	}
	
	function no_items() {
		echo 'No membership level found.';
	}
	
	function prepare_items() {
        global $wpdb; // This is used only if making any database queries

        $per_page = 10;
        $columns = $this->get_columns();
        $hidden = array();
        $sortable = $this->get_sortable_columns();
        $this->_column_headers = array($columns, $hidden, $sortable);
        $this->process_bulk_action();
		
		$current_page = $this->get_pagenum();
        $data = stms_tabledata_levels($per_page);
       	$sql = "SELECT COUNT(*) FROM `{$wpdb->prefix}stms_products` WHERE `product_status` <> 4";
        if ( isset($_REQUEST['s']) && $_REQUEST['s'] != '' ) {
			$s = trim(urldecode($_REQUEST['s']));
			$s = str_replace(array('+', ' '), "_", $s);
			$sql .= " AND (`title` LIKE '%{$s}%' OR `type` LIKE '%{$s}%')";
		}
        $total_items = $wpdb->get_var($sql);
        $this->items = $data;
        $this->set_pagination_args( array(
            'total_items' => $total_items,                  // We have to calculate the total number of items
            'per_page'    => $per_page,                     // We have to determine how many items to show on a page
            'total_pages' => ceil($total_items/$per_page)   // We have to calculate the total number of pages
        ));
    }
}