<?php
if ( !class_exists('WP_List_Table') )
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
    
class stmsMemberList extends WP_List_Table {
	function __construct() {
		global $wpdb;
		// Set parent defaults
        parent::__construct( array(
            'singular'  => 'member',     // singular name of the listed records
            'plural'    => 'members',    // plural name of the listed records
            'ajax'      => true        // does this table support ajax?
        ));
	}
	
	function column_default($item, $column_name) {
		$value = '';
        switch ($column_name) {
            case 'email':
			case 'affiliate':
			case 'account':
                $value = $item[$column_name];
				break;
			/*
			case 'payments':
				if ( $item['payments'] > 0.00 ) {
					$value = '<a href="#members|mode=payments&object_ID=' . $item['ID'] . '"><strong>' . $item['payments'] . '</strong></a>';
				} else {
					$value = 'None';
				}
				break;
			*/
            default:
                return stms_dump($item); // Show the whole array for troubleshooting purposes
        }
		 
		return $value;
    }
	
	
	function column_title($item) {
        // Build row actions
        $actions = array();
        $actions['edit'] = sprintf('<a href="#members|mode=edit&object_ID=%d">Edit</a>', $item['ID']);
		$actions['payments'] = sprintf('<a href="#members|mode=payments&object_ID=%d">Payments</a>', $item['ID']);
		/*
		if ( $item['affiliate'] == 'Yes' ) {
			$actions['affstats'] = sprintf('<a href="#members|mode=affstats&object_ID=%d">Aff. Stats</a>', $item['ID']);
		} else if ( strip_tags($item['affiliate']) == 'Pending' ) {
			$reject = '"This member is about to be denied as an affiliate. If his/her account is not active and have not paid for anything, then it will be deleted too.\nAre you sure you want to continue?"';
			$actions['affact'] = sprintf('<a href="%s&ID=%d" onclick="return confirm(\'Are you sure you want to approve this member as an affiliate?\')">Approve Affiliate</a>', admin_url('admin.php?page=stms-settings&action=stms_memberdata_affapprove'), $item['ID']);
			$actions['affreject'] = sprintf('<a href="%s&ID=%d" onclick=\'return confirm(%s)\'>Deny Affiliate</a>', admin_url('admin.php?page=stms-settings&action=stms_memberdata_affdeny'), $item['ID'], $reject);
		}
		*/
		
		//$actions['access'] = sprintf('<a href="#members|mode=accesslog&object_ID=%d">Access Log</a>', $item['ID']);
		$actions['delete'] = sprintf('<a href="%s&ID=%d" onclick="return confirm(\'Are you sure you want to delete this member?\')">Delete</a>', admin_url('admin.php?page=stms-settings&action=stms_memberdata_delete'), $item['ID']);
		//$actions['login_as'] = sprintf('<a href="%s&ID=%d" target="_blank">Login as member</a>', admin_url('admin.php?page=stms-settings&action=stms_login_as'), $item['ID']);
		
        // Return the title contents
        return sprintf('%1$s %2$s',
            '<a href="#members|mode=edit&object_ID=' . $item['ID'] . '"><strong>' . stms_sanitize($item['title']) . '</strong></a>',
            $this->row_actions($actions)
        );
    }

	
	function column_cb($item) {
        return sprintf(
            '<input type="checkbox" name="%1$s[]" value="%2$s" />',
            /*$1%s*/ $this->_args['singular'],  //Let's simply repurpose the table's singular label
            /*$2%s*/ $item['ID']                //The value of the checkbox should be the record's id
        );
    }
	
	function get_table_classes() {
		return array( 'widefat', 'fixed', 'stms-normal-table' );
	}
	
	function get_columns() {
        $columns = array(
            'cb' => '<input type="checkbox" />', // Render a checkbox instead of text
            'title' => 'Username',
            'email' => 'Email',
            //'payments' => 'Payments',
            'affiliate' => 'Affiliate',
            'account' => 'Account',
        );
        return $columns;
    }
	
	function get_bulk_actions() {
        $actions = array(
        	'stms_bulk_member_payment' => 'Add Payment',
        	'stms_bulk_member_delpayment' => 'Remove Payment',
        	'stms_bulk_member_lock' => 'Lock Account',
        	'stms_bulk_member_unlock' => 'Unlock Account',
        	'stms_bulk_member_setaff' => 'Set as Affiliate',
        	'stms_bulk_member_unsetaff' => 'Set as Non-Affiliate',
        	'stms_bulk_member_setaffex' => 'Set as Exclusive Affiliate',
        	'stms_bulk_member_unsetaffex' => 'Set as Non-Exclusive Affiliate',
        	'stms_bulk_member_approve' => 'Approve Affiliate',
        	'stms_bulk_member_deny' => 'Deny Affiliate',
        	'stms_bulk_member_delete' => 'Delete Member'
        );
        return $actions;
    }
	
	function no_items() {
		echo 'No members found.';
	}
	
	function prepare_items() {
        global $wpdb; // This is used only if making any database queries

        $per_page = ( isset($_GET['row_num']) && is_numeric($_GET['row_num']) ) ? $_GET['row_num'] : 10;
        $columns = $this->get_columns();
        $hidden = array();
        $sortable = $this->get_sortable_columns();
        $this->_column_headers = array($columns, $hidden, $sortable);
		
		$current_page = $this->get_pagenum();
        $data = stms_tabledata_members($per_page);
		
		$total_items = $data['total'];
        $this->items = $data['data'];
        $this->set_pagination_args( array(
            'total_items' => $total_items,                  // We have to calculate the total number of items
            'per_page'    => $per_page,                     // We have to determine how many items to show on a page
            'total_pages' => ceil($total_items/$per_page)   // We have to calculate the total number of pages
        ));
    }
}