<?php
if ( !class_exists('WP_List_Table') )
    require_once( ABSPATH . 'wp-admin/includes/class-wp-list-table.php' );
    
class stmsMemberPayments extends WP_List_Table {
	function __construct() {
		global $wpdb;
		// Set parent defaults
        parent::__construct( array(
            'singular'  => 'payment',     // singular name of the listed records
            'plural'    => 'payments',    // plural name of the listed records
            'ajax'      => true        // does this table support ajax?
        ));
	}
	
	function column_default($item, $column_name) {
		$value = '';
        switch ($column_name) {
            case 'receipt':
			case 'paid':
			case 'status':
			case 'period':
            case 'amount':
			case 'processor':
                $value = $item[$column_name];
				break;
            default:
                return stms_dump($item); // Show the whole array for troubleshooting purposes
        }
		
		return $value;
    }
	
	function column_title($item) {
        // Build row actions
        $actions = array();
        $actions['edit'] = sprintf('<a href="#members|mode=payments&object_ID=%s&payment_id=%s&payment_mode=edit">Edit</a>', $_GET['object_ID'], $item['ID']);
		$actions['delete'] = sprintf('<a href="%s&action=stms_paymentdata_delete&ID=%s" onclick="return confirm(\'Are you sure want to delete this payment?\');">Delete</a>', admin_url('admin.php?page=' . $_REQUEST['page']), $item['ID']);
		
        // Return the title contents
        return sprintf('%1$s %2$s',
            $item['title'],
            $this->row_actions($actions)
        );
    }
	
	function column_cb($item) {
        return sprintf(
            '<input type="checkbox" name="%1$s[]" value="%2$s" />',
            /*$1%s*/ $this->_args['singular'],  //Let's simply repurpose the table's singular label
            /*$2%s*/ $item['ID']                //The value of the checkbox should be the record's id
        );
    }
	
	function get_columns() {
        $columns = array(
            'cb' => '<input type="checkbox" />', // Render a checkbox instead of text
            'title' => 'Product',
            'receipt' => 'Receipt',
            'period'  => 'Period',
            'processor' => 'Processor',
            'status'  => 'Status',
            'paid'  => 'Paid',
            'amount'  => 'Amount',
            
        );
        return $columns;
    }
	
	/*
	function get_sortable_columns() {
        $sortable_columns = array(
            'title' => array('title', false),     // true means it's already sorted
            'receipt'    => array('receipt', false),
            'period'    => array('period', false),
            'paid'    => array('paid', false),
            'status'    => array('status', false),
            'amount'    => array('amount', false)
        );
        return $sortable_columns;
    }
	*/
	
	function get_bulk_actions() {
        $actions = array(
            'stms_bulk_payment_delete' => 'Delete'
        );
        return $actions;
    }
	
	function process_bulk_action() {
		// Do nothing
	}
	
	function no_items() {
		echo 'No payments found.';
	}
	
	function prepare_items() {
        global $wpdb; // This is used only if making any database queries

        $per_page = 15;
        $columns = $this->get_columns();
        $hidden = array();
        $sortable = $this->get_sortable_columns();
        $this->_column_headers = array($columns, $hidden, $sortable);
        $this->process_bulk_action();
        $data = stms_tabledata_payments();
                
        usort($data, 'stms_usort_reorder');
       
        $current_page = $this->get_pagenum();
        $total_items = count($data);
        $data = array_slice($data,(($current_page-1) * $per_page), $per_page);
        $this->items = $data;
        $this->set_pagination_args( array(
            'total_items' => $total_items,                  // We have to calculate the total number of items
            'per_page'    => $per_page,                     // We have to determine how many items to show on a page
            'total_pages' => ceil($total_items/$per_page)   // We have to calculate the total number of pages
        ));
    }
}