<?php

add_action('template_redirect', 'stms_logging_in_member');
function stms_logging_in_member() {
	if ( isset($_REQUEST['_instamember_action']) && $_REQUEST['_instamember_action'] == 'login' ) {
		$login_url = stms_login_url(stms_isset($_REQUEST['redir']));
		if ( !wp_verify_nonce($_POST['_instamember_nonce'], 'stms-nonce') ) {
			wp_redirect($login_url);
			exit;
		}
		
		$logged_in = false;
		$ipaddress = stms_retrieve_ipaddr();
		$username = sanitize_user($_POST['im_username']);
		$password = trim($_POST['im_password']);
		
		do_action('stms_member_logging_in', $username, $ipaddress);
		
		$creds = array();
		$creds['member_login'] = $username;
		$creds['member_pass'] = $password;
		if ( isset($_POST['rememberme']) )
			$creds['remember'] = $_POST['rememberme'];
		
		$login = stms_login($creds);
		if ( $login ) {
			$logged_in = true;
			$redir_url = stms_member_url(stms_isset($_POST['redir']));
			do_action('stms_member_logged_in', $login, $ipaddress);
		} else {
			$logged_in = false;
			$redir_url = $login_url . '&msg=login_failed';
			if ( isset($_POST['tx_id']) )
				$redir_url .= '&tx_id=' . $_POST['tx_id'];
			if ( isset($_POST['insta_token']) )
				$redir_url .= '&insta_token=' . $_POST['insta_token'];
			if ( isset($_POST['flow_id']) )
				$redir_url .= '&flow_id=' . $_POST['flow_id'];
			
			if ( isset($_POST['tx_ids']) && is_array($_POST['tx_ids']) && count($_POST['tx_ids']) ) {
				foreach ( $_POST['tx_ids'] as $trx_id ) {
					$redir_url .= "&tx_ids[]={$trx_id}";
				}
			}
			do_action('stms_member_login_failed', $username, $ipaddress);
		}
		
		// Process transaction for flow 2,1
		// If a member isn't logged in when clicked the payment button...
		// ... and then choose to login when asked to register a new account 
		if ( $logged_in && isset($_POST['tx_id']) && isset($_POST['insta_token']) ) {
			$stms = get_option('stms_settings');
			$token = get_transient('insta_token_' . $_POST['insta_token']);
			if ( !empty($token) ) {
				global $wpdb;
				$tx_id = (int) $_POST['tx_id'];
				if ( $payment = stms_get_payment($tx_id) ) {
					if ( $payment->status == 'completed' && $payment->paid == 1 ) {
						$member = $login;
						$temp_member = stms_member($payment->member_id);
						if ( $member->ID != $payment->member_id && $temp_member->status != 1 ) {
							$temp_id = $payment->member_id;
							// let's fix the transaction data...
							$newdata = array();
							$newdata['member_id'] = $member->ID;
							
							// update the transaction data...
							stms_update_payment($tx_id, $newdata );
							
							// update ipn log
							if ( $ipn = $wpdb->get_row("SELECT `log_id` FROM {$wpdb->prefix}stms_ipn_log WHERE tx_id = {$payment->tx_id} AND receipt = '{$payment->receipt}'") )
								$update = $wpdb->update("{$wpdb->prefix}stms_ipn_log", array('status' => 1, 'member_id' => $member->ID), array('log_id' => $ipn->log_id));
							
							// fix license key data ...
							$settings = stms_get_product_meta($payment->product_id, 'license');
							if ( $settings['enable'] == 1 )
								$wpdb->update("{$wpdb->prefix}stms_licenses", array('member_id' => $member->ID), array('tx_id' => $tx_id));
				
							// remove the temp member...
							stms_delete_member($temp_id, false);
					
							// let's recall this hook again so the receipt can be sent...	
							do_action('stms_payment_completed', $tx_id);
							
							// redirect member
							$product = stms_get_product($payment->product_id);
							$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : '';
							$redir_url = ( filter_var($product->info['thankuri'], FILTER_VALIDATE_URL) ) ? $product->info['thankuri'] : $home_url;
							if ( $redir_url == '' )
								$redir_url = ( !empty($stms['global']['home']) ) ? get_permalink($stms['global']['home']) : get_bloginfo('url');
							
							// Fix upsell items...
							if ( function_exists('sflow_get_cart_items') && isset($_POST['flow_id']) ) {
								if ( isset($_POST['tx_ids']) && is_array($_POST['tx_ids']) && count($_POST['tx_ids']) ) {
									foreach ( $_POST['tx_ids'] as $trx_id ) {
										if ( $trx_id != $_POST['tx_id'] ) {
											$newdata = array();
											$newdata['member_id'] = $member->ID;
											
											// update cart
											sflow_update_member_cart($trx_id, $member->ID);
											
											// update the transaction data...
											stms_update_payment($trx_id, $newdata );
											
											// fix license key data ...
											$wpdb->update("{$wpdb->prefix}stms_licenses", array('member_id' => $member->ID), array('tx_id' => $trx_id));
							
											// let's recall this hook again so the receipt can be sent...	
											do_action('stms_payment_completed', $trx_id);
										}
									}
								}
								$redir_url = stms_format_url($stms['global']['magic'], 'insta_mode=all_access&flow_id=' . $_POST['flow_id']);
							}
							
							if ( isset($_POST['insta_token']) )
								delete_transient('insta_token_' . $_POST['insta_token']);
						}
					}
				}
			}
		}
		@header('Location:' . $redir_url);
		exit;
	}
}

add_action('template_redirect', 'stms_do_logout');
function stms_do_logout() {
	if ( isset($_GET['imhandler']) && $_GET['imhandler'] == 'logout' ) {
		$stms = get_option('stms_settings');
		if ( stms_isset($stms['global']['wp_sync']) == 1 && !is_stms_admin() )
			stms_clear_auth_cookie(true);
		else
			stms_clear_auth_cookie();
		
		do_action('stms_logout');
	}
}

add_action('stms_logout', 'stms_logout_redirect');
function stms_logout_redirect() {
	$stms = get_option('stms_settings');
	$logout_url = stms_isset($stms['global']['after_logout']);
	$redirect_url = ( filter_var($logout_url, FILTER_VALIDATE_URL) ) ? $logout_url : stms_login_url() . '&msg=logged_out';
	wp_redirect($redirect_url);
	exit;
}

add_action('stms_loggedin_cookied', 'stms_wp_login', 10, 2);
function stms_wp_login( $member_id, $ipaddress ) {
	$stms = get_option('stms_settings');
	if ( stms_isset($stms['global']['wp_sync']) != 1 || !($member = stms_member($member_id)) )
		return false;
	
	if ( $member->wpuser_id == 0 || is_stms_admin() )
		return false;
	
	if ( is_user_logged_in() )
		wp_clear_auth_cookie();
	
	wp_set_auth_cookie($member->wpuser_id, false, is_ssl());
}

add_action('stms_clear_auth_cookie', 'stms_wp_logout');
function stms_wp_logout() {
	$stms = get_option('stms_settings');
	if ( stms_isset($stms['global']['wp_sync']) != 1 || is_stms_admin() )
		return false;
	
	wp_clear_auth_cookie();
}

add_action('wp_login', 'stms_sync_wplogin', 10, 2);
function stms_sync_wplogin( $username, $user ) {
	$stms = get_option('stms_settings');
	if ( stms_isset($stms['global']['wp_sync']) != 1 )
		return false;
	
	if ( is_stmsmember_logged_in() && is_stms_admin() ) return false;
	
	global $wpdb;
	$wpuser_id = intval($user->ID);
	if ( $member = $wpdb->get_row("SELECT * FROM `{$wpdb->prefix}stms_members` WHERE `wpuser_id` = {$wpuser_id}") ) {
		if ( is_stmsmember_logged_in() ) stms_clear_auth_cookie();
		
		$remember = ( !empty($_POST['rememberme']) ) ? true : false;
		stms_set_auth_cookie($member->ID, $remember, is_ssl());
	}
}

add_action('wp_logout', 'stms_sync_wplogout');
function stms_sync_wplogout() {
	$stms = get_option('stms_settings');
	if ( stms_isset($stms['global']['wp_sync']) != 1 )
		return false;
	
	stms_clear_auth_cookie(true);
}

add_action('template_redirect', 'stms_protect_member_home', 9);
function stms_protect_member_home() {
	if ( is_category() || is_tag() || is_archive() ) {
		return false;
	}

	global $post;
	$stms = get_option('stms_settings');
	
	if ( is_stms_admin() && stms_isset($stms['global']['admin_unlock']) == 1 )
		return false;
	
	$global_home = stms_isset($stms['global']['home'], 0);
	if ( $global_home != $post->ID )
		return false;
	
	$login_url = stms_login_url();
	if ( !empty($stms['global']['non_redirect']) && filter_var($stms['global']['non_redirect'], FILTER_VALIDATE_URL) ) {
		$login_url = $stms['global']['non_redirect'];
	}
	
	if ( is_stmsmember_logged_in() ) {
		$m = stms_get_current_member();
		if ( $m->status == 4 ) {
			@stms_clear_auth_cookie();
			wp_redirect($login_url);
			exit;
		}
		return false;
	}
		
	
	wp_redirect($login_url);
	exit;
}

add_action('template_redirect', 'stms_post_categories_protection', 11);
function stms_post_categories_protection() {
	if ( !is_single() || !($cats = stms_protected_categories()) )
		return false;
	
	global $wpdb, $post;
	$stms = get_option('stms_settings');
	if ( stms_is_protected($post->ID) )
		return false; // let the single post protection do the job...
	
	$in_cats = false;
	foreach ( $cats as $cat ) {
		if ( in_category($cat, $post->ID) ) {
			if ( !is_array($in_cats) ) $in_cats = array();
			$in_cats[] = $cat;
		}
	}
	
	if ( is_stms_admin() && stms_isset($stms['global']['admin_unlock']) == 1 || !$in_cats )
		return false;
		
	$proto = ( is_ssl() ) ? 'https://' : 'http://';
	$redir = get_permalink($post->ID);
	$login_url = stms_login_url($redir);
	if ( !is_stmsmember_logged_in() ) {
		// immediately redirect to login page
		if ( !empty($stms['global']['non_redirect']) && filter_var($stms['global']['non_redirect'], FILTER_VALIDATE_URL) ) {
			$login_url = $stms['global']['non_redirect'];
		}
		wp_redirect($login_url);
		exit;
	}
	
	foreach ( $in_cats as $cat ) {
		$access = stms_check_category_access($cat);
		if ( $access === TRUE )
			return false;
	}
	
	if ( $access === FALSE ) {
		if ( !empty($stms['global']['noauth_redirect']) && filter_var($stms['global']['noauth_redirect'], FILTER_VALIDATE_URL) ) {
			@header('Location: ' . trim($stms['global']['noauth_redirect']));
			exit;
		} else 
			wp_die('Unauthorize Access.');
	}
	
	if ( $access === 'expired' ) {
		if ( !empty($stms['global']['expired_redirect']) && filter_var($stms['global']['expired_redirect'], FILTER_VALIDATE_URL) ) {
			@header('Location: ' . trim($stms['global']['expired_redirect']));
			exit;
		} else 
			wp_die('Content Expired.');
	}
	
	if ( $access === 'unreleased' ) {
		if ( !empty($stms['global']['unreleased_redirect']) && filter_var($stms['global']['unreleased_redirect'], FILTER_VALIDATE_URL) ) {
			@header('Location: ' . trim($stms['global']['unreleased_redirect']));
			exit;
		} else 
			wp_die('Content hasn\'t been released.');
	}
}

function stms_protected_categories() {
	global $wpdb;
	$cats = $wpdb->get_results($wpdb->prepare("SELECT `cat_id` FROM `{$wpdb->prefix}stms_protects` WHERE `type` = %s ORDER BY `cat_id` ASC", 'categories'));
	if ( !$cats )
		return false;
	
	$categories = array();
	foreach ( $cats as $cat ) {
		$categories[] = $cat->cat_id;
	}
	return $categories;
}

add_action('template_redirect', 'stms_categories_protection', 11);
function stms_categories_protection() {
	global $wp_query;
	if ( !is_category() || !isset($wp_query->query_vars) )
		return false;
	
	$vars = $wp_query->query_vars;
	$cat_id = $vars['cat'];
	
	if ( !stms_is_protected($cat_id, 'categories') )
		return false;
		
	if ( is_stms_admin() && stms_isset($stms['global']['admin_unlock']) == 1 )
		return false;
		
	$proto = ( is_ssl() ) ? 'https://' : 'http://';
	$redir = get_category_link($cat_id);
	$login_url = stms_login_url($redir);
	if ( !is_stmsmember_logged_in() ) {
		// immediately redirect to login page
		if ( !empty($stms['global']['non_redirect']) && filter_var($stms['global']['non_redirect'], FILTER_VALIDATE_URL) ) {
			$login_url = $stms['global']['non_redirect'];
		}
		wp_redirect($login_url);
		exit;
	}
	
	// member is logged in... let's check if it's a temporary account...
	$m = stms_get_current_member();
	if ( $m->status == 4 ) {
		@stms_clear_auth_cookie();
		wp_redirect($login_url);
		exit;
	}
		
	$access = stms_check_category_access($cat_id);
	if ( $access === FALSE )
		wp_die('Unauthorize Access.');
	
	if ( $access === 'expired' )
		wp_die('Content Expired.');
	
	if ( $access === 'unreleased' )
		wp_die('Content hasn\'t been released.');
}

add_action('template_redirect', 'stms_posts_protection', 12);
function stms_posts_protection() {
	@set_time_limit(0);
	
	global $post;

	$stms = get_option('stms_settings');
	$slugs = get_option('stms_slugs');
	$login_id = ( isset($slugs['login']) && $slugs['login'] > 0 ) ? $slugs['login'] : 0;
	if ( is_category() || is_tag() || is_archive() || !stms_is_protected($post->ID) || ($stms['global']['magic'] == $post->ID) || ($login_id == $post->ID) || ($stms['global']['home'] == $post->ID) )
		return false;
	
	if ( is_stms_admin() && stms_isset($stms['global']['admin_unlock']) == 1 )
		return false;
		
	$proto = ( is_ssl() ) ? 'https://' : 'http://';
	$redir = get_permalink($post->ID);
	$login_url = stms_login_url($redir);
	if ( !is_stmsmember_logged_in() ) {
		// immediately redirect to login page
		if ( !empty($stms['global']['non_redirect']) && filter_var($stms['global']['non_redirect'], FILTER_VALIDATE_URL) ) {
			$login_url = $stms['global']['non_redirect'];
		}
		wp_redirect($login_url);
		exit;
	}
	
	// member is logged in... let's check if it's a temporary account...
	$m = stms_get_current_member();
	if ( $m->status == 4 ) {
		@stms_clear_auth_cookie();
		wp_redirect($login_url);
		exit;
	}
		
	$access = stms_check_access();
	if ( $access === FALSE ) {
		if ( !empty($stms['global']['noauth_redirect']) && filter_var($stms['global']['noauth_redirect'], FILTER_VALIDATE_URL) ) {
			@header('Location: ' . trim($stms['global']['noauth_redirect']));
			exit;
		} else 
			wp_die('Unauthorize Access.');
	}
	
	if ( $access === 'expired' ) {
		if ( !empty($stms['global']['expired_redirect']) && filter_var($stms['global']['expired_redirect'], FILTER_VALIDATE_URL) ) {
			@header('Location: ' . trim($stms['global']['expired_redirect']));
			exit;
		} else 
			wp_die('Content Expired.');
	}
	
	if ( $access === 'unreleased' ) {
		if ( !empty($stms['global']['unreleased_redirect']) && filter_var($stms['global']['unreleased_redirect'], FILTER_VALIDATE_URL) ) {
			@header('Location: ' . trim($stms['global']['unreleased_redirect']));
			exit;
		} else 
			wp_die('Content hasn\'t been released.');
	}
}

function stms_check_access( $type = 'content', $is_s3 = false, $post_id = 0 ) {
	global $wpdb, $post;
	
	$member = stms_get_current_member();
	if ( !is_object($member) )
		return false;
	
	$post_id = ( $post_id ) ? $post_id : $post->ID;
	$sql = "SELECT * FROM `{$wpdb->prefix}stms_protects` WHERE";
	$sql .= ( $type == 'file' ) ? " `filename` = %s" : " `post_id` = %d";
	if ( $type == 'file' ) {
		if ( $is_s3 )
			$sql .= " AND `type` = 's3file'";
		else
			$sql .= " AND `type` = 'file'";
	}
	
	$identifier = ( $type == 'file' ) ? $_GET['insta_file'] : stms_isset($post_id);
	if ( !$protects = $wpdb->get_results($wpdb->prepare($sql, $identifier)) ) {
		if ( $type == 'file' )
			return false;
		else
			return true; // allow member to view the content because this post/page isn't protected
	}
	
	$allowed_products = array();
	if ( !$payments = stms_get_member_payments($member->ID) )
		return false; // do not allow member to view the content because he/she haven't paid for anything
	
	foreach ( $protects as $protect ) {
		$allowed_products[] = $protect->product_id;
	}
	
	$access = false;
	foreach ( $payments as $payment ) {
		if ( in_array($payment->product_id, $allowed_products) ) {
			$today = stms_time();
			$expired = $payment->expired_date;
			if ( $payment->status == 'completed' ) {
				$is_expired = false;
				if ( $expired > 0 ) {
					if ( $today > $expired ) {
						$new_data = array();
						$new_data['status'] = 'expired';
						stms_update_payment($payment->tx_id, $new_data);
						wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
						$is_expired = true;
					}
				}
				
				if ( !$is_expired ) {
					$access = true;
					break;
				}
			} else if ( $payment->status == 'cancelled' ) {
				if ( $expired > $today ) {
					// member still have few more days/weeks left before expired...
					$access = true;
					break;
				} else {
					// change the status to expired...
					$new_data = array();
					$new_data['status'] = 'expired';
					stms_update_payment($payment->tx_id, $new_data);
					wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
				}
			}
		}
	}
	
	// Let's check for drip-feed...
	$product_ids = array();
	foreach ( $payments as $payment ) {
		if ( $payment->status == 'completed' )
			$product_ids[] = $payment->product_id;
	}
	
	$start = array();
	$expired = array();	
	$product_ids = array_unique($product_ids);
	foreach ( $protects as $protect ) {
		if ( in_array($protect->product_id, $product_ids) ) {
			$first_payment = stms_get_firstpayment($member->ID, $protect->product_id);
			$first_pay = $first_payment->date;
			$start_str = '+' . $protect->drip_start . ' ' . $protect->drip_start_mode . 's';
			$end_str = '+' . $protect->drip_end . ' ' . $protect->drip_end_mode . 's';
			$start[] = ( $protect->drip_start_mode == 'exact' ) ? strtotime($protect->drip_start) : ( ($protect->drip_start > 0) ? strtotime($start_str, $first_pay) : stms_time() );
			if ( $protect->is_expired == 1 ) {
				$last_payment = stms_get_lastpayment($member->ID, $protect->product_id);
				$last_pay = $last_payment->date;
				$expired[] = ( $protect->drip_end_mode == 'exact' ) ? strtotime($protect->drip_end) : ( ($protect->drip_end > 0) ? strtotime($end_str, $last_pay) : stms_time() );
			}
		}
	}
	asort($start);
	arsort($expired);
	
	$now = stms_time();
	// Check if content is already expired...
	if ( isset($expired[0]) && $expired[0] > 0 && $expired[0] < $now )
		$access = 'expired';
	
	// Check if content is already released...
	if ( isset($start[0]) && $start[0] > $now )
		$access = 'unreleased';
	
	return $access;
}

function stms_check_category_access( $cat_id = 0 ) {
	global $wpdb;
	
	if ( empty($cat_id) )
		return true;
	
	$member = stms_get_current_member();
	if ( !is_object($member) )
		return false;
	
	$sql = "SELECT * FROM `{$wpdb->prefix}stms_protects` WHERE `cat_id` = %d";
	if ( !$protects = $wpdb->get_results($wpdb->prepare($sql, $cat_id)) )
		return true; // allow member to view the content because this category isn't protected
	
	$allowed_products = array();
	if ( !$payments = stms_get_member_payments($member->ID) )
		return false; // do not allow member to view the content because he/she haven't paid for anything
	
	foreach ( $protects as $protect ) {
		$allowed_products[] = $protect->product_id;
	}
	
	$access = false;
	foreach ( $payments as $payment ) {
		if ( in_array($payment->product_id, $allowed_products) ) {
			$today = stms_time();
			$expired = $payment->expired_date;
			if ( $payment->status == 'completed' ) {
				$is_expired = false;
				if ( $expired > 0 ) {
					if ( $today > $expired ) {
						$new_data = array();
						$new_data['status'] = 'expired';
						stms_update_payment($payment->tx_id, $new_data);
						wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
						$is_expired = true;
					}
				}
				
				if ( !$is_expired ) {
					$access = true;
					break;
				}
			} else if ( $payment->status == 'cancelled' ) {
				if ( $expired > $today ) {
					// member still have few more days/weeks left before expired...
					$access = true;
					break;
				} else {
					// change the status to expired...
					$new_data = array();
					$new_data['status'] = 'expired';
					stms_update_payment($payment->tx_id, $new_data);
					wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
				}
			}
		}
	}
	
	// Let's check for drip-feed...
	$product_ids = array();
	foreach ( $payments as $payment ) {
		if ( $payment->status == 'completed' )
			$product_ids[] = $payment->product_id;
	}
	
	$start = array();
	$expired = array();	
	$product_ids = array_unique($product_ids);
	foreach ( $protects as $protect ) {
		if ( in_array($protect->product_id, $product_ids) ) {
			$first_payment = stms_get_firstpayment($member->ID, $protect->product_id);
			$first_pay = $first_payment->date;
			$start_str = '+' . $protect->drip_start . ' ' . $protect->drip_start_mode . 's';
			$end_str = '+' . $protect->drip_end . ' ' . $protect->drip_end_mode . 's';
			$start[] = ( $protect->drip_start_mode == 'exact' ) ? strtotime($protect->drip_start) : ( ($protect->drip_start > 0) ? strtotime($start_str, $first_pay) : stms_time() );
			if ( $protect->is_expired == 1 ) {
				$last_payment = stms_get_lastpayment($member->ID, $protect->product_id);
				$last_pay = $last_payment->date;
				$expired[] = ( $protect->drip_end_mode == 'exact' ) ? strtotime($protect->drip_end) : ( ($protect->drip_end > 0) ? strtotime($end_str, $last_pay) : stms_time() );
			}
		}
	}
	asort($start);
	arsort($expired);
	
	$now = stms_time();
	// Check if content is already expired...
	if ( isset($expired[0]) && $expired[0] > 0 && $expired[0] < $now )
		$access = 'expired';
	
	// Check if content is already released...
	if ( isset($start[0]) && $start[0] > $now )
		$access = 'unreleased';
	
	return $access;
}

function stms_check_s3video_access( $filename ) {
	global $wpdb, $post;
	
	$stms = get_option('stms_settings');
	if ( is_stms_admin() && stms_isset($stms['global']['admin_unlock']) == 1 )
		return true;
	
	$member = stms_get_current_member();
	if ( !is_object($member) )
		return false;
	
	$sql = "SELECT * FROM `{$wpdb->prefix}stms_protects` WHERE `filename` = %s AND `type` = 's3file'";
	if ( !$protects = $wpdb->get_results($wpdb->prepare($sql, $filename)) )
		return false;
	
	$allowed_products = array();
	if ( !$payments = stms_get_member_payments($member->ID) )
		return false; // do not allow member to view the content because he/she haven't paid for anything
	
	foreach ( $protects as $protect ) {
		$allowed_products[] = $protect->product_id;
	}
	
	$access = false;
	foreach ( $payments as $payment ) {
		if ( in_array($payment->product_id, $allowed_products) ) {
			$today = stms_time();
			$expired = $payment->expired_date;
			if ( $payment->status == 'completed' ) {
				$is_expired = false;
				if ( $expired > 0 ) {
					if ( $today > $expired ) {
						$new_data = array();
						$new_data['status'] = 'expired';
						stms_update_payment($payment->tx_id, $new_data);
						wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
						$is_expired = true;
					}
				}
				
				if ( !$is_expired ) {
					$access = true;
					break;
				}
			} else if ( $payment->status == 'cancelled' ) {
				if ( $expired > $today ) {
					// member still have few more days/weeks left before expired...
					$access = true;
					break;
				} else {
					// change the status to expired...
					$new_data = array();
					$new_data['status'] = 'expired';
					stms_update_payment($payment->tx_id, $new_data);
					wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
				}
			}
		}
	}
	
	// Let's check for drip-feed...
	$product_ids = array();
	foreach ( $payments as $payment ) {
		if ( $payment->status == 'completed' )
			$product_ids[] = $payment->product_id;
	}
	
	$start = array();
	$expired = array();	
	$product_ids = array_unique($product_ids);
	foreach ( $protects as $protect ) {
		if ( in_array($protect->product_id, $product_ids) ) {
			$first_payment = stms_get_firstpayment($member->ID, $protect->product_id);
			$first_pay = $first_payment->date;
			$start_str = '+' . $protect->drip_start . ' ' . $protect->drip_start_mode . 's';
			$end_str = '+' . $protect->drip_end . ' ' . $protect->drip_end_mode . 's';
			$start[] = ( $protect->drip_start_mode == 'exact' ) ? strtotime($protect->drip_start) : ( ($protect->drip_start > 0) ? strtotime($start_str, $first_pay) : stms_time() );
			if ( $protect->is_expired == 1 ) {
				$last_payment = stms_get_lastpayment($member->ID, $protect->product_id);
				$last_pay = $last_payment->date;
				$expired[] = ( $protect->drip_end_mode == 'exact' ) ? strtotime($protect->drip_end) : ( ($protect->drip_end > 0) ? strtotime($end_str, $last_pay) : stms_time() );
			}
		}
	}
	asort($start);
	arsort($expired);
	
	$now = stms_time();
	// Check if content is already expired...
	if ( isset($expired[0]) && $expired[0] > 0 && $expired[0] < $now )
		$access = 'expired';
	
	// Check if content is already released...
	if ( isset($start[0]) && $start[0] > $now )
		$access = 'unreleased';
	
	return $access;
}

function stms_check_shortcode_access( $product_ids ) {
	global $post;
	
	$stms = get_option('stms_settings');
	if ( is_stms_admin() && stms_isset($stms['global']['admin_unlock']) == 1 )
		return true;
	
	$member = stms_get_current_member();
	if ( !$member || !is_object($member) )
		return false;
	
	if ( !$payments = stms_get_member_payments($member->ID) )
		return false; // do not allow member to view the content because he/she haven't paid for anything
	
	$allowed_products = explode(",", $product_ids);
	$access = false;
	foreach ( $payments as $payment ) {
		if ( in_array($payment->product_id, $allowed_products) ) {
			$today = stms_time();
			$expired = $payment->expired_date;
			if ( $payment->status == 'completed' ) {
				$is_expired = false;
				if ( $expired > 0 ) {
					if ( $today > $expired ) {
						$new_data = array();
						$new_data['status'] = 'expired';
						stms_update_payment($payment->tx_id, $new_data);
						wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
						$is_expired = true;
					}
				}
				
				if ( !$is_expired ) {
					$access = true;
					break;
				}
			} else if ( $payment->status == 'cancelled' ) {
				if ( $expired > $today ) {
					// member still have few more days/weeks left before expired...
					$access = true;
					break;
				} else {
					// change the status to expired...
					$new_data = array();
					$new_data['status'] = 'expired';
					stms_update_payment($payment->tx_id, $new_data);
					wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
				}
			}
		}
	}
	
	return $access;
}

function stms_check_shortcode_expired( $product_id, $expired = 0, $mode = 'day', $upgrade_id = 0 ) {
	global $post;
	
	$stms = get_option('stms_settings');
	if ( is_stms_admin() && stms_isset($stms['global']['admin_unlock']) == 1 )
		return false;
	
	$member = stms_get_current_member();
	if ( !$member || !is_object($member) )
		return false;
	
	if ( !$payments = stms_get_member_payments($member->ID) )
		return false;
	
	// Let's check for expired...
	$p = false;
	$expire_date = 0;
	foreach ( $payments as $payment ) {
		if ( $payment->product_id == $product_id ) {
			$end_str = '+' . $expired . ' ' . $mode . 's';
			if ( $expired > 0 ) {
				$last_payment = stms_get_lastpayment($member->ID, $payment->product_id);
				$last_pay = $last_payment->date;
				$expire_date = strtotime($end_str, $last_pay);
				$p = $payment;
				break;
			}
		}
	}
	
	$now = stms_time();
	$daysleft = ( $expire_date > $now ) ? round(($expire_date - $now) / 86400) : 0;
	$status = ( isset($expire_date) && $expire_date > 0 && $expire_date < $now ) ? 'expired' : 'active';
	if ( $status == 'expired' && $upgrade_id > 0 ) {
		// let's check if this member already upgrades...
		foreach ( $payments as $payment ) {
			if ( $payment->product_id == $upgrade_id ) {
				$end_str = '+' . $expired . ' ' . $mode . 's';
				if ( $expired > 0 ) {
					$last_payment = stms_get_lastpayment($member->ID, $payment->product_id);
					$last_pay = $last_payment->date;
					$expire_date = strtotime($end_str, $last_pay);
					$p = $payment;
					break;
				}
			}
		}
		$status = ( isset($expire_date) && $expire_date > 0 && $expire_date < $now ) ? 'expired' : 'active';
		$daysleft = ( $expire_date > $now ) ? round(($expire_date - $now) / 86400) : 0;
	}
	
	return array(
		'status' => $status,
		'purchase_date' => ( isset($p->date) ) ? $p->date : 0,
		'remaining' => $daysleft
	);
}

add_filter('pre_get_posts', 'stms_filter_posts', 102);
function stms_filter_posts( $content ) {
	global $wpdb;

	$stms = get_option('stms_settings');
	if ( is_stms_admin() && stms_isset($stms['global']['admin_unlock']) == 1 )
		return false;
	
	$member = stms_get_current_member();
	if ( is_singular() )
		return $content;
	
	if ( !$posts = $wpdb->get_results("SELECT ID FROM {$wpdb->posts} WHERE post_status = 'publish'") )
		return $content;

	$excludes = array();
	$cats = stms_protected_categories();
	foreach ( $posts as $post ) {
		if ( $cats ) {
			foreach ( $cats as $cat ) { // category protection
				if ( in_category($cat, $post->ID) ) {
					if ( !stms_is_protected($post->ID) ) { // the post isn't protected by the category of the post is protected...
						if ( !is_stmsmember_logged_in() ) {
							$excludes[] = $post->ID;
						} else {
							$access = stms_check_category_access($cat);
							if ( $access === TRUE ) continue;
							if ( $access === FALSE || $access === 'expired' || $access === 'unreleased' ) {
								$excludes[] = $post->ID;
							}
						}
					}
				}	
			}
		}
		
		$protected = stms_is_protected($post->ID); // single post/page protection
		if ( $protected ) {
			if ( !is_stmsmember_logged_in() ) {
				$excludes[] = $post->ID;
			} else {
				$access = stms_check_access('content', false, $post->ID);
				if ( $access === FALSE || $access === 'expired' || $access === 'unreleased' ) {
					$excludes[] = $post->ID;
				}
			}
		}
	}
	
	if ( count($excludes) > 0 ) {
		$excludes = array_unique($excludes);
		if ( !empty($content->query_vars['post__not_in']) ) {
			$pre = $content->query_vars['post__not_in'];
			if ( !is_array($pre) )
				$pre = explode(",", $pre);
			
			$excludes = array_merge($excludes, $pre);
		}
		$content->query_vars['post__not_in'] = $excludes;
	}
	
	return $content;
}

function stms_is_protected( $post_id = '', $type = 'content' ) {
	global $wpdb, $post;
	$ID = ( !empty($post_id) ) ? (int) $post_id : $post->ID;
	$col = ( $type == 'categories' ) ? 'cat_id' : 'post_id';
	$protect = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM `{$wpdb->prefix}stms_protects` WHERE `type` = %s AND {$col} = %d", $type, $ID));
	if ( $protect > 0 ) return true;
	return false;
}

add_action('stms_loggedin_cookied', 'stms_log_access', 10, 2);
function stms_log_access($member_id, $ipaddress) {
	global $wpdb;
	
	if ( $member = stms_member($member_id) ) {
		if ( $member->status == 4 )
			return false;
		
		$data = array(
			'member_id' => $member->ID,
			'url' => stms_isset($_SERVER['REQUEST_URI']),
			'ip_addr' => $ipaddress,
			'referrer' => stms_isset($_SERVER['HTTP_REFERER']),
			'date' => stms_time()
		);
		$wpdb->insert("{$wpdb->prefix}stms_access_log", $data);
	}
}

add_action('stms_member_logging_in', 'stms_check_auto_lock', 11, 2);
function stms_check_auto_lock($username, $ipaddress) {
	if ( !$member = stms_get_data_by('login', $username) ) return false;
	if ( $member->is_locked == 'yes' ) {
		$login_url = stms_login_url(stms_isset($_POST['redir']));
		wp_redirect($login_url . '&msg=auto_locked');
		exit;
	}
}

add_action('stms_member_logging_in', 'stms_check_multiple_ip', 12, 2);
function stms_check_multiple_ip($username, $ipaddress) {
	if ( isset($_COOKIE['__insta_ipchecked']) ) return false;
	if ( !$member = stms_get_data_by('login', $username) ) return false;
	if ( $member->is_locked == 'disable' ) return false;
	
	global $wpdb;
	$stms = get_option('stms_settings');
	$mustlock = stms_isset($stms['global']['noshare_lock']);
	$max_count = stms_isset($stms['global']['noshare_ipcount'], 0);
	$minutes = stms_isset($stms['global']['noshare_ipcountmin'], 0);
	
	if ( $max_count <= 0 || $minutes <= 0 )
		return false;
	
	$start = stms_time() - $minutes * 60;
    $end = stms_time();
	$ip = esc_sql($ipaddress);
	$member_id = intval($member->ID);
	$count = $wpdb->get_var(
		"SELECT COUNT(DISTINCT ip_addr) FROM {$wpdb->prefix}stms_access_log
		WHERE ip_addr <> '{$ip}' AND member_id = {$member_id}
		AND date BETWEEN {$start} AND {$end}"
	);
	
	@setcookie('__insta_ipchecked', 1, 0, SITECOOKIEPATH);
	if ( SITECOOKIEPATH != COOKIEPATH )
		@setcookie('__insta_ipchecked', 1, 0, COOKIEPATH);
	
	if ( $count < $max_count )
		return false;
	
	// limit exceeds...
	if ( $mustlock )
		stms_update_member($member->ID, array('is_locked' => 'yes'));
	
	// email notification goes here...
	if ( stms_isset($stms['global']['noshare_admin']) == 1 ) {
		$to = sanitize_email($stms['emails']['admin_email']);
		$from = 'InstaMember <instabot@' . $_SERVER['HTTP_HOST'] . '>';
		
		$subject = get_bloginfo('name') . '- Account Sharing Violation Detected';
		$body = "This is an automated message from InstaMember.\n\n". 
				"An account sharing violation has been detected\n".
				"for the following member:\n\n".
    			"Username: {$member->username}\n".
    			"Email: {$member->email}\n\n";
				
		if ( $mustlock )
			$body .= "Account has been locked by InstaMember.\n\n";
		
		$body .= "Configured login limit from {$max_count} IP within\n".
				"{$minutes} minutes has been reached.\n\n".
				get_bloginfo('name') . "\n\n";
				
		stms_mail($to, $subject, $body, $from);
	}
}

add_action('stms_member_logging_in', 'stms_check_bruteforce', 10, 2);
function stms_check_bruteforce($username, $ipaddress) {
	require_once(STMS_CLASSES . 'stmsBruteForce.php');
	$stms = get_option('stms_settings');
	$max_attempt = stms_isset($stms['global']['brute_attempt']);
	$delay = stms_isset($stms['global']['brute_delay']);

	$brute = new stmsBruteForce($max_attempt, $delay, $ipaddress);
	if ( $brute->checkBrute() ) {
		$login_url = stms_login_url(stms_isset($_POST['redir']));
		wp_redirect($login_url . '&msg=temp_locked');
		exit;
	}
}

add_action('stms_member_logging_in', 'stms_validate_login_recaptcha', 20, 2);
function stms_validate_login_recaptcha($username, $ipaddress) {
	if ( !isset($_POST['recaptcha_challenge_field']) && !isset($_POST['recaptcha_response_field']) ) return false;
	require_once( STMS_SOURCES . '/recaptcha/recaptchalib.php');
	$stms = get_option('stms_settings');
	$privatekey = trim(stms_isset($stms['integration']['recaptcha']['private']));
	$resp = recaptcha_check_answer ($privatekey, $_SERVER["REMOTE_ADDR"], $_POST['recaptcha_challenge_field'], $_POST['recaptcha_response_field']);
	if ( !$resp->is_valid ) {
		stms_count_login_failed($username, $ipaddress);
		$login_url = stms_login_url(stms_isset($_POST['redir']));
		wp_redirect($login_url . '&msg=recaptcha_failed');
		exit;
  	}
}

add_action('stms_member_login_failed', 'stms_count_login_failed', 10, 2);
function stms_count_login_failed($username, $ipaddress) {
	require_once(STMS_CLASSES . 'stmsBruteForce.php');
	$stms = get_option('stms_settings');
	$max_attempt = stms_isset($stms['global']['brute_attempt']);
	$delay = stms_isset($stms['global']['brute_delay']);
	
	// store login failed data
	$brute = new stmsBruteForce($max_attempt, $delay, $ipaddress);
	$brute->setLoginFailed();
}

add_action('stms_member_logged_in', 'stms_clear_failed_login', 11, 2);
function stms_clear_failed_login($login, $ipaddress) {
	delete_transient('stms_brute_' . $ipaddress);
}

add_action('template_redirect', 'stms_forgot_password_request', 2);
function stms_forgot_password_request() {
	if ( isset($_POST['_instamember_action']) && $_POST['_instamember_action'] == 'forgotpass' ) {
		if ( !wp_verify_nonce($_POST['_instamember_nonce'], 'stms-nonce') ) wp_die('Invalid Handler');
		
		$auth_vars = base64_encode(json_encode($_POST));
		$stms = get_option('stms_settings');
		$forgot_url = stms_format_url(stms_isset($stms['global']['magic']), 'imhandler=forgot');
		$error_url = $forgot_url . "&auth_vars={$auth_vars}";
		
		$email = stms_sanitize($_POST['im_forgot_email']);
		$email = sanitize_email($email);
		if ( empty($email) ) {
			$auth_error = base64_encode('Please enter a valid email address.');
			$error_url .= "&auth_error={$auth_error}";
			wp_redirect($error_url);
			exit;
		}

		if ( !$member = stms_get_data_by('email', $email) ) {
			$auth_error = base64_encode('Email "' . $email . '" is not registered.');
			$error_url .= "&auth_error={$auth_error}";
			wp_redirect($error_url);
			exit;
		}
		
		$forgot_key = base64_encode($member->ID . '_' . time());
		set_transient('stms_forgot_' . $member->ID, $forgot_key, 60*60*24);
		
		// send the forgot email...
		$defaults = stms_default_emails();
		$to = sanitize_email($member->email);
		$from = stms_cleanup(stms_isset($stms['emails']['admin_from'])) . ' <' . stms_isset($stms['emails']['admin_email']) . '>';
		$from = ( $from == '' ) ? stms_cleanup(get_option('blogname')) . ' <' . get_option('admin_email') . '>' : $from;
		$from = ( isset($stms['email_templates']['forgot']) ) ? stms_cleanup($stms['email_templates']['forgot']['fromname']) . ' <' . $stms['email_templates']['forgot']['frommail'] . '>' : $from;
	
		$subject = ( isset($stms['email_templates']['forgot']) ) ? stms_cleanup($stms['email_templates']['forgot']['subject']) : $defaults['forgot']['subject'];
		$subject = stms_parse_mail_tags($subject, '', $member->ID);
		
		$body = ( isset($stms['email_templates']['forgot']) ) ? stms_cleanup($stms['email_templates']['forgot']['body']) : $defaults['forgot']['body'];
		$body = stms_parse_mail_tags($body, '', $member->ID);
		
		stms_mail($to, $subject, $body, $from);
		
		wp_redirect($forgot_url . '&requested=true');
		exit;
	}
}

add_action('template_redirect', 'stms_reset_member_password', 2);
function stms_reset_member_password() {
	if ( isset($_POST['_instamember_action']) && $_POST['_instamember_action'] == 'resetpass' ) {
		if ( !wp_verify_nonce($_POST['_instamember_nonce'], 'stms-nonce') ) wp_die('Invalid Handler');
		
		$stms = get_option('stms_settings'); 
		$login_url = stms_login_url();
		$error_url = stms_format_url(stms_isset($stms['global']['magic']), 'imhandler=resetpass') . '&email=' . urlencode($_POST['_email']) . '&key=' . urlencode($_POST['_key']);
		
		$password1 = trim($_POST['im_reset_pass1']);
		$password2 = trim($_POST['im_reset_pass2']);
		if ( empty($password1) ) {
			$auth_error = base64_encode('Please enter a new password.');
			$error_url .= "&auth_error={$auth_error}";
			wp_redirect($error_url);
			exit;
		}
		
		if ( empty($password2) ) {
			$auth_error = base64_encode('Please repeat your new password.');
			$error_url .= "&auth_error={$auth_error}";
			wp_redirect($error_url);
			exit;
		}

		if ( $password1 != $password2 ) {
			$auth_error = base64_encode('Password does not match.');
			$error_url .= "&auth_error={$auth_error}";
			wp_redirect($error_url);
			exit;
		}
		
		$member = stms_get_data_by('email', $_POST['_email']);
		// let's reset the password...
		$data = array('pass' => $password1);
		if ( !$member_id = stms_update_member($member->ID, $data) ) {
			$auth_error = base64_encode('Password reset failed. Please try again later.');
			$error_url .= "&auth_error={$auth_error}";
			wp_redirect($error_url);
		}
		
		// logging in member
		@stms_clear_auth_cookie();
		@stms_set_auth_cookie($member->ID, false, is_ssl());
		
		$profile_url = stms_get_handler_url('profile');
		wp_redirect($profile_url);
		exit;
	}
}

add_action('template_redirect', 'stms_update_profile', 2);
function stms_update_profile() {
	if ( isset($_POST['_instamember_action']) && $_POST['_instamember_action'] == 'update_profile' ) {
		if ( !wp_verify_nonce($_POST['_instamember_nonce'], 'stms-nonce') ) wp_die('Invalid Handler');
		
		$auth_vars = base64_encode(json_encode($_POST));
		
		$stms = get_option('stms_settings'); 
		$profile_url = stms_get_handler_url('profile');
		$error_url = stms_format_url($profile_url, "auth_vars={$auth_vars}");

		$member_id = (int) $_POST['insta_member_id'];
		$member = stms_member($member_id);
		$email = sanitize_email(trim($_POST['im_reg_email']));
		$old_email = sanitize_email(trim($_POST['insta_old_email']));
		if ( empty($email) ) {
			$auth_error = base64_encode('Please enter a valid email address.');
			$error_url .= "&auth_error={$auth_error}";
			wp_redirect($error_url);
			exit;
		}
		
		if ( ($email != $old_email) && stms_email_exists($email) ) {
			$auth_error = base64_encode($email . '" is already registered.');
			$error_url .= "&auth_error={$auth_error}";
			wp_redirect($error_url);
			exit;
		}
		
		$data = array();
		if ( !empty($_POST['im_ch_pass1']) && !empty($_POST['im_ch_pass2']) ) {
			if ( $_POST['im_ch_pass1'] != $_POST['im_ch_pass2'] ) {
				$auth_error = base64_encode('Please enter the same password in the new password fields.');
				$error_url .= "&auth_error={$auth_error}";
				wp_redirect($error_url);
				exit;
			}
			$data['pwd1'] = $_POST['im_ch_pass1'];
		}
		
		$data['member_id'] = $member_id;
		$data['username'] = stms_sanitize($member->username);
		$data['email'] = $email;
		$data['status'] = 1;
		$data['first_name'] = trim(stms_isset($_POST['im_reg_first_name']));
		$data['last_name'] = trim(stms_isset($_POST['im_reg_last_name']));
		$data['street'] = trim(stms_isset($_POST['im_reg_street']));
		$data['city'] = trim(stms_isset($_POST['im_reg_city']));
		$data['state'] = trim(stms_isset($_POST['im_reg_state']));
		$data['zipcode'] = trim(stms_isset($_POST['im_reg_zipcode']));
		$data['country'] = trim(stms_isset($_POST['im_reg_country']));
		$data['title'] = trim(stms_isset($_POST['im_reg_title']));
		$data['phone'] = trim(stms_isset($_POST['im_reg_phone']));
		$data['company'] = trim(stms_isset($_POST['im_reg_company']));
		$data['gender'] = trim(stms_isset($_POST['im_reg_gender']));
		
		$member_id = stms_save_member($data);
		if ( !$member_id )
			wp_die('Update profile failed. Please contact support.');
		
		if ( isset($_POST['im_affs']) )
			stms_update_member_meta($member_id, 'aff_ids', $_POST['im_affs']);

		// Update the cookies if the password changed.
		if ( !empty($data['pwd1']) ) {
			stms_clear_auth_cookie();
			stms_set_auth_cookie($member_id);
		}
		
		$redir_url = stms_format_url($profile_url, 'updated=true');
		wp_redirect($redir_url);
		exit;
	}
}

add_action('template_redirect', 'stms_authorize_upsells_checkout', 5);
function stms_authorize_upsells_checkout() {
	if ( isset($_POST['_instamember_action']) && $_POST['_instamember_action'] == 'upsells_checkout' ) {
		if ( !wp_verify_nonce($_POST['_instamember_nonce'], 'stms-nonce') ) wp_die('Invalid Handler');
		
		$stms = get_option('stms_settings');
		$flow_id = $_POST['insta_flow_id'];
		$auth_vars = base64_encode(json_encode($_POST));
		$member_id = intval($_POST['member_id']);
		$member = stms_member($member_id);
		$reg_url = stms_format_url($stms['global']['magic'], 'imhandler=checkout&flow_id=' . $flow_id);
		
		if ( !$items = sflow_get_cart_items($flow_id) )
			wp_die('ERROR: No items found in the shopping cart.');
		
		// let's do some quick validation first...
		$card_num = stms_sanitize($_POST['im_cardnumber']);
		$card_month = stms_sanitize($_POST['im_cardexpired_month']);
		$card_year = stms_sanitize($_POST['im_cardexpired_year']);
		
		if ( stms_isset($stms['form_signup']['show_tos']) == 1 && stms_isset($stms['form_signup']['tos_url']) && !isset($_POST['im_tos']) ) {
			$auth_error = base64_encode('You must agree to our terms & conditions.');
			$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
			wp_redirect($error_url);
			exit;
		}
		
		if ( empty($_POST['im_cardholder_name']) ) {
			$auth_error = base64_encode('Please enter the full name listed on your credit card.');
			$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
			wp_redirect($error_url);
			exit;
		}
		
		if ( !stms_validate_ccnum($card_num) ) {
			$auth_error = base64_encode('Credit Card Number is Invalid.');
			$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
			wp_redirect($error_url);
			exit;
		}
			
		if ( !stms_validate_ccdate($card_month, $card_year) ) {
			$auth_error = base64_encode('Credit Card Expiration Date is Invalid.');
			$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
			wp_redirect($error_url);
			exit;
		}
			
		if ( stms_isset($stms['payments']['authorize']['ccv']) == 1 && !stms_validate_cvv($card_num, stms_sanitize($_POST['im_cardsecurity'])) ) {
			$auth_error = base64_encode('Credit Card Security Code (CVV) is Invalid.');
			$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
			wp_redirect($error_url);
			exit;
		}
			
		if ( stms_isset($stms['payments']['authorize']['avs']) == 1 ) {
			if ( empty($_POST['im_cardstreet']) || empty($_POST['im_cardcity']) || empty($_POST['im_cardstate']) || empty($_POST['im_cardzip']) || empty($_POST['im_cardcountry']) ) {
				$auth_error = base64_encode('One or more required values in "Billing Information" are missing.');
				$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
				wp_redirect($error_url);
				exit;
			}
		}

		if ( !class_exists('AuthorizeNetRequest') )
				require_once(STMS_SOURCES . 'anet_php_sdk/AuthorizeNet.php');
			
		if ( !defined('AUTHORIZENET_API_LOGIN_ID') && !defined('AUTHORIZENET_TRANSACTION_KEY') ) {
			define('AUTHORIZENET_API_LOGIN_ID', stms_sanitize($stms['payments']['authorize']['login_id'], '', true));
			define('AUTHORIZENET_TRANSACTION_KEY', stms_sanitize($stms['payments']['authorize']['trans_key'], '', true));
		}
		
		$anet_sandbox = ( stms_isset($stms['payments']['authorize']['test']) == 1 ) ? true : false;
		if ( !defined('AUTHORIZENET_SANDBOX') )
			define('AUTHORIZENET_SANDBOX', $anet_sandbox);
		
		$nameoncard = stms_sanitize($_POST['im_cardholder_name']);
		$names = explode(" ", $nameoncard);
		
		$card_fname = ( isset($names[0]) ) ? $names[0] : $nameoncard;
		$card_lname = ( count($names) > 2 ) ? stms_sanitize($names[2]) : stms_sanitize($names[1]);
		$expired_date = $card_month . '/' . $card_year;
		
		$sale = new AuthorizeNetAIM;	
		$x_cards = array(
			'method' => 'CC',
			'cust_id' => $member->ID, // member_id
			'first_name' => $card_fname,
			'last_name' => $card_lname,
			'email' => stms_sanitize($member->email),
			'customer_ip' => $_SERVER['REMOTE_ADDR'],
			'card_num' => $card_num,
			'exp_date' => $expired_date,
		);
		
		if ( stms_isset($stms['payments']['authorize']['ccv']) == 1 ) 
			$x_cards['card_code'] = stms_sanitize($_POST['im_cardsecurity']);
		
		$total_amount = 0;
		$tx_ids = false;
		foreach ( $items as $item ) {
			if ( $product = stms_get_product($item->product_id) ) {
				$first_price = $product->price;
				if ( $product->type == 'recurr' ) {
					$first_price = ( $product->is_trial == 1 ) ? $product->t_price : $product->price;
					if ( !is_array($tx_ids) )
						$tx_ids = array();
				}
				
				$tx_ids[$item->tx_id] = $item->product_id;
				$total_amount += $first_price;
				$sale->addLineItem($product->ID, stms_sanitize($product->title), '', '1', $first_price, 'N');
			}
		}
		
		$x_cards['amount'] = $total_amount;
		
		if ( !empty($member->info['street']) )
			$x_cards['address'] = stms_sanitize($member->info['street']);
		if ( !empty($member->info['city']) )
			$x_cards['city'] = stms_sanitize($member->info['city']);
		if ( !empty($member->info['state']) )
			$x_cards['state'] = stms_sanitize($member->info['state']);
		if ( !empty($member->info['zipcode']) )
			$x_cards['zip'] = stms_sanitize($member->info['zipcode']);
		if ( !empty($member->info['country']) )
			$x_cards['country'] = stms_sanitize($member->info['country']);
		
		if ( stms_isset($stms['payments']['authorize']['avs']) == 1 ) {
			$x_cards['address'] = stms_sanitize($_POST['im_cardstreet']);
			$x_cards['city'] = stms_sanitize($_POST['im_cardcity']);
			$x_cards['state'] = stms_sanitize($_POST['im_cardstate']);
			$x_cards['zip'] = stms_sanitize($_POST['im_cardzip']);
			$x_cards['country'] = stms_sanitize($_POST['im_cardcountry']);
		}
			
		$sale->setFields($x_cards);
		$authcard_first = true;
		if ( $total_amount <= 0 )
			$authcard_first = false;
			
		if ( $authcard_first ) {
			$response = $sale->authorizeOnly(); // Submit CC data to authorize to be validated...
			$auth_error = base64_encode($response->response_reason_text);
			
			if ( !$response->approved ) {
				$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
				wp_redirect($error_url);
				exit;
			}
		
			// CC approved... Let's charge it...
			$auth_code = $response->transaction_id;
			
			// Now capture:
			$capture = new AuthorizeNetAIM;
			$capture_response = $capture->priorAuthCapture($auth_code);
			$auth_code = $capture_response->transaction_id;
		}
		
		if ( !$tx_ids )
			wp_die("ERROR: Payment Failed.");
		
		foreach ( $tx_ids as $tx_id => $product_id ) {
			if ( $product = stms_get_product($product_id) ) {
				$subscription_id = $auth_code;
				if ( $product->type == 'recurr' ) {
					$normal_bill = $product->price;
					$start_date = date("Y-m-d", stms_get_expired($product->ID));
					$rebill_times = stms_isset($product->rebill_times, 0);
					$occurences = ( empty($product->rebill_times) ) ? 9999 : $rebill_times;
					$duration = maybe_unserialize($product->duration);
					
					switch ( $duration['mode'] ) {
						case 'd':
							$int_length = intval($duration['num']);
							$int_unit = 'days';
							break;
							
						case 'w':
							$int_length = intval($duration['num'] * 7);
							$int_unit = 'days';
							break;
						
						case 'm':
							$int_length = intval($duration['num']);
							$int_unit = 'months';
							break;
							
						case 'y':
							$int_length = 12;
							$int_unit = 'months';
							break;
					}
			
					$subscription = new AuthorizeNet_Subscription;
					$subscription->name = stms_cleanup($product->title);
					$subscription->intervalLength = $int_length;
					$subscription->intervalUnit = $int_unit;
					$subscription->startDate = $start_date;
					$subscription->totalOccurrences = $occurences;
					$subscription->amount = $normal_bill;
					$subscription->creditCardCardNumber = $card_num;
					$subscription->creditCardExpirationDate= $_POST['im_cardexpired_year'] . '-' . $_POST['im_cardexpired_month'];
					
					if ( stms_isset($stms['payments']['authorize']['ccv']) == 1 ) 
						$subscription->creditCardCardCode = trim($_POST['im_cardsecurity']);
					
					$subscription->customerId = $member->ID;
					$subscription->customerEmail = $member->email;
			
					$subscription->billToFirstName = $card_fname;
					$subscription->billToLastName = $card_lname;

					if ( !empty($member->info['street']) )
						$subscription->billToAddress = stms_sanitize($member->info['street']);
					if ( !empty($member->info['city']) )
						$subscription->billToCity = stms_sanitize($member->info['city']);
					if ( !empty($member->info['state']) )
						$subscription->billToState = stms_sanitize($member->info['state']);
					if ( !empty($member->info['zipcode']) )
						$subscription->billToZip = stms_sanitize($member->info['zipcode']);
					if ( !empty($member->info['country']) )
						$subscription->billToCountry = stms_sanitize($member->info['country']);
		
					if ( stms_isset($stms['payments']['authorize']['avs']) == 1 ) {
						$subscription->billToAddress = stms_sanitize($_POST['im_cardstreet']);
						$subscription->billToCity = stms_sanitize($_POST['im_cardcity']);
						$subscription->billToState = stms_sanitize($_POST['im_cardstate']);
						$subscription->billToZip = stms_sanitize($_POST['im_cardzip']);
						$subscription->billToCountry = stms_sanitize($_POST['im_cardcountry']);
					}
			
					// Create the subscription.
					$request = new AuthorizeNetARB;
					$response = $request->createSubscription($subscription);
					$subscription_id = $response->getSubscriptionId();

					// save partial payment data
					// get card type
					$cc_type = stms_validate_ccnum($card_num);
					// encrypt cc number so there's no actual cc number that will be saved...
					$length = strlen($card_num) - 4;
					$letter_x = '';
					for ( $i = 0; $i < $length; $i++ ) {
						$letter_x .= 'x';
					}
					$encrypted_ccnum = substr_replace($card_num, $letter_x, 0, $length);
					$cc_data = array(
						'number' => $encrypted_ccnum,
						'type' => $cc_type,
						'month' => $card_month,
						'year' => $card_year
					);
					stms_update_member_meta($member_id, 'authorize_' . $product->ID, $cc_data);
				}

				// let's complete the transaction...
				$args = array(
					'product_id' => $product->ID,
					'member_id' => $member_id,
					'payment_type' => $product->type,
					'paysys' => 'authorize',
					'receipt' => ( $product->type == 'recurr' ) ? $subscription_id : $auth_code,
					'amount' => ( $product->type == 'recurr' && $product->is_trial == 1  ) ? $product->t_price : $product->price,
					'discount' => 0.00,
					'txn_id' => $auth_code,
					'paid' => 1,
					'is_rebill' => 0,
					'status' => 'completed',
					'date' => stms_time(),
					'expired_date' => stms_get_expired($product->ID)
				);
				stms_complete_payment($tx_id, $args);
			}
		}
		
		// redirect to thank you page...
		$thanks_url = stms_format_url(get_bloginfo('url'), 'insta_mode=all_access&flow_id=' . $flow_id);
		wp_redirect($thanks_url);
		exit;
	}
}

add_action('template_redirect', 'stms_register_customer', 3);
function stms_register_customer() {
	if ( isset($_POST['_instamember_action']) && $_POST['_instamember_action'] == 'register' ) {
		if ( !wp_verify_nonce($_POST['_instamember_nonce'], 'stms-nonce') ) wp_die('Invalid Handler');
		
		$product_id = (int) $_POST['product_id'];
		if ( !$product = stms_get_product($product_id) )
			wp_die('Invalid Product');
		
		$is_paid = false;
		if ( isset($_POST['tx_id']) ) {
			if ( !$payment = stms_get_payment($_POST['tx_id']) )
				wp_die('ERROR: Invalid Payment');
				
			if ( $payment->status == 'completed' && $payment->paid == 1 )
				$is_paid = true;
			else {
				if ( isset($_POST['special_id']) ) {
					$regs = get_option('stms_reg_links');
					$special_id = $_POST['special_id'];
					if ( isset($regs[$special_id]) )
						$is_paid = true;
					else
						wp_die('ERROR: Invalid Payment');
				} else {
					wp_die('ERROR: Invalid Payment');
				}
			}
				
		}
		
		$stms = get_option('stms_settings');
		$auth_vars = base64_encode(json_encode($_POST));
		$paysys = $_POST['payment_system'];
		$is_confirm = stms_isset($product->info['free_confirm'], 0);
		$gateway = stms_gateway_type($paysys);
		$reg_key = base64_encode("{$product->ID}|{$paysys}|{$gateway}");
		$_reg_url = stms_get_handler_url('registration');
		$reg_url = stms_format_url($_reg_url, 'key=' . $reg_key);
		if ( isset($_GET['insta_token']) )
			$reg_url .= '&insta_token=' . $_GET['insta_token'];
		
		if ( isset($_GET['flow_id']) )
			$reg_url .= '&flow_id=' . $_GET['flow_id'];
		
		if ( isset($_GET['tx_ids']) && is_array($_GET['tx_ids']) && count($_GET['tx_ids']) ) {
			foreach ( $_GET['tx_ids'] as $trx_id ) {
				$reg_url .= "&tx_ids[]={$trx_id}";
			}
		}
		
		// validate recaptcha...
		if ( isset($_POST['recaptcha_challenge_field']) && isset($_POST['recaptcha_response_field']) ) {
			require_once( STMS_SOURCES . '/recaptcha/recaptchalib.php');
			$privatekey = trim(stms_isset($stms['integration']['recaptcha']['private']));
			$resp = recaptcha_check_answer($privatekey, $_SERVER["REMOTE_ADDR"], $_POST['recaptcha_challenge_field'], $_POST['recaptcha_response_field']);
			if ( !$resp->is_valid ) {
				$auth_error = base64_encode('You\'ve entered wrong captcha code.');
				$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
				@header('Location:' . $error_url);
				exit;
		  	}
		}
		
		if ( $paysys == 'authorize' ) {
			if ( $product->type == 'recurr' ) {
				$dur = maybe_unserialize($product->duration);
				$num = ( isset($dur['num']) ) ? $dur['num'] : 1;
				$mode = ( isset($dur['mode']) ) ? $dur['mode'] : 'd';
				
				if ( $mode == 'd' && $num < 7 ) 
					wp_die('ERROR: Subscription duration must be at least 7 days.');
			}

			// let's do some quick validation first...
			$card_num = stms_sanitize($_POST['im_cardnumber']);
			$card_month = stms_sanitize($_POST['im_cardexpired_month']);
			$card_year = stms_sanitize($_POST['im_cardexpired_year']);
			
			if ( !stms_validate_ccnum($card_num) ) {
				$auth_error = base64_encode('Credit Card Number is Invalid.');
				$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
				@header('Location:' . $error_url);
				exit;
			}
			
			if ( !stms_validate_ccdate($card_month, $card_year) ) {
				$auth_error = base64_encode('Credit Card Expiration Date is Invalid.');
				$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
				@header('Location:' . $error_url);
				exit;
			}
			
			if ( stms_isset($stms['payments']['authorize']['ccv']) == 1 && !stms_validate_cvv($card_num, stms_sanitize($_POST['im_cardsecurity'])) ) {
				$auth_error = base64_encode('Credit Card Security Code (CVV) is Invalid.');
				$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
				@header('Location:' . $error_url);
				exit;
			}
			
			if ( stms_isset($stms['payments']['authorize']['avs']) == 1 ) {
				if ( empty($_POST['im_cardstreet']) || empty($_POST['im_cardcity']) || empty($_POST['im_cardstate']) || empty($_POST['im_cardzip']) || empty($_POST['im_cardcountry']) ) {
					$auth_error = base64_encode('One or more required fields in "Billing Information" are missing values.');
					$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
					@header('Location:' . $error_url);
					exit;
				}
			}
		}
		
		$is_authorize_new = false;
		$is_existing = false;
		if ( is_stmsmember_logged_in() ) {
			$current_member = stms_get_current_member();
			$member_id = $current_member->ID;
			$is_existing = true;
		} else {
			// Quick server-side validation to prevent those who disabled javascript
			if ( empty($_POST['im_reg_username']) ) {
				$auth_error = base64_encode('Please enter a username.');
				$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
				@header('Location:' . $error_url);
				exit;
			}
			
			if ( empty($_POST['im_reg_email']) || !filter_var($_POST['im_reg_email'], FILTER_VALIDATE_EMAIL) ) {
				$auth_error = base64_encode('Please enter a valid email address.');
				$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
				@header('Location:' . $error_url);
				exit;
			}
			
			if ( stms_username_exists($_POST['im_reg_username']) ) {
				$auth_error = base64_encode('Username "' . $_POST['im_reg_username'] . '" is already in use. Please choose another username.');
				$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
				@header('Location:' . $error_url);
				exit;
			}
			
			if ( stms_email_exists($_POST['im_reg_email']) ) {
				$auth_error = base64_encode('"' . $_POST['im_reg_email'] . '" is already registered. Please use a different email address.');
				$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
				@header('Location:' . $error_url);
				exit;
			}
			
			if ( isset($_POST['im_reg_password']) ) {
				if ( empty($_POST['im_reg_password']) ) {
					$auth_error = base64_encode('Please enter a password.');
					$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
					@header('Location:' . $error_url);
					exit;
				}
				
				if ( empty($_POST['im_reg_password2']) ) {
					$auth_error = base64_encode('Please confirm your password.');
					$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
					@header('Location:' . $error_url);
					exit;
				}
				
				if ( $_POST['im_reg_password'] != $_POST['im_reg_password2'] ) {
					$auth_error = base64_encode('Password does not match.');
					$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
					@header('Location:' . $error_url);
					exit;
				}
			}
			
			$username = trim(sanitize_user($_POST['im_reg_username']));
			$email = trim(sanitize_email($_POST['im_reg_email']));
			if ( isset($_POST['im_reg_password']) )
				$password = trim($_POST['im_reg_password']);
			else
				$password = wp_generate_password();
			
			// register the customer
			$data = array();
			$data['member_id'] = ( $is_paid ) ? $payment->member_id : 0;
			$data['email'] = $email;
			$data['username'] = $username;
			$data['password'] = $password;
			$data['status'] = ( $paysys == 'authorize' ) ? 4 : (( $is_confirm ) ? 0 : 1);
			$data['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
			$data['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
			$data['first_name'] = stms_sanitize($_POST['im_reg_first_name']);
			$data['last_name'] = stms_sanitize($_POST['im_reg_last_name']);
			$data['street'] = ( isset($_POST['im_cardstreet']) && !isset($_POST['im_reg_street']) ) ? stms_sanitize($_POST['im_cardstreet']) : stms_sanitize($_POST['im_reg_street']);
			$data['city'] = ( isset($_POST['im_cardcity']) && !isset($_POST['im_reg_city']) ) ? stms_sanitize($_POST['im_cardcity']) : stms_sanitize($_POST['im_reg_city']);
			$data['state'] = ( isset($_POST['im_cardstate']) && !isset($_POST['im_reg_state']) ) ? stms_sanitize($_POST['im_cardstate']) : stms_sanitize($_POST['im_reg_state']);
			$data['zipcode'] = ( isset($_POST['im_cardzip']) && !isset($_POST['im_reg_zipcode']) ) ? stms_sanitize($_POST['im_cardzip']) : stms_sanitize($_POST['im_reg_zipcode']);
			$data['country'] = ( isset($_POST['im_cardcountry']) && !isset($_POST['im_reg_country']) ) ? stms_sanitize($_POST['im_cardcountry']) : stms_sanitize($_POST['im_reg_country']);
			$data['title'] = stms_sanitize($_POST['im_reg_title']);
			$data['phone'] = stms_sanitize($_POST['im_reg_phone']);
			$data['company'] = stms_sanitize($_POST['im_reg_company']);
			$data['gender'] = stms_sanitize($_POST['im_reg_gender']);
			
			if ( $is_paid ) {
				$data['fix_username'] = $username;
				$data['pwd1'] = $password;
			}
			
			$member_id = stms_save_member($data, false, $is_confirm);
			if ( !$member_id )
				wp_die('Registration failed. Please contact support.');
			
			if ( $paysys == 'authorize' )
				$is_authorize_new = true;
			
			// logging in user
			stms_clear_auth_cookie(); // just making sure to cleaning up the login cookie first...
			if ( !$is_confirm ) {
				stms_set_auth_cookie($member_id, false, is_ssl());
			}
		}

		// let's find out if there's any affiliate...
		$aff_id = 0;
		if ( isset($_COOKIE['__insta_aff_id']) ) {
			$aff_id = (int) $_COOKIE['__insta_aff_id']; // there's aff cookie
			if ( $aff_id != $member_id )
				stms_update_member_meta($member_id, 'aff_id', $aff_id); // let's store the cookie into database...
		} else {
			// there's no cookie, but let's check if there's any aff_id value on the db...
			$aff_meta = stms_get_member_meta($member_id, 'aff_id');
			if ( !empty($aff_meta) && $member_id != $aff_meta )
				$aff_id = $aff_meta;
		}
		
		$total_amount = $_POST['total_amount'];
		$normal_price = ( $product->type == 'recurr' && $product->is_trial == 1 ) ? $product->t_price : $product->price;
		$discount = number_format($normal_price - $total_amount, 2);
		
		$steps = stms_isset($product->info['steps']);
		$currency = stms_isset($stms['global']['currency'], 'USD');
		if ( $paysys == 'authorize' ) {
			if ( !class_exists('AuthorizeNetRequest') )
				require_once(STMS_SOURCES . 'anet_php_sdk/AuthorizeNet.php');
			
			if ( !defined('AUTHORIZENET_API_LOGIN_ID') && !defined('AUTHORIZENET_TRANSACTION_KEY') ) {
				define('AUTHORIZENET_API_LOGIN_ID', stms_sanitize($stms['payments']['authorize']['login_id'], '', true));
				define('AUTHORIZENET_TRANSACTION_KEY', stms_sanitize($stms['payments']['authorize']['trans_key'], '', true));
			}
			
			$anet_sandbox = ( stms_isset($stms['payments']['authorize']['test']) == 1 ) ? true : false;
			if ( !defined('AUTHORIZENET_SANDBOX') )
				define('AUTHORIZENET_SANDBOX', $anet_sandbox);
			
			$nameoncard = stms_sanitize($_POST['im_cardholder_name']);
			$names = explode(" ", $nameoncard);
			
			$card_fname = ( isset($names[0]) ) ? $names[0] : $nameoncard;
			$card_lname = ( count($names) > 2 ) ? stms_sanitize($names[2]) : stms_sanitize($names[1]);
			$card_num = stms_sanitize($_POST['im_cardnumber']);
			$card_month = stms_sanitize($_POST['im_cardexpired_month']);
			$card_year = stms_sanitize($_POST['im_cardexpired_year']);
			$expired_date = $card_month . '/' . $card_year;
			
			$buyer = stms_member($member_id);
			$first_price = ( $product->type == 'recurr' && $product->is_trial == 1 ) ? $product->t_price : $product->price;
			
			$sale = new AuthorizeNetAIM;
			
			$x_cards = array(
				'method' => 'CC',
				'cust_id' => $buyer->ID, // member_id
				'amount' => $first_price,
				'first_name' => $card_fname,
				'last_name' => $card_lname,
				'email' => $email,
				'customer_ip' => $_SERVER['REMOTE_ADDR'],
				'card_num' => $card_num,
				'exp_date' => $expired_date,
			);
			
			if ( stms_isset($stms['payments']['authorize']['ccv']) == 1 ) 
				$x_cards['card_code'] = stms_sanitize($_POST['im_cardsecurity']);
				
			if ( !empty($buyer->info['street']) )
				$x_cards['address'] = stms_sanitize($buyer->info['street']);
			if ( !empty($buyer->info['city']) )
				$x_cards['city'] = stms_sanitize($buyer->info['city']);
			if ( !empty($buyer->info['state']) )
				$x_cards['state'] = stms_sanitize($buyer->info['state']);
			if ( !empty($buyer->info['zipcode']) )
				$x_cards['zip'] = stms_sanitize($buyer->info['zipcode']);
			if ( !empty($buyer->info['country']) )
				$x_cards['country'] = stms_sanitize($buyer->info['country']);
			
			if ( stms_isset($stms['payments']['authorize']['avs']) == 1 ) {
				$x_cards['address'] = stms_sanitize($_POST['im_cardstreet']);
				$x_cards['city'] = stms_sanitize($_POST['im_cardcity']);
				$x_cards['state'] = stms_sanitize($_POST['im_cardstate']);
				$x_cards['zip'] = stms_sanitize($_POST['im_cardzip']);
				$x_cards['country'] = stms_sanitize($_POST['im_cardcountry']);
			}

			$sale->setFields($x_cards);
			$sale->addLineItem($product->ID, stms_sanitize($product->title), '', '1', $first_price, 'N');
			
			$authcard_first = true;
			if ( $product->type == 'recurr' && $product->is_trial == 1 && $first_price <= 0.00 )
				$authcard_first = false;
			
			if ( $authcard_first ) {
				$response = $sale->authorizeOnly(); // Submit CC data to authorize to be validated...
				$auth_error = base64_encode($response->response_reason_text);
				
				if ( !$response->approved ) {
					if ( !$is_existing ) {
						// delete the new member temp data...
						stms_clear_auth_cookie();
						@stms_delete_member($buyer->ID, false);
					}
					
					$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
					@header('Location:' . $error_url);
					exit;
				}
			
				// CC approved... Let's charge it...
				$auth_code = $response->transaction_id;
				
				// Now capture:
				$capture = new AuthorizeNetAIM;
				$capture_response = $capture->priorAuthCapture($auth_code);
				$auth_code = $capture_response->transaction_id;
			}
			
			if ( $product->type == 'recurr' ) {
				$normal_bill = $product->price;
				$start_date = date("Y-m-d", stms_get_expired($product->ID));
				$rebill_times = stms_isset($product->rebill_times, 0);
				$occurences = ( empty($product->rebill_times) ) ? 9999 : (( $product->is_trial == 1 ) ? $rebill_times : ($rebill_times - 1));
				$duration = maybe_unserialize($product->duration);
				switch ( $duration['mode'] ) {
					case 'd':
						$int_length = intval($duration['num']);
						$int_unit = 'days';
						break;
						
					case 'w':
						$int_length = intval($duration['num'] * 7);
						$int_unit = 'days';
						break;
					
					case 'm':
						$int_length = intval($duration['num']);
						$int_unit = 'months';
						break;
						
					case 'y':
						$int_length = 12;
						$int_unit = 'months';
						break;
				}
				
				$subscription = new AuthorizeNet_Subscription;
				$subscription->name = stms_sanitize($product->title);
				$subscription->intervalLength = $int_length;
				$subscription->intervalUnit = $int_unit;
				$subscription->startDate = $start_date;
				$subscription->totalOccurrences = $occurences;
				$subscription->amount = $normal_bill;
				$subscription->creditCardCardNumber = $card_num;
				$subscription->creditCardExpirationDate= $_POST['im_cardexpired_year'] . '-' . $_POST['im_cardexpired_month'];
				
				if ( stms_isset($stms['payments']['authorize']['ccv']) == 1 ) 
					$subscription->creditCardCardCode = trim($_POST['im_cardsecurity']);
				
				$subscription->customerId = $buyer->ID;
				$subscription->customerEmail = $email;
				$subscription->billToFirstName = $card_fname;
				$subscription->billToLastName = $card_lname;
				
				if ( !empty($buyer->info['street']) )
					$subscription->billToAddress = stms_cleanup($buyer->info['street']);
				if ( !empty($buyer->info['city']) )
					$subscription->billToCity = stms_cleanup($buyer->info['city']);
				if ( !empty($buyer->info['state']) )
					$subscription->billToState = stms_cleanup($buyer->info['state']);
				if ( !empty($buyer->info['zipcode']) )
					$subscription->billToZip = stms_cleanup($buyer->info['zipcode']);
				if ( !empty($buyer->info['country']) )
					$subscription->billToCountry = stms_cleanup($buyer->info['country']);
			
				if ( stms_isset($stms['payments']['authorize']['avs']) == 1 ) {
					$subscription->billToAddress = stms_sanitize($_POST['im_cardstreet']);
					$subscription->billToCity = stms_sanitize($_POST['im_cardcity']);
					$subscription->billToState = stms_sanitize($_POST['im_cardstate']);
					$subscription->billToZip = stms_sanitize($_POST['im_cardzip']);
					$subscription->billToCountry = stms_sanitize($_POST['im_cardcountry']);
				}
				
				// Create the subscription.
				$request = new AuthorizeNetARB;
				$response = $request->createSubscription($subscription);
				$subscription_id = $response->getSubscriptionId();
				
				// save partial payment data
				// get card type
				$cc_type = stms_validate_ccnum($card_num);
				// encrypt cc number so there's no actual cc number that will be saved...
				$length = strlen($card_num) - 4;
				$letter_x = '';
				for ( $i = 0; $i < $length; $i++ ) {
					$letter_x .= 'x';
				}
				$encrypted_ccnum = substr_replace($card_num, $letter_x, 0, $length);
				
				$cc_data = array(
					'number' => $encrypted_ccnum,
					'type' => $cc_type,
					'month' => $card_month,
					'year' => $card_year
				);
				stms_update_member_meta($buyer->ID, 'authorize_' . $product->ID, $cc_data);
			}

			// let's create the transaction...
			$args = array(
				'product_id' => $product->ID,
				'member_id' => $buyer->ID,
				'aff_id' => $aff_id,
				'payment_type' => $product->type,
				'paysys' => 'authorize',
				'receipt' => ( $product->type == 'recurr' ) ? $subscription_id : $auth_code,
				'currency' => $currency,
				'amount' => $total_amount,
				'discount' => $discount,
				'txn_id' => $auth_code,
				'paid' => 1,
				'is_rebill' => 0,
				'status' => 'completed',
				'date' => stms_time(),
				'expired_date' => stms_get_expired($product->ID)
			);
			$tx_id = stms_add_payment($args);
			
			$steps = '2,1';
			$is_paid = true;
			
			if ( $is_authorize_new ) {
				// update the member status first...
				stms_update_member($buyer->ID, array('status' => 1));
				do_action('stms_member_registered', $buyer->ID, $password);
			}
			do_action('stms_payment_completed', $tx_id);
			do_action('stms_custom_autoresponder', $tx_id); // for custom autoresponder...
			
			if ( isset($_COOKIE['__insta_flow']) ) {
				$flow_id = urldecode($_COOKIE['__insta_flow']);
				do_action('insta_upsells_redirect', $flow_id, $product->ID, $paysys, $buyer->ID);
			}
		}
		
		if ( isset($_POST['special_id']) )
			$steps = '2,1';
			
		if ( $product->type != 'free' && $steps == '2,1' ) {
			if ( isset($_POST['tx_id']) && !isset($_POST['special_id']) )
				do_action('stms_payment_completed', $_POST['tx_id']); // so the receipt can be sent to real email address...

			if ( isset($_POST['insta_token']) )
				delete_transient('insta_token_' . $_POST['insta_token']);
			
			if ( isset($_COOKIE['__insta_flow']) && $paysys != 'authorize' ) {
				$flow_id = urldecode($_COOKIE['__insta_flow']);
				if ( isset($_COOKIE['__insta_flpaysys']) && $_COOKIE['__insta_flpaysys'] == $paysys ) {
					$access_url = stms_format_url($stms['global']['magic'], 'insta_mode=all_access&flow_id=' . $flow_id);
					wp_redirect($access_url);
					exit;
				}
			}
			
			$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_permalink($stms['global']['home']);
			$redirect_url = ( !empty($product->info['thankuri']) ) ? $product->info['thankuri'] : $home_url;
			if ( $redirect_url == '' )
				$redirect_url = ( !empty($stms['global']['home']) ) ? get_permalink($stms['global']['home']) : get_bloginfo('url');
			
			if ( isset($_POST['special_id']) ) {
				$regs = get_option('stms_reg_links');
				$key = trim($_POST['special_id']);
				if ( isset($regs[$key]) ) {
					if ( isset($_POST['tx_id']) ) {
						// complete the transaction...
						$args = array(
							'paid' => 1,
							'is_rebill' => 0,
							'status' => 'completed'
						);
						stms_complete_payment($_POST['tx_id'], $args);
						do_action('stms_custom_autoresponder', $_POST['tx_id']); // for custom autoresponder...
					}
					
					if ( isset($regs[$key]['used']) ) {
						$regs[$key]['used'] = (intval($regs[$key]['used']) + 1);
						update_option('stms_reg_links', $regs);
						
						$special_url = ( empty($home_url) ) ? $redirect_url : $home_url;
						wp_redirect($special_url);
						exit;
					}
				}
			}
			
			wp_redirect($redirect_url);
			exit;
		} else {
			$amount = $total_amount;
			
			$args = array(
				'product_id' => $product->ID,
				'member_id' => $member_id,
				'aff_id' => $aff_id,
				'payment_type' => $product->type,
				'paysys' => $paysys,
				'receipt' => '',
				'currency' => $currency,
				'amount' => $amount,
				'discount' => $discount, 
				'txn_id' => '',
				'paid' => 0,
				'is_rebill' => 0,
				'status' => 'pending',
				'date' => stms_time(),
				'expired_date' => stms_get_expired($product->ID)
			);
			
			if ( $product->type == 'free' && $paysys == 'free' ) {
				$args['amount'] = 0.00;
				$args['paid'] = ( $is_confirm ) ? 0 : 1;
				$args['status'] = ( $is_confirm ) ? 'temp' : 'completed';
				$tx_id = stms_add_payment($args);
				if ( !$tx_id )
					wp_die('Registration failed. Please contact support.');
				
				do_action('stms_custom_autoresponder', $tx_id); // for custom autoresponder...
				
				if ( !$is_confirm ) {
					do_action('stms_payment_completed', $tx_id);
				} else {
					$confirm_code = md5($email);
					set_transient('stms_confirm_' . $member_id, $confirm_code);
					do_action('stms_member_confirm', $tx_id);
				}
				
				if ( $gateway == 'third' )
					stms_other_payments($member_id, $email, $paysys);
			
				$login_url = stms_login_url();
				$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : '';
				$redirect_url = ( filter_var($product->info['thankuri'], FILTER_VALIDATE_URL) ) ? $product->info['thankuri'] : $home_url;
				if ( $redirect_url == '' )
					$redirect_url = ( !empty($stms['global']['home']) ) ? get_permalink($stms['global']['home']) : get_bloginfo('siteurl');
				
				if ( $is_confirm )
					$redirect_url = $login_url . '&msg=confirm_required';
				
				// count usage if this is a special reg URL...
				if ( isset($_POST['special_id']) ) {
					$regs = get_option('stms_reg_links');
					$key = trim($_POST['special_id']);
					if ( isset($regs[$key]) ) {
						if ( isset($regs[$key]['used']) ) {
							$regs[$key]['used'] = (intval($regs[$key]['used']) + 1);
							update_option('stms_reg_links', $regs);
						}
					}
				}
				
				if ( isset($_COOKIE['__insta_flow']) ) {
					$flow_id = urldecode($_COOKIE['__insta_flow']);
					do_action('insta_upsells_redirect', $flow_id, $product->ID, 'free', $member_id);
				}
				
				wp_redirect($redirect_url);
				exit;
			}
			
			$tx_id = stms_add_pending_payment($args);
			if ( !$tx_id )
				wp_die('Registration failed. Please contact support.');
			
			// create check data so payment can be checked on thank you page...
			
			$check_data = array(
				'tx_id' => $tx_id,
				'member_id' => $member_id,
				'acc_type' => 'real',
				'amount' => $total_amount,
				'discount' => $discount
			);
			set_transient('stms_payment_check_' . $tx_id, $check_data, 60*60);
			@setcookie('__insta_payment', $tx_id, time()+3600, SITECOOKIEPATH);
			if ( SITECOOKIEPATH != COOKIEPATH )
				@setcookie('__insta_payment', $tx_id, time()+3600, COOKIEPATH);
			
			// proceed to payment...
			stms_payment_proceed($tx_id, $paysys);
		}
		exit;
	}
}

add_action('template_redirect', 'stms_register_affiliate', 3);
function stms_register_affiliate() {
	if ( isset($_POST['_instamember_action']) && $_POST['_instamember_action'] == 'aff_register' ) {
		if ( !wp_verify_nonce($_POST['_instamember_nonce'], 'stms-nonce') ) wp_die('Invalid Handler');
		
		$stms = get_option('stms_settings');
		$auth_vars = base64_encode(json_encode($_POST));
		$reg_url = stms_get_handler_url('registration');
		
		// validate recaptcha...
		if ( isset($_POST['recaptcha_challenge_field']) && isset($_POST['recaptcha_response_field']) ) {
			require_once( STMS_SOURCES . '/recaptcha/recaptchalib.php');
			$privatekey = trim(stms_isset($stms['integration']['recaptcha']['private']));
			$resp = recaptcha_check_answer($privatekey, $_SERVER["REMOTE_ADDR"], $_POST['recaptcha_challenge_field'], $_POST['recaptcha_response_field']);
			if ( !$resp->is_valid ) {
				$auth_error = base64_encode('You\'ve entered wrong captcha code.');
				$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
				@header('Location:' . $error_url);
				exit;
		  	}
		}
		
		// Quick server-side validation to prevent those who disabled javascript
		if ( empty($_POST['im_reg_username']) ) {
			$auth_error = base64_encode('Please enter a username.');
			$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
			@header('Location:' . $error_url);
			exit;
		}
		
		if ( empty($_POST['im_reg_email']) || !filter_var($_POST['im_reg_email'], FILTER_VALIDATE_EMAIL) ) {
			$auth_error = base64_encode('Please enter a valid email address.');
			$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
			@header('Location:' . $error_url);
			exit;
		}
		
		if ( stms_username_exists($_POST['im_reg_username']) ) {
			$auth_error = base64_encode('Username "' . $_POST['im_reg_username'] . '" is already in use. Please choose another username.');
			$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
			@header('Location:' . $error_url);
			exit;
		}
		
		if ( stms_email_exists($_POST['im_reg_email']) ) {
			$auth_error = base64_encode('"' . $_POST['im_reg_email'] . '" is already registered. Please use a different email address.');
			$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
			@header('Location:' . $error_url);
			exit;
		}
		
		if ( isset($_POST['im_reg_password']) ) {
			if ( empty($_POST['im_reg_password']) ) {
					$auth_error = base64_encode('Please enter a password.');
					$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
					@header('Location:' . $error_url);
					exit;
			}
			
			if ( empty($_POST['im_reg_password2']) ) {
					$auth_error = base64_encode('Please confirm your password.');
					$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
					@header('Location:' . $error_url);
					exit;
			}
			
			if ( $_POST['im_reg_password'] != $_POST['im_reg_password2'] ) {
					$auth_error = base64_encode('Password does not match.');
					$error_url = $reg_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
					@header('Location:' . $error_url);
					exit;
			}
		}
			
		$username = trim(sanitize_user($_POST['im_reg_username']));
		$email = trim(sanitize_email($_POST['im_reg_email']));
		if ( isset($_POST['im_reg_password']) )
			$password = trim($_POST['im_reg_password']);
		else
			$password = wp_generate_password();
			
		// register the affiliate
		$data = array();
		$data['member_id'] = 0;
		$data['email'] = $email;
		$data['username'] = $username;
		$data['password'] = $password;
		$data['status'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : 1;
		$data['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : 1;
		$data['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 1 : 0;
		$data['first_name'] = stms_sanitize($_POST['im_reg_first_name']);
		$data['last_name'] = stms_sanitize($_POST['im_reg_last_name']);
		$data['street'] = ( isset($_POST['im_cardstreet']) && !isset($_POST['im_reg_street']) ) ? stms_sanitize($_POST['im_cardstreet']) : stms_sanitize($_POST['im_reg_street']);
		$data['city'] = ( isset($_POST['im_cardcity']) && !isset($_POST['im_reg_street']) ) ? stms_sanitize($_POST['im_cardcity']) : stms_sanitize($_POST['im_reg_city']);
		$data['state'] = ( isset($_POST['im_cardstate']) && !isset($_POST['im_reg_state']) ) ? stms_sanitize($_POST['im_cardstate']) : stms_sanitize($_POST['im_reg_state']);
		$data['zipcode'] = ( isset($_POST['im_cardzip']) && !isset($_POST['im_reg_zipcode']) ) ? stms_sanitize($_POST['im_cardzip']) : stms_sanitize($_POST['im_reg_zipcode']);
		$data['country'] = ( isset($_POST['im_cardcountry']) && !isset($_POST['im_reg_country']) ) ? stms_sanitize($_POST['im_cardcountry']) : stms_sanitize($_POST['im_reg_country']);
		$data['title'] = stms_sanitize($_POST['im_reg_title']);
		$data['phone'] = stms_sanitize($_POST['im_reg_phone']);
		$data['company'] = stms_sanitize($_POST['im_reg_company']);
		$data['gender'] = stms_sanitize($_POST['im_reg_gender']);
		$data['url'] = stms_sanitize($_POST['im_reg_url']);
			
		$member_id = stms_save_member($data, false, false, 0, true);
		if ( !$member_id )
			wp_die('Registration failed. Please contact support.');
		
		// add affiliate/referrer...
		if ( $_POST['insta_aff_id'] > 0 ) {
			$aff_id = intval($_POST['insta_aff_id']);
			stms_update_member_meta($member_id, 'aff_id', $aff_id);
		}
		
		// submit to autoresponder
		stms_aff_autoresponder_submit($member_id);
		do_action('stms_aff_custom_autoresponder', $member_id); // for custom autoresponder...
			
		// redirect to profile page
		if ( stms_isset($stms['global']['manual_aff']) == 1 ) {
			$redir_url = $reg_url;
			stms_send_affpendingmail($member_id); // send pending email
		} else {
			// logging in user
			stms_set_auth_cookie($member_id);
		
			$redir_url = stms_get_handler_url('profile');
		}
		
			
		@setcookie('__insta_aff_signedup', 1, 0, SITECOOKIEPATH);
		if ( SITECOOKIEPATH != COOKIEPATH )
			@setcookie('__insta_aff_signedup', 1, 0, COOKIEPATH);
			
		wp_redirect($redir_url);
		exit;
	}
}

add_action('init', 'stms_protected_download');
function stms_protected_download() {
	if( !isset($_GET['insta_file']) || empty($_GET['insta_file']) )
		return false;
	
	$is_s3 = false;
	if ( stms_isset($_GET['source']) == 's3' )
		$is_s3 = true;
		
 	$stms = get_option('stms_settings');
	$admin_mode = ( is_stms_admin() && stms_isset($stms['global']['admin_unlock']) == 1 ) ? true : false;
	$download_url = stms_format_url(get_bloginfo('url'), 'insta_file=' . $_GET['insta_file']);
	$download_url .= ( $is_s3 ) ? '&source=s3' : '';
	$login_url = stms_login_url($download_url);
	if ( !$admin_mode && !is_stmsmember_logged_in() ) {
		// immediately redirect to login page
		wp_redirect($login_url);
		exit;
	}
	
	$access = stms_check_access('file', $is_s3);
	if ( !$admin_mode && $access === FALSE )
		wp_die('Unauthorize Access.');
	
	if ( !$admin_mode && $access === 'expired' )
		wp_die('File is Expired.');
	
	if ( !$admin_mode && $access === 'unreleased' )
		wp_die('The File <strong>"' . urldecode($_GET['insta_file']) . '"</strong> is not released yet.');
	
	$filename = rawurldecode($_GET['insta_file']);
	$disposition = ( stms_isset($_GET['stream']) == 'yes' ) ? 'inline' : 'attachment';
	if ( $is_s3 )
		stms_s3_file_download($filename, $disposition);
	
	$file_path  = STMS_DOWNLOAD_PATHS . $filename;
	$path_info = pathinfo($file_path);
	$file_name  = $path_info['basename'];
	$file_ext   = $path_info['extension'];

	if ( is_file($file_path) ) {
		$file_size = filesize($file_path);
		$file = @fopen($file_path, 'rb');
		
		if ( $file ) {
			// set the headers, prevent caching
			header("Pragma: public");
			header("Expires: -1");
			header("Cache-Control: public, must-revalidate, post-check=0, pre-check=0");

	        if ( $disposition == 'attachment' )
				header("Content-Disposition: attachment; filename=\"$file_name\"");
	        else
				header('Content-Disposition: inline;');
 
			$known_mime_types = array(
		 		"pdf" => "application/pdf",
				"exe" => "application/octet-stream",
				"zip" => "application/zip",
				"rar" => "application/rar",
				"doc" => "application/msword",
				"docx"=> "application/msword",
				"xls" => "application/vnd.ms-excel",
				"xlsx"=> "application/vnd.ms-excel",
				"ppt" => "application/vnd.ms-powerpoint",
				"swf" => "application/x-shockwave-flash",
				"mmap"=> "application/mmap",
				"mp3" => "audio/mpeg",
				"wav" => "audio/x-wav",
				"gif" => "image/gif",
				"png" => "image/png",
				"jpeg"=> "image/jpg",
				"jpg" => "image/jpg",
				"txt" => "text/plain",
				"php" => "text/php",
				"html" => "text/html",
				"js" => "text/javascript",
				"css" => "text/style",
				"flv" => "video/x-flv",
				"mp4" => "video/mpeg",
				"mpeg"=> "video/mpeg",
				"mpg" => "video/mpeg",
				"mov" => "video/quicktime",
				"avi" => "video/x-msvideo",
				"wmv" => "video/x-ms-wmv"
		 	);
	        $mime_type = isset($known_mime_types[$file_ext]) ? $known_mime_types[$file_ext] : 'application/octet-stream';
	        header("Content-Type: " . $mime_type);
 
			if ( isset($_SERVER['HTTP_RANGE']) ) {
				list($size_unit, $range_orig) = explode('=', $_SERVER['HTTP_RANGE'], 2);
				if ($size_unit == 'bytes') {
					list($range, $extra_ranges) = explode(',', $range_orig, 2);
				} else {
					$range = '';
					header('HTTP/1.1 416 Requested Range Not Satisfiable');
					exit;
				}
			} else {
				$range = '';
			}
 
			list($seek_start, $seek_end) = explode('-', $range, 2);
	 
			$seek_end   = (empty($seek_end)) ? ($file_size - 1) : min(abs(intval($seek_end)),($file_size - 1));
			$seek_start = (empty($seek_start) || $seek_end < abs(intval($seek_start))) ? 0 : max(abs(intval($seek_start)),0);
	 
			if ( $seek_start > 0 || $seek_end < ($file_size - 1) ) {
				header('HTTP/1.1 206 Partial Content');
				header('Content-Range: bytes '.$seek_start.'-'.$seek_end.'/'.$file_size);
				header('Content-Length: '.($seek_end - $seek_start + 1));
			} else {
			  header("Content-Length: $file_size");
			}
		
			header('Accept-Ranges: bytes');
 
			set_time_limit(0);
			fseek($file, $seek_start);
 
			while( !feof($file) ) {
				print(@fread($file, 1024*8));
				ob_flush();
				flush();
				if ( connection_status()!=0 ) {
					@fclose($file);
					exit;
				}			
			}
			@fclose($file);
			exit;
		} else {
			header("HTTP/1.0 500 Internal Server Error");
			exit;
		}
	} else {
		// file does not exist
		header("HTTP/1.0 404 Not Found");
		exit;
	}
}

add_action('init', 'stms_ip_blocker', 2);
function stms_ip_blocker() {
	$proceed = false;
	if ( isset($_GET['imhandler']) || isset($_GET['insta_mode']) || isset($_GET['insta_file']) )
		$proceed = true;
	
	if ( !$proceed )
		return false;
	
	$stms = get_option('stms_settings');
	$ip = stms_retrieve_ipaddr();
	if ( !$range = stms_get_blocked_ips() )
		return false;
	
	$blocked = stms_ban_ip($range, $ip);
	if ( !$blocked )
		return FALSE;
	
	wp_die( stms_cleanup(stms_isset($stms['blacklist']['ip_message'], "Your IP address has been blocked.")) );
	exit;
}

function stms_ban_ip( $range = array(), $ip = '' ) {
	$ip = stms_longip(trim($ip));
	if ( $ip == FALSE || empty($ip) )
		return FALSE;

	foreach ( $range AS $key => $val ) {
		$temp = explode('-', $val);
        if ( empty($temp[0]) )
			return FALSE;
		else {
			$start_ip = stms_longip(trim($temp[0]));
			if ( $start_ip == FALSE )
				return FALSE;
		}
		if ( empty($temp[1]) ) {
			if( $ip == $start_ip )
				return TRUE;
		} else {
			$stop_ip = stms_longip(trim($temp[1]));
			if ( $stop_ip == FALSE )
				return FALSE;
		}
		if ( $start_ip <= $ip && $ip <= $stop_ip )
			return TRUE;
	}
    return FALSE;
}

function stms_get_blocked_ips() {
	$stms = get_option('stms_settings');
	if ( empty($stms['blacklist']['ip_addresses']) )
		return false;
	
	$ips = explode("\n", trim($stms['blacklist']['ip_addresses']) );
	if ( count($ips) <= 0 )
		return false;
	
	$blocked_ips = array();
	foreach ( $ips as $ip ) {
		$parts = explode(".", $ip);
		if ( stms_isset($parts[3]) == '*' ) {
			$ipc = "{$parts[0]}.{$parts[1]}.{$parts[2]}";
			$blocked_ips[] = "{$ipc}.0 - {$ipc}.255";
		} else {
			$blocked_ips[] = $ip;
		}
	}
	return $blocked_ips;
}

function stms_longip($ip) {
    if( empty($ip) )
		return FALSE;
        
    $block = explode('.', $ip);
    if ( count($block) != 4 )
		return FALSE;
    $i = 3;
    $block_ip = 0;
    foreach ( $block as $k => $v ) {
		$v = filter_var($v, FILTER_VALIDATE_INT);
		if ( $v < 0 ) $v = 0;         
		if( $v > 255 ) $v = 255;
        $block_ip += pow(256, $i) * $v;
        $i--;
    }
    return $block_ip;
}

function stms_get_blocked_emails() {
	$stms = get_option('stms_settings');
	if ( empty($stms['blacklist']['ip_addresses']) )
		return false;
	
	$emails = explode("\n", trim($stms['blacklist']['email_addresses']) );
	if ( count($emails) <= 0 )
		return false;
	
	return $emails;
}

add_action('init', 'stms_email_blocker', 2);
function stms_email_blocker() {
	$proceed = false;
	if ( !empty($_REQUEST['im_reg_email']) )
		$proceed = true;
	
	if ( !$proceed )
		return false;
	
	if ( !$emails = stms_get_blocked_emails() )
		return false;
	
	$cust_email = $_REQUEST['im_reg_email'];
	$blocked = false;
	foreach ( $emails as $email ) {
		$parts = explode("@", $email);
		if ( stms_isset($parts[0]) == '*' ) {
			if ( stristr($email, $cust_email) ) {
				$blocked = true;
				break;
			}
		} else {
			if ( $cust_email == $email ) {
				$blocked = true;
				break;
			}
		}
	}
	
	if ( !$blocked )
		return false;
	
	wp_die( stms_cleanup(stms_isset($stms['blacklist']['email_message'], "Your Email address has been blocked.")) );
	exit;
}

add_action('template_redirect', 'stms_member_activate_affiliate', 6);
function stms_member_activate_affiliate() {
	if ( !is_stmsmember_logged_in() )
		return false;
	
	if ( isset($_GET['action']) && $_GET['action'] == 'aff_activate' ) {
		$member = stms_get_current_member();
		if ( $member->is_aff == 1 )
			return false;
		
		$data = array();
		$stms = get_option('stms_settings');
		if ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) {
			$data['is_aff_pending'] = 1;
			$cookie_val = 3;
			stms_send_affpendingmail($member->ID); // send pending email
		} else {
			$data['is_aff'] = 1;
			$cookie_val = 2;
			
			// submit to autoresponder
			stms_aff_autoresponder_submit($member->ID);
		}
		
		@setcookie('__insta_aff_signedup', $cookie_val, 0, SITECOOKIEPATH);
		if ( SITECOOKIEPATH != COOKIEPATH )
			@setcookie('__insta_aff_signedup', $cookie_val, 0, COOKIEPATH);
		
		if ( !$update = stms_update_member($member->ID, $data) )
			wp_die("ERROR: Cannot activate affiliate account.");
		
		$profile_url = stms_get_handler_url('profile');
		wp_redirect($profile_url);
		exit;
	}
}

// special reg link
add_action('template_redirect', 'stms_special_registration');
function stms_special_registration() {
	if ( isset($_GET['insta_mode']) && $_GET['insta_mode'] == 'freg' && isset($_GET['id']) && isset($_GET['code']) ) {
		$regs = get_option('stms_reg_links');
		$key = $_GET['id'];
		$secret = $_GET['code'];
		$now = stms_time();
		
		if ( !isset($regs[$key]) || ($regs[$key]['secret'] != $secret) || ($regs[$key]['used'] >= $regs[$key]['count']) )
			wp_die("ERROR: Invalid URL");
	
		if ( $regs[$key]['expired'] > 0 && ($now >= $regs[$key]['expired']) )
			wp_die("ERROR: Invalid URL");
		
		$product_id = intval($regs[$key]['product_id']);
		if ( !$product = stms_get_product($product_id) )
			wp_die("ERROR: Invalid URL");
		
		$stms = get_option('stms_settings');
		if ( is_stmsmember_logged_in() ) {
			$member_id = stms_get_current_member_id();
			$acc_type = ( $current_member->status == 4 ) ? 'temp' : 'real';
		} else {
			// let's create a temporary member account...
			if ( !$member_id = stms_create_temp_member() )
				wp_die('ERROR: Invalid Registration URL.');
			
			$acc_type = 'temp';
		}
		
		// let's create the temp trx...
		// Create pending transaction first...
		$args = array(
			'product_id' => $product->ID,
			'member_id' => $member_id,
			'payment_type' => $product->type,
			'paysys' => 'free',
			'currency' => stms_isset($stms['global']['currency'], 'USD'),
			'amount' => 0.00,
			'status' => 'temp',
			'date' => stms_time(),
			'expired_date' => ( $product->type == 'recurr' ) ? strtotime("19 Jan 2038", stms_time()) : 0,
			'ipaddr' => stms_retrieve_ipaddr()
		);
		$tx_id = stms_add_pending_payment($args);
		
		$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_permalink($stms['global']['home']);
		if ( $acc_type == 'real' ) {
			// let's complete the payment...
			$args = array(
				'product_id' => $product->ID,
				'member_id' => $member_id,
				'payment_type' => $product->type,
				'paysys' => 'free',
				'currency' => stms_isset($stms['global']['currency'], 'USD'),
				'amount' => 0.00,
				'paid' => 1,
				'is_rebill' => 0,
				'status' => 'completed',
				'expired_date' => ( $product->type == 'recurr' ) ? strtotime("19 Jan 2038", stms_time()) : 0,
			);
			$tx_id = stms_complete_payment($tx_id, $args);
			wp_redirect($home_url);
			exit;
		}
		
		// proceed to registration
		$session = base64_encode("{$product->ID}|free|direct");
		$insta_token = stms_generate_secret(16, true, true);
		$token_data = array('token' => $insta_token, 'tx_id' => $tx_id);
		set_transient('insta_token_' . $insta_token, $token_data, 21600); // lasted for six hours
		
		$_reg_url = stms_get_handler_url('registration');
		$reg_url = stms_format_url($_reg_url, 'key=' . $session . '&insta_token=' . $insta_token . '&special_id=' . $key);
		wp_redirect($reg_url);
		exit;
	}
}

add_action('template_redirect', 'stms_order_lookup_action');
function stms_order_lookup_action() {
	if ( isset($_POST['_instamember_action']) && $_POST['_instamember_action'] == 'stms_order_check') {
		if ( !wp_verify_nonce($_POST['_instamember_nonce'], 'stms-nonce') ) wp_die('Invalid Handler');
		
		global $wpdb;
		
		$stms = get_option('stms_settings');
		$paysys = $_POST['paysys'];
		$product_id = ( isset($_POST['product_id']) ) ? $_POST['product_id'] : 0;
		$auth_vars = base64_encode(json_encode($_POST));
		$check_url = stms_format_url(stms_get_handler_url('order_lookup'), 'paysys=' . $paysys);
		if ( !empty($product_id) )
			$check_url .= '&product_id=' . $product_id;
		
		if ( empty($_POST['im_order_email']) ) {
			$auth_error = base64_encode('Please enter the email address you used to purchased.');
			$error_url = $check_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
			@header('Location:' . $error_url);
			exit;
		}
		
		$found = false;
		$payer_email = sanitize_email(trim($_POST['im_order_email']));
		$sql = "SELECT * FROM `{$wpdb->prefix}stms_ipn_log` WHERE `paysys` = '{$paysys}' AND `pay_email` = '{$payer_email}'";
		if ( !$ipn = $wpdb->get_row($sql) ) {
			$auth_error = base64_encode('ERROR: Cannot find any order under email: "' . $payer_email . '"');
			$error_url = $check_url . "&auth_error={$auth_error}&auth_vars={$auth_vars}";
			@header('Location:' . $error_url);
			exit;
		}
		
		if ( !$product_id ) {
			$payment = stms_get_payment($ipn->tx_id);
			$product_id = $payment->product_id;
		}
		
		$rcpt = stms_sanitize($ipn->receipt);
		$thanks_url = stms_format_url(get_bloginfo('url'), 'insta_mode=product_access&processor=' . $paysys . '&insta_id=' . $product_id . '&insta_receipt=' . $rcpt);
		
		wp_redirect($thanks_url);
		exit;
	}	
}
