<?php if ( !defined('STMS_PATH') ) wp_die('Access Denied');

require_once( STMS_INC . 'payment-functions.php' ); // load payment functions

/* 
 * ======================== MEMBER REGISTRATION FUNCTIONS ========================
 */
 
function stms_generate_username($email, $first_name, $last_name = '', $loop = false ) {
	$first_name = strtolower(remove_accents($first_name));
	$last_name = strtolower(remove_accents($last_name));
	
	$username = stms_sanitize($first_name);
	if ( !empty($last_name) )
		$username .= "_{$last_name}";
	
	if ( $loop || stms_username_exists($username) || empty($username) ) {
		$parts = explode('@', $email);
		$username = ( !empty($parts[0]) ) ? $parts[0] : $username;
	
		if ( stms_username_exists($username) )
			$username .= '_' . stms_generate_secret(4, true, false, false);
			
		// Let's check again...
		if ( stms_username_exists($username) )
			$username = stms_generate_username($email, $first_name, $last_name, true);
	}
	
	$username = str_replace(array(".", "-", " "), "_", $username);
	return $username;
}

function stms_random_username( $length = 10, $check = true ) {
	$random_id = crypt(uniqid(rand(), 1)); 
	$random_id = strip_tags(stripslashes($random_id)); 
	$random_id = str_replace(".", "", $random_id); 
	$random_id = strrev(str_replace("/", "", $random_id)); 
	$random_id = substr($random_id, 0, $length);
	
	if ( $check && stms_username_exists($random_id) )
		$random_id = stms_random_username($length);
	
	return $random_id;
}

function stms_generate_temporary() {
	$length = rand(6,14);
	$name = stms_generate_secret($length, false );
	
	$length = rand(4,10);
	$name .= '_' . stms_generate_secret($length, true );
	
	return array('username' => $name, 'email' => $name . '@' . $_SERVER['HTTP_HOST']);
}

function stms_create_temp_member() {
	$temp = stms_generate_temporary();
	$username = $temp['username'];
	$email = $temp['email'];
	$password = wp_generate_password();
	
	$data = array();
	$data['member_id'] = 0; // set member to '0' because member didn't exists yet...
	$data['email'] = $email;
	$data['username'] = $username;
	$data['password'] = $password;
	$data['status'] = 4;
	$data['temporary'] = true;
	
	return stms_save_member($data);
}

function stms_auto_register_customer( $tx_id, $email, $first_name, $last_name = '', $paypal_id = '', $paypal_email = '' ) {
	global $wpdb;
	
	$pay = stms_get_payment($tx_id);
	$stms = get_option('stms_settings');
	
	if ( $member = stms_get_data_by('email', $email) )
		return $member->ID;
	
	$sql = "SELECT * FROM `{$wpdb->prefix}stms_members` WHERE `status` = 1 AND `paypal_id` = '{$paypal_id}'";
	if ( !empty($paypal_id) && ($payer = $wpdb->get_row($sql)) )
		return $payer->ID;
		
	$sql = "SELECT * FROM `{$wpdb->prefix}stms_members` WHERE `status` = 1 AND `paypal_email` = '{$paypal_email}'";
	if ( !empty($paypal_email) && ($payer = $wpdb->get_row($sql)) )
		return $payer->ID;

	// no member found... let's create a new one...
	$new_username = stms_generate_username($email, $first_name, $last_name);
	$new_password = wp_generate_password(8, false, false);
	$userdata = array();
	$userdata['member_id'] = ( isset($pay->member_id) ) ? $pay->member_id : 0;
	$userdata['username'] = $new_username;
	$userdata['password'] = $new_password;
	if ( isset($pay->member_id) && $pay->member_id > 0 ) {
		$userdata['fix_username'] = $new_username;
		$userdata['pwd1'] = $new_password;
	}
	$userdata['email'] = $email;
	$userdata['first_name'] = $first_name;
	$userdata['last_name'] = $last_name;
	$userdata['status'] = 1;
	$userdata['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
	$userdata['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
	
	if ( !empty($paypal_id) )
		$userdata['paypal_id'] = $paypal_id;
	
	if ( !empty($paypal_email) )
		$userdata['paypal_email'] = $paypal_email;
	
	$member_id = stms_save_member($userdata);
	return $member_id;
}

function stms_save_member( $userdata, $manual = false, $confirm = false, $tx_id = 0, $is_aff = false ) {
	if ( !is_array($userdata) )
		return false;
	
	global $wpdb;
	$username = esc_sql($userdata['username']);
	$username = sanitize_user($username, true);
	$plain_password = trim(stms_isset($userdata['password']));
	$email = esc_sql($userdata['email']);
	
	$data = array();
	$data['email'] = $email;
	$data['status'] = stms_isset($userdata['status'], 1);
	$data['is_locked'] = stms_isset($userdata['is_locked'], 'no');
	$data['is_aff'] = stms_isset($userdata['is_aff'], 1);
	$data['is_aff_pending'] = stms_isset($userdata['is_aff_pending'], 0);
	
	$data['first_name'] = isset($userdata['first_name']) ? sanitize_text_field($userdata['first_name']) : '';
	$data['last_name'] = isset($userdata['last_name']) ? sanitize_text_field($userdata['last_name']) : '';
	
	if ( isset($userdata['paypal_id']) )
		$data['paypal_id'] = sanitize_email($userdata['paypal_id']);
	
	if ( isset($userdata['paypal_email']) )
		$data['paypal_email'] = sanitize_email($userdata['paypal_email']);
	
	if ( isset($userdata['wpuser_id']) )
		$data['wpuser_id'] = sanitize_email($userdata['wpuser_id']);
	
	// Are we updating or creating?
	$member_id = intval($userdata['member_id']);
	if ( $member_id > 0 ) {
		$ID = $member_id;
		$update = true;
		
		$meta = stms_get_member_meta($ID, 'info');
		$user_key = stms_sanitize($meta['user_key']);
		if ( isset($userdata['fix_username']) ) {
			$fix_username = esc_sql($userdata['fix_username']);
			$fix_username = sanitize_user($fix_username, true);
			$data['username'] = trim($fix_username);
		}
		$old_member_data = stms_get_data_by('id', $ID);
		if ( !empty($userdata['pwd1']) ) {
				$plaintext_pass = trim($userdata['pwd1']);
				$data['pass'] = wp_hash_password($plaintext_pass);
				$user_key = stms_string_enc($plaintext_pass, AUTH_SALT);
		}
		$wpdb->update("{$wpdb->prefix}stms_members", $data, compact('ID'));
	} else {
		$update = false;
		$data['username'] = trim($username);
		$data['pass'] = ( stms_isset($userdata['is_aff_pending'], 0) == 1 ) ? $plain_password : wp_hash_password($plain_password);
		if ( $confirm )
			$data['pass'] = $plain_password;
		$data['registered'] = date("Y-m-d H:i:s", stms_time());
		$data['date'] = stms_time();
		
		$user_key = stms_string_enc($plain_password, AUTH_SALT);
		$wpdb->insert("{$wpdb->prefix}stms_members", $data);
		$ID = $wpdb->insert_id;
	}
	
	$metadata = array();
	if ( isset($userdata['street']) && $userdata['street'] != '' )
		$metadata['street'] = stms_sanitize($userdata['street']);
	if ( isset($userdata['city']) && $userdata['city'] != '' )
		$metadata['city'] = stms_sanitize($userdata['city']);
	if ( isset($userdata['state']) && $userdata['state'] != '' )
		$metadata['state'] = stms_sanitize($userdata['state']);
	if ( isset($userdata['zipcode']) && $userdata['zipcode'] != '' )
		$metadata['zipcode'] = stms_sanitize($userdata['zipcode']);
	if ( isset($userdata['country']) && $userdata['country'] != '' )
		$metadata['country'] = stms_sanitize($userdata['country']);
	if ( isset($userdata['title']) && $userdata['title'] != '' )
		$metadata['title'] = stms_sanitize($userdata['title']);
	if ( isset($userdata['phone']) && $userdata['phone'] != '' )
		$metadata['phone'] = stms_sanitize($userdata['phone']);
	if ( isset($userdata['company']) && $userdata['company'] != '' )
		$metadata['company'] = stms_sanitize($userdata['company']);
	if ( isset($userdata['gender']) && $userdata['gender'] != '' )
		$metadata['gender'] = stms_sanitize($userdata['gender']);
	if ( isset($userdata['url']) && $userdata['url'] != '' )
		$metadata['url'] = stms_sanitize($userdata['url']);
	
	if ( isset($user_key) && $user_key != '' )
		$metadata['user_key'] = $user_key;

	// Update additional member information
	if ( count($metadata) > 0 )
		stms_update_member_meta($ID, 'info', $metadata);
	
	// delete caches
	delete_transient('instamem_member_' . $ID);
	wp_cache_delete($ID, 'stms_members');
	wp_cache_delete($username, 'stms_memberlogins');
	wp_cache_delete($email, 'stms_memberemail');

	$current_member = stms_get_current_member();
	if ( stms_isset($current_member->ID) == $ID ) {
		if ( isset($plaintext_pass) ) {
			stms_clear_auth_cookie();
			stms_set_auth_cookie($ID);
		}
	}
	
	$call_hook = true;
	if ( $is_aff && stms_isset($userdata['is_aff_pending']) == 1 || isset($userdata['temporary']) || $manual || $confirm )
		$call_hook = false;
	
	if ( $call_hook ) {
		if ( $update ) {
			if ( isset($userdata['fix_username']) )
				do_action('stms_member_registered', $ID, $plaintext_pass, $tx_id);
			else
				do_action('stms_profile_updated', $ID, $old_user_data);
		} else
			do_action('stms_member_registered', $ID, $plain_password, $tx_id);
	}

	do_action('stms_member_saved', $ID, $userdata); // hook for wp sync...
	
	return $ID;
}

function stms_delete_member( $member_id, $redirect = true ) {
	global $wpdb;

	$stms = get_option('stms_settings');
	
	$id = (int) $member_id;
	$member = stms_member($id);
	if ( !$member ) {
		if ( $redirect ) {
			@header( 'Location:' . admin_url('admin.php?page=stms-settings') . '#members');
			exit;
		}
		return false;
	}
	
	do_action('stms_member_delete', $id);

	if ( stms_isset($stms['global']['wp_sync']) == 1 ) {
		$wpuser_id = $member->wpuser_id;
		if ( !user_can($wpuser_id, 'edit_posts') ) {
			wp_delete_user($wpuser_id);
		}
	}
	
	stms_clean_member_cache($member);
	$wpdb->delete("{$wpdb->prefix}stms_members", array('ID' => $id));
	
	do_action('stms_member_deleted', $id);
	if ( $redirect ) {
		@header( 'Location:' . admin_url('admin.php?page=stms-settings') . '#members|member=deleted' );
		exit;
	}
	return true;
}

function stms_delete_old_members() {
	global $wpdb;
	$old_time = strtotime("7 days ago", stms_time());
	if ( $members = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}stms_members` WHERE `status` = 4 AND `date` <= {$old_time}") ) {
		foreach ( $members as $m ) {
			$wpdb->delete("{$wpdb->prefix}stms_members", array('ID' => $m->ID));
			
			// delete caches
			stms_clean_member_cache($m);
		}
	}
}

function stms_update_member( $member_id, $data = array() ) {
	if ( count($data) <= 0 )
		return false;
	
	$stms = get_option('stms_settings');
	$member = stms_member($member_id);
	if ( !$member )
		return false;
	
	$ID = $member->ID;
	global $wpdb;
	
	if ( isset($data['pass']) ) {
		$plain_password = trim($data['pass']);
		$data['pass'] = wp_hash_password($plain_password);
	}
	$wpdb->update("{$wpdb->prefix}stms_members", $data, compact('ID'));
	
	if ( stms_isset($stms['global']['wp_sync']) == 1 && !empty($plain_password) ) {
		$wpuser_id = $member->wpuser_id;
		if ( $wpuser_id )
			wp_update_user( array('user_pass' => trim($plain_password)) );
	}
	
	// delete caches
	delete_transient('instamem_member_' . $ID);
	wp_cache_delete($ID, 'stms_members');
	wp_cache_delete($member->username, 'stms_memberlogins');
	wp_cache_delete($member->email, 'stms_memberemail');

	do_action('stms_member_updated', $ID);
	
    return $ID;
}

add_action('template_redirect', 'stms_confirm_member', 2);
function stms_confirm_member() {
	if ( isset($_GET['insta_confirm']) && isset($_GET['id']) && isset($_GET['tx']) ) {
		$stms = get_option('stms_settings');
		$login_url = stms_login_url();
		$code = urldecode($_GET['insta_confirm']);
		$member_id = intval($_GET['id']);
		$tx_id = intval($_GET['tx']);
		
		if ( !$member = stms_member($member_id) ) {
			wp_redirect($login_url . '&msg=invalid_confirm_url');
			exit;
		}
		
		if ( !$payment = stms_get_payment($tx_id) ) {
			wp_redirect($login_url . '&msg=invalid_confirm_url');
			exit;
		}
		
		if ( !$product = stms_get_product($payment->product_id) ) {
			wp_redirect($login_url . '&msg=invalid_confirm_url');
			exit;
		}
		
		$confirm = get_transient('stms_confirm_' . $member->ID);
		if ( empty($confirm) ) {
			wp_redirect($login_url . '&msg=invalid_confirm_url');
			exit;
		}
		
		if ( $confirm != $code ) {
			wp_redirect($login_url . '&msg=invalid_confirm_url');
			exit;
		}
		
		// update member
		$password = stms_string_dec($member->info['user_key'], AUTH_SALT);
		$data = array(
			'member_id' => $member->ID,
			'username' => stms_sanitize($member->username),
			'fix_username' => stms_sanitize($member->username),
			'pwd1' => $password,
			'status' => 1,
			'email' => $member->email,
			'first_name' => stms_sanitize($member->first_name),
			'last_name' => stms_sanitize($member->last_name)
		);
		$member_id = stms_save_member($data);
		
		// update payment (for access)
		$new_data = array(
			'status' => 'completed',
			'paid' => 1
		);
		$tx_id = stms_update_payment($tx_id, $new_data);
		wp_cache_delete($member->ID, 'stms_member_payments'); // delete cache
		
		do_action('stms_payment_completed', $tx_id);
		
		$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : '';
		$redirect_url = ( filter_var($product->info['thankuri'], FILTER_VALIDATE_URL) ) ? $product->info['thankuri'] : $home_url;
		if ( $redirect_url == '' )
			$redirect_url = ( !empty($stms['global']['home']) ) ? get_permalink($stms['global']['home']) : get_bloginfo('siteurl');
				
		delete_transient('stms_confirm_' . $member->ID);		
		if ( is_stmsmember_logged_in() ) // just a pre-caution...
			stms_clear_auth_cookie();
		
		stms_set_auth_cookie($member_id, false, is_ssl());
		
		if ( isset($_COOKIE['__insta_flow']) ) {
			$flow_id = urldecode($_COOKIE['__insta_flow']);
			do_action('insta_upsells_redirect', $flow_id, $product->ID, 'free', $member->ID);
		}
		
		wp_redirect($redirect_url);
		exit;
	}
}

add_action('user_register', 'stms_sync_new_wpuser');
function stms_sync_new_wpuser( $user_id ) {
	global $wpdb;
	
	$stms = get_option('stms_settings');
	if ( stms_isset($stms['global']['wp_sync']) != 1 )
		return false;
		
	$wpuser = new WP_User($user_id);	
	if ( !$wpuser || !is_object($wpuser) )
		return false;
	
	$username = $wpuser->data->user_login;
	if ( stms_username_exists($username) )
		return false;
	
	$first_name = get_user_meta($wpuser->ID, 'first_name', true);
	$last_name = get_user_meta($wpuser->ID, 'last_name', true);
	$encrypted_pass = $wpuser->data->user_pass;
	$userdata = array(
		'ID' => 0,
		'username' => $username,
		'email' => $wpuser->data->user_email,
		'pass' => 'temp_pass',
		'status' => 1,
		'first_name' => $first_name,
		'last_name' => $last_name,
		'wpuser_id' => $wpuser->ID,
		'is_aff' => ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1),
		'is_aff_pending' => ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1)
	);
	$member_id = stms_save_member($userdata, true);
	
	// Update the password...
	$wpdb->update("{$wpdb->prefix}stms_members", array('pass' => $encrypted_pass), array('ID' => $member_id));
	$member = stms_member($member_id);
	
	// clean up caches...
	stms_clean_member_cache($member);
	return true;
}

add_action('profile_update', 'stms_sync_wpuser_data', 10, 2);
function stms_sync_wpuser_data( $user_id, $old_user_data ) {
	$stms = get_option('stms_settings');
	if ( stms_isset($stms['global']['wp_sync']) != 1 )
		return false;
	
	global $wpdb;
	
	$current_member_id = stms_get_current_member_id();
	$wpuser_id = intval($user_id);
	$wpuser = get_userdata($wpuser_id);
	if ( $member = $wpdb->get_row("SELECT * FROM `{$wpdb->prefix}stms_members` WHERE `wpuser_id` = {$wpuser_id}") ) {
		if ( $wpuser->user_pass != $member->pass ) {
			if ( $current_member_id && ($current_member_id == $member->ID) ) {
				stms_clear_auth_cookie(true);
			}
		}
		
		$data = array(
			'email' => sanitize_email($wpuser->user_email),
			'pass' => $wpuser->user_pass,
			'first_name' => stms_sanitize($wpuser->first_name),
			'last_name' => stms_sanitize($wpuser->last_name)
		);
		$wpdb->update("{$wpdb->prefix}stms_members", $data, array('ID' => $member->ID));
		
		// delete caches
		delete_transient('instamem_member_' . $member->ID);
		wp_cache_delete($member->ID, 'stms_members');
		wp_cache_delete($member->username, 'stms_memberlogins');
		wp_cache_delete($member->email, 'stms_memberemail');
	
		if ( $wpuser->user_pass != $member->pass ) {
			if ( $current_member_id && ($current_member_id == $member->ID) ) {
				stms_set_auth_cookie($member->ID, false, is_ssl(), true);
			}
		}
	}
}

add_action('stms_member_saved', 'stms_register_wpuser', 10, 2);
function stms_register_wpuser( $member_id, $userdata ) {
	global $wpdb;
	
	$stms = get_option('stms_settings');
	if ( stms_isset($stms['global']['wp_sync']) != 1 )
		return false;
		
	$member = stms_member($member_id);
	if ( $member->status == 4 || $member->status == 0 )
		return false;
	
	$username = esc_sql($userdata['username']);
	$username = sanitize_user($username, true);
	$plain_password = trim(stms_isset($userdata['password']));
	$email = esc_sql($userdata['email']);
	$email = sanitize_email($email);
	
	$wpuser_id = username_exists($username);
	$update = ( $wpuser_id || email_exists($email) ) ? true : false;
	
	if ( $member->wpuser_id > 0 ) {
		if ( $wpuser_id && ($wpuser_id != $member->wpuser_id) || user_can($wpuser_id, 'edit_posts') ) return false; // let's not take the risk updating someone else account...
		$wpuser_id = $member->wpuser_id;
		$update = true;
	}
	
	if ( !$update ) {
		$username = ( !empty($userdata['fix_username']) ) ? sanitize_user(esc_sql($userdata['fix_username'])) : $username;
		$password = ( !empty($userdata['pwd1']) ) ? trim($userdata['pwd1']) : $plain_password;
		$wpuser_id = wp_create_user($username, $password, $email);
	}
		
	if ( $wpuser_id ) {
		$data = array(
			'ID' => $wpuser_id,
			'first_name' => stms_sanitize($userdata['first_name']),
			'last_name' => stms_sanitize($userdata['last_name'])
		);
		
		if ( !user_can($wpuser_id, 'edit_posts') ) {
			$data['show_admin_bar_front'] = 'false';
			$data['role'] = 'subscriber';
		}
		
		if ( !user_can($wpuser_id, 'edit_posts') && !empty($userdata['pwd1']) )
			$data['user_pass'] = trim($userdata['pwd1']);
		
		if ( !$update )
			$data['user_registered'] = date("Y-m-d H:i:s", $member->date);
		
		@wp_update_user($data);
		$wpdb->update("{$wpdb->prefix}stms_members", array('wpuser_id' => $wpuser_id), array('ID' => $member->ID));
	}
	return true;
}

/* 
 * ======================== LOGIN/AUTHENTICATION FUNCTIONS ========================
 */
function stms_login_url( $redir = '' ) {
	$stms = get_option('stms_settings');
	$slugs = get_option('stms_slugs');
	$url = ( !empty($redir) ) ? rawurlencode($redir) : get_permalink(stms_isset($stms['global']['home']));
	//$redirect_path = str_replace(get_bloginfo('url'), '', $url);
	$login_id = ( isset($slugs['login']) && $slugs['login'] > 0 ) ? $slugs['login'] : 0;
	$handler_id = stms_isset($stms['global']['magic']);
	if ( $login_id )
		$login_url = stms_format_url($login_id, 'redir=' . rawurlencode($url));
	else
		$login_url = stms_format_url($handler_id, 'imhandler=login&redir=' . rawurlencode($url));
	
	return $login_url;
}

function stms_member_url( $redir = '' ) {
	$stms = get_option('stms_settings');
	$home_url = get_permalink(stms_isset($stms['global']['home']));
	//$member_url = ( !empty($redir) ) ? get_bloginfo('url') . rawurlencode($redir) : $home_url;
	$member_url = ( !empty($redir) ) ? rawurldecode($redir) : $home_url;
	
	return $member_url;
}

function stms_login($credentials, $secure_cookie = '') {
	if ( empty($credentials) || !is_array($credentials) ) return false;
	if ( !empty($credentials['remember']) )
		$credentials['remember'] = true;
	else
		$credentials['remember'] = false;

	if ( '' === $secure_cookie )
		$secure_cookie = is_ssl(); // is_ssl() is a WP function
	
	$member = stms_authenticate( $credentials['member_login'], $credentials['member_pass'] );
	if ( !$member )
		return false;

	stms_set_auth_cookie($member->ID, $credentials['remember'], $secure_cookie);
	return $member;
}

function stms_authenticate( $username, $password ) {
	if ( !function_exists('wp_hash_password') )
		require_once(ABSPATH . 'wp-includes/pluggable.php');
	
	$username = sanitize_user($username);
	$password = trim($password);

	if ( empty($username) || empty($password) )
		return false;
	
	$datatype = 'login';
	if ( sanitize_email($username) )
		$datatype = 'email';
	
	$memberdata = stms_get_data_by($datatype, $username);
	// if member not found...
	if ( !$memberdata )
		return false;
	
	// check account status
	if ( $memberdata->status != 1 )
		return false;
	
	// check password
	if ( !class_exists('PasswordHash') )
		require_once( ABSPATH . 'wp-includes/class-phpass.php');
	
	$wp_hasher = new PasswordHash(8, TRUE);
	$hashed_pass = $memberdata->pass;
	$plain_pass = $password;
	
	if ( !$wp_hasher->CheckPassword($plain_pass, $hashed_pass) )
	    return false;

	$member = stms_member($memberdata->ID);
	return $member;
}

function stms_set_auth_cookie( $member_id, $remember = false, $secure = '', $is_wp = false ) {
	if ( $remember ) {
		$expiration = $expire = time() + 1209600;
	} else {
		$expiration = time() + 172800;
		$expire = 0;
	}

	if ( '' === $secure )
		$secure = is_ssl();  // is_ssl() is a WP function
	
	if ( $secure ) {
		$auth_cookie_name = 'stms_sec_' . COOKIEHASH;
		$scheme = 'secure_auth';
	} else {
		$auth_cookie_name = 'stms_' . COOKIEHASH;
		$scheme = 'auth';
	}

	$auth_cookie = stms_generate_auth_cookie($member_id, $expiration, $scheme);
	$logged_in_cookie = stms_generate_auth_cookie($member_id, $expiration, 'logged_in');
	
	$cookie_domain = str_replace(array('http://', 'https://'), '', $_SERVER['HTTP_HOST']);
	$cookie_domain = str_replace('/', '', $cookie_domain);

	@setcookie($auth_cookie_name, $auth_cookie, $expire, PLUGINS_COOKIE_PATH, COOKIE_DOMAIN, $secure, true);
	@setcookie($auth_cookie_name, $auth_cookie, $expire, ADMIN_COOKIE_PATH, COOKIE_DOMAIN, $secure, true);
	@setcookie($auth_cookie_name, $auth_cookie, $expire, COOKIEPATH, COOKIE_DOMAIN, $secure, true);
	@setcookie('stms_logged_in_' . COOKIEHASH, $logged_in_cookie, $expire, COOKIEPATH, COOKIE_DOMAIN, $secure, true);
	if ( COOKIEPATH != SITECOOKIEPATH ) {
		@setcookie($auth_cookie_name, $auth_cookie, $expire, SITECOOKIEPATH, COOKIE_DOMAIN, $secure, true);
		@setcookie('stms_logged_in_' . COOKIEHASH, $logged_in_cookie, $expire, SITECOOKIEPATH, COOKIE_DOMAIN, $secure, true);
		
	}
		
	$ipaddr = stms_retrieve_ipaddr();
	
	do_action('stms_loggedin_cookied', $member_id, $ipaddr);
	do_action('stms_claim_payments', $member_id);
	return true;
}

function stms_generate_auth_cookie($member_id, $expiration, $scheme = 'auth') {
	if ( !function_exists('wp_hash') )
		require_once(ABSPATH . 'wp-includes/pluggable.php');
	
	$member = stms_get_memberdata($member_id);
	$pass_frag = substr($member->pass, 8, 4);
	$key = wp_hash($member->username . $pass_frag . '|' . $expiration, $scheme); // wp_hash is a wp function
	$hash = hash_hmac('md5', $member->username . '|' . $expiration, $key);

	$cookie = $member->username . '|' . $expiration . '|' . $hash;
	return $cookie;
}

function stms_validate_auth_cookie($cookie = '', $scheme = '') {
	if ( !$cookie_elements = stms_parse_auth_cookie($cookie, $scheme) )
		return false;

	extract($cookie_elements, EXTR_OVERWRITE);
	$expired = $expiration;

	// Quick check to see if an honest cookie has expired
	if ( $expired < time() ) {
		stms_clear_auth_cookie();
		return false;
	}
	
	$member = stms_get_data_by('login', $username);
	if ( !$member ) {
		stms_clear_auth_cookie();
		return false;
	}

	$pass_frag = substr($member->pass, 8, 4);

	if ( !function_exists('wp_hash') )
		require_once(ABSPATH . 'wp-includes/pluggable.php');
		
	$key = wp_hash($username . $pass_frag . '|' . $expiration, $scheme);
	$hash = hash_hmac('md5', $username . '|' . $expiration, $key);

	if ( $hmac != $hash ) {
		stms_clear_auth_cookie();
		return false;
	}

	return $member->ID;
}

function stms_parse_auth_cookie($cookie = '', $scheme = '') {
	if ( empty($cookie) ) {
		switch ($scheme){
			case 'auth':
				$cookie_name = 'stms_' . COOKIEHASH;
				break;
			case 'secure_auth':
				$cookie_name = 'stms_sec_' . COOKIEHASH;
				break;
			case "logged_in":
				$cookie_name = 'stms_logged_in_' . COOKIEHASH;
				break;
			default:
				if ( is_ssl() ) {
					$cookie_name = 'stms_sec_' . COOKIEHASH;
					$scheme = 'secure_auth';
				} else {
					$cookie_name = 'stms_' . COOKIEHASH;
					$scheme = 'auth';
				}
	    }

		if ( empty($_COOKIE[$cookie_name]) )
			return false;
		$cookie = $_COOKIE[$cookie_name];
	}

	$cookie_elements = explode('|', $cookie);
	if ( count($cookie_elements) != 3 )
		return false;

	list($username, $expiration, $hmac) = $cookie_elements;
	return compact('username', 'expiration', 'hmac', 'scheme');
}

function stms_clear_auth_cookie( $is_wp = false ) {
	if ( $is_wp )
		do_action('stms_clear_auth_cookie');
	
	//$debug = debug_backtrace();
	//stms_error_handler("Logging Out - " . __FILE__ . " - " . maybe_serialize($debug), 0, '', 'logout', false);
	
	@setcookie('stms_' . COOKIEHASH, ' ', time() - 31536000, PLUGINS_COOKIE_PATH, COOKIE_DOMAIN);
	@setcookie('stms_sec_' . COOKIEHASH, ' ', time() - 31536000, PLUGINS_COOKIE_PATH, COOKIE_DOMAIN); // for secure cookie
	
	@setcookie('stms_' . COOKIEHASH, ' ', time() - 31536000, ADMIN_COOKIE_PATH, COOKIE_DOMAIN);
	@setcookie('stms_sec_' . COOKIEHASH, ' ', time() - 31536000, ADMIN_COOKIE_PATH, COOKIE_DOMAIN); // for secure cookie
	
	@setcookie('stms_' . COOKIEHASH, ' ', time() - 31536000, SITECOOKIEPATH, COOKIE_DOMAIN);
	@setcookie('stms_sec_' . COOKIEHASH, ' ', time() - 31536000, SITECOOKIEPATH, COOKIE_DOMAIN); // for secure cookie
	@setcookie('stms_logged_in_' . COOKIEHASH, ' ', time() - 31536000, SITECOOKIEPATH, COOKIE_DOMAIN);
	
	@setcookie('stms_' . COOKIEHASH, ' ', time() - 31536000, COOKIEPATH, COOKIE_DOMAIN);
	@setcookie('stms_sec_' . COOKIEHASH, ' ', time() - 31536000, COOKIEPATH, COOKIE_DOMAIN); // for secure cookie
	@setcookie('stms_logged_in_' . COOKIEHASH, ' ', time() - 31536000, COOKIEPATH, COOKIE_DOMAIN);
}

function is_stmsmember_logged_in() {
	if ( $member = stms_get_current_member() )
		return true;
	
	if ( $member_id = stms_validate_auth_cookie('', 'logged_in') )
		return true;
	
	return false;
}

function stms_auth_redirect() {
	$secure = is_ssl(); // is_ssl() is a WP function

	// Redirect to https if forced to use SSL
	if ( $use_ssl = get_option('use_ssl') && !is_ssl() ) {
		wp_redirect('https://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);
		exit;
	}

	if ( $member_id = stms_validate_auth_cookie('', 'logged_in') ) {
		if ( $use_ssl = get_option('use_ssl') && !is_ssl() ) {
			wp_redirect('https://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);
			exit();
		}

		return;  // The cookie is good so we're done
	}

	// The cookie is no good so force login
	nocache_headers(); // This is a WP function

	if ( is_ssl() )
		$proto = 'https://';
	else
		$proto = 'http://';

	$redirect = $proto . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];
	$login_url = stms_login_url($redirect);

	wp_redirect($login_url);
	exit;
}

/* 
 * ======================== MEMBER DATA FUNCTIONS ========================
 */
function stms_set_current_member($id, $name = '') {
	$current_member = stms_member($id, $name);
	return $current_member;
}

function stms_get_current_member() {
	if ( $member_id = stms_validate_auth_cookie('', 'logged_in') ) {
		$current_member = stms_member($member_id);
		return $current_member;
	}
}

function stms_currentmemberinfo() {
	if ( !$member = stms_validate_auth_cookie() ) {
		 if ( empty( $_COOKIE['stms_logged_in_' . COOKIEHASH] ) || !$member = stms_validate_auth_cookie( $_COOKIE['stms_logged_in_' . COOKIEHASH], 'logged_in' ) ) {
		 	stms_set_current_member(0);
		 	return false;
		 }
	}

	$current_member = stms_set_current_member($member);
	return $current_member;
}

function stms_get_current_member_id() {
	if ( !is_stmsmember_logged_in() ) return 0;
	$member = stms_get_current_member();
	return ( isset($member->ID) ) ? $member->ID : 0;
}

function stms_get_memberdata( $member_id ) {
	return stms_get_data_by( 'id', $member_id );
}

function stms_member( $id = 0, $name = '' ) {
	if ( !empty($id) && !is_numeric($id) ) {
		$name = $id;
		$id = 0;
	}
	if ( $id )
		$data = stms_get_data_by( 'id', $id );
	else
		$data = stms_get_data_by( 'login', $name );

	if ( !$data || !is_object($data) )
		return false;
	if ( isset($data->metadata) )
		unset($data->metadata);
	
	$data->info = stms_get_member_meta($data->ID, 'info');
	return $data;
}

function stms_username_exists( $username ) {
	if ( $member = stms_get_data_by('login', $username ) )
		return $member->ID;

	return false;
}

function stms_email_exists( $email ) {
	$email = sanitize_email($email);
	if ( $member = stms_get_data_by('email', $email) )
		return $member->ID;

	return false;
}

function stms_get_data_by( $field, $value ) {
	global $wpdb;

	if ( 'id' == $field ) {
		// Make sure the value is numeric to avoid casting objects, for example,
		// to int 1.
		if ( !is_numeric($value) )
			return false;
		$value = absint($value);
	} else {
		$value = trim($value);
	}

	if ( !$value )
		return false;

	switch ( $field ) {
		case 'id':
			$member_id = $value;
			$db_field = 'ID';
			break;
		case 'email':
			$member_id = wp_cache_get($value, 'stms_memberemail');
			$db_field = 'email';
			break;
		case 'login':
			$value = sanitize_user($value);
			$member_id = wp_cache_get($value, 'stms_memberlogins');
			$db_field = 'username';
			break;
		default:
			return false;
	}

	if ( false !== $member_id ) {
		if ( $member = wp_cache_get( $member_id, 'stms_members' ) )
			return $member;
	}

	if ( !$member = $wpdb->get_row($wpdb->prepare("SELECT * FROM `{$wpdb->prefix}stms_members` WHERE `$db_field` = %s", $value)) )
		return false;

	stms_update_member_caches( $member );
	return $member;
}

function stms_update_member_caches($member) {
	wp_cache_add($member->ID, $member, 'stms_members');
	wp_cache_add($member->username, $member->ID, 'stms_memberlogins');
	wp_cache_add($member->email, $member->ID, 'stms_memberemail');
}

function stms_clean_member_cache( $member ) {
	if ( is_numeric($member) )
		$member = stms_member($member);
	if ( !is_object($member) ) return;
	
	wp_cache_delete($member->ID, 'stms_members');
	wp_cache_delete($member->username, 'stms_memberlogins');
	wp_cache_delete($member->email, 'stms_memberemail');
}

/* 
 * ======================== PRODUCT FUNCTIONS ========================
 */
function stms_get_products() {
	global $wpdb;
	
	$sql = "SELECT `ID` FROM `{$wpdb->prefix}stms_products` WHERE `product_status` <> 4 ORDER BY date DESC";
	$ids = $wpdb->get_results($sql);
	if ( !$ids || empty($ids) )
		return false;

	$products = array();
	foreach ( $ids as $id ) {
		$products[] = stms_get_product($id->ID);
	}
	return $products;
}

function stms_find_product_by_externalid( $paysys, $item, $vendor = '' ) {
	global $wpdb;
	
	$table = "{$wpdb->prefix}stms_third_items";
	if ( $wpdb->get_var("SHOW TABLES LIKE '{$table}'") == $table ) { // check if table exists...
		$items = $wpdb->get_results($wpdb->prepare("SELECT * FROM `{$table}` WHERE `item_id` = %s AND `vendor_id` = %s AND `paysys` = %s", $item, $vendor, $paysys));
		if ( !$items ) return false;
		foreach ( $items as $item ) {
			$p = stms_get_product($item->product_id);
			$info = $p->info;
			if ( isset($info['third_pay'][$paysys]) && $info['third_pay'][$paysys] == 1 )
				return $p;
		}
	} else {
		if ( !$products = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}stms_products` ORDER BY `ID` DESC") )
			return false;
		
		foreach ( $products as $product ) {
			$meta = maybe_unserialize($product->metadata);
			$info = ( isset($meta['main']) ) ? $meta['main'] : array();
			$pay_settings = ( isset($meta['pay_settings']) ) ? $meta['pay_settings'] : array();
			$orig_item = stms_sanitize($pay_settings[$paysys]['item']);
			if ( $vendor != '' ) {
				$orig_vendor = stms_sanitize($pay_settings[$paysys]['vendor']);
				if ( $orig_item == $item && stms_isset($orig_vendor) == $vendor && stms_isset($info['third_pay'][$paysys]) == 1 )
					return stms_get_product($product->ID);
			} else {
				if ( $orig_item == $item && stms_isset($info['third_pay'][$paysys]) == 1 )
					return stms_get_product($product->ID);
			}
		}
	}
	return false;
}

function stms_get_product_by( $field, $value ) {
	global $wpdb;

	if ( 'id' == $field ) {
		// Make sure the value is numeric to avoid casting objects, for example,
		// to int 1.
		if ( !is_numeric($value) )
			return false;
		$value = absint($value);
	} else {
		$value = trim($value);
	}

	if ( !$value )
		return false;

	switch ( $field ) {
		case 'id':
			$product_id = $value;
			$db_field = 'ID';
			break;
		case 'title':
			$value = addslashes($value);
			$product_id = wp_cache_get($value, 'stms_producttitle');
			$db_field = 'title';
			break;
		default:
			return false;
	}

	if ( false !== $product_id ) {
		if ( $product = wp_cache_get( $product_id, 'stms_products' ) )
			return $product;
	}

	if ( !$product = $wpdb->get_row($wpdb->prepare("SELECT * FROM `{$wpdb->prefix}stms_products` WHERE `$db_field` = %s", $value)) )
		return false;
	
	if ( isset($product->metadata) ) {
		$product->info = stms_get_product_meta($product->ID, 'main');
		unset($product->metadata);
	}
	
	wp_cache_add($product->ID, $product, 'stms_products');
	wp_cache_add($product->title, $product->ID, 'stms_producttitle');
	return $member;
}

function stms_get_product_bytitle( $title ) {
	global $wpdb;
	if ( !$products = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}stms_products` ORDER BY `ID` DESC") )
		return false;
	
	foreach ( $products as $product ) {
		$orig_title = stms_sanitize($product->title);
		$title = stms_sanitize($title);
		$match = similar_text($orig_title, $title, $percent);
		if ( $percent > 98 )
			return stms_get_product($product->ID);
	}
	return false;
}

function stms_get_productname( $product_id ) {
	if ( !$product = stms_get_product($product_id) ) 
		return false;
	
	return stms_cleanup($product->title);
}

function stms_get_product($product_id = '') {
	if ( empty($product_id) )
		return false;
	
	global $wpdb;
	
	$product_id = intval($product_id);
	$product = wp_cache_get($product_id, 'stms_products');
	if ( !$product ) {
		// grab product data from database, if there's no product in cache
		$sql = "SELECT * FROM `{$wpdb->prefix}stms_products` WHERE `ID` = %d";
		$product = $wpdb->get_row($wpdb->prepare($sql, $product_id));
		if ( !$product || !is_object($product) )
			return false;
		
		if ( isset($product->metadata) )
			unset($product->metadata);
		
		$product->info = stms_get_product_meta($product_id, 'main');
		$product->pay_settings = stms_get_paysettings($product_id);
		
		// add $data to cache
		wp_cache_add($product_id, $product, 'stms_products');
		wp_cache_add($product->title, $product->ID, 'stms_producttitle');
	}
	return $product;
}

function stms_delete_product($id, $redirect = true ) {
	$id = (int) $id;
	if ( !$product = stms_get_product($id) ) {
		if ( $redirect ) {
			@header( 'Location:' . admin_url('admin.php?page=stms-settings') . '#level' );
			exit;
		} else
			return false;
	}
	
	global $wpdb;
	
	do_action('stms_product_delete', $id);
	
	// delete protection first...
	$wpdb->delete("{$wpdb->prefix}stms_protects", array('product_id' => $id));
	
	// archive the product
	$wpdb->update("{$wpdb->prefix}stms_products", array('product_status' => 4), array('ID' => $id));
	//delete cache
	wp_cache_delete($id, 'stms_products');
	wp_cache_delete($product->title, 'stms_producttitle');
	
	do_action('stms_product_deleted', $id);
	if ( $redirect ) {
		@header( 'Location:' . admin_url('admin.php?page=stms-settings') . '#level|product=deleted' );
		exit;
	}
	return true;
}
/* 
 * ======================== METADATA FUNCTIONS ========================
 */
function stms_update_metadata($type, $object_id, $key, $value) {
	global $wpdb;
	
	$table = "{$wpdb->prefix}stms_{$type}";
	$id = intval($object_id);
	$chk = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM {$table} WHERE ID = %d", $id));
	if ( $chk < 1 )
		return false;
	
	$data = $wpdb->get_row($wpdb->prepare("SELECT `metadata` FROM {$table} WHERE ID = %d", $id));
	$meta = maybe_unserialize($data->metadata);
	if ( !is_array($meta) )
		$meta = array();
	
	$key = stripslashes($key);
	$meta[$key] = stripslashes_deep($value);
	$wpdb->update(
		$table,
		array('metadata' => maybe_serialize($meta)),
		array('ID' => $id)
	);
	
	wp_cache_delete($object_id, 'stms_' . $type . '_meta'); // delete the cache
	return true;
}

function stms_get_metadata($type, $object_id, $key) {
	global $wpdb;
	
	$table = "{$wpdb->prefix}stms_{$type}";
	$id = intval($object_id);
	
	// get from cache first
	$data = wp_cache_get($object_id, 'stms_' . $type . '_meta');
	if ( !$data ) {
		$data = $wpdb->get_row($wpdb->prepare("SELECT `metadata` FROM {$table} WHERE ID = %d", $id));
		if ( !$data )
			return false;
		
		$data = $data->metadata;
	}
	
	$meta = maybe_unserialize($data);
	if ( !is_array($meta) || !isset($meta[$key]) )
		return false;
	
	wp_cache_set($object_id, $meta, 'stms_' . $type . '_meta'); // cache the data
	return $meta[$key];
}

function stms_delete_metadata($type, $object_id, $key) {
	global $wpdb;
	
	$table = "{$wpdb->prefix}stms_{$type}";
	$id = intval($object_id);
	
	$data = $wpdb->get_row($wpdb->prepare("SELECT `metadata` FROM {$table} WHERE ID = %d", $id));
	$meta = maybe_unserialize($data->metadata);
	if ( !isset($meta[$key]) )
		return false;
	
	unset($meta[$key]);
	if ( count($meta) < 1 )
		$meta = '';
	
	$wpdb->update(
		$table,
		array('metadata' => maybe_serialize($meta)),
		array('ID' => $id)
	);
	wp_cache_delete($object_id, 'stms_' . $type . '_meta'); // delete the cache
	
	return true;
}

/* 
 * ======================== MEMBERS META FUNCTIONS ========================
 */
function stms_delete_member_meta($member_id, $meta_key, $meta_value = '') {
	return stms_delete_metadata('members', $member_id, $meta_key);
}

function stms_get_member_meta($member_id, $key = '') {
	return stms_get_metadata('members', $member_id, $key);
}

function stms_update_member_meta($member_id, $meta_key, $meta_value) {
	return stms_update_metadata('members', $member_id, $meta_key, $meta_value);
}

/* 
 * ======================== PRODUCT META FUNCTIONS ========================
 */
function stms_delete_product_meta($product_id, $meta_key, $meta_value = '') {
	return stms_delete_metadata('products', $product_id, $meta_key);
}

function stms_get_product_meta($product_id, $key = '') {
	return stms_get_metadata('products', $product_id, $key);
}

function stms_update_product_meta($product_id, $meta_key, $meta_value) {
	return stms_update_metadata('products', $product_id, $meta_key, $meta_value);
}

/* 
 * ======================== S3 FUNCTIONS ========================
 */
function stms_s3_get_files() {
	$stms = get_option('stms_settings');
	$aws = stms_isset($stms['integration']['s3']);
	if ( empty($aws['access_key']) || empty($aws['secret_key']) || empty($aws['bucket']) )
		return false;
	
	if ( !class_exists('S3') )
		require_once(STMS_SOURCES . 'S3.php');
	
	$access_key = stms_cleanup($aws['access_key'], '', false, false, true);
	$secret_key = stms_cleanup($aws['secret_key'], '', false, false, true);
	
	$files = get_transient('_stms_s3_files');
	if ( empty($files) ) {
		$files = array();
		$s3 = new S3($access_key, $secret_key);
		if ( !$buckets = $s3->getBucket($aws['bucket']) )
			return false;
		
		if ( !is_array($buckets) || count($buckets) <= 0 )
			return false;
		
		foreach ( $buckets as $file ) {
			$files[] = $file['name'];
		}
		set_transient('_stms_s3_files', $files, 60);
	}
	return $files;
}

function stms_s3_file_download( $filename, $disposition = 'attachment' ) {
	if ( empty($filename) )
		return '';
	$url = stms_s3_get_authenticated_url($filename);
	wp_redirect($url);
	exit;
}

function stms_s3_get_authenticated_url( $filename ) {
	if ( empty($filename) )
		return false;
	
	if ( !class_exists('S3') )
		require_once(STMS_SOURCES . 'S3.php');
	
	$stms = get_option('stms_settings');
	$aws = stms_isset($stms['integration']['s3']);
	$access_key = stms_cleanup($aws['access_key'], '', false, false, true);
	$secret_key = stms_cleanup($aws['secret_key'], '', false, false, true);
	
	$auth_url = get_transient('im-s3-' . $filename);
	if ( !$auth_url ) {
		$s3 = new S3($access_key, $secret_key);
		$auth_url = $s3->getAuthenticatedURL($aws['bucket'], $filename, 3600, false, true);
		set_transient('im-s3-' . $filename, $auth_url, 60*10);
	}
	return $auth_url;
}

function stms_s3_get_object_info( $filename ) {
	if ( empty($filename) )
		return false;
	
	if ( !class_exists('S3') )
		require_once(STMS_SOURCES . 'S3.php');
	
	$stms = get_option('stms_settings');
	$aws = stms_isset($stms['integration']['s3']);
	$access_key = stms_cleanup($aws['access_key'], '', false, false, true);
	$secret_key = stms_cleanup($aws['secret_key'], '', false, false, true);
	
	$s3 = new S3($access_key, $secret_key);
	return $s3->getObjectInfo($aws['bucket'], $filename);
}

function stms_s3_get_file_acp( $filename ) {
	if ( empty($filename) )
		return false;
	
	if ( !class_exists('S3') )
		require_once(STMS_SOURCES . 'S3.php');
	
	$stms = get_option('stms_settings');
	$aws = stms_isset($stms['integration']['s3']);
	$access_key = stms_cleanup($aws['access_key'], '', false, false, true);
	$secret_key = stms_cleanup($aws['secret_key'], '', false, false, true);
	
	$s3 = new S3($access_key, $secret_key);
	return $s3->getAccessControlPolicy($aws['bucket'], $filename);
}

function stms_s3_set_file_acp( $filename, $mode = 'protect' ) {
	if ( empty($filename) )
		return false;
	
	if ( !class_exists('S3') )
		require_once(STMS_SOURCES . 'S3.php');
	
	$stms = get_option('stms_settings');
	$aws = stms_isset($stms['integration']['s3']);
	$access_key = stms_cleanup($aws['access_key'], '', false, false, true);
	$secret_key = stms_cleanup($aws['secret_key'], '', false, false, true);
	
	$s3 = new S3($access_key, $secret_key);
	$acp = $s3->getAccessControlPolicy($aws['bucket'], $filename);
	$new_acp = array();
	$new_acp['owner'] = $acp['owner'];
	$new_acp['acl'] = array();
	
	// Authenticated Users Read
	$new_acp['acl'][] = array(
		'type' => 'Group',
		'uri' => 'http://acs.amazonaws.com/groups/global/AuthenticatedUsers',
		'permission' => 'READ'
	); 
	
	// Authenticated Users Read ACP
	$new_acp['acl'][] = array(
		'type' => 'Group',
		'uri' => 'http://acs.amazonaws.com/groups/global/AuthenticatedUsers',
		'permission' => 'READ_ACP'
	); 
	
	// Owner Permission
	$new_acp['acl'][] = array(
		'type' => 'CanonicalUser',
		'name' => $acp['owner']['name'],
		'id' => $acp['owner']['id'],
		'permission' => 'FULL_CONTROL'
	);

	if ( $mode == 'unprotect' ) {
		$new_acp['acl'][] = array(
			'type' => 'Group',
			'uri' => 'http://acs.amazonaws.com/groups/global/AllUsers',
			'permission' => 'READ'
		);
	}
	
	if ( !$set = $s3->setAccessControlPolicy($aws['bucket'], $filename, $new_acp) )
		return false;
	return true;
}

/* 
 * ======================== LICENSING FUNCTIONS ========================
 */

function stms_get_license_by( $field, $value ) {
	if ( 'id' == $field || 'tx_id' == $field ) {
		if ( !is_numeric($value) )
			return false;
		$value = absint($value);
	} else {
		$value = trim($value);
	}
	
	if ( !$value )
		return false;
	global $wpdb;
	switch ( $field ) {
		case 'id':
			$license_id = $value;
			$db_field = 'ID';
			break;
		case 'tx_id':
			$license_id = wp_cache_get($value, 'stms_tx_licenses');
			$db_field = 'tx_id';
			break;
		case 'key':
			$license_id = wp_cache_get($value, 'stms_key_licenses');
			$db_field = 'license';
			break;
		default:
			return false;
	}
	if ( false !== $license_id ) {
		if ( $license = wp_cache_get($license_id, 'stms_licenses') )
			return $license;
	}
	if ( !$license = $wpdb->get_row($wpdb->prepare("SELECT * FROM `{$wpdb->prefix}stms_licenses` WHERE `$db_field` = %s", $value)) )
		return false;
	
	stms_update_license_caches($license);
	return $license;
}

function stms_update_license_caches($license) {
	wp_cache_add($license->ID, $license, 'stms_licenses');
	wp_cache_add($license->tx_id, $license->ID, 'stms_tx_licenses');
	wp_cache_add($license->license, $license->ID, 'stms_key_licenses');
}

function stms_clean_license_cache($license) {
	if ( is_numeric($license) )
		$license = stms_get_memberdata($license);
	if ( !is_object($license) )
		return false;
	
	wp_cache_delete($license->ID, 'stms_licenses');
	wp_cache_delete($license->tx_id, 'stms_tx_licenses');
	wp_cache_delete($license->license, 'stms_key_licenses');
}

function stms_get_licensedata( $license_id ) {
	return stms_get_license_by('id', $license_id);
}

function stms_get_licenselog( $license_id, $field, $start = 0, $limit = 15 ) {
	global $wpdb;
	
	$license_id = intval($license_id);
	$sql = "SELECT * FROM `{$wpdb->prefix}stms_license_log`
			WHERE `license_id` = {$license_id} ORDER BY `{$field}`
			DESC LIMIT {$start}, {$limit}";

	if ( !$logs = $wpdb->get_results($sql) )
		return false;
	
	return $logs;
}

add_action('stms_payment_completed', 'stms_create_license', 2, 1);
function stms_create_license( $tx_id ) {
	if ( !$payment = stms_get_payment($tx_id) )
		return false;
	
	// do we need to create a license?
	$settings = stms_get_product_meta($payment->product_id, 'license');
	if ( empty($settings) || ($settings['enable'] != 1) || ($payment->payment_type == 'recurr' && $payment->is_rebill == 1) ) // there's no need to create a license for this transaction...
		return false;

	global $wpdb;
	$table = "{$wpdb->prefix}stms_licenses";
	$count = $wpdb->get_var($wpdb->prepare("SELECT ID FROM {$table} WHERE `product_id` = %d AND `tx_id` = %d", $payment->product_id, $tx_id));
	if ( $count > 0 ) // license for this transaction is already exists...
		return false;
	
	$license_key = stms_generate_license_key( stms_isset($settings['length'], 10), $tx_id, stms_isset($settings['prefix'], ''), stms_isset($settings['suffix'], '') );
	$utc_str = gmdate("M d Y H:i:s", time());
	$utc = strtotime($utc_str);
	$data = array(
			'license' => $license_key,
			'product_id' => $payment->product_id,
			'tx_id' => $payment->tx_id,
			'member_id' => $payment->member_id,
			'status' => 1,
			'date' => $utc
		);
	$wpdb->insert($table, $data);
	return true;
}

function stms_generate_license_key( $length = 10, $tx_id, $prefix = '', $suffix = '' ) {
	$start = mt_rand(0, 5);
	$length = ( $length > 20 ) ? 20 : $length;
	$secret = stms_generate_secret(10, true, false, true);
	$license = sha1($tx_id . $secret . stms_time());
	$license = substr($license, $start, $length);
	
	return $prefix . $license . $suffix;
}

if ( !is_admin() ) add_action('init', 'stms_rest_api', 5);
function stms_rest_api() {
	$slug = array();
	$site_url = get_bloginfo('url');
	$wp_dir = explode('/', str_replace(array('http://', 'https://'), '', $site_url));
	$start = count($wp_dir);
	$dir  = dirname($_SERVER['PHP_SELF']);
	$file = explode('/', $_SERVER['REQUEST_URI']);
	
	for ( $i = $start; $i < count($file); $i++ ) {
		$slug[] = $file[$i];
	}

	if ( count($slug) < 1 || stms_isset($slug[0]) != '!' && stms_isset($slug[1]) != 'instamember_api' || empty($slug[2]) )
		return false;

	$mode = stms_sanitize($slug[2]);
	$object_id = stms_sanitize($slug[3]);
	switch ( $mode ) {
		case 'license':
			$request_data = ( isset($slug[4]) && $slug[4] == 'data' ) ? true : false;
			stms_license_api($object_id, $request_data);
			break;
		case 'update':
			//
			break;
		default:
			return false;
	}
	exit;
}

function stms_license_api( $key = '', $request_data = false ) {
	require_once(STMS_CLASSES . 'stmsRestUtils.php');
	global $wpdb;
	
	if ( empty($key) ) {
		stmsRestUtils::sendResponse(400, json_encode(array('status' => 'Error', 'message' => 'License key is empty.')), 'application/json');
		exit;
	}
	
	$stms = get_option('stms_settings');
	$rest = stmsRestUtils::processRequest();
	$method = $rest->getMethod();
	$accept = $rest->getHttpAccept();
	$vars = $rest->getRequestVars();
	
	if ( $method == 'post' && empty($vars) ) {
		stmsRestUtils::sendResponse(400, json_encode(array('status' => 'Error', 'message' => 'Required fields are empty')), 'application/json');
		exit;
	}
	
	// post data
	$secret = stms_isset($stms['global']['licensing_secret']);
	$domain = ( $method == 'post' ) ? trim(rawurldecode($vars['domain'])) : '';
	$userip = ( $method == 'post' ) ? trim(rawurldecode($vars['userip'])) : '';
	$servip = ( $method == 'post' ) ? trim(rawurldecode($vars['servip'])) : '';
	$type = ( $method == 'post' ) ? 'activation' : 'validation';
	
	if ( !$license = stms_get_license_by('key', $key) ) {
		stmsRestUtils::sendResponse(401, json_encode(array('status' => 'Error', 'message' => 'Invalid license key')), 'application/json');
		exit;
	}

	$username = $_SERVER['PHP_AUTH_USER'];
	$password = $_SERVER['PHP_AUTH_PW'];
	$md5_hash = md5($key . $secret);
	if ( $username != $secret || $password != $md5_hash ) {
		stmsRestUtils::sendResponse(401, json_encode(array('status' => 'Unauthorized', 'message' => 'Authentication failed')), 'application/json');
		exit;
	}
	
	// The license is exist...  get the settings...
	$settings = stms_get_product_meta($license->product_id, 'license');
	if ( stms_isset($settings['enable']) != 1 ) {
		// no license protection for this product...
		stmsRestUtils::sendResponse(200, json_encode(array('status' => 'Success', 'message' => 'License key is active')), 'application/json');
		exit;
	}

	// quick check - is it inactive...
	if ( $license->status == 0 ) {
		// license is not active...
		stmsRestUtils::sendResponse(200, json_encode(array('status' => 'Inactive', 'message' => 'License key is inactive or disabled')), 'application/json');
		exit;
	}
	
	// quick check - is it suspended...
	if ( $license->status == 2 ) {
		// license is suspended...
		stmsRestUtils::sendResponse(200, json_encode(array('status' => 'Suspended', 'message' => 'License key has been suspended')), 'application/json');
		exit;
	}
	
	// quick check - is it expired...
	if ( $license->status == 3 ) {
		// license is expired...
		stmsRestUtils::sendResponse(200, json_encode(array('status' => 'Expired', 'message' => 'License key has been expired')), 'application/json');
		exit;
	}

	// license status on the database is active...
	// let's do some detail check to make sure its availability...
	$usage = stms_isset($settings['usage']);
	$mode = stms_isset($settings['usage_mode']);
	$expire = stms_isset($settings['expire']);
	$exp_mode = stms_isset($settings['expire_mode']);
	
	// check if license should be expired by now...
	if ( $exp_mode != 'never' && is_numeric($expire) && $expire > 0 ) {
		$s = ( $expire > 1 ) ? 's' : '';
		$expiration = strtotime("+{$expire} {$exp_mode}{$s}", $license->date);
		$utc_str = gmdate("M d Y H:i:s", time());
		$utc_time = strtotime($utc_str);
		if ( $utc_time >= $expiration  ) {
			// license has expired...
			// update the status...
			stms_license_action($license->ID, 'expired', false);
			stmsRestUtils::sendResponse(200, json_encode(array('status' => 'Expired', 'message' => 'License key has been expired')), 'application/json');
			exit;
		}
	}

	
	$status = json_encode(array('status' => 'Success', 'message' => 'License key is active'));
	if ( $type == 'validation' && $request_data ) {
		$all_data = stms_get_requested_license_data($license);
		$license_data = (array) $all_data;
		$status = json_encode(array('status' => 'Success', 'message' => 'License key is active', 'data' => $license_data));
	}
	
	$write = true;
	if ( $type == 'activation' ) {
		// quick check - is it used/maxed...
		if ( $license->status == 4 ) {
			// Check if this is a re-install...
			switch ( $mode ) {
				case 'domain':
					$value = $domain;
					break;
				case 'servip':
					$value = $servip;
				case 'userip':
					$value = $userip;
					break;
			}
			
			$count = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM `{$wpdb->prefix}stms_license_log` WHERE `license_id` = %d AND `$mode` = %s", $license->ID, $value));
			if ( $count < 1 ) {
				$status = json_encode(array('status' => 'Maxed', 'message' => 'License key is active but has reached the maximum usage limit'));
				$write = false;
			}
		} else {
			if ( $mode != 'unlimited' ) {
				// license is still active but limited ... let's check if this license already used...
				switch ( $mode ) {
					case 'domain':
						$value = $domain;
						break;
					case 'servip':
						$value = $servip;
					case 'userip':
						$value = $userip;
						break;
				}
				
				$count = $wpdb->get_var($wpdb->prepare("SELECT COUNT(DISTINCT `{$mode}`) FROM `{$wpdb->prefix}stms_license_log` WHERE `license_id` = %d", $license->ID));
				if ( $count >= $usage ) {
					// this license is active but maxed... so it cannot be used...
					// update the status...
					stms_license_action($license->ID, 'maxed', false);
					$status = json_encode(array('status' => 'Maxed', 'message' => 'License key is active but has reached the maximum usage limit'));
					$write = false;
				} else if ( $count < $usage ) {
					// This license has not reach the limit...
					$status = json_encode(array('status' => 'Success', 'message' => 'License key is active'));
					$write = true;
				}
			}
		}
		
		if ( $write ) {
			$log_data = array(
				'license_id' => $license->ID,
				'product_id' => $license->product_id,
				'member_id' => $license->member_id,
				'domain' => $domain,
				'servip' => $servip,
				'userip' => $userip,
				'date' => time(),
			);
			$wpdb->insert("{$wpdb->prefix}stms_license_log", $log_data);
		}
	}
	stmsRestUtils::sendResponse(200, $status, 'application/json');
	exit;
}

function stms_get_requested_license_data( $license ) {
	if ( !is_object($license) )
		return $license;
	
	if ( !$product = stms_get_product($license->product_id) )
		return $license;
	
	if ( !$settings = stms_get_product_meta($license->product_id, 'license') )
		return $license;
		
	if ( $settings['enable'] != 1 )
		return $license;
	
	$expire_mode = stms_isset($settings['expire']) . ' ' . stms_isset($settings['expire_mode']);
	$license->expiration = ( stms_isset($settings['expire_mode']) == 'never' ) ? 'Never' : ( stms_isset($settings['expire']) > 1 ? $expire_mode . 's' : $expire_mode );
	
	$usage_mode = stms_isset($settings['usage']) . ' ' . stms_isset($settings['usage_mode']);
	$license->limit = ( stms_isset($settings['usage_mode']) == 'unlimited' ) ? 'Unlimited' : ( stms_isset($settings['usage']) > 1 ? $usage_mode . 's' : $usage_mode );
	$license->created = date("Y-m-d H:i:s", $license->date);
	
	if ( $settings['expire_mode'] != 'never' && is_numeric($settings['expire']) && $settings['expire'] > 0 ) {
		$s = ( $settings['expire'] > 1 ) ? 's' : '';
		$expired = strtotime("+{$settings['expire']} {$settings['expire_mode']}{$s}", $license->date);
		$license->expired = $expired;
		$license->expired_date = date("Y-m-d H:i:s", $expired);
	}
	return $license;
}

function stms_license_action( $id, $type = 'delete', $redirect = true ) {
	if ( !$license = stms_get_licensedata($id) ) {
		if ( $redirect ) {
			@header( 'Location:' . admin_url('admin.php?page=stms-settings') . '#licenses|mode=manage');
			exit;
		}
		return false;
	}
	
	global $wpdb;
	$msg = '';
	switch ( $type ) {
		case 'delete':
			stms_clean_license_cache($license);
			$wpdb->delete("{$wpdb->prefix}stms_licenses", array('ID' => $id));
			$msg = '&license=deleted';
			break;
		case 'suspend':
			if ( $license->status == 1 ) {
				$wpdb->update("{$wpdb->prefix}stms_licenses", array('status' => 2), array('ID' => $id));
				$msg = '&license=suspended';
			}
			break;
		case 'reactivate':
			if ( $item['status_code'] != 1 && $item['status_code'] != 3 && $item['status_code'] != 4 ) {
				$wpdb->update("{$wpdb->prefix}stms_licenses", array('status' => 1), array('ID' => $id));
				$msg = '&license=reactivated';
			}
			break;
		case 'deactivate':
			if ( $license->status == 1 || $license->status == 2 ) {
				$wpdb->update("{$wpdb->prefix}stms_licenses", array('status' => 0), array('ID' => $id));
				$msg = '&license=deactivated';
			}
			break;
		case 'expired':
			$wpdb->update("{$wpdb->prefix}stms_licenses", array('status' => 3), array('ID' => $id));
			$msg = '&license=expired';
			break;
		case 'maxed':
			$wpdb->update("{$wpdb->prefix}stms_licenses", array('status' => 4), array('ID' => $id));
			$msg = '&license=maxed';
			break;
	}

	// clean caches ...
	if ( $type != 'delete' ) stms_clean_license_cache($license);
	
	if ( $redirect ) {
		@header( 'Location:' . admin_url('admin.php?page=stms-settings') . '#licenses|mode=manage' . $msg);
		exit;
	}
	return true;
}

add_action('stms_payment_deleted', 'stms_license_force_remove', 20);
function stms_license_force_remove( $tx_id ) {
	if ( !$lic = stms_get_license_by('tx_id', $tx_id) )
		return false;
	
	return stms_license_action($lic->ID, 'delete', false);
}

add_action('stms_payment_updated', 'stms_license_onpaymentupdate', 20);
function stms_license_onpaymentupdate( $tx_id ) {
	if ( !$py = stms_get_payment($tx_id) ) 
		return false;
	
	if ( !$lic = stms_get_license_by('tx_id', $tx_id) )
		return false;
	
	if ( $py->status == 'refunded' )
		return stms_license_action($lic->ID, 'delete', false);
	else
		return false;
}

/* 
 * ======================== EMAIL FUNCTIONS ========================
 */
function stms_parse_mail_tags( $content, $pass = '', $member_id = 0, $tx_id = 0, $aff_comm = 0.00 ) {
	$content = stms_sanitize($content);
	$stms = get_option('stms_settings');
	$m = stms_member($member_id);
	
	if ( empty($pass) )
		$pass = stms_string_dec(stms_sanitize($m->info['user_key']), AUTH_SALT);
	
	// Parse member data
	$content = str_replace('%member_id%', stms_sanitize($m->ID), $content);
	$content = str_replace('%member_login%', stms_sanitize($m->username), $content);
	$content = str_replace('%member_email%', stms_sanitize($m->email), $content);
	$content = str_replace('%member_pass%', stms_sanitize($pass), $content);
	$content = str_replace('%member_firstname%', stms_sanitize($m->first_name), $content);
	$content = str_replace('%member_lastname%', stms_sanitize($m->last_name), $content);
	$content = str_replace('%member_street%', stms_sanitize($m->info['street']), $content);
	$content = str_replace('%member_city%', stms_sanitize($m->info['city']), $content);
	$content = str_replace('%member_state%', stms_sanitize($m->info['state']), $content);
	$content = str_replace('%member_zip%', stms_sanitize($m->info['zipcode']), $content);
	$content = str_replace('%member_country%', stms_sanitize($m->info['country']), $content);
	$content = str_replace('%member_phone%', stms_sanitize($m->info['phone']), $content);
	$content = str_replace('%member_company%', stms_sanitize($m->info['company']), $content);
	$content = str_replace('%member_title%', stms_sanitize($m->info['title']), $content);
	$content = str_replace('%member_gender%', stms_sanitize($m->info['gender']), $content);
	
	if ( $tx_id )
		$tx = stms_get_payment($tx_id);
	
	$content = str_replace('%aff_commission%', $aff_comm, $content);
	$content = str_replace('%aff_id%', $m->ID, $content);

	$aff_link = stms_format_url( get_bloginfo('url'), 'iaff=' . $m->ID );
	$aff_link .= ( $tx_id ) ? '&p=' . stms_isset($tx->product_id) : '';
	$aff_stats_url = stms_format_url($stms['global']['magic'], 'imhandler=stats');
	
	$content = str_replace('%aff_link%', $aff_link, $content);
	$content = str_replace('%aff_stats_url%', $aff_stats_url, $content);
	
	$aff_page_url = ( isset($stms['global']['aff_home']) ) ? get_permalink($stms['global']['aff_home']) : $aff_stats_url;
	$content = str_replace('%aff_page_url%', $aff_page_url, $content);
	
	if ( $tx_id ) {
		// Parse transaction data
		$content = str_replace('%txn_receipt%', stms_sanitize($tx->receipt), $content);
		$content = str_replace('%txn_invoice%', stms_parse_invoice_tag($tx->tx_id), $content);
		$content = str_replace('%txn_product%', stms_sanitize($tx->title), $content);
		$content = str_replace('%txn_begin_date%', date("F d, Y", stms_isset($tx->date)), $content);
		if ( stms_isset($tx->expired_date) <= 0 )
			$content = str_replace('%txn_expired_date%', 'Lifetime', $content);
		else
			$content = str_replace('%txn_expired_date%', date("F d, Y", stms_isset($tx->expired_date)), $content);
		$content = str_replace('%txn_amount%', stms_sanitize($tx->amount, '0.00') . ' ' . stms_sanitize($tx->currency, 'USD'), $content);
		
		$product = stms_get_product($tx->product_id);
		$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_permalink($stms['global']['home']);
		
	}
	
	$global_home = intval(stms_isset($stms['global']['home']));
	$member_url = ( $global_home > 0 ) ? get_permalink($global_home) : get_bloginfo('url');
	if ( $tx_id ) {
		$product = stms_get_product($tx->product_id);
		if ( $product ) {
			if ( !empty($product->home_id) || $product->home_id > 0 )
				$member_url = get_permalink($product->home_id);
		}
		
		$download_url = $member_url;
		$gateway = stms_gateway_type($tx->paysys);
		if ( $gateway == 'third' )
			$download_url = stms_format_url(get_bloginfo('url'), 'insta_mode=product_access&processor=' . $tx->paysys . '&insta_id=' . $tx->product_id . '&insta_receipt=' . $tx->receipt);
			
		$content = str_replace('%txn_download_url%', $download_url, $content);
	}
	
	$confirm_code = get_transient('stms_confirm_' . stms_isset($m->ID));
	$confirm_url = stms_format_url( get_option('siteurl'), 'insta_confirm=' . $confirm_code . '&id=' . stms_isset($m->ID) . '&tx=' . stms_isset($tx_id, 0) );

	// Parse site data
	$content = str_replace('%site_title%', stms_sanitize(get_option('blogname')), $content);
	$content = str_replace('%site_member_url%', $member_url, $content);
	$content = str_replace('%site_confirm_url%', $confirm_url, $content);
	$content = str_replace('%site_admin_email%', stms_sanitize($stms['emails']['admin_email']), $content);
	$content = str_replace('%site_admin_email_from%', stms_sanitize($stms['emails']['admin_from']), $content);
	$content = str_replace('%site_support%', stms_sanitize($stms['global']['support']), $content);
	
	// Parse other general data...
	$business_name = stms_sanitize($stms['global']['business_name'], get_bloginfo('name'));
	$business_addr = stms_sanitize($stms['global']['business_addr'], '');
	$content = str_replace('%business_name%', $business_name, $content);
	$content = str_replace('%business_address%', $business_addr, $content);
	
	// Parse forgot password link ...
	$forgot_key = get_transient('stms_forgot_' . $member_id);
	$forgot_url = stms_format_url($stms['global']['magic'], 'imhandler=resetpass&email=' . urlencode($m->email) . '&key=' . urlencode($forgot_key));
	$content = str_replace('%resetpass_url%', $forgot_url, $content);
	
	return $content;
}

function stms_parse_invoice_tag( $tx_id ) {
	if ( !$payment = stms_get_payment($tx_id) )
		return '';
	
	$stms = get_option('stms_settings');
	$invoice = '';
	$product = stms_get_product($payment->product_id);
	$subtotal = $payment->amount;
	$discount = stms_isset($payment->discount, 0.00);
	$price = $payment->amount + $discount;
	
	if ( $payment->is_rebill != 1 ) {
		if ( $license = stms_get_license_by('tx_id', $payment->tx_id) ) {
			$invoice .= "Your License Key: {$license->license}\n";
		}
	}
	
	$invoice .= "\n===============================================\n\n";
	$invoice .= "Payment for invoice #{$payment->receipt} on " . date("F d, Y", $payment->date) . "\n\n";
	$invoice .= "Product Name: " . stms_sanitize($payment->title) . "\n";
	$invoice .= ( $payment->payment_type == 'recurr' ) ? 'Period: ' . date("m/d/Y", $payment->date) . ' - ' . date("m/d/Y", $payment->expired_date) . "\n" : '';
	$invoice .= "\n----------------------------------\n";
	$invoice .= "Sub Total: " . number_format($price, 2) . " {$payment->currency}\n";
	$invoice .= ( $discount > 0.00 ) ? "Discount: {$discount}\n" : '';
	$invoice .= "----------------------------------\n";
	$invoice .= "Total: " . number_format($payment->amount, 2) . " {$payment->currency}\n\n";
	$invoice .= "===============================================\n\n";
	
	return $invoice;
}

function stms_mail_tags( $type = 'global', $pass = false ) {
	$tags = array();
	
	$tags['general_group'] = 'General Macro';
	$tags['%site_title%'] = 'Site Title/Name (as configured in "Settings -> General")';
	$tags['%site_member_url%'] = 'Membership home page or download URL';
	$tags['%site_confirm_url%'] = 'Registration confirm URL';
	$tags['%site_admin_email%'] = 'Admin\'s email address';
	$tags['%site_admin_email_from%'] = 'Admin\'s email "From Name"';
	$tags['%site_support%'] = 'Support email or URL';
	if ( $type == 'forgot' )
		$tags['%resetpass_url%'] = 'Reset Password URL';
	$tags['%business_name%'] = 'Your business name (as configured in "InstaMember -> Configuration")';
	$tags['%business_address%'] = 'Your business address (as configured in "InstaMember -> Configuration")';
	$tags['general_end'] = '';
	
	$tags['member_group'] = 'Member Macro';
	$tags['%member_id%'] = 'Member ID#';
	$tags['%member_login%'] = 'Member\'s username';
	$tags['%member_pass%'] = 'Member\'s password';

	$tags['%member_email%'] = 'Member\'s email address';
	$tags['%member_firstname%'] = 'Member\'s first name';
	$tags['%member_lastname%'] = 'Member\'s last name';
	$tags['%member_street%'] = 'Member\'s street address';
	$tags['%member_city%'] = 'Member\'s city';
	$tags['%member_state%'] = 'Member\'s state/province';
	$tags['%member_zip%'] = 'Member\'s zip code';
	$tags['%member_country%'] = 'Member\'s country';
	$tags['%member_phone%'] = 'Member\'s phone#';
	$tags['%member_company%'] = 'Member\'s company name';
	$tags['%member_title%'] = 'Member\'s title';
	$tags['%member_gender%'] = 'Member\'s gender';
	$tags['member_end'] = '';
	
	$tags['aff_group'] = 'Affiliate Macro';
	if ( $type == 'aff_transactional' )
		$tags['%aff_commission%'] = 'Affiliate commission amount';
	
	$tags['%aff_link%'] = 'Affiliate Link';
	$tags['%aff_stats_url%'] = 'Affiliate Stats URL';
	$tags['aff_end'] = '';
	
	if ( $type == 'transactional' || $type == 'aff_transactional' ) {
		$tags['txn_group'] = 'Transaction/Payment Macro';
		$tags['%txn_product%'] = 'Name of the purchased product';
		$tags['%txn_download_url%'] = 'Product Access URL';
		$tags['%txn_receipt%'] = 'Payment/Subscription receipt#';
		$tags['%txn_invoice%'] = 'Invoice/Payment/Order details)';
		$tags['%txn_begin_date%'] = 'Payment/Subscription start date';
		$tags['%txn_expired_date%'] = 'Payment/Subscription end date (will display as \'Lifetime\' if the product has no expiration date)';
		$tags['%txn_amount%'] = 'Payment/Subscription amount';
		$tags['txn_end'] = '';
	}
	return $tags;
}

function stms_default_emails() {
	$email = array();
	$email['confirm']['subject'] = '%site_title% - Complete Your Registration';
	$email['confirm']['body'] = "Hi %member_firstname% %member_lastname%\n\n".
								"You just registered an account on %site_title%.\n\n".
								"Please click on the link below to confirm your registration:\n".
								"%site_confirm_url%\n\n".
								"Thank You,\n".
								"%business_name%\n".
								"%business_address%\n\n";
								
	$email['signedup']['subject'] = 'Welcome To %site_title%';
	$email['signedup']['body'] = "Hi %member_firstname% %member_lastname%\n\n".
								"Thank you for registration on %site_title%!\n\n".
								"Username: %member_login%\n".
								"Password: %member_pass%\n\n".
								"Please login to the members area at:\n".
								"%site_member_url%\n\n".
								"Thank You,\n".
								"%business_name%\n".
								"%business_address%\n\n";
								
	$email['receipt']['subject'] = '%txn_product% - Payment Receipt';
	$email['receipt']['body'] = "Hi %member_firstname% %member_lastname%\n\n".
								"Thank you for your purchase. Here's your payment details:\n\n".
								"%txn_invoice%\n\n".
								"Product Access URL:\n%txn_download_url%\n\n".
								"Thank You,\n".
								"%business_name%\n".
								"%business_address%\n\n";
								
	$email['upgraded']['subject'] = 'Your Account Has Been Upgraded';
	$email['upgraded']['body'] = "Hi %member_firstname% %member_lastname%\n\n".
								"Your account has been upgraded successfully.\n\n".
								"You can view the details by login to the members area:\n".
								"%site_member_url%\n\n".
								"Thank You,\n".
								"%business_name%\n".
								"%business_address%\n\n";
								
	$email['forgot']['subject'] = 'Password Reset Assistance';
	$email['forgot']['body'] = "Hi %member_firstname% %member_lastname%\n\n".
								"We received a reset password request for your account.\n\n".
								"Please click the link below to reset your password:\n".
								"%resetpass_url%\n\n".
								"If clicking the link above does not work, copy and paste the URL in\n".
								"a new browser window instead. The URL will expire in 24 hours for security\n".
								"reasons.\n\n".
								"Please disregard this message if you did not make a password reset request.\n\n".
								"Thank You,\n".
								"%business_name%\n".
								"%business_address%\n\n";
								
	$email['cancel']['subject'] = 'Membership Cancelled';
	$email['cancel']['body'] = "Hi %member_firstname% %member_lastname%\n\n".
								"Your subscription to %txn_product% has been cancelled.\n\n".
								"We're sad to see you go!\n\n".
								"Thank You,\n".
								"%business_name%\n".
								"%business_address%\n\n";
								
	$email['commission']['subject'] = 'Congratulation %member_firstname%! You\'ve just got a sale!';
	$email['commission']['body'] = "Hi %member_firstname% %member_lastname%\n\n".
								"New sale have been made using your affiliate link.\n\n".
								"Product Sold: %txn_product%\n".
								"Commission Earned: %aff_commission%\n\n".
								"Your affiliate link:\n".
								"%aff_link%\n\n".
								"Your stats:\n".
								"%aff_stats_url%\n\n".
								"Thank You,\n".
								"%business_name%\n".
								"%business_address%\n\n";
								
	$email['aff_pending']['subject'] = '%site_title% - Affiliate Request Received';
	$email['aff_pending']['body'] = "Hi %member_firstname% %member_lastname%\n\n".
								"Thank you for signing up for our affiliate program.\n\n".
								"Please note that we do evaluate every affiliate\n".
								"request. If your application is approved, you will be\n".
								"notified via e-mail. Approval time is usually 2-3\n".
								"business days.\n\n".
								"We appreciate your interest in our affiliate program and\n".
								"thank you for choosing to signup at %site_title%.\n\n".
								"Thank You,\n".
								"%business_name%\n".
								"%business_address%\n\n";
	
	$email['aff_approved']['subject'] = '%site_title% - Affiliate Status Approved';
	$email['aff_approved']['body'] = "Hi %member_firstname% %member_lastname%\n\n".
								"Congratulations. We've reviewed your application and you've\n".
								"been approved as %site_title% affiliate.\n\n".
								"You can login at the URL below to get started:\n".
								"%aff_page_url%\n\n".
								"If you're an existing customer or previously have joined\n".
								"our site, you can login using your existing membership account.\n\n".
								"If you're new, an email containing your login info should be\n".
								"arrived shortly.\n\n".
								"Thank You,\n".
								"%business_name%\n".
								"%business_address%\n\n";
								
	$email['admin_receipt']['subject'] = '%site_title% - New Payment';
	$email['admin_receipt']['body'] = "New payment completed:\n".
								"%txn_invoice%\n\n".
								"User Detail:\n".
								"Username: %member_login%\n".
								"Email: %member_email%\n".
								"Full Name: %member_firstname% %member_lastname%\n\n".
								"%site_title%\n\n";
								
	$email['admin_cancel']['subject'] = '%site_title% - Member Subscription Cancel';
	$email['admin_cancel']['body'] = "User:\n".
								"Username: %member_login%\n".
								"Email: %member_email%\n".
								"Full Name: %member_firstname% %member_lastname%\n\n".
								"has cancelled subscription to:\n".
								"Product name: %txn_product%\n\n".
								"%site_title%\n\n";
								
	$email['admin_commission']['subject'] = '%site_title% - Affiliate Sale';
	$email['admin_commission']['body'] = "An affiliate just referred a sale\n\n".
								"Details:\n\n".
								"Aff. Username: %member_login%\n".
								"Aff. Full Name: %member_firstname% %member_lastname%\n".
								"Aff. Commission: %aff_commission%\n\n".
								"%txn_invoice%\n\n".
								"%site_title%\n\n";
	
	// Support Desk Emails	
	$email['support_reply']['subject'] = 'New reply on ticket %ticket_num% ';
	$email['support_reply']['body'] = "Dear %firstname% %lastname%\n\n".
								"You have received a new reply for your ticket.\n\n".
								"Follow the link below to view this message:\n".
								"%ticket_url%\n\n".
								"%business_name%\n".
								"%business_address%\n\n";
									
	$email['admin_support']['subject'] = 'Ticket %ticket_num% needs your response';
	$email['admin_support']['body'] = "Dear Admin\n\n".
								"You have received a new message on support desk.\n\n".
								"Please click the link below to respond:\n".
								"%ticket_url%\n\n".
								"%site_title%\n\n";
													
	return $email;
}

function stms_mail( $to, $subject, $message, $from = '', $add_headers = '' ) {
	/* For debugging smtp only
	global $phpmailer;
	
	if ( !is_object( $phpmailer ) || !is_a( $phpmailer, 'PHPMailer' ) ) {
		require_once ABSPATH . WPINC . '/class-phpmailer.php';
		require_once ABSPATH . WPINC . '/class-smtp.php';
		$phpmailer = new PHPMailer();
	}

	$phpmailer->SMTPDebug = 2;
	*/
	
	$stms = get_option('stms_settings');
	$mailsys = stms_isset($stms['emails']['mailsys']);
	
	$message = stms_sanitize(strip_tags($message));
	$default_from = get_option('blogname') . ' <' . get_option('admin_email') . '>';
	$_from = ( !empty($stms['emails']['admin_from']) && !empty($stms['emails']['admin_email']) ) ? stms_sanitize($stms['emails']['admin_from']) . ' <' . stms_isset($stms['emails']['admin_email']) . '>' : $default_from;
	$email_from = stms_sanitize($stms['emails']['admin_from']);
	$email_addr = stms_sanitize($stms['emails']['admin_email']);
	
	if ( $from != '' ) {
		if ( !empty($stms['email_templates'][$from]['fromname']) && !empty($stms['email_templates'][$from]['frommail']) ) {
			$_from = stripslashes(stms_sanitize($stms['email_templates'][$from]['fromname'])) . ' <' . sanitize_email($stms['email_templates'][$from]['frommail']) . '>';
			$email_from = stms_sanitize($stms['email_templates'][$from]['fromname']);
			$email_addr = stms_sanitize($stms['email_templates'][$from]['frommail']);
		}
		
		if ( $from == 'instabot' ) {
			$_from = 'InstaMember <instabot@' . $_SERVER['HTTP_HOST'] . '>';
			$email_from = 'InstaMember';
			$email_addr = 'instamember@' . $_SERVER['HTTP_HOST'];
		}
	}
	
	if ( $mailsys == 'ses' ) {
		$default_from = get_option('admin_email');
		$_from = ( !empty($stms['emails']['admin_email']) ) ? stms_isset($stms['emails']['admin_email']) : $default_from;
		if ( $from != '' ) {
			if ( !empty($stms['email_templates'][$from]['frommail']) )
				$_from = sanitize_email($stms['email_templates'][$from]['frommail']);
		}

		require_once(STMS_SOURCES . 'ses.php');
		$aws_key = stms_isset($stms['integration']['s3']['access_key'], '', true);
		$aws_secret = stms_sanitize($stms['integration']['s3']['secret_key'], '', true);
		$ses = new SimpleEmailService($aws_key, $aws_secret);
		
		$verifies = $ses->listVerifiedEmailAddresses();
		if ( isset($verifies['Addresses']) && !in_array($_from, $verifies['Addresses']) )
			return false;
		
		$email = new SimpleEmailServiceMessage();
		$email->addTo($to);
		$email->setFrom($_from);
		$email->setSubject($subject);
		$email->setMessageFromString($message);
		$send = $ses->sendEmail($email);
		if ( !$send )
			return false;
	} else {
			if ( !function_exists('wp_mail') )
				require_once(ABSPATH . 'wp-includes/pluggable.php');
			
			$headers = array();
			$headers[] = "From: {$email_from} <{$email_addr}>";
			$send = wp_mail($to, $subject, $message, $headers);
			if ( !$send ) {
				stms_error_handler("Email cannot be send to {$to} From {$_from}", 0, '', 'email', false);
				return false;
			}
	}
	return true;
}

add_action('phpmailer_init','stms_activate_smtp');
function stms_activate_smtp( $phpmailer ) {
	$stms = get_option('stms_settings');
	if( stms_isset($stms['emails']['mailsys']) == 'smtp' ) {
		$phpmailer->IsSMTP();
		$phpmailer->SMTPSecure = stms_isset($stms['emails']['smtp_security']);
		$phpmailer->Host = trim(stms_isset($stms['emails']['smtp_host']));
		$phpmailer->Port = trim(stms_isset($stms['emails']['smtp_port']));
	
		$use_auth = ( !empty($stms['emails']['smtp_username']) != '' && !empty($stms['emails']['smtp_password']) ) ? TRUE : FALSE;
		if ( $use_auth ) {
			$phpmailer->SMTPAuth = TRUE;
			$phpmailer->Username = trim($stms['emails']['smtp_username']);
			$phpmailer->Password = trim($stms['emails']['smtp_password']);
		}
	} else {
		$phpmailer->IsMail();
	}
}

function stms_send_broadcast_email( $email, $subject, $body ) {
	$stms = get_option('stms_settings');
	$from = stms_sanitize($stms['emails']['admin_from']) . ' <' . sanitize_email($stms['emails']['admin_email']) . '>';
	$from = ( $from == '' ) ? stms_sanitize(get_option('blogname')) . ' <' . get_option('admin_email') . '>' : $from;
	if ( $mail = stms_mail( $email, $subject, $body, $from) )
		return true;
	return false;
}

add_action('template_redirect', 'stms_broadcast_unsubscribe');
function stms_broadcast_unsubscribe() {
	if ( isset($_GET['insta_mode']) && $_GET['insta_mode'] == 'unsubscribe' ) {
		global $wpdb;
		
		if ( empty($_GET['sub']) )
			wp_die('ERROR: Page Not Found');
		
		$email = base64_decode(rawurldecode($_GET['sub']));
		if ( !$m = stms_get_data_by('email', $email) )
			wp_die('ERROR: Email address ' . $email . ' is not registered.');
			
		$unsubscribed = get_option('stms_unsubscribed');
		if ( !is_array($unsubscribed) )
			$unsubscribed = array();
		
		if ( !in_array($email, $unsubscribed) ) {
			$unsubscribed[] = $email;
			update_option('stms_unsubscribed', $unsubscribed);
		}
		
		// also remove from queue...
		$wpdb->delete("{$wpdb->prefix}stms_email_queue", array('email' => $email));
		
		wp_die('<div class="insta-update-msg"><p>Email address <strong>"' . $email . '"</strong> has been unsubscribed from our mailing system.</p></div>', 'Unsubscribed');
	}	
}

add_action('stms_member_registered', 'stms_send_signupmail', 10, 3);
function stms_send_signupmail( $member_id, $password, $tx_id = 0 ) {
	$stms = get_option('stms_settings');
	$defaults = stms_default_emails();
	if ( !$member = stms_member($member_id) )
		return false;
	
	if ( $member->status == 4 || $member->status == 0 ) // do not send email if it's a temporary or inactive member account...
		return false;
	
	$to = sanitize_email($member->email);
	$subject = ( isset($stms['email_templates']['signedup']) ) ? stms_sanitize($stms['email_templates']['signedup']['subject']) : $defaults['signedup']['subject'];
	$subject = stms_parse_mail_tags($subject, $password, $member->ID, $tx_id);
	
	$body = ( isset($stms['email_templates']['signedup']) ) ? stms_sanitize($stms['email_templates']['signedup']['body']) : $defaults['signedup']['body'];
	$body = stms_parse_mail_tags($body, $password, $member->ID, $tx_id);
	
	stms_mail($to, $subject, $body, 'signedup');
	return true;
}

add_action('stms_member_confirm', 'stms_send_confirmmail', 10, 2);
function stms_send_confirmmail( $tx_id ) {
	$stms = get_option('stms_settings');
	$defaults = stms_default_emails();
	
	stms_error_handler("Got Hit Confirm Before", 0, '', 'confirm_email', false);
	
	if ( !$payment = stms_get_payment($tx_id) ) {
		return false;
	}
		
	if ( !$member = stms_member($payment->member_id) ) {
		return false;
	}
	
	if ( $member->status == 4 ) {
		return false;
	}

	stms_error_handler("Got Hit Confirm After", 0, '', 'confirm_email', false);
	
	$to = sanitize_email($member->email);
	$subject = ( isset($stms['email_templates']['confirm']) ) ? stms_sanitize($stms['email_templates']['confirm']['subject']) : $defaults['confirm']['subject'];
	$subject = stms_parse_mail_tags($subject, '', $member->ID, $payment->tx_id);
	
	$body = ( isset($stms['email_templates']['confirm']) ) ? stms_sanitize($stms['email_templates']['confirm']['body']) : $defaults['confirm']['body'];
	$body = stms_parse_mail_tags($body, '', $member->ID, $payment->tx_id);
	
	stms_mail($to, $subject, $body, 'confirm');
	return true;
}

add_action('stms_payment_completed', 'stms_send_receiptmail', 20, 1);
function stms_send_receiptmail( $tx_id ) {
	$stms = get_option('stms_settings');
	$defaults = stms_default_emails();
	if ( !$payment = stms_get_payment($tx_id) )
		return false;
	
	if ( !$member = stms_member($payment->member_id) )
		return false;
	
	$product = stms_get_product($payment->product_id);
	
	$send_email = true;
	if ( $member->status == 4 || $member->status == 0 || $payment->paysys == 'free' ) // do not send receipt email if it's a temporary or free member account...
		$send_email = false;
	
	$gateway = stms_gateway_type($payment->paysys);
	if ( $gateway == 'third' && stms_isset($product->info['auto_register']) == 1 ) {
		$send_email = true;
	}
	
	if ( !$send_email )
		return false;
	
	$to = sanitize_email($member->email);
	$subject = ( isset($stms['email_templates']['receipt']) ) ? stms_sanitize($stms['email_templates']['receipt']['subject']) : $defaults['receipt']['subject'];
	$subject = stms_parse_mail_tags($subject, '', $member->ID, $payment->tx_id);
	
	$body = ( isset($stms['email_templates']['receipt']) ) ? stms_sanitize($stms['email_templates']['receipt']['body']) : $defaults['receipt']['body'];
	$body = stms_parse_mail_tags($body, '', $member->ID, $payment->tx_id);
	
	stms_mail($to, $subject, $body, 'receipt');

	if ( stms_isset($stms['emails']['send_admin_receipt']) == 1 ) {
		$to = sanitize_email(stms_isset($stms['emails']['admin_email'], get_option('admin_email')));
		$subject = ( isset($stms['email_templates']['admin_receipt']) ) ? stms_sanitize($stms['email_templates']['admin_receipt']['subject']) : $defaults['admin_receipt']['subject'];
		$subject = stms_parse_mail_tags($subject, '', $member->ID, $tx_id);
		
		$body = ( isset($stms['email_templates']['admin_receipt']) ) ? stms_sanitize($stms['email_templates']['admin_receipt']['body']) : $defaults['admin_receipt']['body'];
		$body = stms_parse_mail_tags($body, '', $member->ID, $tx_id);
		
		stms_mail($to, $subject, $body, 'instabot');
	}
	
	return true;
}

add_action('stms_subscr_cancelled', 'stms_send_cancelmail');
function stms_send_cancelmail($tx_id) {
	$stms = get_option('stms_settings');
	$defaults = stms_default_emails();
	if ( !$payment = stms_get_payment($tx_id) )
		return false;
	
	$member = stms_member($payment->member_id);
	if ( stms_isset($stms['emails']['send_cancel']) == 1 ) :
		$to = sanitize_email($member->email);
		$subject = ( isset($stms['email_templates']['cancel']) ) ? stms_sanitize($stms['email_templates']['cancel']['subject']) : $defaults['cancel']['subject'];
		$subject = stms_parse_mail_tags($subject, '', $member->ID, $tx_id);
		
		$body = ( isset($stms['email_templates']['cancel']) ) ? stms_sanitize($stms['email_templates']['cancel']['body']) : $defaults['cancel']['body'];
		$body = stms_parse_mail_tags($body, '', $member->ID, $tx_id);
		stms_mail($to, $subject, $body, 'cancel');
	endif;
	
	if ( stms_isset($stms['emails']['send_admin_cancel']) == 1 ) :
		$to = sanitize_email(stms_isset($stms['emails']['admin_email'], get_option('admin_email')));
		$subject = ( isset($stms['email_templates']['admin_cancel']) ) ? stms_cleanup($stms['email_templates']['admin_cancel']['subject']) : $defaults['admin_cancel']['subject'];
		$subject = stms_parse_mail_tags($subject, '', $member->ID, $tx_id);
		
		$body = ( isset($stms['email_templates']['admin_cancel']) ) ? stms_cleanup($stms['email_templates']['admin_cancel']['body']) : $defaults['admin_cancel']['body'];
		$body = stms_parse_mail_tags($body, '', $member->ID, $tx_id);
		
		stms_mail($to, $subject, $body, 'instabot');
	endif;
	return true;
}

function stms_send_affpendingmail( $member_id ) {
	$stms = get_option('stms_settings');
	$defaults = stms_default_emails();
	$member = stms_member($member_id);

	if ( stms_isset($stms['emails']['send_aff_pending'], 1) == 1 ) :
		$to = sanitize_email($member->email);
		$subject = ( isset($stms['email_templates']['aff_pending']) ) ? stms_sanitize($stms['email_templates']['aff_pending']['subject']) : $defaults['aff_pending']['subject'];
		$subject = stms_parse_mail_tags($subject, '', $member->ID);
		
		$body = ( isset($stms['email_templates']['aff_pending']) ) ? stms_sanitize($stms['email_templates']['aff_pending']['body']) : $defaults['aff_pending']['body'];
		$body = stms_parse_mail_tags($body, '', $member->ID);
		
		stms_mail($to, $subject, $body, 'aff_pending');
	endif;
	return true;
}

function stms_send_affapprovedmail( $member_id ) {
	$stms = get_option('stms_settings');
	$defaults = stms_default_emails();
	$member = stms_member($member_id);
	$to = sanitize_email($member->email);
	$subject = ( isset($stms['email_templates']['aff_approved']) ) ? stms_sanitize($stms['email_templates']['aff_approved']['subject']) : $defaults['aff_approved']['subject'];
	$subject = stms_parse_mail_tags($subject, '', $member->ID);
	
	$body = ( isset($stms['email_templates']['aff_approved']) ) ? stms_sanitize($stms['email_templates']['aff_approved']['body']) : $defaults['aff_approved']['body'];
	$body = stms_parse_mail_tags($body, '', $member->ID);
	
	stms_mail($to, $subject, $body, 'aff_approved');
	return true;
}

add_action('stms_aff_commissioned_1', 'stms_send_commissionmail', 10, 3);
add_action('stms_aff_commissioned_2', 'stms_send_commissionmail', 10, 3);
function stms_send_commissionmail( $tx_id, $aff_id, $comm ) {
	$stms = get_option('stms_settings');
	$defaults = stms_default_emails();

	if ( !$payment = stms_get_payment($tx_id) )
		return false;
	
	if ( $aff_id <= 0 )
		return false;
	
	$member = stms_member($aff_id);
	if ( stms_isset($stms['emails']['send_commission']) == 1 ) :
		$to = sanitize_email($member->email);
		$subject = ( isset($stms['email_templates']['commission']) ) ? stms_cleanup($stms['email_templates']['commission']['subject']) : $defaults['commission']['subject'];
		$subject = stms_parse_mail_tags($subject, '', $aff_id, $tx_id, $comm);
		
		$body = ( isset($stms['email_templates']['commission']) ) ? stms_cleanup($stms['email_templates']['commission']['body']) : $defaults['commission']['body'];
		$body = stms_parse_mail_tags($body, '', $aff_id, $tx_id, $comm);
		
		stms_mail($to, $subject, $body, 'commission');
	endif;

	if ( stms_isset($stms['emails']['send_admin_commission']) == 1 ) :
		$to = sanitize_email(sanitize_email(stms_isset($stms['emails']['admin_email'], get_option('admin_email'))));
		$from = 'InstaMember <instabot@' . $_SERVER['HTTP_HOST'] . '>';
		
		$subject = ( isset($stms['email_templates']['admin_commission']) ) ? stms_cleanup($stms['email_templates']['admin_commission']['subject']) : $defaults['admin_commission']['subject'];
		$subject = stms_parse_mail_tags($subject, '', $aff_id, $tx_id, $comm);
		
		$body = ( isset($stms['email_templates']['admin_commission']) ) ? stms_cleanup($stms['email_templates']['admin_commission']['body']) : $defaults['admin_commission']['body'];
		$body = stms_parse_mail_tags($body, '', $aff_id, $tx_id, $comm);
		
		stms_mail($to, $subject, $body, 'instabot');
	endif;
	return true;
}

/* 
 * ======================== AFFILIATE-RELATED FUNCTIONS ========================
 */
function stms_affiliate_track($aff_id) {
	if ( empty($aff_id) )
		return false;
	
	$aff_id = (int) $aff_id;
	global $wpdb;
	$data = array(
		'aff_id' => $aff_id,
		'product_id' => ( isset($_GET['p']) ) ? intval($_GET['p']) : 0,
		'url' => $_SERVER['REQUEST_URI'],
		'ip_addr' => stms_retrieve_ipaddr(),
		'referrer' => stms_isset($_SERVER['HTTP_REFERER']),
		'date' => stms_time()
	);
	$wpdb->insert("{$wpdb->prefix}stms_aff_clicks", $data);
	return true;
}

add_action('template_redirect', 'stms_affiliate_link', 3);
function stms_affiliate_link() {
	if ( isset($_GET['iaff']) ) {
		$stms = get_option('stms_settings');
		$set = true;
		if ( is_stmsmember_logged_in() ) {
			$member = stms_get_current_member();
			if ( $member->ID == $_GET['iaff'] || $member->is_aff != 1 )
				$set = false;
		}
		$cookie_days = ( isset($stms['global']['cookie_age']) ) ? intval($stms['global']['cookie_age']) : 365;
		$cookie_age = time() + (60 * 60 * 24 * $cookie_days);
		if ( $set ) {
			@setcookie('__insta_aff_id', $_GET['iaff'], $cookie_age, SITECOOKIEPATH);
			if ( SITECOOKIEPATH != COOKIEPATH )
				@setcookie('__insta_aff_id', $_GET['iaff'], $cookie_age, COOKIEPATH);
			stms_affiliate_track($_GET['iaff']); // affiliate click track
			if ( is_stmsmember_logged_in() ) { // update affiliate data
				$member_id = stms_get_current_member_id();
				if ( $aff_id != $member_id )
					stms_update_member_meta($member_id, 'aff_id', $_GET['iaff']);
			}
		}
		if ( isset($_GET['u']) ) {
			$u = urldecode($_GET['u']);
			if ( filter_var($u, FILTER_VALIDATE_URL) ) {
				wp_redirect($u);
				exit;
			}
		}
		$redirect_url = get_bloginfo('url');
		if ( isset($_GET['p']) && $_GET['p'] > 0 ) {
			if ( $product = stms_get_product($_GET['p']) ) {
				stms_drop_third_affiliates($product->ID, $_GET['iaff']);
				if ( filter_var(stms_isset($product->info['salesuri']), FILTER_VALIDATE_URL) )
					$redirect_url = $product->info['salesuri'];
			}
		}
		wp_redirect($redirect_url);
		exit;
	}
}

function is_stms_affiliate( $member_id ) {
	if ( !$member = stms_member($member_id) ) return false;
	if ( $member->is_aff == 1 ) return true;
	return false;
}

/* 
 * ======================== AUTORESPONDER FUNCTIONS ========================
 */
function stms_update_aweber_lists() {
	$stms = get_option('stms_settings');
	$aw_consumer = stms_sanitize($stms['integration']['aweber']['consumer_key'], '', true);
	$aw_secret = stms_sanitize($stms['integration']['aweber']['secret_key'], '', true);
	$aweber_token = get_option('stms_aweber_token');
	if ( empty($aw_consumer) && empty($aw_secret) || !is_array($aweber_token) )
		return false;
	
	require_once(STMS_SOURCES . 'aweber_api/aweber_api.php');
	$aweber = new AWeberAPI($aw_consumer, $aw_secret);
	try {
		$account = $aweber->getAccount($aweber_token['accessToken'], $aweber_token['accessTokenSecret']);
	} catch ( AWeberAPIException $exc ) {
		return false;
	}
	
	$lists = array();
	foreach ( $account->lists as $key => $value ) {
		$lists[$value->id] = $value->name;
	}

	return $lists;
}

function stms_update_gr_lists() {
	require_once(STMS_SOURCES . 'jsonRPCClient.php');
	$stms = get_option('stms_settings');
	$api_key = stms_sanitize($stms['integration']['gr']['api_key'], '', true);
	if ( empty($api_key) )
		return false;
	
	$api_url = 'http://api2.getresponse.com';
	$client = new jsonRPCClient($api_url);
	try {
		$campaigns = $client->get_campaigns($api_key);
	} catch ( Exception $e ) {
		return false;
	}
	
	if ( count($campaigns) <= 0 )
		return false;
	
	$lists = array();
	foreach ( $campaigns as $campaign ) {
		$lists[$campaign['name']] = $campaign['name'];
	}
	
	return $lists;
}

function stms_update_mc_lists() {
	require_once(STMS_SOURCES . 'MCAPI.class.php');
	$stms = get_option('stms_settings');
	$api_key = stms_sanitize($stms['integration']['mc']['api_key'], '', true);
	if ( empty($api_key) )
		return false;
	
	$mc = new MCAPI($api_key);
	$retval = $mc->lists();
	if ( $mc->errorCode )
		return false;
	
	$mc_lists = $retval['data'];
	if ( sizeof($mc_lists) <= 0 )
		return false;
	
	$lists = array();
	foreach ( $mc_lists as $list) {
		$lists[$list['id']] = $list['name'];
	}
	return $lists;
}

function stms_update_imc_lists() {
	require_once(STMS_SOURCES . 'imnicamail.php');
	$stms = get_option('stms_settings');
	$username = stms_sanitize($stms['integration']['imc']['username'], '', true);
	$password = stms_sanitize($stms['integration']['imc']['pass'], '', true);
	if ( empty($username) || empty($password) )
		return false;
	
	// set post parameters for logging in...
	$post_parameters = array();
	$post_parameters[] = "Command=User.Login";
	$post_parameters[] = "ResponseFormat=JSON";
	$post_parameters[] = "Username={$username}";
	$post_parameters[] = "Password={$password}";
	
	$login = callImnicaMail('POST', $post_parameters);
	if ( stms_isset($login['code']) != 200 )
		return false;
	
	// set post parameters for getting lists data...
	$session_id = $login['data']['SessionID'];
	$post_parameters = array();
	$post_parameters[] = "Command=Lists.Get";
	$post_parameters[] = "ResponseFormat=JSON";
	$post_parameters[] = "SessionID={$session_id}";
	$post_parameters[] = "OrderField=ListID";
	$post_parameters[] = "OrderType=ASC";

	$call = callImnicaMail('POST', $post_parameters);
	$imc_lists = stms_isset($call['data']['Lists']);
	
	if ( empty($imc_lists) )
		return false;
	
	$lists = array();
	foreach ( $imc_lists as $list) {
		$lists[$list['ListID']] = $list['Name'];
	}
	return $lists;
}

function stms_update_ic_lists() {
	require_once(STMS_SOURCES . 'icontact.php');
	$stms = get_option('stms_settings');
	$api_key = stms_sanitize($stms['integration']['ic']['api_key'], '', true);
	$api_pass = stms_sanitize($stms['integration']['ic']['pass'], '', true);
	$username = stms_sanitize($stms['integration']['ic']['username'], '', true);
	$token = get_option('stms_icontact_token');
	if ( empty($api_key) || empty($api_pass) || empty($username) || !is_array($token) )
		return false;

	$accountId = $token['account_id'];
	$clientFolderId = $token['folder_id'];
	$resource = "/a/{$accountId}/c/{$clientFolderId}/lists";
	$call = callResource($config, $resource, 'GET');
	if ( $call['code'] != 200 )
		return false;
	
	$ic_lists = $call['data']['lists'];
	if ( !$ic_lists || count($ic_lists) <= 0 )
		return false;
	
	$lists = array();
	foreach( $ic_lists as $list ) {
		$lists[$list['listId']] = $list['name'];
	}
	return $lists;
}

function stms_update_sr_lists() {
	$stms = get_option('stms_settings');
	$key = stms_sanitize($stms['integration']['sr']['app_key'], '', true);
	$secret = stms_sanitize($stms['integration']['sr']['app_secret'], '', true);
	$userid = stms_sanitize($stms['integration']['sr']['userid'], '', true);
	if ( empty($key) || empty($secret) || empty($userid) )
		return false;

	$query = 'http://api.sendreach.com/index.php?key=' . $key . '&secret=' . $secret . '&action=lists_view';
	$call = @file_get_contents($query);
	
	$sr_lists = json_decode($call);
	if ( !$sr_lists || count($sr_lists) <= 0 )
		return false;
	
	$lists = array();
	foreach( $sr_lists as $list ) {
		$lists[$list->id] = $list->list_name;
	}
	return $lists;
}

function stms_available_autoresponders() {
	$stms = get_option('stms_settings');
	$resp = array();
	
	if ( stms_isset($stms['integration']['aweber']['consumer_key']) != ''
		&& stms_isset($stms['integration']['aweber']['secret_key']) != '' ) {
		$aweber_token = get_option('stms_aweber_token');
		if ( is_array($aweber_token) )
			$resp['aweber'] = 'Aweber'; 
	}
	
	if ( stms_isset($stms['integration']['gr']['api_key']) != '' )
			$resp['gr'] = 'GetResponse';
	
	if ( stms_isset($stms['integration']['mc']['api_key']) != '' )
			$resp['mc'] = 'MailChimp';
			
	$cc_api_key = stms_isset($stms['integration']['cc']['consumer_key']);
	$cc_secret_key = stms_isset($stms['integration']['cc']['secret_key']);
	$constant_token = get_option('stms_constant_token');
	if ( !empty($cc_api_key) && !empty($cc_secret_key) && is_array($constant_token) )
		$resp['cc'] = 'ConstantContact';

	$ic_api_key = stms_isset($stms['integration']['ic']['api_key']);
	$ic_api_pass = stms_isset($stms['integration']['ic']['pass']);
	$ic_username = stms_isset($stms['integration']['ic']['username']);
	$ic_token = get_option('stms_icontact_token');
	if ( !empty($ic_api_key) && !empty($ic_api_pass) && !empty($ic_username) && is_array($ic_token) )
		$resp['ic'] = 'iContact';
	
	$imc_username = stms_isset($stms['integration']['imc']['username']);
	$imc_pass = stms_isset($stms['integration']['imc']['pass']);
	if ( !empty($imc_username) && !empty($imc_pass) )
		$resp['imc'] = 'ImnicaMail';
	
	$inf_appname = stms_isset($stms['integration']['inf']['appname']);
	$inf_api = stms_isset($stms['integration']['inf']['api']);
	if ( !empty($inf_appname) && !empty($inf_api) )
		$resp['inf'] = 'InfusionSoft';
	
	$sr_key = stms_isset($stms['integration']['sr']['app_key']);
	$sr_secret = stms_isset($stms['integration']['sr']['app_secret']);
	$sr_userid = stms_isset($stms['integration']['sr']['userid']);
	if ( !empty($sr_key) && !empty($sr_secret) && !empty($sr_userid) )
		$resp['sr'] = 'SendReach';
	
	$resp['custom'] = 'Custom Autoresponder';
	return $resp;
}

add_action('init', 'stms_aweber_connect');
function stms_aweber_connect() {
	if ( isset($_REQUEST['stms_connect']) && $_REQUEST['stms_connect'] == 'aweber' ) {
		require_once(STMS_SOURCES . 'aweber_api/aweber_api.php');
		$stms = get_option('stms_settings');
		$aweber = new AWeberAPI(stms_isset($stms['integration']['aweber']['consumer_key']), stms_isset($stms['integration']['aweber']['secret_key']));
		$aweber_token = get_option('stms_aweber_token');
		if ( isset($_REQUEST['_instamember_nonce']) ) {
			if ( !wp_verify_nonce($_REQUEST['_instamember_nonce'], 'stms-nonce') || !empty($aweber_token) ) {
				@header('Location: ' . admin_url('admin.php?page=stms-settings') . '#integration');
				exit;
			}
			$callbackUrl = stms_format_url(get_option('siteurl'), 'stms_connect=aweber');
			try {
    			list($requestToken, $requestTokenSecret) = $aweber->getRequestToken($callbackUrl);
				@setcookie('__stms_aweber_ts', $requestTokenSecret, 0, COOKIEPATH);
				if ( SITECOOKIEPATH != COOKIEPATH )
					@setcookie('__stms_aweber_ts', $requestTokenSecret, 0, SITECOOKIEPATH);
				wp_redirect($aweber->getAuthorizeUrl());
				exit;
			} catch(AWeberAPIException $exc) {
				@header('Location: ' . admin_url('admin.php?page=stms-settings') . '#integration');
				exit;
			}
		} else if ( isset($_REQUEST['oauth_token']) ) {
			$aweber->user->tokenSecret = stms_isset($_COOKIE['__stms_aweber_ts']);
    		$aweber->user->requestToken = $_REQUEST['oauth_token'];
    		$aweber->user->verifier = stms_isset($_REQUEST['oauth_verifier']);
    		
    		list($accessToken, $accessTokenSecret) = $aweber->getAccessToken();
			$account = $aweber->getAccount($accessToken, $accessTokenSecret);
			$aweber_token = array(
						'accessToken' => $accessToken,
						'accessTokenSecret' => $accessTokenSecret,
						'accountID' => $account->id
			);
			update_option('stms_aweber_token', $aweber_token);
			
			// update aweber lists
			if ( function_exists('stms_update_aweber_lists') ) {
				$stms_lists = get_option('stms_ar_lists');
				if ( !is_array($stms_lists) )
					$stms_lists = array();
				$stms_lists['aweber'] = stms_update_aweber_lists();
				update_option('stms_ar_lists', $stms_lists);
			}
			
			// redirect back to integration page...
    		@header('Location: ' . admin_url('admin.php?page=stms-settings') . '#integration');
			exit;
		}
	}
}

add_action('init', 'stms_constantcontact_connect');
function stms_constantcontact_connect() {
	if ( isset($_REQUEST['instamember_connect']) && $_REQUEST['instamember_connect'] == 'oauth2' && isset($_REQUEST['provider']) && $_REQUEST['provider'] == 'constantcontact' ) {
		if ( !class_exists('ConstantContact') )
			require_once(STMS_SOURCES . 'ctct/autoload.php');
		
		$stms = get_option('stms_settings');
		$api_key = stms_isset($stms['integration']['cc']['consumer_key']);
		$secret_key = stms_isset($stms['integration']['cc']['secret_key']);
		$redirect_url = stms_format_url(get_option('siteurl'), 'instamember_connect=oauth2&provider=constantcontact');
		
		$oauth = new CTCTOauth2($api_key, $secret_key, $redirect_url);
		if ( isset($_GET['code']) ) {
			try {
        		$accessToken = $oauth->getAccessToken($_GET['code']);
    		} catch ( OAuth2Exception $ex ) {
        		echo '<p style="color:#cc0000">Error: ' . $ex->getMessage() . '</p>';
        		die();
    		}

			$session_consumer = array('username' => $_GET['username'], 'access_token' => $accessToken );
			update_option('stms_constant_token', $session_consumer);
			
			// update constantcontact lists
			if ( function_exists('stms_update_cc_lists') ) {
				$stms_lists = get_option('stms_ar_lists');
				if ( !is_array($stms_lists) )
					$stms_lists = array();
				$stms_lists['cc'] = stms_update_cc_lists();
				update_option('stms_ar_lists', $stms_lists);
			}

			stms_dump($accessToken);
			exit;
		}

		// redirect back to integration page...
		@header('Location: ' . admin_url('admin.php?page=stms-settings') . '#integration');
		exit;
	}
}

add_action('stms_payment_completed', 'stms_autoresponder_submit', 11, 1);
function stms_autoresponder_submit( $tx_id ) {
	if ( !$payment = stms_get_payment($tx_id) )
		return false;
	
	if ( !$product = stms_get_product($payment->product_id) )
		return false;
	
	if ( $product->info['is_responder'] != 1 || $payment->is_rebill == 1 || $payment->status == 'temp' )
		return false;
	
	$data = array(
		'member_id' => $payment->member_id,
		'ipaddr' => $payment->ipaddr,
		'ad_tracking' => strtolower(str_replace(" ", "_", esc_attr($product->title))),
		'note' => stms_sanitize($product->title)
	);
	
	$resp = stms_sanitize($product->info['autoresponder']);
	$submit = false;
	switch ( $resp ) {
		case 'aweber':
			if ( empty($product->info['aweber_list']) ) return false;
			$data['list'] = stms_sanitize($product->info['aweber_list']);
			$submit = stms_submit_aweber($data);
			break;
			
		case 'gr':
			if ( empty($product->info['gr_list']) ) return false;
			$data['list'] = stms_sanitize($product->info['gr_list']);
			$submit = stms_submit_getresponse($data);
			break;
			
		case 'mc':
			if ( empty($product->info['mc_list']) ) return false;
			$data['list'] = stms_sanitize($product->info['mc_list']);
			$submit = stms_submit_mailchimp($data);
			break;
		
		case 'ic':
			if ( empty($product->info['ic_list']) ) return false;
			$data['list'] = stms_sanitize($product->info['ic_list']);
			$submit = stms_submit_icontact($data);
			break;
			
		case 'cc':
			if ( empty($product->info['cc_list']) ) return false;
			$data['list'] = stms_sanitize($product->info['cc_list']);
			$submit = stms_submit_constantcontact($data);
			break;
			
		case 'imc':
			if ( empty($product->info['imc_list']) )
				return false;
			$data['list'] = stms_sanitize($product->info['imc_list']);
			$submit = stms_submit_imnica($data);
			break;
			
		case 'inf':
			$data['list'] = stms_sanitize($product->info['inf_list'], '');
			$submit = stms_submit_infusionsoft($data);
			break;
		
		case 'sr':
			$data['list'] = stms_sanitize($product->info['sr_list'], '');
			$submit = stms_submit_sendreach($data);
			break;
	}
	return $submit;
}

function stms_aff_autoresponder_submit( $member_id ) {
	$stms = get_option('stms_settings');
	if ( stms_isset($stms['global']['is_aff_responder'], 0) != 1 )
		return false;
	
	$data = array(
		'member_id' => $member_id,
		'ipaddr' => stms_retrieve_ipaddr(),
		'note' => stms_sanitize(get_bloginfo('blogname'))
	);
	$resp = stms_sanitize($stms['global']['aff_responder'], '');
	switch ( $resp ) {
		case 'aweber':
			if ( empty($stms['global']['aweber_aff_list']) )
				return false;
			$data['list'] = stms_sanitize($stms['global']['aweber_aff_list']);
			$submit = stms_submit_aweber($data);
			break;
			
		case 'gr':
			if ( empty($stms['global']['gr_aff_list']) )
				return false;
			$data['list'] = stms_sanitize($stms['global']['gr_aff_list']);
			$submit = stms_submit_getresponse($data);
			break;
			
		case 'mc':
			if ( empty($stms['global']['mc_aff_list']) )
				return false;
			$data['list'] = stms_sanitize($stms['global']['mc_aff_list']);
			$submit = stms_submit_mailchimp($data);
			break;
		
		case 'ic':
			if ( empty($stms['global']['ic_aff_list']) )
				return false;
			$data['list'] = stms_sanitize($stms['global']['ic_aff_list']);
			$submit = stms_submit_icontact($data);
			break;
			
		case 'cc':
			if ( empty($stms['global']['cc_aff_list']) )
				return false;
			$data['list'] = stms_sanitize($stms['global']['cc_aff_list']);
			$submit = stms_submit_icontact($data);
			break;
			
		case 'imc':
			if ( empty($stms['global']['imc_aff_list']) )
				return false;
			$data['list'] = stms_sanitize($stms['global']['imc_aff_list']);
			$submit = stms_submit_imnica($data);
			break;
			
		case 'inf':
			$data['list'] = stms_sanitize($stms['global']['inf_aff_list'], '');
			$submit = stms_submit_infusionsoft($data);
			
		case 'sr':
			$data['list'] = stms_sanitize($stms['global']['sr_aff_list'], '');
			$submit = stms_submit_sendreach($data);
			
	}
	return $submit;
}

function stms_submit_aweber( $data ) {
	if ( !is_array($data) )
		return false;
	
	require_once(STMS_SOURCES . 'aweber_api/aweber_api.php');
	$stms = get_option('stms_settings');
	$token = get_option('stms_aweber_token');

	if ( empty($data['list'])
		|| empty($token)
		|| empty($stms['integration']['aweber']['consumer_key'])
		|| empty($stms['integration']['aweber']['secret_key'])
	) {
		return false;
	}
	
	if ( !$member = stms_member($data['member_id']) ) 
		return false;
	
	$consumer_key = stms_sanitize($stms['integration']['aweber']['consumer_key'], '', true);
	$secret_key = stms_sanitize($stms['integration']['aweber']['secret_key'], '', true);
	
	$aweber = new AWeberAPI($consumer_key, $secret_key);
	try {
		$account = $aweber->getAccount($token['accessToken'], $token['accessTokenSecret']);
	} catch ( AWeberAPIException $exc ) {
		return false;
	}
	
	$account_id = $token['accountID'];
	$list_id = stms_sanitize($data['list']);

	$listURL = "/accounts/{$account_id}/lists/{$list_id}";
	try {
		$list = $account->loadFromUrl($listURL);
	} catch ( AWeberAPIException $exc ) {
		return false;
	}
	
	$subscriber_name = ( !empty($member->first_name) ) ? stms_cleanup($member->first_name) : $member->username;
	$params = array(
        	'email' => sanitize_email($member->email),
        	'ip_address' => $data['ipaddr'],
        	'misc_notes' => stms_sanitize($data['note']),
        	'name' => $subscriber_name
    	);

    $subscribers = $list->subscribers;
	try {
		$new_subscriber = $subscribers->create($params);
	} catch ( AWeberAPIException $exc ) {
		return false;
	}
	return true;
}

function stms_submit_getresponse( $data ) {
	if ( !is_array($data) )
		return false;
	
	require_once(STMS_SOURCES . 'jsonRPCClient.php');
	$stms = get_option('stms_settings');
	if ( empty($data['list'])
		|| empty($stms['integration']['gr']['api_key'])
	) {
		return false;
	}
	
	if ( !$member = stms_member($data['member_id']) ) 
		return false;
	
	$api_key = stms_sanitize($stms['integration']['gr']['api_key'], '', true);
	$api_url = 'http://api2.getresponse.com';
	$list_name = $data['list'];
	
	$client = new jsonRPCClient($api_url);
	try {
		$result = $client->get_campaigns(
			$api_key,
			array ( 'name' => array ( 'EQUALS' => $list_name ) )
		);
	} catch (Exception $e) {
		return false;
	}
	
	$subscriber_name = ( !empty($member->first_name) ) ? stms_cleanup($member->first_name) : $member->username;
	$campaign_id = array_pop(array_keys($result));
	
	// let's check if contact already added...
	/*
	try {
		$result = $client->get_contacts(
			$api_key,
			array(
				'campaigns' => array($CAMPAIGN_ID),
				'email' => array ('EQUALS' => sanitize_email($member->email))
			)
		);
	} catch (Exception $e) {
		return false;
	}
	
	if ( is_array($result) && count($result) > 0 ) {
		foreach ( $result as $contact ) {
			if ( $member->email == $contact['email'] && $CAMPAIGN_ID == $contact['campaign'] )
				return false; // contact already exists
		}
	}
	*/
	
	try {
    	$result = $client->add_contact(
    		$api_key,
    		array (
    			'campaign'  => $campaign_id,
    			'name'      => $subscriber_name,
    			'email'     => sanitize_email($member->email),
    			'cycle_day' => 0
    		)
		);
	} catch (Exception $e) {
    	return false;
	}
	return true;
}

function stms_submit_mailchimp( $data ) {
	if ( !is_array($data) )
		return false;
	
	require_once(STMS_SOURCES . 'MCAPI.class.php');
	$stms = get_option('stms_settings');
	if ( empty($data['list'])
		|| empty($stms['integration']['mc']['api_key'])
	) {
		return false;
	}
	
	if ( !$member = stms_member($data['member_id']) ) 
		return false;
	
	$api_key = stms_sanitize($stms['integration']['mc']['api_key'], '', true);
	$api = new MCAPI($api_key);
	$list_id = $data['list'];
	$email = sanitize_email($member->email);
	$first_name = ( !empty($member->first_name) ) ? $member->first_name : $member->username;
	$last_name = ( !empty($member->last_name) ) ? $member->last_name : '';

	$merge_vars = array(
			'FNAME' => $first_name,
			'LNAME' => $last_name,
			'OPTIN_TIME' => date("Y-M-D H:i:s", time())
	);

	// subscribe user to mail chimp
	$retval = $api->listSubscribe($list_id, $email, $merge_vars, 'html', false);
	if ( $retval )
		return true;
	else
		return false;
}

function stms_submit_icontact( $data ) {
	if ( !is_array($data) )
		return false;
	
	require_once(STMS_SOURCES . 'icontact.php');

	$stms = get_option('stms_settings');
	$token = get_option('stms_icontact_token');
	
	$config = array();
	$config['apiKey'] = stms_sanitize($stms['integration']['ic']['api_key'], '', true);
	$config['username'] = stms_sanitize($stms['integration']['ic']['pass'], '', true);
	$config['password'] = stms_sanitize($stms['integration']['ic']['username'], '', true);
	if ( empty($data['list'])
		|| empty($token)
		|| empty($config['apiKey'])
		|| empty($config['username'])
		|| empty($config['password'])
	) {
		return false;
	}
	
	if ( !$member = stms_member($data['member_id']) ) 
		return false;
	
	$accountId = $token['account_id'];
	$clientFolderId = $token['folder_id'];
	$listId = $data['list'];
	$email = sanitize_email($member->email);
	$first_name = ( !empty($member->first_name) ) ? $member->first_name : $member->username;
	$last_name = ( !empty($member->last_name) ) ? $member->last_name : '';

	// add to contact first
	$contactId = null;
	$resource = "/a/{$accountId}/c/{$clientFolderId}/contacts";
	$data = array(
			'firstName' => $first_name,
			'lastName'    => $last_name,
			'email'    => $email,
		);
	$call = callResource($config, $resource, 'POST', array($data));

	if ( $call['code'] == 200 ) {
		$contactId = $call['data']['contacts'][0]['contactId'];
		$resource = "/a/{$accountId}/c/{$clientFolderId}/subscriptions";
		$data = array(	
					array(
						'contactId' => $contactId,
						'listId'    => $listId,
						'status'    => 'normal',
					)
				);
		$call = callResource($config, $resource, 'POST', $data);
	}
}

function stms_submit_imnica( $data ) {
	if ( !is_array($data) )
		return false;
	
	require_once(STMS_SOURCES . 'imnicamail.php');
	$stms = get_option('stms_settings');
	if ( empty($data['list'])
		|| empty($stms['integration']['imc']['username'])
		|| empty($stms['integration']['imc']['pass'])
	) {
		return false;
	}

	if ( !$member = stms_member($data['member_id']) ) 
		return false;
		
	$username = stms_sanitize($stms['integration']['imc']['username'], '', true);
	$password = stms_sanitize($stms['integration']['imc']['pass'], '', true);
	
	$list_id  = $data['list'];
	
	// set post parameters for logging in...
	$post_parameters = array();
	$post_parameters[] = "Command=User.Login";
	$post_parameters[] = "ResponseFormat=JSON";
	$post_parameters[] = "Username={$username}";
	$post_parameters[] = "Password={$password}";

	$login = callImnicaMail( 'POST', $post_parameters );
	if ( $login['code'] != 200 ) return false;
	$session_id = $login['data']['SessionID'];
	
	$post_parameters = array();
	$post_parameters[] = "Command=Subscriber.Subscribe";
	$post_parameters[] = "ResponseFormat=JSON";
	$post_parameters[] = "SessionID={$session_id}";
	$post_parameters[] = "ListID={$list_id}";
	$post_parameters[] = "EmailAddress={$member->email}";
	$post_parameters[] = "IPAddress={$data['ipaddr']}";

	$subscribe = callImnicaMail( 'POST', $post_parameters );
	if ( $subscribe['code'] != 200 ) return false;
	return true;
}

function stms_submit_infusionsoft( $data ) {
	if ( !is_array($data) ) return false;
	
	require_once(STMS_SOURCES . 'xmlrpc-3.0/lib/xmlrpc.inc');
	$stms = get_option('stms_settings');
	if ( empty($stms['integration']['inf']['appname'])
		|| empty($stms['integration']['inf']['api'])
	) {
		return false;
	}
	
	if ( !$member = stms_member($data['member_id']) ) 
		return false;
	
	$app_name = stms_sanitize($stms['integration']['inf']['appname'], '', true);
	$api_key = stms_sanitize($stms['integration']['inf']['api'], '', true);
	$list_id = stms_isset($data['list'], '');
	$email = sanitize_email($member->email);
	$first_name = ( !empty($member->first_name) ) ? $member->first_name : $member->username;
	$last_name = ( !empty($member->last_name) ) ? $member->last_name : '';

	$api_url = "https://{$app_name}.infusionsoft.com/api/xmlrpc";
	
	$client = new xmlrpc_client($api_url);
	$client->return_type = "phpvals";
	$client->setSSLVerifyPeer(FALSE);
			
	// let's check if this email is exists...
	$contactId = false;
	$call = new xmlrpcmsg('ContactService.findByEmail',
		array(
			php_xmlrpc_encode($api_key), // API encrypted Key
			php_xmlrpc_encode($email), // Email address
			php_xmlrpc_encode(
				array(
					'Id', 
					'FirstName', 
					'LastName'
				)
			)
		)
	);
	
	$result = $client->send($call);
	if( !$result->faultCode() ) {
		$result = $result->value();
	    $contactId = ( !empty($result[0]['Id']) ) ? $result[0]['Id'] : false;
	}

	if ( !$contactId ) {
		// Let's add the email to InfusionSoft contact
		$call = new xmlrpcmsg('ContactService.add',
			array(
				php_xmlrpc_encode($api_key), // API encrypted Key
				php_xmlrpc_encode(
					array(
						'Email' => $email, 
						'FirstName' => $first_name, 
						'LastName' => $last_name
					)
				)
			)
		);
		$result = $client->send($call);
		if( !$result->faultCode() ) $contactId = $result->value();
		else return false;
		
		// Opt-In to InfusionSoft email service...
		$call = new xmlrpcmsg('APIEmailService.optIn',
			array(
				php_xmlrpc_encode($api_key), // API encrypted Key
				php_xmlrpc_encode($email),
				php_xmlrpc_encode('API OptIn via InstaMember (Site URL: ' . get_bloginfo('url') . ')')
			)
		);
		$result = $client->send($call);
		if( $result->faultCode() ) return false;
	}

	if ( $contactId && !empty($list_id) && is_numeric($list_id) ) {
		// let's add to InfusionSoft campaign
		$call = new xmlrpcmsg('ContactService.addToCampaign',
			array(
				php_xmlrpc_encode($api_key), // API encrypted Key
				php_xmlrpc_encode($contactId),
				php_xmlrpc_encode($list_id)
			)
		);
		$result = $client->send($call);
		if( $result->faultCode() ) return false;
	}
	return true;
}

function stms_submit_sendreach( $data ) {
	if ( !is_array($data) )
		return false;
	
	$stms = get_option('stms_settings');
	$key = stms_sanitize($stms['integration']['sr']['app_key'], '', true);
	$secret = stms_sanitize($stms['integration']['sr']['app_secret'], '', true);
	$userid = stms_sanitize($stms['integration']['sr']['userid'], '', true);
	if ( empty($data['list'])
		|| empty($key)
		|| empty($secret)
		|| empty($userid)
	) {
		return false;
	}
	
	if ( !$member = stms_member($data['member_id']) ) 
		return false;
	
	$list_id = $data['list'];
	$email = sanitize_email($member->email);
	$first_name = ( !empty($member->first_name) ) ? $member->first_name : $member->username;
	$last_name = ( !empty($member->last_name) ) ? $member->last_name : '';

	$query = 'http://api.sendreach.com/index.php?key=' . $key
		. '&secret=' . $secret
		. '&action=subscriber_add&user_id=' . $userid
		. '&list_id=' . $list_id
		. '&first_name=' . $first_name
		. '&last_name=' . $last_name
		. '&email='. $email
		. '&client_ip=' . stms_retrieve_ipaddr();
		
	$call = @file_get_contents($query);
}

add_action('stms_custom_autoresponder', 'stms_drop_ar_cookie', 10, 1);
function stms_drop_ar_cookie( $tx_id ) {
	if ( !$p = stms_get_payment($tx_id) )
		return false;
	
	$content = '';
	if ( isset($_COOKIE['__insta_ar_cookie']) )
		$content .= rawurldecode($_COOKIE['__insta_ar_cookie']) . '|';
	
	$content .= $p->tx_id;
	@setcookie('__insta_ar_cookie', $content, stms_time()+172800, SITECOOKIEPATH);
	if ( SITECOOKIEPATH != COOKIEPATH )
		@setcookie('__insta_ar_cookie', $content, stms_time()+172800, COOKIEPATH);
	return true;
}

add_action('wp_footer', 'stms_submit_custom_ar');
function stms_submit_custom_ar() {
	if ( isset($_COOKIE['__insta_ar_cookie']) ) {
		$html = '';
		$content = rawurldecode($_COOKIE['__insta_ar_cookie']);
		$tx_ids = explode("|", $content);
		if ( $tx_ids ) {
			$i = 0;
			$t = 200;
			$html .= '<script type="text/javascript">var timer;</script>' . "\n";
			foreach ( $tx_ids as $tx_id ) {
				if ( $py = stms_get_payment($tx_id) ) {
					$pr = stms_get_product($py->product_id);
					$m = stms_member($py->member_id);
					if ( $m && $m->status == 1 ) {
						$email = sanitize_email($m->email);
						$fname = stms_sanitize($m->first_name);
						if ( empty($fname) ) {
							$name = explode("@", $email);
							$fname = stms_sanitize($name[0]);
						}
							
						$_code = html_entity_decode(stms_sanitize($pr->info['custom_list']));
						$code = stms_extract_fields($_code);
						if ( $code ) {
							if ( stristr(stms_isset($code['action']), 'gogvo.com') ) {
								$url = 'https://www.gogvo.com/subscribe.php?';
								$params = array();
								if ( isset($code['hiddens']) && is_array($code['hiddens']) && count($code['hiddens']) > 0 ) {
									foreach ( $code['hiddens'] as $k => $v ) {
										$params[$k] = $v;
									}
								}
								
								if ( isset($code['fields']) && is_array($code['fields']) && count($code['fields']) > 0 ) {
									foreach ( $code['fields'] as $field ) {
										$value = ( stristr($field, 'mail') || stristr($field, 'from') ) ? $email : $fname;
										$params[$field] = $value;
									}
								}
								$url .= http_build_query($params);
								$send = wp_remote_get($url, array('timeout' => 120, 'httpversion' => '1.1'));
							} else {
								$html .= '<form method="post" name="stms-ar-form' . $i . '" id="stms-ar-form' . $i . '" action="' . stms_isset($code['action']) . '" target="stms_ar_frame' . $i . '">' . "\n";
								if ( isset($code['fields']) && is_array($code['fields']) && count($code['fields']) > 0 ) {
									foreach ( $code['fields'] as $field ) {
										$value = ( stristr($field, 'mail') || stristr($field, 'from') ) ? $email : $fname;
										$html .= '<input type="hidden" name="' . $field . '" value="' . $value . '" />' . "\n";
									}
								}
								
								if ( isset($code['hiddens']) && is_array($code['hiddens']) && count($code['hiddens']) > 0 ) {
									foreach ( $code['hiddens'] as $k => $v ) {
										$html .= '<input type="hidden" name="' . $k . '" value="' . $v . '" />' . "\n";
									}
								}
								$html .= '</form>' . "\n";
								$html .= '<iframe name="stms_ar_frame' . $i . '" id="stms_ar_frame' . $i . '" style="display:none" height="1" width="1" frameborder="0" scrolling="no"></iframe>' . "\n";
								$html .= '<script>' . "\n";
								$html .= "
										window.clearTimeout(timer);
										timer = setTimeout(function redir(){
											document.getElementById('stms-ar-form{$i}').submit();
										}, {$t});
								";
								$html .= '</script>' . "\n";
							}
							$i += 1;
							$t += 100;
						}
					}
				}
			}
		}
		echo $html;
		@setcookie('__insta_ar_cookie', 0, time()-259200, SITECOOKIEPATH);
		@setcookie('__insta_ar_cookie', 0, time()-259200, COOKIEPATH);
	}
}

add_action('stms_aff_custom_autoresponder', 'stms_drop_ar_affcookie', 10, 1);
function stms_drop_ar_affcookie( $member_id ) {
	if ( !$m = stms_member($member_id) )
		return false;
	
	@setcookie('__insta_ar_affcookie', $member_id, stms_time()+172800, SITECOOKIEPATH);
	if ( SITECOOKIEPATH != COOKIEPATH )
		@setcookie('__insta_ar_affcookie', $member_id, stms_time()+172800, COOKIEPATH);
	return true;
}

add_action('wp_footer', 'stms_submit_affcustom_ar');
function stms_submit_affcustom_ar() {
	if ( isset($_COOKIE['__insta_ar_affcookie']) ) {
		$html = '';
		$member_id = rawurldecode($_COOKIE['__insta_ar_affcookie']);
		$stms = get_option('stms_settings');
		if ( $m = stms_member($member_id) ) {
			if ( $m->status == 1 ) {
				$email = sanitize_email($m->email);
				$fname = stms_sanitize($m->first_name);
				if ( empty($fname) ) {
					$name = explode("@", $email);
					$fname = stms_sanitize($name[0]);
				}
							
				$_code = html_entity_decode(stms_sanitize($stms['global']['custom_aff_list']));
				$code = stms_extract_fields($_code);
				if ( $code ) {
					if ( stristr(stms_isset($code['action']), 'gogvo.com') ) {
						$url = 'https://www.gogvo.com/subscribe.php?';
						$params = array();
						if ( isset($code['hiddens']) && is_array($code['hiddens']) && count($code['hiddens']) > 0 ) {
							foreach ( $code['hiddens'] as $k => $v ) {
								$params[$k] = $v;
							}
						}
						
						if ( isset($code['fields']) && is_array($code['fields']) && count($code['fields']) > 0 ) {
							foreach ( $code['fields'] as $field ) {
								$value = ( stristr($field, 'mail') || stristr($field, 'from') ) ? $email : $fname;
								$params[$field] = $value;
							}
						}
						$url .= http_build_query($params);
						$send = wp_remote_get($url, array('timeout' => 120, 'httpversion' => '1.1'));
					} else {
						$html .= '<form method="post" name="stms-affar-form" id="stms-affar-form" action="' . stms_isset($code['action']) . '" target="stms_affar_frame">' . "\n";
						if ( isset($code['fields']) && is_array($code['fields']) && count($code['fields']) > 0 ) {
							foreach ( $code['fields'] as $field ) {
								$value = ( stristr($field, 'mail') || stristr($field, 'from') ) ? $email : $fname;
								$html .= '<input type="hidden" name="' . $field . '" value="' . $value . '" />' . "\n";
							}
						}
						
						if ( isset($code['hiddens']) && is_array($code['hiddens']) && count($code['hiddens']) > 0 ) {
							foreach ( $code['hiddens'] as $k => $v ) {
								$html .= '<input type="hidden" name="' . $k . '" value="' . $v . '" />' . "\n";
							}
						}
						$html .= '</form>' . "\n";
						$html .= '<iframe name="stms_affar_frame" id="stms_affar_frame" style="display:none" height="1" width="1" frameborder="0" scrolling="no"></iframe>' . "\n";
						$html .= '<script>' . "\n";
						$html .= "
								setTimeout(function redir(){
									document.getElementById('stms-affar-form').submit();
								}, 200);
						";
						$html .= '</script>' . "\n";
					}
				}
			}
		}
		echo $html;
		@setcookie('__insta_ar_affcookie', 0, time()-259200, SITECOOKIEPATH);
		@setcookie('__insta_ar_affcookie', 0, time()-259200, COOKIEPATH);
	}
}

function stms_extract_fields( $code ) {
	if ( $code == '' ) return false;
	
	$code = html_entity_decode(stripslashes($code));
	if ( !preg_match('/<form\s[^>]*action=[\'"]([^\'"]+)[\'"]/i', $code, $form) ) {
		if ( stristr($code, 'iframe') && function_exists('file_get_contents') ) {
			preg_match('/<iframe\s[^>]*src=[\'"]([^\'"]+)[\'"]/i', $code, $matches);
			if ( isset($matches[1]) ) {
				$iframe_url = html_entity_decode(urldecode($matches[1]));
				$content = @file_get_contents($iframe_url);
				
				if ( !empty($content) )
					$code = $content;
			}
		} 
	
		if ( stristr($code, '<script') && function_exists('file_get_contents') ){
			preg_match('/<script\s[^>]*src=[\'"]([^\'"]+)[\'"]/i', $code, $matches);
			if ( isset($matches[1]) ) {
				$js_url = html_entity_decode(urldecode($matches[1]));
				$content = @file_get_contents($js_url);
				if ( !empty($content) && stristr($content, 'document.write') ) {
					$code = stripslashes($content);
				}
			}
		}
	}

	preg_match('/<form\s[^>]*action=[\'"]([^\'"]+)[\'"]/i', $code, $form);
	preg_match_all('/<input\s[^>]*type=[\'"]?hidden[^>]*>/i', $code, $hiddens);
	preg_match_all('/<input\s[^>]*type=([\'"])?(text|email)[^>]*>/i', $code, $texts);
	
	// Text fields
	$fields = '';
	if ( !empty($texts[0]) ) {
		foreach( $texts[0] as $text ) {
			$name  = stms_extract_attribute($text, 'name');

			if ( !is_array($fields) )
				$fields = array();

			$fields[] = $name;
		}
	}

	// Hidden fields
	$values = '';
	if ( !empty($hiddens[0]) ) {
		foreach( $hiddens[0] as $hidden ) {
			$name  = stms_extract_attribute($hidden, 'name');
			$value = stms_extract_attribute($hidden, 'value');

			if ( !is_array($values) )
				$values = array();

			$values[$name] = $value;
		}
	}

	$post_data['action'] = stms_isset($form[1]);
	$post_data['fields'] = $fields;
	$post_data['hiddens'] = $values;

	return $post_data;
}

function stms_extract_attribute( $field, $attrib ) {
	$value = '';
	$pattern = ( $attrib == 'name' ) ? '/<input\s[^>]*name=[\'"]([^\'"]+)[\'"]/i' : '/<input\s[^>]*value=[\'"]([^\'"]+)[\'"]/i';
	preg_match($pattern, stripslashes($field), $matches);
	$value = ( !isset($matches[1]) ) ? '' : $matches[1];
	return $value;
}

/* 
 * ======================== ADD-ONS FUNCTIONS ========================
 */

add_action('stms_load_addons', 'stms_load_addons');
function stms_load_addons() {
	$addons_path = INSTA_ADDONS;
	if ( file_exists($addons_path) ) {
		if ( $addons_dir = opendir($addons_path) ) {
			while ( ( $addon = readdir($addons_dir) ) !== false ) {
				if ( $addon != '.' && $addon != '..' && $addon != 'index.php' && $addon != 'index.html' ) {
					$dir_path = $addons_path . $addon . '/';
					if ( file_exists($dir_path . 'index.php') )
						require_once($dir_path . 'index.php');
				}
			}
		}
	}
}
/* 
 * ======================== MISC. FUNCTIONS ========================
 */

function stms_get_access_log( $member_id, $start = 0, $limit = 15 ) {
	global $wpdb;
	
	$member_id = intval($member_id);
	$results = $wpdb->get_results(
					"SELECT * FROM `{$wpdb->prefix}stms_access_log`
					WHERE `member_id` = {$member_id} ORDER BY date DESC LIMIT {$start},{$limit}"
				);
	if ( !$results ) return false;
	return $results;
}

function stms_check_member_access( $member_id, $product_id ) {
	global $wpdb;
	
	$member_id = intval($member_id);
	$product_id = intval($product_id);
	if ( !$member = stms_member($member_id) )
		return false;
	
	$chk = $wpdb->get_var(
			"SELECT COUNT(*) FROM `{$wpdb->prefix}stms_product_access`
			WHERE `member_id` = {$member_id} AND `product_id` = {$product_id}
			AND `status` = 'active'"
		);
	if ( $chk > 0 ) return true;
	return false;
}

function stms_save_member_access( $member_id, $product_id, $status = 'active' ) {
	global $wpdb;
	
	$member_id = intval($member_id);
	$product_id = intval($product_id);
	if ( !$member = stms_member($member_id) )
		return false;
		
	$chk = $wpdb->get_var(
			"SELECT COUNT(*) FROM `{$wpdb->prefix}stms_product_access`
			WHERE `member_id` = {$member_id} AND `product_id` = {$product_id}"
		);
	$update = ( $count > 0 ) ? true : false;
	$data = array(
		'product_id' => $product_id,
		'member_id' => $member_id,
		'status' => $status
	);
	if ( $update )
		$wpdb->update("{$wpdb->prefix}stms_product_access", $data, array('product_id' => $product_id, 'member_id' => $member_id));
	else
		$wpdb->update("{$wpdb->prefix}stms_product_access", $data);
	return true;
}

add_action('stms_payment_completed', 'stms_webinar_register', 21, 1);
function stms_webinar_register( $tx_id ) {
	$stms = get_option('stms_settings');
	if ( !$payment = stms_get_payment($tx_id) ) return false;
	if ( !$member = stms_member($payment->member_id) ) return false;
	if ( !$product = stms_member($payment->product_id) ) return false;
	if ( $payment->is_rebill == 1 || stms_isset($product->info['is_webinar']) != 1 ) return false;
	
	if ( is_array($product->info['webinar_url']) && count($product->info['webinar_url']) > 0 ) {
		foreach ( $product->info['webinar_url'] as $gtw_url ) {
			if ( $data = stms_webinar_data($gtw_url) ) {
				$postdata = array(
					"Name_First" => stms_sanitize($member->first_name),
					"Name_Last"	=> stms_sanitize($member->last_name),
					"Email" => sanitize_email($member->email),
					"WebinarKey" => stms_isset($data['webid']),
					"Template" => "island/webinar/registration.tmpl",
					"Form" => "webinarRegistrationForm",
					"ViewArchivedWebinar" => "false",
					"registrant" => "",
					"RegistrantTimeZoneKey"	=> 8
				);
				
				$ch = curl_init();
				curl_setopt($ch, CURLOPT_URL, $data['url']);
				curl_setopt($ch, CURLOPT_POST, 1);
				curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($postdata));
			 	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			  	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
			  	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
  				$result = curl_exec($ch);
				
				// interval between webinar registration...
				sleep(2);
			}
		}
	}
}

function stms_webinar_data( $url ) {
	if ( !$parts = @parse_url($url) )
		return false;
	
	$host = explode(".", $parts['host']);
	$path = explode("/", $parts['path']);
	$subdomain = stms_isset($host[0], 'www1');
	$webid = end($path);
	
	return array (
		"url" => "https://{$subdomain}.gotomeeting.com/en_US/island/webinar/registration.flow",
		"webid" => $webid
	);
}

function stms_all_files() {
	return array(
		'dashboard' => 'Dashboard',
		'config' => 'Setup/Configuration',
		'level' => 'Products',
		'members' => 'Members/Affiliates',
		//'access' => 'Members Access',
		'reports' => 'Sales Report',
		'affstats' => 'Affiliate Stats',
		'licenses' => 'Manage Licenses',
		'support' => 'Support Desk',
		'broadcast' => 'Broadcast Emails',
		'urls' => 'Membership URLs',
	);
}

function stms_find_week_dates( $weekNumber = null, $year = null ) {
	// receives a specific Week Number (0 to 52) and returns that week's day dates in an array.
	// If no week specified returns this week's dates.
	$weekNumber = ($weekNumber=='') ? date('W'): $weekNumber ;
	$year = ($year=='') ? date('Y'): $year ;

	$weekNumber=sprintf("%02d", $weekNumber);
	for ( $i=1; $i<=7; $i++ ) {
    	$arrdays[] = strtotime($year . 'W' . $weekNumber . $i, stms_time());
	}
	return $arrdays;
}

function stms_find_weekly_dates() {
	$y = date("Y", stms_time());
	$w = date("W", stms_time());
	$res = array();
	for ( $i = 0; $i < 48; $i++ ) {
		$week_days = stms_find_week_dates($w, $y);
		$starting_date = date('m/d/Y', $week_days[0]);
		$ending_date = date('m/d/Y', $week_days[6]);
		
		$res[] = array(
					'name' => date("F d", $week_days[0]) . '-' . date("d, Y", $week_days[6]),
					'start' => $starting_date,
					'end' => $ending_date
				);

		$w--;
		if ( $w <= 0 ) { $w = 52; $y--; }
	}

	return $res;
}

function stms_find_monthly_dates() {
	$months = array();
	$start = strtotime(date("Y") . '/' . date("m") . '/01', stms_time());
	$end = strtotime('last day next month', $start);
	$months[] = array(
		'name' => date("F d", $start) . '-' . date("d, Y", $end),
		'start' => date("m/d/Y", $start),
		'end' => date("m/d/Y", $end)
	);
	for ( $i = 0; $i < 12; $i++ ) {
		$start = strtotime("1 month ago", $start);
		$end = strtotime('last day next month', $start);
		$months[] = array(
			'name' => date("F d", $start) . '-' . date("d, Y", $end),
			'start' => date("m/d/Y", $start),
			'end' => date("m/d/Y", $end)
		); 
	}
	return $months;
}

function stms_default_forms() {
	$forms = array(
		'signup' => array(
			'title' => 'Membership Registration',
			'before_text' => '',
			'after_text' => '',
			'card_title' => 'Billing Information',
			'cardname_label' => 'Name On Card',
			'cardnum_label' => 'Credit Card Number',
			'cardexp_label' => 'Expiration Date',
			'cardccv_label' => 'Security Code',
			'cardccv_txt' => 'What\'s This?',
			'cardccv_cnt' => "For American Express, it's the four digits in the front of the card.\n\nFor MasterCard, Visa or Discover, it's the last three digits on the signature area on the back of the card.",
			'billaddress_label' => 'Billing Address',
			'billcity_label' => 'City',
			'billstate_label' => 'State',
			'billzipcode_label' => 'Zip Code',
			'billcountry_label' => 'Country',
			'paymethod_label' => 'Payment Method',
			'captcha_label' => 'Enter Captcha:',
			'tos_text' => 'I have read and agree to [tos_link]Terms and Conditions[/tos_link]',
			'logexists_msg' => 'Already have an account? Please [login_link]Login[/login_link] to continue',
			'backtoreg_msg' => '&laquo; Back to [reg_link]registration form[/reg_link]',
			'paysuccess_msg' => "<strong>Congratulations On Your Purchase!</strong>\nIf you're an existing member, please [login_link]click here[/login_link] to login.\n\nDon't have an account yet? Please create a member's account using the form below:",
			'label' => 'Next Step',
			'upbutton_label' => 'Pay Now',
			'fields' => array(
				'username' => array('username', 'Username', 'no', 'yes'),
				'password' => array('password', 'Password', 'yes', 'yes'),
				'email' => array('email', 'Email Address', 'no', 'yes'),
				'title' => array('title', 'Title', 'yes', 'no'),
				'first_name' => array('first_name', 'First Name', 'yes', 'no'),
				'last_name' => array('last_name', 'Last Name', 'yes', 'no'),
				'street' => array('street', 'Street Address', 'yes', 'no'),
				'city' => array('city', 'City', 'yes', 'no'),
				'state' => array('state', 'State', 'yes', 'no'),
				'country' => array('country', 'Country', 'yes', 'no'),
				'zipcode' => array('zipcode', 'Zip Code', 'yes', 'no'),
				'phone' => array('phone', 'Phone', 'yes', 'no'),
				'company' => array('company', 'Company', 'yes', 'no'),
				'gender' => array('gender', 'Gender', 'yes', 'no'),
				'url' => array('url', 'Site URL', 'yes', 'no')
			),
			'active' => array('username', 'password', 'email', 'first_name', 'last_name')
		),
		'profile' => array(
			'title' => 'Edit Profile',
			'before_text' => '',
			'after_text' => '',
			'username_label' => 'Username',
			'changepass_title' => 'Change Password',
			'changepass_txt' => 'Leave these fields blank if you don\'t want to change your password.',
			'changepass_label' => 'New Password',
			'changepass2_label' => 'Repeat New Password',
			'aff_title' => 'Affiliate Program Settings',
			'afflink_label' => 'Your Affiliate Link',
			'viewstats_txt' => 'view stats',
			'affpaypal_label' => 'PayPal Email Address',
			'affpayza_label' => 'Payza Email Address',
			'affinactive_msg' => "Your affiliate account status is currently in <span style='color:#cc0000;font-weight:bold'>inactive</span>.\n\n<strong>[aff_activate_link]Click here[/aff_activate_link] to activate your affiliate account.</strong>",
			'affpending_msg' => "Your affiliate account status is currently in <span style='color:#000099;font-weight:bold'>pending</span>. You'll be notified via email when we approved your affiliate application.",
			'updated_msg' => '<strong>Your profile has been updated</strong>',
			'label' => 'Update Profile',
			'fields' => array(
				'email' => array('email', 'Email Address', 'no', 'yes'),
				'title' => array('title', 'Title', 'yes', 'no'),
				'first_name' => array('first_name', 'First Name', 'yes', 'yes'),
				'last_name' => array('last_name', 'Last Name', 'yes', 'yes'),
				'street' => array('street', 'Street Address', 'yes', 'no'),
				'city' => array('city', 'City', 'yes', 'no'),
				'state' => array('state', 'State', 'yes', 'no'),
				'country' => array('country', 'Country', 'yes', 'no'),
				'zipcode' => array('zipcode', 'Zip Code', 'yes', 'no'),
				'phone' => array('phone', 'Phone', 'yes', 'no'),
				'company' => array('company', 'Company', 'yes', 'no'),
				'gender' => array('gender', 'Gender', 'yes', 'no')
			),
			'active' => array('email', 'first_name', 'last_name', 'street', 'city', 'state', 'country', 'zipcode')
		),
		'affiliate' => array(
			'title' => 'Affiliate Registration',
			'before_text' => '',
			'after_text' => '',
			'captcha_label' => 'Enter Captcha:',
			'tos_text' => 'I have read and agree to [tos_link]Terms and Conditions[/tos_link]',
			'affactivated_msg' => '<strong>Your affiliate account has been activated. Please complete your profile and affiliate program settings below:</strong>',
			'affreview_msg' => '<strong>Thank you for your interest in our affiliate program. We\'ll evaluate your affiliate application, and you\'ll be notified via email when approved.</strong>',
			'label' => 'Register',
			'fields' => array(
				'username' => array('username', 'Username', 'no', 'yes'),
				'password' => array('password', 'Password', 'yes', 'yes'),
				'email' => array('email', 'Email Address', 'no', 'yes'),
				'title' => array('title', 'Title', 'yes', 'no'),
				'first_name' => array('first_name', 'First Name', 'yes', 'no'),
				'last_name' => array('last_name', 'Last Name', 'yes', 'no'),
				'street' => array('street', 'Street Address', 'yes', 'no'),
				'city' => array('city', 'City', 'yes', 'no'),
				'state' => array('state', 'State', 'yes', 'no'),
				'country' => array('country', 'Country', 'yes', 'no'),
				'zipcode' => array('zipcode', 'Zip Code', 'yes', 'no'),
				'phone' => array('phone', 'Phone', 'yes', 'no'),
				'company' => array('company', 'Company', 'yes', 'no'),
				'gender' => array('gender', 'Gender', 'yes', 'no'),
				'url' => array('url', 'Site URL', 'yes', 'no')
			),
			'active' => array('username', 'password', 'email', 'first_name', 'last_name', 'url')
		),
		'login' => array(
			'title' => 'Member Login',
			'label' => 'Log In',
			'before_text' => '',
			'after_text' => '',
			'username_label' => 'Username or Email Address',
			'password_label' => 'Password',
			'remember_label' => 'Remember Me',
			'captcha_label' => 'Please Enter Captcha:',
			'forgotpass_txt' => 'Forgot Your Password?',
			'logrequired_msg' => '<strong>Please Login</strong>',
			'button_label' => 'Log In',
			'button_label2' => 'Log In and Continue &raquo;',
		),
		'forgotpass' => array(
			'title' => 'Forgot Password Form',
			'before_text' => '',
			'after_text' => '',
			'email_label' => 'Email Address',
			'label' => 'Submit',
			'back_txt' => '&laquo; Back to login page',
		),
		'resetpass' => array(
			'title' => 'Reset Your Password',
			'before_text' => 'To reset your password, please create a new password below:',
			'after_text' => '',
			'password_label' => 'Enter New Password',
			'password2_label' => 'Repeat New Password',
			'passchanged_msg' => '<strong>Your password has been changed. Please login.</strong>',
			'label' => 'Change Password'
		)
	);
	
	return $forms;
}

function stms_build_form_fields($fields, $type = 'signup') {
	$defaults = stms_default_forms();
 	$forms = array();
	$forms['title'] = ( isset($fields['form_title']) ) ? stms_cleanup($fields['form_title']) : $defaults[$type]['title'];
	$forms['before_text'] = ( isset($fields['before_text']) ) ? stms_cleanup($fields['before_text'], '<strong><em><u><i><b><a><img><span><div><ul><ol><li>') : $defaults[$type]['before_text'];
	$forms['after_text'] = ( isset($fields['after_text']) ) ? stms_cleanup($fields['after_text'], '<strong><em><u><i><b><a><img><span><div><ul><ol><li>') : $defaults[$type]['after_text'];
	$forms['label'] = ( isset($fields['button_label']) ) ? stms_cleanup($fields['button_label']) : $defaults[$type]['label'];
	
	if ( $type == 'login' ) {
		$forms['username_label'] = ( isset($fields['username_label']) ) ? stms_cleanup($fields['username_label']) : $defaults[$type]['username_label'];
		$forms['password_label'] = ( isset($fields['password_label']) ) ? stms_cleanup($fields['password_label']) : $defaults[$type]['password_label'];
		$forms['remember_label'] = ( isset($fields['remember_label']) ) ? stms_cleanup($fields['remember_label']) : $defaults[$type]['remember_label'];
		$forms['captcha_label'] = ( isset($fields['captcha_label']) ) ? stms_cleanup($fields['captcha_label']) : $defaults[$type]['captcha_label'];
		$forms['forgotpass_txt'] = ( isset($fields['forgotpass_txt']) ) ? stms_cleanup($fields['forgotpass_txt']) : $defaults[$type]['forgotpass_txt'];
		$forms['button_label2'] = ( isset($fields['button_label2']) ) ? stms_cleanup($fields['button_label2']) : $defaults[$type]['button_label2'];
		
		$forms['logrequired_msg'] = ( isset($fields['logrequired_msg']) ) ? stms_cleanup($fields['logrequired_msg'], '<span><strong><em><u><i><b>') : $defaults[$type]['logrequired_msg'];
	}
	
	if ( $type == 'profile' ) {
		$forms['username_label'] = ( isset($fields['username_label']) ) ? stms_cleanup($fields['username_label']) : $defaults[$type]['username_label'];
		$forms['changepass_title'] = ( isset($fields['changepass_title']) ) ? stms_cleanup($fields['changepass_title']) : $defaults[$type]['changepass_title'];
		$forms['changepass_txt'] = ( isset($fields['changepass_txt']) ) ? stms_cleanup($fields['changepass_txt']) : $defaults[$type]['changepass_txt'];
		$forms['changepass_label'] = ( isset($fields['changepass_label']) ) ? stms_cleanup($fields['changepass_label']) : $defaults[$type]['changepass_label'];
		$forms['changepass2_label'] = ( isset($fields['changepass2_label']) ) ? stms_cleanup($fields['changepass2_label']) : $defaults[$type]['changepass2_label'];
		
		$forms['aff_title'] = ( isset($fields['aff_title']) ) ? stms_cleanup($fields['aff_title']) : $defaults[$type]['aff_title'];
		$forms['afflink_label'] = ( isset($fields['afflink_label']) ) ? stms_cleanup($fields['afflink_label']) : $defaults[$type]['afflink_label'];
		$forms['viewstats_txt'] = ( isset($fields['viewstats_txt']) ) ? stms_cleanup($fields['viewstats_txt']) : $defaults[$type]['viewstats_txt'];
		$forms['affpaypal_label'] = ( isset($fields['affpaypal_label']) ) ? stms_cleanup($fields['affpaypal_label']) : $defaults[$type]['affpaypal_label'];
		$forms['affpayza_label'] = ( isset($fields['affpayza_label']) ) ? stms_cleanup($fields['affpayza_label']) : $defaults[$type]['affpayza_label'];
		
		$forms['affinactive_msg'] = ( isset($fields['affinactive_msg']) ) ? stms_cleanup($fields['affinactive_msg'], '<span><strong><em><u><i><b>') : $defaults[$type]['affinactive_msg'];
		$forms['affpending_msg'] = ( isset($fields['affpending_msg']) ) ? stms_cleanup($fields['affpending_msg'], '<span><strong><em><u><i><b>') : $defaults[$type]['affpending_msg'];
	
		$forms['updated_msg'] = ( isset($fields['updated_msg']) ) ? stms_cleanup($fields['updated_msg'], '<span><strong><em><u><i><b>') : $defaults[$type]['updated_msg'];
	}
	
	if ( $type == 'forgotpass' ) {
		$forms['email_label'] = ( isset($fields['email_label']) ) ? stms_cleanup($fields['email_label']) : $defaults[$type]['email_label'];
		$forms['back_txt'] = ( isset($fields['back_txt']) ) ? stms_cleanup($fields['back_txt']) : $defaults[$type]['back_txt'];
	}
	
	if ( $type == 'resetpass' ) {
		$forms['password_label'] = ( isset($fields['password_label']) ) ? stms_cleanup($fields['password_label']) : $defaults[$type]['password_label'];
		$forms['password2_label'] = ( isset($fields['password2_label']) ) ? stms_cleanup($fields['password2_label']) : $defaults[$type]['password2_label'];
		$forms['passchanged_msg'] = ( isset($fields['passreset_msg']) ) ? stms_cleanup($fields['passreset_msg'], '<span><strong><em><u><i><b>') : $defaults[$type]['passreset_msg'];
	}
	
	if ( $type == 'signup' || $type == 'affiliate' ) {
		$forms['tos_text'] = ( isset($fields['tos_text']) ) ? stms_cleanup($fields['tos_text']) : $defaults[$type]['tos_text'];
		$forms['captcha_label'] = ( isset($fields['captcha_label']) ) ? stms_cleanup($fields['captcha_label']) : $defaults[$type]['captcha_label'];
	}
	
	if ( $type == 'affiliate' ) {
		$forms['affactivated_msg'] = ( isset($fields['affactivated_msg']) ) ? stms_cleanup($fields['affactivated_msg']) : $defaults[$type]['affactivated_msg'];
		$forms['affreview_msg'] = ( isset($fields['affreview_msg']) ) ? stms_cleanup($fields['affreview_msg']) : $defaults[$type]['affreview_msg'];
	}
	
	if ( $type == 'signup' ) {
		$forms['paymethod_label'] = ( isset($fields['paymethod_label']) ) ? stms_cleanup($fields['paymethod_label']) : $defaults[$type]['paymethod_label'];
		
		$forms['card_title'] = ( isset($fields['card_title']) ) ? stms_cleanup($fields['card_title']) : $defaults[$type]['card_title'];
		$forms['cardname_label'] = ( isset($fields['cardname_label']) ) ? stms_cleanup($fields['cardname_label']) : $defaults[$type]['cardname_label'];
		$forms['cardnum_label'] = ( isset($fields['cardnum_label']) ) ? stms_cleanup($fields['cardnum_label']) : $defaults[$type]['cardnum_label'];
		$forms['cardexp_label'] = ( isset($fields['cardexp_label']) ) ? stms_cleanup($fields['cardexp_label']) : $defaults[$type]['cardexp_label'];
		$forms['cardccv_label'] = ( isset($fields['cardccv_label']) ) ? stms_cleanup($fields['cardccv_label']) : $defaults[$type]['cardccv_label'];
		$forms['cardccv_txt'] = ( isset($fields['cardccv_txt']) ) ? stms_cleanup($fields['cardccv_txt']) : $defaults[$type]['cardccv_txt'];
		$forms['cardccv_cnt'] = ( isset($fields['cardccv_cnt']) ) ? stms_cleanup($fields['cardccv_cnt'], '<strong><em><u><i><b>') : $defaults[$type]['cardccv_cnt'];
		$forms['upbutton_label'] = ( isset($fields['upbutton_label']) ) ? stms_cleanup($fields['upbutton_label']) : $defaults[$type]['upbutton_label'];
		
		$forms['billaddress_label'] = ( isset($fields['billaddress_label']) ) ? stms_cleanup($fields['billaddress_label']) : $defaults[$type]['billaddress_label'];
		$forms['billcity_label'] = ( isset($fields['billcity_label']) ) ? stms_cleanup($fields['billcity_label']) : $defaults[$type]['billcity_label'];
		$forms['billstate_label'] = ( isset($fields['billstate_label']) ) ? stms_cleanup($fields['billstate_label']) : $defaults[$type]['billstate_label'];
		$forms['billzipcode_label'] = ( isset($fields['billzipcode_label']) ) ? stms_cleanup($fields['billzipcode_label']) : $defaults[$type]['billzipcode_label'];
		$forms['billcountry_label'] = ( isset($fields['billcountry_label']) ) ? stms_cleanup($fields['billcountry_label']) : $defaults[$type]['billcountry_label'];
		
		$forms['logexists_msg'] = ( isset($fields['logexists_msg']) ) ? stms_cleanup($fields['logexists_msg'], '<strong><em><u><i><b>') : $defaults[$type]['logexists_msg'];
		$forms['backtoreg_msg'] = ( isset($fields['backtoreg_msg']) ) ? stms_cleanup($fields['backtoreg_msg'], '<strong><em><u><i><b>') : $defaults[$type]['backtoreg_msg'];
		$forms['paysuccess_msg'] = ( isset($fields['paysuccess_msg']) ) ? stms_cleanup($fields['paysuccess_msg'], '<strong><em><u><i><b>') : $defaults[$type]['paysuccess_msg'];
	}
	
	if ( $type != 'login' && $type != 'forgotpass' && $type != 'resetpass' ) {
		$new_fields = array();
		$forms['active'] = $fields['fields'];
		$unremove = array('username', 'email', 'message');
		for( $i = 0; $i < count($fields['fields']); $i++ ) {
			$removeable = ( in_array($fields['fields'][$i], $unremove) ) ? 'no' : 'yes';
			$checked = ( stms_isset($fields['fields'][$i]) == 'password' || isset($fields['required'][$fields['fields'][$i]]) ) ? 'yes' : 'no';
			$checked = ( in_array($fields['fields'][$i], $unremove) ) ? 'yes' : $checked;
			$new_fields[$fields['fields'][$i]] = array(
				$fields['fields'][$i],
				$fields['labels'][$fields['fields'][$i]],
				$removeable,
				$checked
			);
		}
		$forms['fields'] = wp_parse_args($new_fields, stms_isset($defaults[$type]['fields']));
	}
	return $forms;
}

function stms_string_enc( $string, $key ) {
    $key = sha1($key);
    $strLen = strlen($string);
    $keyLen = strlen($key);
	$j = 0;
	$hash = '';
    for ( $i = 0; $i < $strLen; $i++ ) {
        $ordStr = ord(substr($string,$i,1));
        if ( $j == $keyLen ) { $j = 0; }
        $ordKey = ord(substr($key,$j,1));
        $j++;
        $hash .= strrev(base_convert(dechex($ordStr + $ordKey),16,36));
    }
    return $hash;
}

function stms_string_dec( $string, $key ) {
    $key = sha1($key);
    $strLen = strlen($string);
    $keyLen = strlen($key);
	$j = 0;
	$hash = '';
    for ($i = 0; $i < $strLen; $i+=2) {
        $ordStr = hexdec(base_convert(strrev(substr($string,$i,2)),36,16));
        if ( $j == $keyLen ) { $j = 0; }
        $ordKey = ord(substr($key,$j,1));
        $j++;
        $hash .= chr($ordStr - $ordKey);
    }
    return $hash;
}

function stms_generate_secret($length = 16, $number = true, $capital = false, $alpha = true ) {
	if ( $length < 1 )
		return false;
	
	$characters = '';
	if ( $alpha )
    	$characters .= 'abcdefghijklmnopqrstuvwxyz';
	if ( $number )
		$characters .= '1234567890';
	if ( $capital )
		$characters .= 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
		
    $secret = '';    
	for ( $p = 0; $p < $length; $p++ ) {
   		$secret .= $characters[mt_rand(0, strlen($characters)-1)];
	}
    return $secret;
}

function stms_get_handler_url( $handler = 'login' ) {
	$url = stms_login_url();
	$stms = get_option('stms_settings');
	$slugs = get_option('stms_slugs');
	$handler_url = stms_format_url(stms_isset($stms['global']['magic']), 'imhandler=');
	$home_url = get_permalink(stms_isset($stms['global']['home']));
	$logout_url = $handler_url . 'logout';
	$support_url = $handler_url . 'support';
	$profile_url = $handler_url . 'profile';
	$order_lookup_url = $handler_url . 'order_lookup';
	$registration_url = $handler_url . 'registration';
	$stats_url = $handler_url . 'stats';

	switch ( $handler ) {
		case 'home':
			$url = $home_url;
			break;
			
		case 'logout':
			$logout_id = ( isset($slugs['logout']) && $slugs['logout'] > 0 ) ? $slugs['logout'] : 0;
			$url = ( $logout_id ) ? get_permalink($logout_id) : $logout_url;
			break;
			
		case 'support':
			$support_id = ( isset($slugs['support']) && $slugs['support'] > 0 ) ? $slugs['support'] : 0;
			$url = ( $support_id ) ? get_permalink($support_id) : $support_url;
			break;
			
		case 'profile':
			$profile_id = ( isset($slugs['profile']) && $slugs['profile'] > 0 ) ? $slugs['profile'] : 0;
			$url = ( $profile_id ) ? get_permalink($profile_id) : $profile_url;
			break;
			
		case 'order_lookup':
			$order_lookup_id = ( isset($slugs['order_lookup']) && $slugs['order_lookup'] > 0 ) ? $slugs['order_lookup'] : 0;
			$url = ( $order_lookup_id ) ? get_permalink($order_lookup_id) : $order_lookup_url;
			break;
			
		case 'registration':
			$registration_id = ( isset($slugs['registration']) && $slugs['registration'] > 0 ) ? $slugs['registration'] : 0;
			$url = ( $registration_id ) ? get_permalink($registration_id) : $registration_url;
			break;
			
		case 'stats':
			$stats_id = ( isset($slugs['stats']) && $slugs['stats'] > 0 ) ? $slugs['stats'] : 0;
			$url = ( $stats_id ) ? get_permalink($stats_id) : $stats_url;
			break;
			
		case 'login':
		default:
			$url = stms_login_url();
			break;
	}
	return $url;
}

function stms_reset_handler_url( $handler = 'login' ) {
	$slugs = get_option('stms_slugs');
	if ( !is_array($slugs) )
		$slugs = array();
	
	if ( isset($slugs[$handler]) )
		unset($slugs[$handler]);
	
	update_option('stms_slugs', $slugs);
}

function stms_sanitize( &$value, $default = null, $is_key = false ) {
	if ( !isset($value) ) $value = $default;
	if ( $is_key ) 
		$value = str_replace(" ", "+", $value);
	return trim(stripslashes($value));
}

function stms_cleanup(&$str, $allowed_tags = '', $remove_lines = false, $texturize = false, $is_key = false ) {
	if ( !isset($str) )
		return '';
		
	$value = stms_sanitize($str);
	if ( $remove_lines )
		$value = str_replace(array("\n", "\r", "\r\n"), " ", $value);
	if ( empty($allowed_tags) )
		$value = strip_tags($value);
	else {
		$value = strip_tags($str, $allowed_tags);
		$value = $value;
	}
	
	if ( $is_key ) 
		$value = str_replace(" ", "+", $value);
	
	if ( $texturize )
		$value = wptexturize($value);
	
	return $value;
}
function stms_world_countries() {
	$countries = array(
		'US'=>'United States',
		'DZ'=>'Algeria',
		'AO'=>'Angola',
		'BJ'=>'Benin',
		'BW'=>'Botswana',
		'BF'=>'Burkina Faso',
		'BI'=>'Burundi',
		'CM'=>'Cameroon',
		'CV'=>'Cape Verde',
		'CF'=>'Central African Republic',
		'TD'=>'Chad',
		'KM'=>'Comoros',
		'CD'=>'Congo [DRC]',
		'CG'=>'Congo [Republic]',
		'DJ'=>'Djibouti',
		'EG'=>'Egypt',
		'GQ'=>'Equatorial Guinea',
		'ER'=>'Eritrea',
		'ET'=>'Ethiopia',
		'GA'=>'Gabon',
		'GM'=>'Gambia',
		'GH'=>'Ghana',
		'GN'=>'Guinea',
		'GW'=>'Guinea-Bissau',
		'CI'=>'Ivory Coast',
		'KE'=>'Kenya',
		'LS'=>'Lesotho',
		'LR'=>'Liberia',
		'LY'=>'Libya',
		'MG'=>'Madagascar',
		'MW'=>'Malawi',
		'ML'=>'Mali',
		'MR'=>'Mauritania',
		'MU'=>'Mauritius',
		'YT'=>'Mayotte',
		'MA'=>'Morocco',
		'MZ'=>'Mozambique',
		'NA'=>'Namibia',
		'NE'=>'Niger',
		'NG'=>'Nigeria',
		'RW'=>'Rwanda',
		'RE'=>'Réunion',
		'SH'=>'Saint Helena',
		'SN'=>'Senegal',
		'SC'=>'Seychelles',
		'SL'=>'Sierra Leone',
		'SO'=>'Somalia',
		'ZA'=>'South Africa',
		'SD'=>'Sudan',
		'SZ'=>'Swaziland',
		'ST'=>'São Tomé and Príncipe',
		'TZ'=>'Tanzania',
		'TG'=>'Togo',
		'TN'=>'Tunisia',
		'UG'=>'Uganda',
		'EH'=>'Western Sahara',
		'ZM'=>'Zambia',
		'ZW'=>'Zimbabwe',
		'AQ'=>'Antarctica',
		'BV'=>'Bouvet Island',
		'TF'=>'French Southern Territories',
		'HM'=>'Heard Island and McDonald Island',
		'GS'=>'South Georgia and the South Sandwich Islands',
		'AF'=>'Afghanistan',
		'AM'=>'Armenia',
		'AZ'=>'Azerbaijan',
		'BH'=>'Bahrain',
		'BD'=>'Bangladesh',
		'BT'=>'Bhutan',
		'IO'=>'British Indian Ocean Territory',
		'BN'=>'Brunei',
		'KH'=>'Cambodia',
		'CN'=>'China',
		'CX'=>'Christmas Island',
		'CC'=>'Cocos [Keeling] Islands',
		'GE'=>'Georgia',
		'HK'=>'Hong Kong',
		'IN'=>'India',
		'ID'=>'Indonesia',
		'IR'=>'Iran',
		'IQ'=>'Iraq',
		'IL'=>'Israel',
		'JP'=>'Japan',
		'JO'=>'Jordan',
		'KZ'=>'Kazakhstan',
		'KW'=>'Kuwait',
		'KG'=>'Kyrgyzstan',
		'LA'=>'Laos',
		'LB'=>'Lebanon',
		'MO'=>'Macau',
		'MY'=>'Malaysia',
		'MV'=>'Maldives',
		'MN'=>'Mongolia',
		'MM'=>'Myanmar [Burma]',
		'NP'=>'Nepal',
		'KP'=>'North Korea',
		'OM'=>'Oman',
		'PK'=>'Pakistan',
		'PS'=>'Palestinian Territories',
		'PH'=>'Philippines',
		'QA'=>'Qatar',
		'SA'=>'Saudi Arabia',
		'SG'=>'Singapore',
		'KR'=>'South Korea',
		'LK'=>'Sri Lanka',
		'SY'=>'Syria',
		'TW'=>'Taiwan',
		'TJ'=>'Tajikistan',
		'TH'=>'Thailand',
		'TR'=>'Turkey',
		'TM'=>'Turkmenistan',
		'AE'=>'United Arab Emirates',
		'UZ'=>'Uzbekistan',
		'VN'=>'Vietnam',
		'YE'=>'Yemen',
		'AL'=>'Albania',
		'AD'=>'Andorra',
		'AT'=>'Austria',
		'BY'=>'Belarus',
		'BE'=>'Belgium',
		'BA'=>'Bosnia and Herzegovina',
		'BG'=>'Bulgaria',
		'HR'=>'Croatia',
		'CY'=>'Cyprus',
		'CZ'=>'Czech Republic',
		'DK'=>'Denmark',
		'EE'=>'Estonia',
		'FO'=>'Faroe Islands',
		'FI'=>'Finland',
		'FR'=>'France',
		'DE'=>'Germany',
		'GI'=>'Gibraltar',
		'GR'=>'Greece',
		'GG'=>'Guernsey',
		'HU'=>'Hungary',
		'IS'=>'Iceland',
		'IE'=>'Ireland',
		'IM'=>'Isle of Man',
		'IT'=>'Italy',
		'JE'=>'Jersey',
		'XK'=>'Kosovo',
		'LV'=>'Latvia',
		'LI'=>'Liechtenstein',
		'LT'=>'Lithuania',
		'LU'=>'Luxembourg',
		'MK'=>'Macedonia',
		'MT'=>'Malta',
		'MD'=>'Moldova',
		'MC'=>'Monaco',
		'ME'=>'Montenegro',
		'NL'=>'Netherlands',
		'NO'=>'Norway',
		'PL'=>'Poland',
		'PT'=>'Portugal',
		'RO'=>'Romania',
		'RU'=>'Russia',
		'SM'=>'San Marino',
		'RS'=>'Serbia',
		'CS'=>'Serbia and Montenegro',
		'SK'=>'Slovakia',
		'SI'=>'Slovenia',
		'ES'=>'Spain',
		'SJ'=>'Svalbard and Jan Mayen',
		'SE'=>'Sweden',
		'CH'=>'Switzerland',
		'UA'=>'Ukraine',
		'GB'=>'United Kingdom',
		'VA'=>'Vatican City',
		'AX'=>'Åland Islands',
		'AI'=>'Anguilla',
		'AG'=>'Antigua and Barbuda',
		'AW'=>'Aruba',
		'BS'=>'Bahamas',
		'BB'=>'Barbados',
		'BZ'=>'Belize',
		'BM'=>'Bermuda',
		'BQ'=>'Bonaire, Saint Eustatius and Saba',
		'VG'=>'British Virgin Islands',
		'CA'=>'Canada',
		'KY'=>'Cayman Islands',
		'CR'=>'Costa Rica',
		'CU'=>'Cuba',
		'CW'=>'Curacao',
		'DM'=>'Dominica',
		'DO'=>'Dominican Republic',
		'SV'=>'El Salvador',
		'GL'=>'Greenland',
		'GD'=>'Grenada',
		'GP'=>'Guadeloupe',
		'GT'=>'Guatemala',
		'HT'=>'Haiti',
		'HN'=>'Honduras',
		'JM'=>'Jamaica',
		'MQ'=>'Martinique',
		'MX'=>'Mexico',
		'MS'=>'Montserrat',
		'AN'=>'Netherlands Antilles',
		'NI'=>'Nicaragua',
		'PA'=>'Panama',
		'PR'=>'Puerto Rico',
		'BL'=>'Saint Barthélemy',
		'KN'=>'Saint Kitts and Nevis',
		'LC'=>'Saint Lucia',
		'MF'=>'Saint Martin',
		'PM'=>'Saint Pierre and Miquelon',
		'VC'=>'Saint Vincent and the Grenadines',
		'SX'=>'Sint Maarten',
		'TT'=>'Trinidad and Tobago',
		'TC'=>'Turks and Caicos Islands',
		'VI'=>'U.S. Virgin Islands',
		'AR'=>'Argentina',
		'BO'=>'Bolivia',
		'BR'=>'Brazil',
		'CL'=>'Chile',
		'CO'=>'Colombia',
		'EC'=>'Ecuador',
		'FK'=>'Falkland Islands',
		'GF'=>'French Guiana',
		'GY'=>'Guyana',
		'PY'=>'Paraguay',
		'PE'=>'Peru',
		'SR'=>'Suriname',
		'UY'=>'Uruguay',
		'VE'=>'Venezuela',
		'AS'=>'American Samoa',
		'AU'=>'Australia',
		'CK'=>'Cook Islands',
		'TL'=>'East Timor',
		'FJ'=>'Fiji',
		'PF'=>'French Polynesia',
		'GU'=>'Guam',
		'KI'=>'Kiribati',
		'MH'=>'Marshall Islands',
		'FM'=>'Micronesia',
		'NR'=>'Nauru',
		'NC'=>'New Caledonia',
		'NZ'=>'New Zealand',
		'NU'=>'Niue',
		'NF'=>'Norfolk Island',
		'MP'=>'Northern Mariana Islands',
		'PW'=>'Palau',
		'PG'=>'Papua New Guinea',
		'PN'=>'Pitcairn Islands',
		'WS'=>'Samoa',
		'SB'=>'Solomon Islands',
		'TK'=>'Tokelau',
		'TO'=>'Tonga',
		'TV'=>'Tuvalu',
		'UM'=>'U.S. Minor Outlying Islands',
		'VU'=>'Vanuatu',
		'WF'=>'Wallis and Futuna'
	);
	ksort($countries);
	return $countries;
}
		
function stms_time() {
	$wp_timezone = get_option('timezone_string');
	$timezone = ( !empty($wp_timezone) ) ? $wp_timezone : date_default_timezone_get();
	$new_date = new DateTime('now', new DateTimeZone($timezone));
	$now = $new_date->format('H:i:s');

	return strtotime($now);
}

function stms_retrieve_ipaddr() {
	$unique_ip = trim(getenv('HTTP_X_FORWARDED_FOR'));
	$uv_id = $unique_ip;

	if ( !preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/", $unique_ip) ) {
    		$unique_ip = $_SERVER['REMOTE_ADDR'];
    		$uv_id = $unique_ip;
	}

	if ( !preg_match("/^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$/", $unique_ip) ) {
    		@$uv_id = gethostbyaddr($unique_ip);
	}

	return $uv_id;
}

function stms_drop_third_affiliates( $product_id, $aff_id, $paysys = '' ) {
	if ( !$product = stms_get_product($product_id) )
		return false;
	
	if ( !$aff = stms_member($aff_id) )
		return false;
	
	$stms = get_option('stms_settings');
	$pay_settings = stms_get_paysettings($product->ID);
	$aff_ids = stms_get_member_meta($aff->ID, 'aff_ids');
	if ( !empty($paysys) ) {
		$item = stms_isset($pay_settings[$paysys]['item']);
		$e_aff_id = stms_isset($aff_ids[$paysys]);
		$vendor_id = '';
		if ( $paysys == 'clickbank' )
			$vendor_id = stms_isset($pay_settings[$paysys]['vendor']);
		else if ( $paysys == 'ejunkie' )
			$vendor_id = stms_isset($stms['payments']['ejunkie']['id']);
		
		do_action('stms_drop_affiliate_' . $paysys, $vendor_id, $e_aff_id, $item);
	} else if ( empty($paysys) && $pay_settings ) {
		foreach ( $pay_settings as $key => $val ) {
			$item = stms_isset($pay_settings[$key]['item']);
			$e_aff_id = stms_isset($aff_ids[$key]);
			$vendor_id = '';
			if ( $key == 'clickbank' )
				$vendor_id = stms_isset($pay_settings[$key]['vendor']);
			else if ( $key == 'ejunkie' )
				$vendor_id = stms_isset($stms['payments']['ejunkie']['id']);
			
			do_action('stms_drop_affiliate_' . $key, $vendor_id, $e_aff_id, $item);
		}
	}
}

function stms_error_handler( $error, $member_id = 0, $receipt = '', $type = 'normal', $exit = true ) {
	$log_error = stms_log_error($error, $member_id, $receipt, $type);
	if ( $exit ) exit;
}

function stms_log_error( $error, $member_id = 0, $receipt = '', $type = 'normal' ) {
	global $wpdb;
	if ( empty($error) )
		return false;
	
	$data = array(
		'member_id' => $member_id,
		'receipt' => $receipt,
		'url' => esc_sql($_SERVER['REQUEST_URI']),
		'ip_addr' => stms_retrieve_ipaddr(),
		'referrer' => esc_sql(stms_isset($_SERVER['HTTP_REFERER'])),
		'error' => $error,
		'type' => $type,
		'date' => stms_time()
	);
	$wpdb->insert("{$wpdb->prefix}stms_error_log", $data);
	return true;
}

function stms_save_third_item( $product_id, $item, $vendor = '', $paysys = '' ) {
	global $wpdb;
	
	$table = "{$wpdb->prefix}stms_third_items";
	$data = array(
		'product_id' => $product_id,
		'item_id' => trim($item),
		'vendor_id' => trim($vendor),
		'paysys' => trim($paysys)
	);
			
	$item = $wpdb->get_var($wpdb->prepare("SELECT * FROM `{$table}` WHERE `product_id` = %d AND `paysys` = %s", $product_id, $paysys));
	$item_id = ( $item ) ? $item->ID : 0;
	if ( $item_id  ) { // update
		$wpdb->update($table, $data, array('ID' => $item_id));
	} else {
		$wpdb->insert($table, $data);
		$item_id = $wpdb->insert_id;
	}
	return $item_id;
}

function stms_get_paysettings( $product_id ) {
	global $wpdb;
	
	$meta = stms_get_product_meta($product_id, 'pay_settings');
	if ( !$items = $wpdb->get_results($wpdb->prepare("SELECT * FROM `{$wpdb->prefix}stms_third_items` WHERE `product_id` = %d", $product_id)) )
		return $meta;
	
	$settings = array();
	foreach ( $items as $item ) {
		$settings[$item->paysys] = array(
			'vendor' => stms_sanitize($item->vendor_id),
			'item' => stms_sanitize($item->item_id)
		);
	}
	
	if ( is_array($meta) )
		return array_merge($settings, $meta);
	return $settings;
}

// move 3rd-party payment item ids from meta to table
if ( is_admin() ) add_action('admin_init', 'stms_migrate_items'); else add_action('template_redirect', 'stms_migrate_items');
function stms_migrate_items() {
	global $wpdb;
	$table = "{$wpdb->prefix}stms_third_items";
	if ( $wpdb->get_var("SHOW TABLES LIKE '{$table}'") == $table ) { // check if table exists...
		if ( $products = stms_get_products() ) {
			if ( !($items = $wpdb->get_var("SELECT COUNT(ID) FROM `{$table}` ORDER BY `ID` DESC")) && count($products) > 0 ) {
				foreach ( $products as $product ) {
					$settings = stms_get_product_meta($product->ID, 'pay_settings');
					if ( $settings ) {
						foreach ( $settings as $k => $v ) {
							$item = stms_save_third_item($product->ID, stms_isset($v['item'], ''), stms_isset($v['vendor'], ''), $k);
						}
					}
				}
			}
		}
	}
}
