<?php if ( !defined('STMS_PATH') ) wp_die('Access Denied');
function stms_get_pay_plugins( $type = 'directs' ) {
	$plugin_path = ( $type == 'thirds' ) ? STMS_PAYTHIRDS : STMS_PAYDIRECTS;
	$plugins = array();
	if ( is_dir($plugin_path) ) {
		if ( $plugin_dir = opendir($plugin_path) ) {
			while ( ( $plugin = readdir($plugin_dir) ) !== false ) {
				if ( $plugin != '.' && $plugin != '..' )
					$plugins[] = $plugin;
			}
		}
	}
	
	$payments = false;
	if ( count($plugins) > 0 ) {
		$payments = array();
		for ( $i = 0; $i < count($plugins); $i++ ) {
			$plugin_file = $plugin_path . $plugins[$i];
			if ( $data = stms_get_payment_data($plugin_file) ) {
				$payments[$data['Key']] = $data['Name'];
			}
		}
	}
	
	return $payments;
}

function stms_get_direct_payments() {
	$stms = get_option('stms_settings');
	$directs = stms_get_pay_plugins('directs');
	$processors = array();
	if ( $directs ) {
		foreach ( $directs as $k => $v ) {
			if ( stms_isset($stms['payments']['is_' . $k]) == 1 )
				$processors[$k] = $v;
		}
	}
	return $processors;
}

function stms_get_third_payments() {
	$stms = get_option('stms_settings');
	$directs = stms_get_pay_plugins('thirds');
	$processors = array();
	if ( $directs ) {
		foreach ( $directs as $k => $v ) {
			if ( stms_isset($stms['payments']['is_' . $k]) == 1 )
				$processors[$k] = $v;
		}
		ksort($processors);
	}
	return $processors;
}

add_action('stms_load_payment_files', 'stms_load_payments');
function stms_load_payments() {
	$directs = stms_get_pay_plugins();
	if ( $directs ) {
		foreach ( $directs as $k => $v ) {
			$file = STMS_PAYDIRECTS . "{$k}.php";
			if ( file_exists($file) )
				require_once($file);
		}
	}
	
	$thirds = stms_get_pay_plugins('thirds');
	if ( $thirds ) {
		foreach ( $thirds as $k => $v ) {
			$file = STMS_PAYTHIRDS . "{$k}.php";
			if ( file_exists($file) )
				require_once($file);
		}
	}
}

function stms_get_payment_data( $file_path ) {
	if ( !file_exists($file_path) )
		return false;
	
	$defaults = array(
		'Name' => 'Payment Name',
		'Key' => 'Payment Key',
		'OneTime' => 'One Time',
		'Recurring' => 'Recurring',
		'FlowType' => 'Flow Type'
	);
	return get_file_data($file_path, $defaults);
}

function stms_payment_abilities( $paysys ) {
	$type = stms_gateway_type($paysys);
	if ( !$type )
		return false;
	
	$file_path = ( $type == 'direct' ) ? STMS_PAYDIRECTS . "{$paysys}.php" : STMS_PAYTHIRDS . "{$paysys}.php";
	if ( !$data = stms_get_payment_data($file_path) )
		return false;
	
	return array(
		'onetime' => stms_isset($data['OneTime']),
		'recurring' => stms_isset($data['Recurring']),
		'flow' => stms_isset($data['FlowType']),
	);
}

function stms_log_payment() {
	$log = get_option('stms_payment_log');
	if ( !empty($log['content']) ) return true;
	return false;
}

function stms_other_payments( $member_id, $payer_email, $paysys ) {
	global $wpdb;
	
	$ipns = $wpdb->get_results(
				$wpdb->prepare(
					"SELECT * FROM `{$wpdb->prefix}stms_ipn_log`
					WHERE `paysys` = %s AND `status` = %d
					AND `pay_email` = %s",
					$paysys, 0, $payer_email
				)
			);
			
	if ( !$ipns )
		return false;
	
	$tx_ids = false;
	if ( count($ipns) > 0 ) {
		$tx_ids = array();
		foreach ( $ipns as $ipn ) {
			if ( $p = stms_get_payment($ipn->tx_id) ) {
				$tx_ids[] = $p->tx_id;
				$new_data = array();
				$new_data['member_id'] = $member_id;
				stms_update_payment($p->tx_id, $new_data);
				
				$update = $wpdb->update("{$wpdb->prefix}stms_ipn_log", array('status' => 1, 'member_id' => $member_id), array('log_id' => $ipn->log_id));	
				do_action('stms_payment_completed', $p->tx_id);
				do_action('stms_custom_autoresponder', $p->tx_id); // for custom autoresponder...
			}
		}
	}
	return $tx_ids;
}

function stms_forward_ipns() {
	if ( !function_exists('curl_init') )
		return false;
	
	$stms = get_option('stms_settings');
	if ( isset($stms['payments']['is_forward_ipn']) && $stms['payments']['is_forward_ipn'] == 1 ) {
		if ( !empty($stms['payments']['ipnforward_urls']) ) {
			$vars = ( empty($_POST['cverify']) && !empty($_REQUEST['cverify']) ) ? $_REQUEST : $_POST;
			unset($vars['insta_mode'], $vars['processor'], $vars['insta_id'], $vars['product_id']);
			foreach ( $vars as $k => $v ) {
				if ( stristr($k, 'insta_') )
					unset($vars[$k]);
				if ( stristr($k, 'stms_') )
					unset($vars[$k]);	
			}
			
			if ( !empty($vars) ) {
				$urls = explode("\n", stms_sanitize($stms['payments']['ipnforward_urls']));
				if ( $urls ) {
					foreach ( $urls as $url ) {
						if ( filter_var($url, FILTER_VALIDATE_URL) ) {
							$curl = curl_init();
							curl_setopt($curl, CURLOPT_URL, $url);
							curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
							curl_setopt($curl, CURLOPT_TIMEOUT, 45);
							curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, 0);
							curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
							curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
							curl_setopt($curl, CURLOPT_POST, true);
							curl_setopt($curl, CURLOPT_POSTFIELDS, http_build_query($vars));
							$result = curl_exec($curl);
						}
					}
				}
			}
		}
	}
}

function stms_log_ipn( $vars, $tx_id = 0, $paysys = 'paypal', $status = 1, $member_id = 0, $receipt = '' ) {
	global $wpdb;
	
	$sql = $wpdb->prepare(
				"SELECT COUNT(*) FROM `{$wpdb->prefix}stms_ipn_log`
				WHERE `tx_id` = %d AND `paysys` = %s",
				$tx_id, $paysys
			);
	
	$count = $wpdb->get_var($sql);
	if ( $count > 0 )
		return false;
	
	// get email on transaction ...
	$payer_email = '';
	switch ( $paysys ) {
		case 'wsopro':
		case 'ejunkie':
		case 'oneshopping':
			$payer_email = stms_isset($vars['payer_email']);
			break;
		case 'clickbank':
		case 'dgr':
		case 'jvzoo':
			$payer_email = stms_isset($vars['ccustemail']);
			break;
		case 'nanacast':
			$payer_email = stms_isset($vars['u_email']);
			break;
		case 'productpay':
			$payer_email = stms_isset($vars['user_email']);
			break;
		case 'dealguardian':
			$payer_email = stms_isset($vars['buyer_email']);
			break;
		case 'wsoproa':
			$payer_email = stms_isset($vars['WP_BUYER_EMAIL']);
			break;
	}
	
	$payer_email = apply_filters('stms_payer_email', $payer_email);
	$data = array(
		'tx_id' => $tx_id,
		'member_id' => $member_id,
		'paysys' => $paysys,
		'pay_email' => $payer_email,
		'receipt' => $receipt,
		'status' => $status,
		'postdata' => maybe_serialize($vars),
		'date' => stms_time()
	);
	
	$wpdb->insert("{$wpdb->prefix}stms_ipn_log", $data);
	return $wpdb->insert_id;
}

function stms_get_ipn_log( $paysys, $receipt ) {
	global $wpdb;
	$ipn = $wpdb->get_row(
		"SELECT * FROM `{$wpdb->prefix}stms_ipn_log`
		WHERE `paysys` = '{$paysys}' AND `receipt` = '{$receipt}'"
	);
	if ( !$ipn ) return false;
	return $ipn;
}

function stms_get_ipnurl( $sys = 'clickbank' ) {
	$site = trailingslashit(get_bloginfo('url'));
	return stms_format_url($site, 'insta_mode=ipn&processor=' . $sys);
}

function stms_get_returnurl( $sys = 'clickbank', $alternate = false, $access = false, $product_id = 0 ) {
	$stms = get_option('stms_settings');

	$url = stms_format_url($stms['global']['magic'], 'insta_mode=thanks&processor=' . $sys);
	if ( $alternate )
		$url = stms_format_url($stms['global']['magic'], 'insta_mode=thank_you&processor=' . $sys);
	if ( $access )
		$url = stms_format_url($stms['global']['magic'], 'insta_mode=product_access&processor=' . $sys);

	if ( $product_id > 0 )
		$url .= '&insta_id=' . $product_id;
		
	return $url;
}

function stms_complete_payment( $tx_id, $args ) {
	if ( !$tx_id || !is_array($args) )
		return false;
	
	if ( !$payment = stms_get_payment($tx_id) )
		return false;
	
	global $wpdb;
	$defaults = array(
		'product_id' => $payment->product_id,
		'member_id' => $payment->member_id,
		'aff_id' => $payment->aff_id,
		'payment_type' => $payment->payment_type,
		'paysys' => $payment->paysys,
		'receipt' => $payment->receipt,
		'currency' => $payment->currency,
		'amount' => $payment->amount,
		'fee' => $payment->fee,
		'discount' => $payment->discount,
		'txn_id' => $payment->txn_id,
		'paid' => $payment->paid,
		'is_rebill' => $payment->is_rebill,
		'status' => $payment->status,
		'ipaddr' => $payment->ipaddr,
		'expired_date' => $payment->expired_date,
		'coupon' => ''
	);
	$args = wp_parse_args( $args, $defaults );
	extract($args, EXTR_SKIP);
	
	$tx_id = intval($tx_id);
    if ( $payment->status == 'completed' && $payment->paid == 1 )
		return false;
			
	unset($args['coupon']);
  	$wpdb->update( "{$wpdb->prefix}stms_transactions", $args, compact('tx_id') );
	
	//$is_temp = ( $status == 'temp' ) ? true : false;
	
	//delete caches
	wp_cache_delete($tx_id, 'stms_payments');
	wp_cache_delete($member_id, 'stms_member_payments');

	//if ( !$is_temp )
	do_action('stms_payment_completed', $tx_id);
	
    return $tx_id;
}

function stms_add_pending_payment( $args ) {
	if ( !is_array($args) )
		return false;
	
	$defaults = array(
			'product_id' => 0,
			'member_id' => 0,
			'aff_id' => 0,
			'payment_type' => 'onetime',
			'paysys' => 'manual',
			'receipt' => '',
			'currency' => 'USD',
			'amount' => '0.00',
			'discount' => '0.00',
			'txn_id' => '',
			'paid' => 0,
			'is_rebill' => 0,
			'status' => 'pending',
			'ipaddr' => stms_retrieve_ipaddr(),
			'date' => stms_time(),
			'expired_date' => strtotime('+30 days')
		);
	$args = wp_parse_args( $args, $defaults );
	$tx_id = stms_add_payment($args);
	if ( !$tx_id )
		return false;

	do_action('stms_pending_payment_added', $tx_id);
	return $tx_id;
}

function stms_add_payment($args) {
	global $wpdb;
	$defaults = array(
		'product_id' => 0,
		'member_id' => 0,
		'aff_id' => 0,
		'payment_type' => 'onetime',
		'paysys' => 'manual',
		'receipt' => '',
		'currency' => 'USD',
		'amount' => '0.00',
		'fee' => '0.00',
		'discount' => '0.00',
		'txn_id' => '',
		'paid' => 0,
		'is_rebill' => 0,
		'status' => 'pending',
		'ipaddr' => stms_retrieve_ipaddr(),
		'date' => stms_time(),
		'expired_date' => 0
	);
	$args = wp_parse_args( $args, $defaults );
	extract($args, EXTR_SKIP);
		
    if ( !$product_id )
    	return false;
	
  	$wpdb->insert("{$wpdb->prefix}stms_transactions", $args);
    $tx_id = $wpdb->insert_id;
	
	// delete cache
	wp_cache_delete($member_id, 'stms_member_payments');

	if ( $args['paid'] == 1 && $args['status'] == 'completed' && $args['is_rebill'] == 1 )
		do_action('stms_payment_completed', $tx_id);
		
    return $tx_id;
}

function stms_update_payment($tx_id, $data = array() ) {
	if ( count($data) <= 0 )
		return false;
	
	global $wpdb;
	$wpdb->update( "{$wpdb->prefix}stms_transactions", $data, compact('tx_id') );
	
	// delete caches
	wp_cache_delete($tx_id, 'stms_payments');
	if ( isset($data['member_id']) )
		wp_cache_delete($member_id, 'stms_member_payments');
	
	do_action('stms_payment_updated', $tx_id);
	
    return $tx_id;
}

function stms_delete_payment($tx_id, $redirect = true ) {
	$tx_id = (int) $tx_id;
	if ( !$payment = stms_get_payment($tx_id) ) {
		if ( $redirect ) {
			@header( 'Location:' . admin_url('admin.php?page=stms-settings') . '#members|mode=payments&object_ID=' . $member_id );
			exit;
		} else
			return false;
	}
	
	global $wpdb;
	
	do_action('stms_payment_delete', $tx_id);
	
	$member_id = $payment->member_id;
	$wpdb->delete("{$wpdb->prefix}stms_transactions", compact('tx_id'));
	
	// void affiliate commission
	stms_void_affiliate_commission($tx_id);
	
	//delete caches
	wp_cache_delete($tx_id, 'stms_payments');
	wp_cache_delete($member_id, 'stms_member_payments');

	do_action('stms_payment_deleted', $tx_id);
	
	if ( $redirect ) {
		@header( 'Location:' . admin_url('admin.php?page=stms-settings') . '#members|mode=payments&object_ID=' . $member_id . '&payment=deleted' );
		exit;
	}
	return true;
}

function stms_delete_old_payments() {
	global $wpdb;
	$old_time = strtotime("7 days ago", stms_time());
	if ( $payments = $wpdb->get_results("SELECT * FROM `{$wpdb->prefix}stms_transactions` WHERE `status` = 'temp' AND `date` <= {$old_time}") ) {
		foreach ( $payments as $py ) {
			$wpdb->delete("{$wpdb->prefix}stms_transactions", array('tx_id' => $py->tx_id));
			
			//delete caches
			wp_cache_delete($py->tx_id, 'stms_payments');
			wp_cache_delete($py->member_id, 'stms_member_payments');
		}
	}
}

function stms_get_firstpayment( $member_id, $product_id ) {
	global $wpdb;
	
	$member_id = (int) $member_id;
	$product_id = (int) $product_id;
	$first_payment = $wpdb->get_row(
		$wpdb->prepare("SELECT * FROM `{$wpdb->prefix}stms_transactions`
			WHERE `product_id` = %d AND `member_id` = %d
			ORDER BY tx_id ASC",
			$product_id,
			$member_id
		)
	);
	return $first_payment;
}

function stms_get_lastpayment( $member_id, $product_id ) {
	global $wpdb;
	
	$member_id = (int) $member_id;
	$product_id = (int) $product_id;
	$first_payment = $wpdb->get_row(
		$wpdb->prepare("SELECT * FROM `{$wpdb->prefix}stms_transactions`
			WHERE `product_id` = %d AND `member_id` = %d AND `status` = %s
			ORDER BY tx_id DESC",
			$product_id,
			$member_id,
			'completed'
		)
	);
	return $first_payment;
}

function stms_get_payment( $tx_id ) {
	if ( empty($tx_id) )
		return false;

	global $wpdb;
	$tx_id = (int) $tx_id;
	$payment = wp_cache_get($tx_id, 'stms_payments');
	if ( !$payment ) {
	    $payment = $wpdb->get_row("SELECT
			tx.*, m.username, p.title
			FROM {$wpdb->prefix}stms_transactions tx
			LEFT JOIN {$wpdb->prefix}stms_members m ON (m.ID = tx.member_id)
			LEFT JOIN {$wpdb->prefix}stms_products p ON (p.ID = tx.product_id)
			WHERE tx.tx_id = {$tx_id}");
	
		if ( !$payment )
			return false;
		
		wp_cache_add($tx_id, $payment, 'stms_payments');
	}
	
	// process expired payment...
	if ( $payment->payment_type == 'recurr' ) {
		$now = stms_time();
		$expired = $payment->expired_date;
		if ( $now > $expired ) {
			$wpdb->update("{$wpdb->prefix}stms_transactions", array('status' => 'expired'), compact('tx_id'));
			$payment->status = 'expired';
			wp_cache_set($tx_id, $payment, 'stms_payments');
		}
	}
	
	return $payment;
}

function stms_get_payment_byreceipt( $paysys, $receipt ) {
	global $wpdb;
    $payment = $wpdb->get_row(
    				"SELECT * FROM {$wpdb->prefix}stms_transactions
    				WHERE paysys = {$paysys} AND receipt = '{$receipt}'
    				AND is_rebill = 0"
    			);

	if ( !$payment )
		return false;
		
	return $payment;
}

function stms_count_member_payments($member_id) {
	if ( empty($member_id) )
		return false;
	
	global $wpdb;
	
	$id = (int) $member_id;
	$count = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM `{$wpdb->prefix}stms_transactions` WHERE `member_id` = %d", $id));
	
	return $count;
}

function stms_get_member_payments( $member_id ) {
	global $wpdb;
	
	$member_id = (int) $member_id;
	$payments = wp_cache_get($member_id, 'stms_member_payments');
	if ( !$payments ) {
		$payments = $wpdb->get_results("SELECT
			tx.*, p.title
			FROM {$wpdb->prefix}stms_transactions tx
			LEFT JOIN {$wpdb->prefix}stms_products p ON (p.ID = tx.product_id)
			WHERE tx.member_id = {$member_id} ORDER BY tx.date DESC"
		);
		
		if ( !$payments )
			return false;
		
		wp_cache_add($member_id, $payments, 'stms_member_payments');
	}

	return $payments;
}

function stms_get_member_access( $member_id ) {
	global $wpdb;
	
	$member_id = (int) $member_id;
	$payments = $wpdb->get_results("SELECT
		tx.*, p.title, p.home_id
		FROM {$wpdb->prefix}stms_transactions tx
		LEFT JOIN {$wpdb->prefix}stms_products p ON (p.ID = tx.product_id)
		WHERE tx.member_id = {$member_id} AND tx.is_rebill <> 1
		ORDER BY p.title ASC"
	);
		
	if ( !$payments )
		return false;

	return $payments;
}

function stms_member_has_access( $member_id, $product_id ) {
	global $wpdb;
	
	$member_id = intval($member_id);
	$product_id = intval($product_id);
	$payment = $wpdb->get_row("SELECT * FROM `{$wpdb->prefix}stms_transactions`
		WHERE `member_id` = {$member_id} AND `product_id` = {$product_id}
		AND `paid` = 1 ORDER BY `tx_id` DESC LIMIT 1");
	
	if ( !$payment )
		return false;
	
	if ( $payment->status == 'refunded' && $payment->status == 'expired' )
		return false;
	
	$now = stms_time();
	$expired = $payment->expired_date;
	if ( $payment->payment_type == 'recurr' && ($now > $expired) )
		return false;
	
	return true;
}

function stms_refund_payment( $tx_id, $gateway = 'direct' ) {
	$tx_id = (int) $tx_id;
	if ( !$payment = stms_get_payment($tx_id) )
		return false;
		
	$new_data = array();
	$new_data['status'] = 'refunded';
	stms_update_payment($tx_id, $new_data );
	wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
	
	if ( $gateway == 'direct' )
		stms_void_affiliate_commission($tx_id);
}

add_action('stms_payment_completed', 'stms_add_affiliate_commission');
function stms_add_affiliate_commission( $tx_id ) {
	global $wpdb;

	$stms = get_option('stms_settings');
    $payment = stms_get_payment($tx_id);
    if ( !$payment = stms_get_payment($tx_id) )
    	return false;

	$cust = stms_member($payment->member_id);
	$aff_id = $payment->aff_id;
	if ( !is_stms_affiliate($aff_id) || $cust->status == 4 )
		return false;
	
	// check if commission for this tx has been processed...
	$chk = $wpdb->get_var($wpdb->prepare("SELECT COUNT(*) FROM `{$wpdb->prefix}stms_aff_commissions` WHERE tx_id = %d", $payment->tx_id));	
	$gateway_type = stms_gateway_type($payment->paysys);
	if ( !$aff_id || $payment->member_id == $aff_id || $gateway_type == 'third' || $chk > 0 )
		return false;

	$amount = $payment->amount;
	if ( $payment->paysys == 'paypal' || $payment->paysys == 'payza' ) {
		if ( stms_isset($stms['global']['fee_applied']) == 1 ) {
			$amount = @($amount - stms_isset($payment->fee, 0.00));
		}
	}
	
	if ( $amount <= 0.00 )
		return false;
	
	$commission = stms_calculate_commission($amount, $payment->product_id, $aff_id);
	if ( $commission <= 0 )
		return false;
	
	// Calculate Pay Date...
	$product = stms_get_product($payment->product_id);
	$refund = stms_isset($product->info['refund'], 30);
	$pay_date = strtotime("+{$refund} days", stms_time());
	
	$data = array(
		'aff_id' => $aff_id,
		'tx_id' => $payment->tx_id,
		'product_id' => $payment->product_id,
		'receipt' => $payment->receipt,
		'commission' => doubleval($commission),
		'tier'  => 1,
		'void'  => 0,
		'status' => 0,
		'pay_date' => $pay_date,
		'date' => stms_time()
	);
	$wpdb->insert("{$wpdb->prefix}stms_aff_commissions", $data);
	do_action('stms_aff_commissioned_1', $payment->tx_id, $aff_id, $commission);
	
	// is two tier...
	if ( stms_isset($product->info['is_twotier']) == 1 ) {
		$aff_meta = stms_get_member_meta($aff_id, 'aff_id');
		if ( !empty($aff_meta) && $aff_id != $aff_meta ) {
			if ( !is_stms_affiliate($aff_meta) )
				return false;
			
			$comm = stms_calculate_commission($amount, $payment->product_id, $aff_meta, true);
			if ( $comm <= 0 )
				return false;
			
			$data = array(
				'aff_id' => $aff_meta,
				'tx_id' => $payment->tx_id,
				'product_id' => $payment->product_id,
				'receipt' => $payment->receipt,
				'commission' => doubleval($comm),
				'tier'  => 2,
				'void'  => 0,
				'status' => 0,
				'pay_date' => $pay_date,
				'date' => stms_time()
			);
			$wpdb->insert("{$wpdb->prefix}stms_aff_commissions", $data);
			do_action('stms_aff_commissioned_2', $payment->tx_id, $aff_meta, $comm);
		}
	}
	return true;
}

function stms_void_affiliate_commission( $tx_id ) {
	global $wpdb;

	// process tier 1 commission...
	$tier1 = false;
	if ( $comm = stms_get_affcommission($tx_id) )
		$tier1 = true;
	
	if ( $tier1 ) {
		$data = array(
			'aff_id' => $comm->aff_id,
			'tx_id' => $comm->tx_id,
			'product_id' => $comm->product_id,
			'receipt' => $comm->receipt,
			'commission' => $comm->commission,
			'tier'  => 1,
			'void'  => 1,
			'status' => 0,
			'pay_date' => 0,
			'date' => stms_time()
		);
		
		if ( $chk = stms_get_affcommission($tx_id, 'void') )
			$wpdb->update("{$wpdb->prefix}stms_aff_commissions", $data, array('ID' => $comm->ID));
		else
			$wpdb->insert("{$wpdb->prefix}stms_aff_commissions", $data);
		
		do_action('stms_aff_tier1_voided', $tx_id);
	}
	
	// process tier 2 commission...
	$tier2 = false;
	if ( $comm = stms_get_affcommission($tx_id, '', 2) )
		$tier2 = true;
	
	if ( $tier2 ) {
		$data = array(
			'aff_id' => $comm->aff_id,
			'tx_id' => $comm->tx_id,
			'product_id' => $comm->product_id,
			'receipt' => $comm->receipt,
			'commission' => $comm->commission,
			'tier'  => 2,
			'void'  => 1,
			'status' => 0,
			'pay_date' => 0,
			'date' => stms_time()
		);
		if ( $chk = stms_get_affcommission($tx_id, 'void', 2) )
			$wpdb->update("{$wpdb->prefix}stms_aff_commissions", $data, array('ID' => $comm->ID));
		else
			$wpdb->insert("{$wpdb->prefix}stms_aff_commissions", $data);
		do_action('stms_aff_tier2_voided', $tx_id);
	}
	
	if ( !$tier1 && !$tier2 ) return false;
    return true;
}

function stms_get_affcommission( $tx_id, $type = '', $tier = 'all', $status = 'all' ) {
	global $wpdb;
	$tx_id = (int) $tx_id;
	$cache = ( $type == 'void' ) ? $tx_id . '_void_' . $tier : $tx_id . '_' . $tier;
	$comm = wp_cache_get($cache, 'stms_aff_commission');
	if  ( !$comm ) {
		$void = ( $type == 'void' ) ? intval(1) : intval(0);
		$sql = "SELECT * FROM `{$wpdb->prefix}stms_aff_commissions` WHERE `tx_id` = {$tx_id} AND `void` = {$void}";
		
		if ( $tier != 'all' ) {
			$tier = intval($tier);
			$sql .= " AND `tier` = {$tier}";
		}
		
		if ( $status != 'all' ) {
			$status = intval($status);
			$sql .= " AND `status` = {$status}";
		}
		
		if ( !$comm = $wpdb->get_row($sql) )
			return false;
		
		// add to cache...
		wp_cache_add($cache, $comm, 'stms_aff_commission');
	}
	return $comm;
}

function stms_calculate_commission($amount, $product_id, $aff_id, $two_tier = false ) {
	if ( !$product = stms_get_product($product_id) )
		return false;
	
	$exclusive = stms_get_member_meta($aff_id, 'exclusive_aff');
	if ( $two_tier )
		$percent = ( $exclusive == 1 ) ? stms_isset($product->info['s_aff_comm2']) : stms_isset($product->info['aff_comm2']);
	else
		$percent = ( $exclusive == 1 ) ? stms_isset($product->info['s_aff_comm']) : stms_isset($product->info['aff_comm']);
	$commission = @( ($amount * $percent) / 100 );
	return $commission;
}

function stms_get_commissions( $member_id = 0, $start = 0, $end = 0 ) {
	global $wpdb;

	$sql = "SELECT ac.*, p.title, m.username FROM {$wpdb->prefix}stms_aff_commissions ac
			LEFT JOIN {$wpdb->prefix}stms_products p ON (p.ID = ac.product_id)
			LEFT JOIN {$wpdb->prefix}stms_members m ON (m.ID = ac.aff_id)
			WHERE 1";
			
	if ( $member_id )
		$sql .= " AND ac.aff_id = {$member_id}";
	
	if ( $start && !$end )
		$sql .= " AND ac.date >= {$start}";
	
	if ( !$start && $end )
		$sql .= " AND ac.date <= {$end}";
	
	if ( $start && $end )
		$sql .= " AND ac.date BETWEEN {$start} AND {$end}";
	
	$sql .= " ORDER BY ac.ID, ac.void DESC";
	if ( !$comms = $wpdb->get_results($sql) )
		return false;

	return $comms;
}

function stms_member_affstats( $member_id, $start, $end ) {
	global $wpdb;
	
	if ( empty($member_id) )
		return false;
	
	$start = strtotime($start, stms_time());
	$end = strtotime($end . " 23:59", stms_time());
	
	$member_id = intval($member_id);
	$comm_sql = "SELECT cmt.aff_id, cmt.product_id,
					(SELECT SUM(commission) FROM {$wpdb->prefix}stms_aff_commissions
						WHERE void = 0 AND product_id = cmt.product_id AND aff_id = cmt.aff_id
						AND date BETWEEN {$start} AND {$end})
					AS commissions,
					(SELECT SUM(commission) FROM {$wpdb->prefix}stms_aff_commissions
						WHERE void = 1 AND product_id = cmt.product_id AND aff_id = cmt.aff_id
						AND date BETWEEN {$start} AND {$end})
					AS corrections
				FROM {$wpdb->prefix}stms_aff_commissions cmt WHERE cmt.aff_id = {$member_id} GROUP BY cmt.product_id";
				
	$sales_sql = "SELECT sl.aff_id, sl.product_id,
					(SELECT COUNT(*) FROM {$wpdb->prefix}stms_transactions
						WHERE paid = 1 AND aff_id = sl.aff_id
						AND product_id = sl.product_id AND date BETWEEN {$start} AND {$end})
					AS sales,
					(SELECT COUNT(*) FROM {$wpdb->prefix}stms_transactions
						WHERE paid = 1 AND status = 'refunded' AND aff_id = sl.aff_id
						AND product_id = sl.product_id AND date BETWEEN {$start} AND {$end})
					AS refund
				FROM {$wpdb->prefix}stms_transactions sl
				WHERE sl.aff_id = {$member_id} GROUP BY sl.product_id";
	$clicks_sql = "SELECT aff_id, product_id, COUNT(*) AS rawclicks, COUNT(DISTINCT ip_addr) AS clicks FROM {$wpdb->prefix}stms_aff_clicks WHERE aff_id = {$member_id} AND date BETWEEN {$start} AND {$end} GROUP BY product_id";
	
	$sql = "SELECT p.title, p.ID, p.product_status,
				IFNULL(cm.commissions, 0) as commissions,
				IFNULL(cm.corrections, 0) as corrections,
				IFNULL(tx.sales, 0) as sales,
				IFNULL(tx.refund, 0) as refund,
				IFNULL(c.rawclicks, 0) as rawclicks,
				IFNULL(c.clicks, 0) as clicks
			FROM {$wpdb->prefix}stms_products p
			LEFT JOIN ({$comm_sql}) cm ON (cm.product_id = p.ID)
			LEFT JOIN ({$sales_sql}) tx ON (tx.product_id = p.ID)
			LEFT JOIN ({$clicks_sql}) c ON (c.product_id = p.ID)
			WHERE p.product_status <> 4 GROUP BY p.ID, p.title ORDER BY p.title ASC";
			
	if ( !$results = $wpdb->get_results($sql) )
		return false;
		
	return $results;
}


function stms_affiliate_summary( $member_id ) {
	global $wpdb;
	
	if ( empty($member_id) )
		return false;
	
	$stms = get_option('stms_settings');
	$min_pay = stms_isset($stms['global']['min_payout'], 10.00);
	$aff_id = intval($member_id);
	$defaults = array(
		'all_commissions' => 0.00,
		'all_voids' => 0.00,
		'payouts' => 0.00,
		'paid' => 0.00,
		'new_comms' => 0.00,
		'new_voids' => 0.00,
		'balance' => 0.00,
		'available' => 0.00,
		'pending' => 0.00
	);
	
	$today = strtotime("today 23:59", stms_time());
	$sql = "SELECT
				aff_id,
					IFNULL((SELECT SUM(commission) FROM {$wpdb->prefix}stms_aff_commissions
					WHERE void = 0 AND aff_id = acm.aff_id), 0.00)
				AS commissions,
				 	IFNULL((SELECT SUM(commission)
					FROM {$wpdb->prefix}stms_aff_commissions
					WHERE void = 1 AND aff_id = acm.aff_id), 0.00)
				AS voids,
					IFNULL(IFNULL((SELECT SUM(commission) FROM {$wpdb->prefix}stms_aff_commissions
					WHERE status = 1 AND void = 0 AND aff_id = acm.aff_id), 0.00) - IFNULL((SELECT SUM(commission)
					FROM {$wpdb->prefix}stms_aff_commissions WHERE status = 1 AND void = 1
					AND aff_id = acm.aff_id), 0.00), 0.00)
				AS paid,
					IFNULL((SELECT SUM(commission) FROM {$wpdb->prefix}stms_aff_commissions
					WHERE status = 0 AND void = 0 AND aff_id = acm.aff_id), 0.00)
				AS gross,
					IFNULL((SELECT SUM(commission) FROM {$wpdb->prefix}stms_aff_commissions
					WHERE status = 0 AND void = 0 AND aff_id = acm.aff_id
					AND pay_date <= {$today}), 0.00)
				AS unpaid,
					IFNULL((SELECT SUM(commission)
					FROM {$wpdb->prefix}stms_aff_commissions
					WHERE status = 0 AND void = 1 AND aff_id = acm.aff_id), 0.00)
				AS new_voids
			FROM {$wpdb->prefix}stms_aff_commissions acm
			WHERE acm.aff_id = $aff_id GROUP BY acm.aff_id";
	
	if ( $result = $wpdb->get_row($sql) ) {
		$payouts = number_format($result->commissions - $result->voids, 2);
		$balance = number_format($result->gross - $result->new_voids, 2);
		$available = number_format($result->unpaid - $result->new_voids, 2);
		$available = ( $available >= $min_pay ) ? $available : '0.00';
		$pending = number_format($balance - $available, 2);
		$summary = array(
			'all_commissions' => $result->commissions,
			'all_voids' => $result->voids,
			'payouts' => $payouts,
			'paid' => $result->paid,
			'new_comms' => $result->gross,
			'new_voids' => $result->new_voids,
			'balance' => $balance,
			'available' => $available,
			'pending' => $pending
		);
		
		$defaults = array_merge($defaults, $summary);
	}
	
	return $defaults;
}

function stms_generate_receipt($product_id, $is_rebill = false) {
	global $wpdb;
	
	$product = stms_get_product($product_id);
	$receipt = ( stms_isset($product->type) == 'onetime' ) ? 'P-' : ( ($is_rebill) ? 'R-' : 'S-');
	$receipt .= date("ymd", stms_time());
	$receipt .= stms_generate_secret($length = 8, true, true, false );
	
	return $receipt;
}

function stms_get_expired( $product_id, $first_bill = true ) {
	if ( !$product = stms_get_product($product_id) )
		return false;
	
	if ( $product->type == 'recurr' ) {
		$is_trial = ( $first_bill && $product->is_trial == 1 ) ? true : false;
		$dur = ( $is_trial ) ? maybe_unserialize($product->t_duration) : maybe_unserialize($product->duration);
		$num = ( isset($dur['num']) ) ? $dur['num'] : 1;
		$mode = ( isset($dur['mode']) ) ? $dur['mode'] : 'd';
		$str_time = stms_duration_str($num, $mode);
		return strtotime($str_time);
	}
	/*
	else if ( $product->type == 'onetime' ) {
		if ( empty($product->info['ot_expired']) ) return intval(0);
		$num = ( isset($product->info['ot_expnum']) ) ? $product->info['ot_expnum'] : 1;
		$mode = ( isset($product->info['ot_exptype']) ) ? $product->info['ot_exptype'] : 'y';
		$str_time = stms_duration_str($num, $mode);
		return strtotime($str_time);
	}
	*/ 
	return intval(0);
}

function stms_duration_str( $num, $mode, $use_num = true ) {
	if ( $mode == 'd' )
		$m = 'day';
	else if ( $mode == 'w' )
		$m = 'week';
	else if ( $mode == 'm' )
		$m = 'month';
	else if ( $mode == 'y' )
		$m = 'year';
	
	$str = '';
	if ( $use_num )
		$str .= "+{$num} ";
	$str .= "{$m}";
	if ( $num > 1 )
		$str .= 's';
	
	return $str;
}

function stms_payment_get_lock( $tx_id ) {
	register_shutdown_function('stms_payment_release_lock', $tx_id);
	set_transient('stms_lock_payment_' . $tx_id, 'locked', 30);
}

function stms_payment_release_lock( $tx_id ) {
    delete_transient( 'stms_lock_payment_' . $tx_id );
}

function stms_is_payment_locked($tx_id) {
	if ( get_transient( 'stms_lock_payment_' . $tx_id ) == 'locked' )
		return true;
	return false;
}

function stms_get_processors( $product, $third = true ) {
	if ( !is_object($product) || !isset($product->info) )
		return false;
	
	$pay_direct = ( is_array($product->info['direct_pay']) ) ? $product->info['direct_pay'] : array();
	if ( !$third )
		return $pay_direct;
	
	$pay_third = ( is_array($product->info['third_pay']) ) ? $product->info['third_pay'] : array();
	return array_merge($pay_direct, $pay_third);
}

function stms_gateway_type( $paysys ) {
	if ( $paysys == 'global' )
		return 'global';
	
	$direct_pays = stms_get_direct_payments();
	if ( isset($direct_pays[$paysys]) )
		return 'direct';
	
	$third_pays = stms_get_third_payments();
	if ( isset($third_pays[$paysys]) )
		return 'third';
	
	return false;
}

function stms_getpaymentname( $key ) {
	if ( empty($key) )
		return 'Unknown';
	
	$direct_pays = stms_get_direct_payments();
	$third_pays = stms_get_third_payments();
	$pay_direct = ( is_array($direct_pays) ) ? $direct_pays : array();
	$pay_third = ( is_array($third_pays) ) ? $third_pays : array();
	$pay_names = array_merge($pay_direct, $pay_third);
	if ( isset($pay_names[$key]) )
		return stms_cleanup($pay_names[$key]);
	
	return 'Unknown';
}

// -- PAYMENT BUTTONS --
add_action('template_redirect', 'stms_payment_buttons', 1);
function stms_payment_buttons() {
	$process_pay = false;
	$pay_method = 'normal';
	if ( !empty($_REQUEST['insta_pay']) && !empty($_REQUEST['id']) )
		$process_pay = true;
	
	if ( stms_isset($_GET['insta_mode']) == 'order' && !empty($_GET['id']) ) {
		$process_pay = true;
		$pay_method = 'auto';
	}
	
	if ( !$process_pay ) return;
	$stms = get_option('stms_settings');
	$product_id = (int) $_GET['id'];
	if ( !$product = stms_get_product($product_id) )
		wp_die('ERROR: Product Not Found.');
	
	if ( isset($_GET['insta_pay']) && $_GET['insta_pay'] == 'free' ) {
		$session = base64_encode("{$product->ID}|free|direct");
		$_reg_url = stms_get_handler_url('registration');
		$reg_url = stms_format_url($_reg_url, 'key=' . $session);
		
		if ( isset($_GET['fl']) ) {
			$flow_id = urldecode($_GET['fl']);
			do_action('insta_upsells_front', $flow_id, $product->ID, $_GET['insta_pay']);
		}
		
		wp_redirect($reg_url);
		exit;
	}
	
	if ( $pay_method == 'auto' && !isset($_COOKIE['__insta_processor']) ) {
		$default_paysys = 'unknown';
		if ( is_array($product->info['direct_pay']) && count($product->info['direct_pay']) > 0 ) {
			foreach ( $product->info['direct_pay'] as $k => $v ) {
				if ( !empty($k) ) {
					$default_paysys = $k;
					break;
				}
			}
		}
		$_COOKIE['__insta_processor'] = $default_paysys;
	}
	
	$processor = ( $pay_method == 'auto' ) ? $_COOKIE['__insta_processor'] : $_GET['insta_pay'];
	$gateway = stms_gateway_type($processor);
	if ( !$gateway )
		wp_die('ERROR: Unknown Payment System.');
	
	if ( $gateway == 'third' )
		wp_die('ERROR: Invalid Payment System.');
	
	do_action('insta_payment_clicked', $product->ID, $processor);
	
	// upsell action for certain payment systems
	if ( isset($_GET['fl']) ) {
		$flow_id = urldecode($_GET['fl']);
		do_action('insta_upsells_front', $flow_id, $product->ID, $processor);
	}
	
	if ( !isset($_GET['fl']) && isset($_COOKIE['__insta_flow']) ) {
		$flow_id = urldecode($_COOKIE['__insta_flow']);
		do_action('insta_upsells_cart', $flow_id, $product->ID, $processor);
	}

	if ( !isset($product->info['direct_pay'][$processor]) )
		wp_die('ERROR: Invalid Payment System.');
	
	$steps = stms_isset($product->info['steps']);
	if ( $processor == 'authorize' )
		$steps = '1,2'; // force to register first if it's Authorize.net
	
	// let's find out if there's any affiliate...
	$aff_id = 0;
	if ( isset($_COOKIE['__insta_aff_id']) )
		$aff_id = (int) $_COOKIE['__insta_aff_id']; // there's aff cookie
	
	$payabs = stms_payment_abilities($processor);
	if ( $steps == '2,1' ) {
		$is_autoreg = ( stms_isset($product->info['auto_register']) == 1 ) ? true : false;
		// Direct to payment proccessing...
		if ( is_stmsmember_logged_in() ) {
			$current_member = stms_get_current_member();
			$member_id = $current_member->ID;
			$acc_type = ( $current_member->status == 4 ) ? 'temp' : 'real';
		} else {
			// let's create a temporary member account...
			if ( !$member_id = stms_create_temp_member() )
				wp_die('ERROR: Order Failed.');
			
			// logging in user
			stms_set_auth_cookie($member_id);
			$acc_type = 'temp';
		}
		
		$currency = stms_isset($stms['global']['currency'], 'USD');
		$normal_price = ( $product->type == 'recurr' && $product->is_trial == 1 ) ? $product->t_price : $product->price;
		$amount = apply_filters('insta_total_checkout', $normal_price, $normal_price, $currency, $product->ID);
		$discount = number_format($normal_price - $amount, 2);
		
		// Create pending payment...
		$args = array(
			'product_id' => $product->ID,
			'member_id' => $member_id,
			'aff_id' => $aff_id,
			'payment_type' => $product->type,
			'paysys' => $processor,
			'receipt' => '',
			'currency' => $currency,
			'amount' => $amount,
			'discount' => $discount,
			'txn_id' => '',
			'paid' => 0,
			'is_rebill' => 0,
			'status' => 'temp',
			'date' => stms_time(),
			'expired_date' => stms_get_expired($product->ID)
		);
		$tx_id = stms_add_pending_payment($args);
		if ( !$tx_id )
			wp_die('ERROR: Order Failed.');
		
		// create check data so payment can be checked on thank you page...
		$check_data = array(
			'tx_id' => $tx_id,
			'member_id' => $member_id,
			'acc_type' => $acc_type,
			'amount' => $amount,
			'discount' => $discount
		);
		set_transient('stms_payment_check_' . $tx_id, $check_data, 60*60);
		@setcookie('__insta_payment', $tx_id, time()+3600, SITECOOKIEPATH);
		if ( SITECOOKIEPATH != COOKIEPATH )
			@setcookie('__insta_payment', $tx_id, time()+3600, COOKIEPATH);
		
		// proceed to payment...
		stms_payment_proceed($tx_id, $processor);
	} else {
		// Redirect to registration URL...
		$session = base64_encode("{$product_id}|{$processor}|{$gateway}");
		$_reg_url = stms_get_handler_url('registration');
		$reg_url = stms_format_url($_reg_url, 'key=' . $session);
		wp_redirect($reg_url);
	}
	exit;
}

add_action('template_redirect', 'stms_payment_redirect', 1);
function stms_payment_redirect() {
	$display = false;
	if ( isset($_GET['insta_mode']) && $_GET['insta_mode'] == 'payment_redirect' && isset($_GET['trans_id']) )
		$display = true;
	
	if ( !$display )
		return false;
	
	$tx_id = intval(stms_isset($_REQUEST['trans_id']));
	if ( !$payment = stms_get_payment($tx_id) )
		return false;

	do_action('stms_payment_redirect_begin', $payment);
?>
<!DOCTYPE html>
<!--[if IE 6]>
<html id="ie6" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 7]>
<html id="ie7" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 8]>
<html id="ie8" <?php language_attributes(); ?>>
<![endif]-->
<!--[if !(IE 6) | !(IE 7) | !(IE 8)  ]><!-->
<html <?php language_attributes(); ?>>
<!--<![endif]-->
<head>
<meta charset="<?php bloginfo( 'charset' ); ?>" />
<title>Secure Payment Redirection ...</title>
<!--[if lt IE 9]>
	<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->
<link rel="profile" href="http://gmpg.org/xfn/11" />
<meta name="HandheldFriendly" content="True" />
<meta name="viewport" content="width=device-width, initial-scale=1"/>
<!-- Adding "maximum-scale=1" fixes the Mobile Safari auto-zoom bug: http://filamentgroup.com/examples/iosScaleBug/ -->
<link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Droid+Sans" type="text/css" media="all" />
<?php do_action('stms_payment_redirect_head', $payment); ?>
<style type="text/css">
body {
	margin:0;
	padding:0;
	font-size:14px;
	font-family:'Droid Sans', sans-serif;
	background-color:#F5F5F5;
	color:#808080;
}
#redirect_msg {
	margin:25px auto 0;
	padding:15px 20px;
	width:40%;
	background-color:#FFF;
	text-align:center;
}
</style>
</head>
<body>
<div id="redirect_msg">
	<p><strong>Please wait while we redirect you to a secure payment page...</strong></p>
	<p><img src="<?php echo STMS_URL; ?>images/admin-loader.gif" border="0" /></p>
	<?php do_action('stms_payment_redirect_body', $payment); ?>
</div>
<?php do_action('stms_payment_redirect_footer', $payment); ?>
</body>
</html>
<?php
	exit;
}

add_action('template_redirect', 'stms_payment_notification', 1);
function stms_payment_notification() {
	if ( stms_isset($_GET['insta_mode']) == 'ipn' && stms_isset($_GET['processor']) != '' ) {
		stms_forward_ipns();
		do_action('stms_ipn_' . $_GET['processor']);
		exit;
	}
}

function stms_payment_proceed( $tx_id, $paysys = 'paypal' ) {
	// let's call the function
	do_action('stms_payment_proceed_' . $paysys, $tx_id);
}

add_action('template_redirect', 'stms_payment_thanks', 1);
function stms_payment_thanks() {
	if ( stms_isset($_GET['insta_mode']) == 'thanks' && stms_isset($_GET['processor']) != '' ) {
		$member_url = stms_member_url();
		if ( !isset($_COOKIE['__insta_payment']) ) {
			// simply redirect them to member home page...
			wp_redirect($member_url);
			exit;
		}
	
		$tx_id = (int) $_COOKIE['__insta_payment'];
		if ( !$payment = stms_get_payment($tx_id) )
			wp_die('ERROR: Cannot Find Payment Data.');
		
		$stms = get_option('stms_settings');
		$support = stms_get_handler_url('support');
		if ( $payment->status == 'completed' && $payment->paid == 1 ) {
			do_action('stms_custom_autoresponder', $payment->tx_id); // for custom autoresponder...
			// Payment success... proceed to next step
			if ( !$product = stms_get_product($payment->product_id) )
				wp_die('ERROR: Payment cannot be processed. Product Unknown.');
			
			@setcookie('__insta_payment', 0, time()-84600, SITECOOKIEPATH); // delete payment cookie
			@setcookie('__insta_payment', 0, time()-84600, COOKIEPATH);
			
			$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_permalink($stms['global']['home']);
			$redirect_url = ( !empty($product->info['thankuri']) ) ? $product->info['thankuri'] : $home_url;
			
			delete_transient('stms_thanks_retries_' . $tx_id);
			if ( $product->info['steps'] == '2,1' ) {
				$check = get_transient('stms_payment_check_' . $tx_id);
				if ( stms_isset($check['acc_type']) == 'real' ) {
					if ( isset($_COOKIE['__insta_flow']) ) {
						$flow_id = urldecode($_COOKIE['__insta_flow']);
						do_action('insta_upsells_redirect', $flow_id, $product->ID, $payment->paysys, $payment->member_id);
					}

					wp_redirect($redirect_url);
					exit;
				}
				
				if ( stms_isset($product->info['auto_register']) == 1 ) {
					$temp_member_id = stms_get_current_member_id();
					if ( $temp_member_id != $payment->member_id ) {
						stms_delete_member($temp_member_id, false);
					}
					stms_clear_auth_cookie(); // logged out the member first...
					stms_set_auth_cookie($payment->member_id); // re-logging in member...
					if ( isset($_COOKIE['__insta_flow']) ) {
						$flow_id = urldecode($_COOKIE['__insta_flow']);
						do_action('insta_upsells_redirect', $flow_id, $product->ID, $payment->paysys, $payment->member_id);
					}
					wp_redirect($redirect_url);
					exit;
				}
				
				if ( isset($_COOKIE['__insta_flow']) ) {
					$flow_id = urldecode($_COOKIE['__insta_flow']);
					do_action('insta_upsells_redirect', $flow_id, $product->ID, $payment->paysys, $payment->member_id);
				}
				
				// proceed to registration
				$gateway = stms_gateway_type($payment->paysys);
				$session = base64_encode("{$product->ID}|{$payment->paysys}|{$gateway}");
				$insta_token = stms_generate_secret(16, true, true);
				$token_data = array('token' => $insta_token, 'tx_id' => $payment->tx_id);
				set_transient('insta_token_' . $insta_token, $token_data, 21600); // lasted for six hours
				
				if ( is_stmsmember_logged_in() ) stms_clear_auth_cookie();
				
				$_reg_url = stms_get_handler_url('registration');
				$reg_url = stms_format_url($_reg_url, 'key=' . $session . '&insta_token=' . $insta_token);
				wp_redirect($reg_url);
				exit;
			} else {
				if ( isset($_COOKIE['__insta_flow']) ) {
					$flow_id = urldecode($_COOKIE['__insta_flow']);
					do_action('insta_upsells_redirect', $flow_id, $product->ID, $payment->paysys, $payment->member_id);
				}
				
				wp_redirect($redirect_url);
				exit;
			}
		} else {
			$retries = get_transient('stms_thanks_retries_' . $tx_id);
			$thanks_url = stms_format_url($stms['global']['magic'], 'insta_mode=thanks&processor=' . $payment->paysys);
			if ( $retries >= 15 ) {
				$page_title = 'Payment Error';
				$refresh = '';
				$message = '
					<p style="text-align:center;"><strong>Payment Error</strong></p>
					<p>We\'re sorry, but we cannot confirm your payment. Please contact our <a href="' . $support . '">support</a> to resolve this issue.</p>
				';
			} else {
				$count = $retries + 1;
				set_transient('stms_thanks_retries_' . $tx_id, $count, 60*15);
				$page_title = 'Waiting For Payment Confirmation...';
				$refresh = '<meta http-equiv="refresh" content="5;URL=\'' . $thanks_url . '\'" />';
				$message = '
					<h2 style="text-align:center;"><strong>Waiting For Payment Confirmation...</strong></h2>
					<p style="text-align:center;"><img src="' . STMS_URL . 'images/admin-loader.gif" border="0" /></p>
					<p style="text-align:center;">This page will be auto-refresh every 5 seconds until your payment confirmed.</p>
					<p style="text-align:center;"><strong>Please DO NOT close this page.</strong></p>
				';
			}
?>
<!DOCTYPE html>
<!--[if IE 6]>
<html id="ie6" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 7]>
<html id="ie7" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 8]>
<html id="ie8" <?php language_attributes(); ?>>
<![endif]-->
<!--[if !(IE 6) | !(IE 7) | !(IE 8)  ]><!-->
<html <?php language_attributes(); ?>>
<!--<![endif]-->
<head>
<meta charset="<?php bloginfo( 'charset' ); ?>" />
<title><?php echo $page_title; ?></title>
<?php echo $refresh; ?>
<!--[if lt IE 9]>
	<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->
<link rel="profile" href="http://gmpg.org/xfn/11" />
<meta name="HandheldFriendly" content="True" />
<meta name="viewport" content="width=device-width, initial-scale=1"/>
<!-- Adding "maximum-scale=1" fixes the Mobile Safari auto-zoom bug: http://filamentgroup.com/examples/iosScaleBug/ -->
<link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Droid+Sans" type="text/css" media="all" />
<style type="text/css">
body {
	margin:0;
	padding:0;
	font-size:14px;
	font-family:'Droid Sans', sans-serif;
	background-color:#F5F5F5;
	color:#808080;
}
#redirect_msg {
	margin:25px auto 0;
	padding:15px 20px;
	width:40%;
	background-color:#FFF;
}
</style>
</head>
<body>
<div id="redirect_msg">
	<?php echo $message; ?>
</div>
</body>
</html>
<?php
		}
		exit;
	}
}

add_action('template_redirect', 'stms_payment_thanks_alternate', 1);
function stms_payment_thanks_alternate() {
	if ( stms_isset($_GET['insta_mode']) == 'thank_you' && stms_isset($_GET['processor']) != '' ) {
		$processor = stms_isset($_GET['processor']);
		$product_id = 0;
		if ( isset($_GET['product_id']) ) {
			$product = stms_get_product($_GET['product_id']);
			$product_id = $product->ID;
		}
		
		$is_upsell = false;
		$member_url = stms_member_url();
		if ( !isset($_COOKIE['__insta_payment']) ) {
			$refuse = true;
			if ( isset($_COOKIE['__insta_payment_' . $product_id]) )
				$refuse = false;
			
			if ( isset($_COOKIE['__insta_flow']) && stms_isset($_COOKIE['__insta_flpaysys']) == $processor ) {
				$refuse = false;
				$is_upsell = true;
			}
		
			if ( $refuse ) {
				// simply redirect them to member home page...
				wp_redirect($member_url);
				exit;
			}
		}
		
		
		$tx_id = ( isset($_COOKIE['__insta_payment_' . $product_id]) ) ? intval($_COOKIE['__insta_payment_' . $product_id]) : stms_isset($_COOKIE['__insta_payment']);
		if ( !$is_upsell ) {
			$check = get_transient('stms_payment_check_' . $tx_id);
			if ( empty($check) ) {
				wp_redirect($member_url);
				exit;
			}
		}
		
		do_action('stms_validate_thanks_' . $processor, $tx_id);
		exit;
	}
}

add_action('template_redirect', 'stms_payment_thanks_third', 1);
function stms_payment_thanks_third() {
	if ( stms_isset($_GET['insta_mode']) == 'product_access' && stms_isset($_GET['processor']) != '' ) {
		//do_action('stms_validate_access_' . $_GET['processor']);
		stms_third_thanks_handler($_GET['processor']);
		exit;
	}
}

add_action('template_redirect', 'stms_auto_switch_payment');
function stms_auto_switch_payment() {
	if ( stms_isset($_GET['insta_mode']) == 'landing' && stms_isset($_GET['payment']) != '' && stms_isset($_GET['id']) > 0 ) {
		if ( !$product = stms_get_product($_GET['id']) )
			return false;
		
		if ( !$gateway = stms_gateway_type($_GET['payment']) )
			return false;
			
		$sales_url = trim($product->info['salesuri']);
		@setcookie('__insta_processor', $_GET['payment'], time()+3600*24*365, SITECOOKIEPATH);
		if ( SITECOOKIEPATH != COOKIEPATH )
			@setcookie('__insta_processor', $_GET['payment'], time()+3600*24*365, COOKIEPATH);
		
		wp_redirect($sales_url);
		exit;
	}
}

add_action('template_redirect', 'stms_auto_switch_js');
function stms_auto_switch_js() {
	if ( stms_isset($_GET['insta_mode']) == 'js_auto' && stms_isset($_GET['id']) > 0 ) {
		$stms = get_option('stms_settings');
		if ( !$product = stms_get_product($_GET['id']) )
			return false;
		
		$text = stms_isset($_GET['text'], 'addtocart');
		$color = stms_isset($_GET['color'], 'orange');
		$paysys = 'unknown';
		
		if ( !isset($_COOKIE['__insta_processor']) ) {
			if ( is_array($product->info['direct_pay']) && count($product->info['direct_pay']) > 0 ) {
				$directs = $product->info['direct_pay'];
				if ( isset($directs['paypal']) && $directs['paypal'] == 1 )
					$paysys = 'paypal';
				else {
					$direct_pays = stms_get_direct_payments();
					foreach ( $direct_pays as $k => $v ) {
						if ( $directs[$k] == 1 ) {
							$paysys = $k;
							break;
						}
					}
				}
			}

			if ( $paysys == 'unknown' && is_array($product->info['third_pay']) && count($product->info['third_pay']) > 0 ) {
				foreach ( $product->info['third_pay'] as $k => $v ) {
					if ( $v == 1 ) {
						$paysys = $k;
						break;
					}
				}
			}
		} else {
			$paysys = $_COOKIE['__insta_processor'];
		}

		if ( $paysys == 'unknown' )
			return false;
		
		$code = '';
		$settings = stms_get_paysettings($product->ID);
		switch ( $paysys ) {
			case 'zaxaa':
				$code = stms_sanitize($settings['zaxaa']['code']);
				$code = str_replace('"', "'", html_entity_decode($code));
				break;
				
			case 'jvzoo':
				$code = stms_sanitize($settings['jvzoo']['code']);
				if ( empty($code) ) {
					// lets create it using the provided item id
					$item = stms_isset($settings['jvzoo']['item'], 0);
					$code = "<a href='https://www.jvzoo.com/b/0/{$item}/2?insta_id={$product->ID}'><img src='http://i.jvzoo.com/0/{$item}/2' border='0' style='-webkit-box-shadow: none; box-shadow: none' /></a>";
				}
				$code = str_replace('"', "'", html_entity_decode($code));
				break;
				
			case 'dgr':
				$code = stms_sanitize($settings['dgr']['code']);
				if ( empty($code) ) {
					// lets create it using the provided item id
					$item = stms_isset($settings['dgr']['item'], 0);
					$code = "<a href='http://www.digiresults.com/buy/{$item}?insta_id={$product->ID}'><img src='http://www.digiresults.com/buy-button/{$item}' border='0' style='-webkit-box-shadow: none; box-shadow: none' /></a>";
				}
				$code = str_replace('"', "'", html_entity_decode($code));
				break;
				
			case 'wsopro':
				$code = stms_sanitize($settings['wsopro']['code']);
				if ( empty($code) ) {
					// lets create it using the provided item id
					$item = stms_isset($settings['wsopro']['item'], 0);
					$code = "<a href='http://www.warriorplus.com/buywso/{$item}'><img src='" . STMS_URL . "images/buttons/big-{$color}-{$text}-circle.png' border='0' style='-webkit-box-shadow: none; box-shadow: none' /></a>";
				}
				$code = str_replace('"', "'", html_entity_decode($code));
				break;
				
			case 'productpay':
				$code = stms_sanitize($settings['productpay']['code']);
				if ( empty($code) ) {
					// lets create it using the provided item id
					$item = stms_isset($settings['productpay']['item'], 0);
					$code = "<a href='https://www.productpay.com/product/{$item}'><img src='" . STMS_URL . "images/buttons/big-{$color}-{$text}-circle.png' border='0' style='-webkit-box-shadow: none; box-shadow: none' /></a>";
				}
				$code = str_replace('"', "'", html_entity_decode($code));
				break;
				
			case 'nanacast':
				$code = stms_sanitize($settings['nanacast']['code']);
				if ( empty($code) ) {
					// lets create it using the provided item id
					$item = stms_isset($settings['nanacast']['item'], 0);
					$code = "<script src='http://nanacast.com/s/?req=create_form&lid={$item}&l_show_paid=1&l_paid_image=" . urlencode(STMS_URL . "images/buttons/big-{$color}-{$text}-circle.png"). "'></script>";
				}
				$code = str_replace('"', "'", html_entity_decode($code));
				break;
				
			case 'dealguardian':
				$code = stms_sanitize($settings['dealguardian']['code']);
				$code = str_replace('"', "'", html_entity_decode($code));
				break;
				
			default:
				$payment_url = '';
				$gateway = stms_gateway_type($paysys);
				if ( $gateway == 'third' ) {
					$vendor = ( $paysys == 'clickbank' && isset($settings[$paysys]['vendor']) ) ? $settings[$paysys]['vendor'] : '';
					$item = ( isset($settings[$paysys]['item']) ) ? $settings[$paysys]['item'] : 0;
					if ( $paysys == 'clickbank' ) {
						$payment_url = ( isset($settings['clickbank']['payurl']) ) ? $settings['clickbank']['payurl'] : "http://{$item}.{$vendor}.pay.clickbank.net/";
					} else if ( $paysys == 'oneshopping' ) {
						$payment_url = ( isset($settings['oneshopping']['payurl']) ) ? $settings['oneshopping']['payurl'] : 'https://';
					} else if ( $paysys == 'ejunkie' ) {
						$client_id = stms_sanitize($stms['payments']['ejunkie']['id'], '', true);
						$payment_url = ( isset($settings['ejunkie']['payurl']) ) ? $settings['ejunkie']['payurl'] : "http://www.e-junkie.com/ecom/gb.php?i={$item}&c=single&cl={$client_id}&custom={$product->ID}";
					}
				} else {
					$payment_url = stms_format_url(get_bloginfo('url'), 'insta_pay=' . $paysys . '&id=' . $product->ID);
				}
				$code = "<a href='{$payment_url}'><img src='" . STMS_URL . "images/buttons/big-{$color}-{$text}-circle.png' border='0' style='-webkit-box-shadow: none; box-shadow: none' /></a>";
				break;
		}

		if ( $code == '' )
			return false;
		
		header("Content-type:text/javascript");
		
		echo 'document.write("<div style=\'text-align:center; margin:10px auto 18px; max-width: 560px; min-width:240px;\'>")' . "\n";
		echo 'document.write("' . html_entity_decode($code) . '")' . "\n";
		echo 'document.write("</div>")' . "\n";
		exit;
	}
}

function stms_validate_ccnum( $cardnumber ) {
	$cardnumber = str_replace(array(" ", "-"), "", trim($cardnumber));
	$card_patterns = array(
		'mastercard' => "/^5[1-5][0-9]{14}$/",
		'visa' => "/^4[0-9]{12}([0-9]{3})?$/",
		'amex' => "/^3[47][0-9]{13}$/",
		'dinners' => "/^3(0[0-5]|[68][0-9])[0-9]{11}$/",
		'discovers' => "/^6011[0-9]{12}$/",
		'jcb' => "/^(3[0-9]{4}|2131|1800)[0-9]{11}$/",
	);
	
	$card_type = false;
	foreach ( $card_patterns as $key => $value ) {
		if ( preg_match($value, $cardnumber) ) {
			$card_type = $key;
			break;
		}
	}
	
	if ( !$card_type )
		return false;
	
	/*  MOD 10 Checksum Algorithm  */
	settype($cardnumber, 'string');
	$revcode = strrev($cardnumber);
	$checksum = 0;
	 
	for ( $i = 0; $i < strlen($revcode); $i++ ) {
		$current_num = intval($revcode[$i]);
		if ( $i & 1 ) /* Odd position */
			$current_num *= 2;
		
		/* Split digits and add. */
		$checksum += $current_num % 10;
		if ( $current_num > 9 )
			$checksum += 1;
	}
	
	if ( $checksum % 10 == 0)
		return $card_type;
	else
		return false;
}

function stms_validate_ccdate( $month, $year ) {
	$month_now = date("m", stms_time());
	$year_now = date("Y", stms_time());
	if ( $year < $year_now ) return false;
	if ( $month < $month_now && $year == $year_now ) return false;
	return true;
}

function stms_validate_cvv( $cardnumber, $cvv ) {
	$firstnumber = (int) substr($cardnumber, 0, 1);
	if ( $firstnumber === 3 ) {
		if ( !preg_match("/^\d{4}$/", $cvv) ) return false;
	} else {
		if ( !preg_match("/^\d{3}$/", $cvv) ) return false;
	}
	return true;
}

function stms_third_thanks_handler( $paysys ) {
	if ( !file_exists(STMS_PAYMENTS . 'thirds/' . $paysys . '.php') )
		wp_die('ERROR: Invalid Payment System');
	
	global $wpdb;
	
	$args = apply_filters('stms_thanks_setup_' . $paysys, array());
	$defaults = array(
		'flow' => 'normal',
		'rcpt' => 'cbreceipt',
		'email_check' => false,
		'email_var' => 'email',
		'fullname_var' => 'none',
		'firstname_var' => 'first_name',
		'lastname_var' => 'last_name',
		'username_var' => 'username',
		'password_var' => 'password',
	);
	$args = wp_parse_args($args, $defaults);
	
	$product_id = ( isset($_REQUEST['product_id']) ) ? $_REQUEST['product_id'] : intval($_REQUEST['insta_id']);
	$stms = get_option('stms_settings');
	$rcpt = ( isset($_REQUEST[$args['rcpt']]) ) ? stms_sanitize($_REQUEST[$args['rcpt']]) : stms_sanitize($_GET['insta_receipt'], '');

	if ( $paysys == 'productpay' && empty($rcpt) ) {
		$find_receipt = '';
		foreach ( $_GET as $k => $v ) {
			if ( stristr($_GET[$k], '?cbreceipt') ) {
				$find_receipt = rawurldecode(trim($_GET[$k]));
				break;
			}
		}
		
		$parts = explode("?", $find_receipt);
		$ppreceipt = explode("=", stms_isset($parts[1], ''));
		$rcpt = stms_isset($ppreceipt[1], '');
	}
	
	if ( $paysys == 'zaxaa' && empty($rcpt) && isset($_REQUEST['tx']) ) {
		if ( !function_exists('curl_init') )
			wp_die('cURL extension is NOT installed in your server. Please contact your host administrator.');
			
		$signature = ( !empty($stms['payments']['zaxaa']['signature']) ) ? trim($stms['payments']['zaxaa']['signature']) : '';
		$tx = $_REQUEST['tx']; // token transaction
		$postfields = 'sid=' . $signature . '&tx=' . $tx;

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, 'https://www.zaxaa.com/notify/pdt');
		curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
		curl_setopt($ch, CURLOPT_TIMEOUT, 10);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_POST, true);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $postfields);

		$response = curl_exec($ch);
		$code = curl_getinfo($ch, CURLINFO_HTTP_CODE);

		curl_close($ch);
		
		if ( $code == 200 ) {
			$vars = urldecode($response);
			parse_str($vars, $zpost);
	
			$rcpt = $zpost['trans_receipt'];
			$_GET['rcpt'] = $rcpt;
		}
	}

	if ( empty($rcpt) ) {
		if ( $product = stms_get_product($product_id) ) {
			$check_url = stms_format_url(stms_get_handler_url('order_lookup'), 'product_id=' . $product->ID . '&paysys=' . $paysys);
			wp_redirect($check_url);
			exit;
		} else {
			wp_die("ERROR: Page Not Found.");
		}
	}
	
	$support = stms_get_handler_url('support');
	if ( $ipn = stms_get_ipn_log($paysys, $rcpt) ) {
		$vars = maybe_unserialize($ipn->postdata);
		if ( !$payment = stms_get_payment($ipn->tx_id) )
			wp_die('ERROR: Invalid Payment');
		
		$product = stms_get_product($payment->product_id);
		$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_permalink($stms['global']['home']);
		$redirect_url = ( !empty($product->info['thankuri']) ) ? $product->info['thankuri'] : $home_url;
	
		if ( $ipn->status == 1 ) { // check if IPN has been processed...
			// IPN has been processed... then redirect to members download page...
			wp_redirect($redirect_url);
			exit;
		}
		$member_id = ( is_stmsmember_logged_in() ) ? stms_get_current_member_id() : $payment->member_id;
		$is_logged_in = ( is_stmsmember_logged_in() ) ? true : false;
		$is_existing = false;
		
		$temp_member = stms_member($payment->member_id);
		if ( $is_logged_in ) {
			// logging out member first...
			stms_clear_auth_cookie();
			if ( stms_isset($product->info['auto_register']) != 1 && ($member_id != $payment->member_id) ) {
				// fix license key data ...
				$settings = stms_get_product_meta($payment->product_id, 'license');
				if ( $settings['enable'] == 1 )
					$wpdb->update("{$wpdb->prefix}stms_licenses", array('member_id' => $member_id), array('tx_id' => $payment->tx_id));
				
				// Update member id trx
				$new_data = array();
				$new_data['member_id'] = $member_id;
				stms_update_payment($payment->tx_id, $new_data);
				wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
				
				do_action('stms_payment_completed', $payment->tx_id); // so the receipt can be resend...
				
				if ( $temp_member->status == 4 )
					stms_delete_member($temp_member->ID, false);
			}
			
			$is_existing = true;
		}
		
		// Recall payment data to make sure that we use latest data...
		$payment = stms_get_payment($ipn->tx_id);
		// let's update ipn log...
		$update = $wpdb->update("{$wpdb->prefix}stms_ipn_log", array('status' => 1, 'member_id' => $payment->member_id), array('log_id' => $ipn->log_id));
		// remove retries data...
		delete_transient('stms_thanks_retries_' . $rcpt);
		do_action('stms_custom_autoresponder', $payment->tx_id); // for custom autoresponder...
		
		if ( $is_existing ) {
			// check and complete other payments (if any)
			//stms_other_payments($member_id, $vars[$args['email_var']], $paysys);

			// re-logging in...
			stms_set_auth_cookie($payment->member_id);
			
			// now redirecting...
			wp_redirect($redirect_url);
			exit;
		}
		
		if ( $args['fullname_var'] != 'none' ) {
			$full_name = stms_sanitize($vars[$args['fullname_var']]);
			$names = explode(" ", $full_name, 2);
			$first_name = ucwords(strtolower(stms_sanitize($names[0])));
			$last_name = ( isset($names[1]) ) ? ucwords(strtolower(stms_sanitize($names[1]))) : '';
		} else {
			$first_name = ucwords(strtolower(stms_sanitize($vars[$args['firstname_var']])));
			$last_name = ucwords(strtolower(stms_sanitize($vars[$args['lastname_var']])));
		}
		
		if ( stms_isset($product->info['auto_register']) == 1 || $args['flow'] == 'auto_only' ) {
			$meta = stms_get_member_meta($payment->member_id, 'info');
			$new_pass = ( !empty($meta['user_key']) ) ? stms_string_dec($meta['user_key'], AUTH_SALT) : wp_generate_password(8, false);
			$m = stms_member($payment->member_id);
			$new_username = ( stms_isset($product->info['steps']) == '2,1' && !empty($first_name) && !empty($m->username) ) ? stms_sanitize($m->username) : stms_generate_username($vars[$args['email_var']], $first_name, $last_name);
			
			if ( $m->status != 1 ) {
				// update member data...
				$userdata = array();
				$userdata['member_id'] = $payment->member_id;
				$userdata['email'] = sanitize_email($vars[$args['email_var']]);
				$userdata['status'] = 1;
				$userdata['username'] = ( !empty($vars[$args['username_var']]) ) ? stms_sanitize($vars[$args['username_var']]) : $new_username;
				$userdata['fix_username'] = ( !empty($vars[$args['username_var']]) ) ? stms_sanitize($vars[$args['username_var']]) : $new_username;
				$userdata['pwd1'] = ( !empty($vars[$args['password_var']]) ) ? stms_sanitize($vars[$args['password_var']]) : $new_pass;
				$userdata['first_name'] = $first_name;
				$userdata['last_name'] = $last_name;
				$userdata['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
				$userdata['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
				
				stms_save_member($userdata, false, false, $payment->tx_id);
			}

			// check and complete other payments (if any)
			//stms_other_payments($member_id, $vars[$args['email_var']], $paysys);
			
			// clean any login cookies first...
			stms_clear_auth_cookie();
			
			// logging in and then redirect...
			stms_set_auth_cookie($payment->member_id);
			wp_redirect($redirect_url);
			exit;
		} else {
			// Proceed to registration
			$session = base64_encode("{$product->ID}|{$paysys}|third");
			$insta_token = stms_generate_secret(16, true, true );
			$token_data = array( 'token' => $insta_token, 'tx_id' => $payment->tx_id);
			set_transient('insta_token_' . $insta_token, $token_data, 86400);
			
			$_reg_url = stms_get_handler_url('registration');
			$reg_url = stms_format_url($_reg_url, 'key=' . $session . '&insta_token=' . $insta_token);
			// build reg vars...
			$reg_vars = array();
			$reg_vars['im_reg_email'] = sanitize_email($vars[$args['email_var']]);
			$reg_vars['im_reg_first_name'] = $first_name;
			$reg_vars['im_reg_last_name'] = $last_name;
			$auth_vars = base64_encode(json_encode($reg_vars));
			
			wp_redirect($reg_url . '&auth_vars=' . $auth_vars);
			exit;
		}
	} else {
		$retries = get_transient('stms_thanks_retries_' . $rcpt);
		$thanks_url = stms_format_url($stms['global']['magic'], http_build_query($_GET));
		if ( $retries >= 15 ) {
			$page_title = 'Payment Error';
			$refresh = '';
			$message = '
				<p style="text-align:center;"><strong>Payment Error</strong></p>
				<p>We\'re sorry, but there\'s a problem in validating your payment. Please contact our <a href="' . $support . '">support</a> to resolve this issue.</p>
			';
		} else {
			$count = $retries + 1;
			set_transient('stms_thanks_retries_' . $rcpt, $count, 60*15);
			$page_title = 'Waiting For Payment Confirmation...';
			$refresh = '<meta http-equiv="refresh" content="5;URL=\'' . $thanks_url . '\'" />';
			$message = '
				<h2 style="text-align:center;"><strong>Waiting For Payment Confirmation...</strong></h2>
				<p style="text-align:center;"><img src="' . STMS_URL . 'images/admin-loader.gif" border="0" /></p>
				<p style="text-align:center;">This page will be auto-refresh every 5 seconds until your payment confirmed.</p>
				<p style="text-align:center;"><strong>Please DO NOT close this page.</strong></p>
			';
		}
?>
<!DOCTYPE html>
<!--[if IE 6]>
<html id="ie6" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 7]>
<html id="ie7" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 8]>
<html id="ie8" <?php language_attributes(); ?>>
<![endif]-->
<!--[if !(IE 6) | !(IE 7) | !(IE 8)  ]><!-->
<html <?php language_attributes(); ?>>
<!--<![endif]-->
<head>
<meta charset="<?php bloginfo( 'charset' ); ?>" />
<title><?php echo $page_title; ?></title>
<?php echo $refresh; ?>
<!--[if lt IE 9]>
	<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->
<link rel="profile" href="http://gmpg.org/xfn/11" />
<meta name="HandheldFriendly" content="True" />
<meta name="viewport" content="width=device-width, initial-scale=1"/>
<!-- Adding "maximum-scale=1" fixes the Mobile Safari auto-zoom bug: http://filamentgroup.com/examples/iosScaleBug/ -->
<link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Droid+Sans" type="text/css" media="all" />
<style type="text/css">
body {
	margin:0;
	padding:0;
	font-size:14px;
	font-family:'Droid Sans', sans-serif;
	background-color:#F5F5F5;
	color:#808080;
}
#redirect_msg {
	margin:25px auto 0;
	padding:15px 20px;
	width:40%;
	background-color:#FFF;
}
</style>
</head>
<body>
<div id="redirect_msg">
	<?php echo $message; ?>
</div>
</body>
</html>
<?php
	}
}

//add_action('stms_claim_payments', 'stms_third_check_unclaim');
function stms_third_check_unclaim( $member_id ) {
	global $wpdb;
	
	// get pay_email data of this member...
	$member_id = intval($member_id);
	$sql = "SELECT * FROM `{$wpdb->prefix}stms_ipn_log` WHERE `member_id` = {$member_id} AND `status` = 1";
	if ( !$ipn = $wpdb->get_row($sql) )
		return false;
	
	// check for unclaimed ipn of the pay_email
	$pay_email = sanitize_email($ipn->pay_email);
	$sql = "SELECT * FROM `{$wpdb->prefix}stms_ipn_log` WHERE `status` = 0 AND `pay_email` = '{$pay_email}'";
	if ( !$ipns = $wpdb->get_results($sql) )
		return false;
	
	// let's fix the data...
	foreach ( $ipns as $ipn ) {
		if ( $p = stms_get_payment($ipn->tx_id) ) {
			$count = $wpdb->get_var("SELECT COUNT(*) FROM `{$wpdb->prefix}stms_transactions`
					WHERE `member_id` = {$member_id} AND `product_id` = {$p->product_id}
					AND `paid` = 1 AND (`paysys = 'manual' OR `paysys` = 'free')");
			
			if ( $count < 1 ) {
				
				$temp_member = stms_member($p->member_id);
				// fix license key data ...
				$settings = stms_get_product_meta($payment->product_id, 'license');
				if ( $settings['enable'] == 1 )
					$wpdb->update("{$wpdb->prefix}stms_licenses", array('member_id' => $member_id), array('tx_id' => $p->tx_id));
					
				// Update member id trx
				$new_data = array();
				$new_data['member_id'] = $member_id;
				stms_update_payment($p->tx_id, $new_data);
				wp_cache_delete($member_id, 'stms_member_payments'); // delete cache
					
				$hour_ago = stms_time() - 3600;
				if ( $p->date > $hour_ago )
					do_action('stms_payment_completed', $payment->tx_id); // so the receipt can be resend...
					
				if ( isset($temp_member->status) && $temp_member->status == 4 )
					stms_delete_member($temp_member->ID, false);
					
				// update the ipn log
				$wpdb->update("{$wpdb->prefix}stms_ipn_log", array('status' => 1), array('log_id' => $ipn->log_id));
			}
		}
	}
}
