<?php
add_shortcode('insta_s3_video', 'insta_s3_video_handler');
function insta_s3_video_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'file' => '',
		'player' => 'flowplayer',
		'width' => '640',
		'height' => '360',
		'autoplay' => 0,
		'controls' => 1,
		'autohide' => 1,
		'splash' => ''
	), $atts ) );
	
	if ( empty($file) )
		return '<p>Video file is empty</p>';
	
	if ( $player == 'jwplayer' && !instamember_jwplayer_installed() )
		return '<p>Video Error: Please install JW Player Plugin for WordPress.</p>';
	
	$access = stms_check_s3video_access($file);
	if ( $access === FALSE ) {
		$permalink = get_permalink();
		$login_url = stms_login_url($permalink);
		if ( !is_stmsmember_logged_in() )
			return '<div class="insta-error-msg"><p><strong>You have to <a href="' . $login_url . '">login</a> to watch this video.</strong></p></div>';
		else
			return '<div class="insta-error-msg"><p><strong>Video is not available.</strong></p></div>';
	}
	
	if ( $access === 'expired' ) {
		$permalink = get_permalink();
		$login_url = stms_login_url($permalink);
		return '<div class="insta-error-msg"><p><strong>Video has been expired.</strong></p></div>';
	}

	if ( $access === 'unreleased' ) {
		$permalink = get_permalink();
		$login_url = stms_login_url($permalink);
		return '<div class="insta-error-msg"><p><strong>Video hasn\'t been released yet.</strong></p></div>';
	}
	
	$file = strip_tags(html_entity_decode($file));
	$parts = explode(".", $file);
	$mp4 = end($parts);
	
	$url = stms_s3_get_authenticated_url($file);
	$video = array();
	$video['video_url'] = $url;
	$video['video_width'] = $width;
	$video['video_height'] = $height;
	$video['autoplay'] = $autoplay;
	$video['autohide'] = $autohide;
	$video['disable_control'] = ( $controls == 1 ) ? 0 : 1;
	$video['video_scr'] = $splash;
	$video['mp4'] = ( $mp4 == 'mp4' ) ? 1 : 0;
	
	$show_video = '<div class="insta-video-shadow" style="margin:10px auto 16px; width:' . $width . 'px; height:' . $height . 'px">';
	if ( $player == 'flowplayer' ) {
		$show_video .= instamember_flowplayer($video);
	} else if ( $player == 'jwplayer' ) {
		$show_video .= instamember_jwplayer($video);
	}
	$show_video .= '</div>';

	return str_replace(array("\r", "\n"), '', $show_video);
}

add_shortcode('insta_license', 'insta_license_handler');
function insta_license_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'id' => '',
	), $atts ) );
	
	global $wpdb;
	
	$proto = ( is_ssl() ) ? 'https://' : 'http://';
	$login_url = stms_login_url($proto . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);
	if ( !is_stmsmember_logged_in() )
		return 'Please <a href="' . $login_url . '">login</a> to see this content.<br />';
	
	$stms = get_option('stms_settings');
	$member_id = stms_get_current_member_id();
	$member = stms_member($member_id);
	$sql = $wpdb->prepare("SELECT * FROM `{$wpdb->prefix}stms_transactions`
			WHERE `member_id` = %d AND `status` = %s
			ORDER BY `tx_id` DESC", $member_id, 'completed');
			
	if ( !$payments = $wpdb->get_results($sql) )
		return 'No License Key Found.';
	
	$tx_id = 0;
	$product_id = intval($id);
	foreach ( $payments as $py ) {
		if ( $py->product_id == $product_id && $py->paid == 1 && $py->is_rebill != 1 ) {
			$tx_id = $py->tx_id;
			break;
		}
	}
	
	if ( !$tx_id )
		return 'No License Key Found.';
	
	if ( !$l = stms_get_license_by('tx_id', $tx_id) )
		return 'No License Key Found.';
	
	$html = $l->license;
	if ( $l->status == 0 ) {
		$html .= ' (status: Inactive)';
	} else if ( $l->status == 2 ) {
		$html .= ' (status: Suspended)';
	} else if ( $l->status == 3 ) {
		$html .= ' (status: Expired)';
	} else {
		$html .= ' (status: Active)';
	}
	
	return $html;
}

add_shortcode('insta_member', 'insta_member_handler');
function insta_member_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'type' => 'username'
	), $atts ) );
	
	global $wpdb;
	
	$stms = get_option('stms_settings');
	if ( !is_stmsmember_logged_in() )
		return '';

	$m = stms_get_current_member();
	switch ( $type ) {
		case 'title':
			$text = ucwords(stms_sanitize($m->info['title'])) . '.';
			break;
		case 'address':
			$text = stms_sanitize($m->info['street']);
			break;
		case 'city':
			$text = ucwords(stms_sanitize($m->info['city']));
			break;
		case 'state':
			$text = ucwords(stms_sanitize($m->info['state']));
			break;
		case 'country':
			$countries = stms_world_countries();
			$text = ( isset($m->info['country']) && isset($countries[$m->info['country']]) ) ? stms_sanitize($countries[$m->info['country']]) : '';
			break;
		case 'zipcode':
			$text = stms_sanitize($m->info['zipcode']);
			break;
		case 'phone':
			$text = stms_sanitize($m->info['phone']);
			break;
		case 'gender':
			$text = ucwords(stms_sanitize($m->info['gender']));
			break;
		case 'company':
			$text = stms_sanitize($m->info['company']);
			break;
		case 'firstname':
			$text = stms_sanitize($m->first_name);
			break;
		case 'lastname':
			$text = stms_sanitize($m->last_name);
			break;
		case 'email':
			$text = stms_sanitize($m->email);
			break;
		case 'username':
		default:
			$text = stms_sanitize($m->username);
			break;
	}
	return $text;
}

add_shortcode('insta_purchases', 'insta_purchases_handler');
function insta_purchases_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'file' => ''
	), $atts ) );
	
	global $wpdb;
	
	$stms = get_option('stms_settings');
	$permalink = get_permalink();
	$login_url = stms_login_url($permalink);
	if ( !is_stmsmember_logged_in() )
		return '<div class="insta-error-msg"><p><strong>You have to <a href="' . $login_url . '">login</a> to view this content.</strong></p></div>';

	$member_id = stms_get_current_member_id();
	$products = stms_get_products();
	$purchases = array();
	if ( $products ) {
		foreach ( $products as $pr ) {
			$payment = $wpdb->get_row("SELECT * FROM `{$wpdb->prefix}stms_transactions` WHERE `product_id` = {$pr->ID} AND `member_id` = {$member_id} AND `paid` = 1 ORDER BY `tx_id` DESC");
			if ( !$payment || $payment->status == 'temp' ) continue;
			$title = stms_cleanup($pr->title);
			$_status = $payment->status;
			$expire = 'Lifetime';
			if ( $payment->expired_date > 0 ) {
				$e = $wpdb->get_row("SELECT * FROM `{$wpdb->prefix}stms_transactions`
					WHERE `product_id` = {$payment->product_id} AND `member_id` = {$member_id}
					AND `paid` = 1 ORDER BY `tx_id` DESC LIMIT 1"
				);
				$exp = stms_isset($e->expired_date, stms_time());
				$expire = date("m/d/Y", $exp);
				$_status = $e->status;
			}
			
			if ( isset($exp) && (($_status == 'completed' || $_status == 'cancelled')) ) {
				$now = stms_time();
				if ( $now > $exp )
					$_status = 'expired'; 
			}
			
			$_status = ( $_status == 'completed' ) ? 'active' : $_status;
			$status = ucwords($_status);
			$home_id = ( $pr->home_id ) ? $pr->home_id : stms_isset($stms['global']['home']);
			$access = get_permalink($home_id);
			
			$purchases[] = array(
				'title' => $title,
				'start' => date("m/d/Y", $payment->date),
				'end' => $expire,
				'status' => $status,
				'access' => $access
			);
			
			if ( isset($exp) ) unset($exp);
		}
	}

	$html = '';
	$html .= '
		<table cellpadding="0" cellspacing="0" border="0" class="display" id="insta-commissions" width="100%">
		<thead>
			<tr>
				<th>Product</th>
				<th>Begin</th>
				<th>Expire</th>
				<th>Status</th>
				<th>Access</th>
			</tr>
		</thead>
		<tfoot>
			<tr>
				<th>Product</th>
				<th>Begin</th>
				<th>Expire</th>
				<th>Status</th>
				<th>Access</th>
			</tr>
		</tfoot>
		<tbody>
	';
	
	if ( $purchases ) {
		$i = 0;
		foreach ( $purchases as $purchase ) {
			$style = ( $i%2 ) ? 'background-color:#F5F5F5' : 'background-color:#FFF';
			$html .= "
				<tr style='{$style}'>
					<td>{$purchase['title']}</td>
					<td class='center'>{$purchase['start']}</td>
					<td class='center'>{$purchase['end']}</td>
					<td class='center'>{$purchase['status']}</td>
					<td class='center'><a href='{$purchase['access']}' target='_blank'>Access</a></td>
				</tr>
			";
			$i++;
		}
	}
	$html .= '</table>';
	return $html;
}

add_shortcode('insta_offers', 'insta_offers_handler');
function insta_offers_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'exclude_ids' => '',
	), $atts ) );
	
	global $wpdb;
	
	$stms = get_option('stms_settings');
	$permalink = get_permalink();
	$login_url = stms_login_url($permalink);
	if ( !is_stmsmember_logged_in() )
		return '';

	$member_id = stms_get_current_member_id();
	$products = stms_get_products();
	$excludes = ( !empty($exclude_ids) ) ? explode(",", $exclude_ids) : array();
	
	$offers = array();
	if ( $products ) {
		foreach ( $products as $pr ) {
			$payment = $wpdb->get_row("SELECT * FROM `{$wpdb->prefix}stms_transactions` WHERE `product_id` = {$pr->ID} AND `member_id` = {$member_id} AND `status` = 'completed' AND `paid` = 1");
			if ( $payment || in_array($pr->ID, $excludes) || $pr->type == 'free' ) continue;
			$offers[] = $pr;
		}
	}
	
	$html = '';
	if ( $offers ) {
		$html .= '<ul>';
		foreach ( $offers as $offer ) {
			$html .= '<li><a href="' . $offer->info['salesuri'] . '" target="_blank">' . stms_sanitize($offer->title) . '</a></li>';
		}
		$html .= '</ul>';
	}
	
	return $html;
}

add_shortcode('insta_download', 'insta_download_handler');
function insta_download_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'file' => '',
		'type' => 'normal',
		'target' => '_self',
	), $atts ) );
	
	$file = strip_tags(html_entity_decode($file));
	$s3 = ( $type == 's3' ) ? '&source=s3' : '';
	$download_url = stms_format_url(get_bloginfo('url'), 'insta_file=' . $file . $s3);
	$label = ( !empty($content) ) ? $content : $download_url;
	$html = '';
	$html .= "<a href='{$download_url}' target='{$target}'>{$label}</a>";
	return $html;
}

add_shortcode('insta_aff_swipe', 'insta_aff_swipe_handler');
function insta_aff_swipe_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'subject' => '',
		'center' => 'no',
		'width' => '560',
		'height' => '480'
	), $atts ) );
	
	if ( !is_stmsmember_logged_in() )
		return '';
	
	$stms = get_option('stms_settings');
	$member_id = stms_get_current_member_id();
	$member = stms_member($member_id);
	$profile_url = stms_get_handler_url('profile');
	$aff_activate_url = stms_format_url($profile_url, 'action=aff_activate');
	if ( $member->is_aff != 1 ) {
		if ( $member->is_aff_pending == 1 )
			return '<p><em>Your affiliate account status is currently in <strong>pending</strong>. You\'ll be able to view this content when we approved your affiliate application.</em></p>';
		else
			return '<p><em>You have to become our affiliate first to view this content. <a href="' . $aff_activate_url . '">Click here</a> to activate your affiliate account.</em></p>';
	}
	
	$margin = ( $center == 'yes' ) ? '10px auto 16px' : '10px 0 16px';
	$html = '';
	if ( !empty($subject) ) {
		$html .= '<div style="width:' . $width . 'px; margin:' . $margin . '; padding:0;"><strong>Subject:</strong><br />';
		$html .= '<input type="text" style="width:100%" value="' . esc_attr($subject) . '"/>';
		$html .= '</div>';
	}
	$html .= '<div style="width:' . $width . 'px; height:' . $height . 'px; overflow: scroll; margin:' . $margin . '; padding:0 10px 10px; border:1px solid #E5E5E5;">';
	$html .= do_shortcode(trim($content));
	$html .= '</div>';
	return $html;
}

add_shortcode('insta_aff_banner', 'insta_aff_banner_handler');
function insta_aff_banner_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'id' => '',
		'image' => '',
		'url' => '',
		'target' => '_blank',
		'width' => '468',
		'height' => '60'
	), $atts ) );
	
	if ( !is_stmsmember_logged_in() )
		return '';
	
	if ( !$path_info = pathinfo($image) )
		return 'Invalid Image URL';
	
	$ext = stms_isset($path_info['extension'], '');
	$allowed = array('jpg', 'jpeg', 'gif', 'bmp', 'png');
	if ( !in_array(strtolower($ext), $allowed) )
		return 'Invalid Image URL';
	
	$stms = get_option('stms_settings');
	$member_id = stms_get_current_member_id();
	$member = stms_member($member_id);
	$profile_url = stms_get_handler_url('profile');
	$aff_activate_url = stms_format_url($profile_url, 'action=aff_activate');
	if ( $member->is_aff != 1 ) {
		if ( $member->is_aff_pending == 1 )
			return '<p><em>Your affiliate account status is currently in <strong>pending</strong>. You\'ll be able to view this content when we approved your affiliate application.</em></p>';
		else
			return '<p><em>You have to become our affiliate first to view this content. <a href="' . $aff_activate_url . '">Click here</a> to activate your affiliate account.</em></p>';
	}
	
	$aff_url = stms_format_url(get_bloginfo('url'), 'iaff=' . $member_id);
	if ( filter_var($url, FILTER_VALIDATE_URL) ) {
		$aff_url .= '&u=' . urlencode($url);
	} else {
		if ( !empty($id) )
			$aff_url .= '&p=' . $id;
	}
	
	$div_width = ( $width > 600 ) ? $width : 600;
	$width = ( empty($width) ) ? '100%' : $width;
	$height = ( empty($height) ) ? 'auto' : $height;
	$code = esc_textarea("<a href='{$aff_url}' target='{$target}'><img src='{$image}' width='{$width}' height='{$height}' border='0' /></a>");
	$html = '<div style="width:' . $div_width . 'px; margin:10px auto 16px; text-align:center">';
	$html .= "<a href='{$aff_url}' target='{$target}'><img src='{$image}' width='{$width}' height='{$height}' border='0' class='insta-img' /></a>";
	$html .= '<p><strong>HTML Code:</strong><br /><textarea style="width:520px;height:auto">' . $code . '</textarea></p>';
	$html .= '</div>';
	return $html;
}

add_shortcode('insta_aff_link', 'insta_aff_link_handler');
function insta_aff_link_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'id' => '',
		'url' => '',
		'target' => '_blank',
	), $atts ) );
	
	if ( !is_stmsmember_logged_in() )
		return '';
	
	$stms = get_option('stms_settings');
	$member_id = stms_get_current_member_id();
	$member = stms_member($member_id);
	$profile_url = stms_get_handler_url('profile');
	$aff_activate_url = stms_format_url($profile_url, 'action=aff_activate');
	if ( $member->is_aff != 1 ) {
		if ( $member->is_aff_pending == 1 )
			return '<em>Your affiliate account status is currently in <strong>pending</strong>. You\'ll be able to view this content when we approved your affiliate application.</em>';
		else
			return '<em>You have to become our affiliate first to view this content. <a href="' . $aff_activate_url . '">Click here</a> to activate your affiliate account.</em>';
	}

	$aff_url = stms_format_url(get_bloginfo('url'), 'iaff=' . $member_id);
	if ( filter_var($url, FILTER_VALIDATE_URL) ) {
		$aff_url .= '&u=' . urlencode($url);
	} else {
		if ( !empty($id) )
			$aff_url .= '&p=' . $id;
	}
	
	$label = ( !empty($content) ) ? $content : $aff_url;
	$html = '';
	$html .= "<a href='{$aff_url}' target='{$target}'>{$label}</a>";
	return $html;
}

add_shortcode('insta_login', 'insta_login_handler');
function insta_login_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'redirect' => '',
		'title' => '',
		'label_username' => '',
		'label_password' => '',
		'label_button' => '',
		'label_remember' => '',
		'remember' => 'yes',
		'redirect_url' => ''
	), $atts ) );
	
	$stms = get_option('stms_settings');
	$defaults = stms_default_forms();
	$fields = ( isset($stms['form_login']) ) ? stms_build_form_fields($stms['form_login'], 'login') : stms_isset($defaults['login']);
	$nonce = wp_create_nonce('stms-nonce');
	$ipaddress = stms_retrieve_ipaddr();
	$forgotpass_url = stms_format_url($stms['global']['magic'], 'imhandler=forgot');
	$msg = stms_isset($_REQUEST['msg']);
	
	$login_title = ( isset($stms['form_login']['form_title']) ) ? stms_cleanup($stms['form_login']['form_title']) : stms_isset($defaults['login']['title']);
	$username_label = ( isset($stms['form_login']['username_label']) ) ? stms_cleanup($stms['form_login']['username_label']) : stms_isset($defaults['login']['username_label']);
	$password_label = ( isset($stms['form_login']['password_label']) ) ? stms_cleanup($stms['form_login']['password_label']) : stms_isset($defaults['login']['password_label']);
	$remember_label = ( isset($stms['form_login']['remember_label']) ) ? stms_cleanup($stms['form_login']['remember_label']) : stms_isset($defaults['login']['remember_label']);
	$captcha_label = ( isset($stms['form_login']['captcha_label']) ) ? stms_cleanup($stms['form_login']['captcha_label']) : stms_isset($defaults['login']['captcha_label']);
	$forgotpass_txt = ( isset($stms['form_login']['forgotpass_txt']) ) ? stms_cleanup($stms['form_login']['forgotpass_txt']) : stms_isset($defaults['login']['forgotpass_txt']);
	$button_label = ( isset($stms['form_login']['button_label']) ) ? stms_cleanup($stms['form_login']['button_label']) : $defaults['login']['label'];
	
	$login_title = ( !empty($title) ) ? $title : '';
	$username_label = ( empty($label_username) ) ? $username_label : esc_attr($label_username);
	$password_label = ( empty($label_password) ) ? $password_label : esc_attr($label_password);
	$remember_label = ( empty($label_remember) ) ? $remember_label : esc_attr($label_remember);
	$button_label = ( empty($label_button) ) ? $button_label : esc_attr($label_button);
	
	$form = '<div class="insta-login-sc">';
	if ( wp_get_theme() == 'InstaSpark' )
		$form .= '<div class="insta-login-center">' . "\n";
		
	$form .= '<form method="post" name="instamem-log-form" id="instamem-log-form">' . "\n";
	if ( !empty($login_title) ) $form .= '<h3>' . $login_title . '</h3>';
	$form .= apply_filters('stms_login_messages', '', $msg);
	$form .= '<input type="hidden" name="_instamember_action" value="login" />' . "\n";
	$form .= '<input type="hidden" name="_instamember_nonce" value="' . $nonce . '" />' . "\n";
	if ( isset($_GET['tx_id']) && $_GET['tx_id'] > 0 )
		$form .= '<input type="hidden" name="tx_id" value="' . $_GET['tx_id'] . '" />' . "\n";
	
	if ( isset($_GET['insta_token']) )
		$form .= '<input type="hidden" name="insta_token" value="' . rawurldecode($_GET['insta_token']) . '" />' . "\n";
	
	if ( isset($_GET['flow_id']) )
		$form .= '<input type="hidden" name="flow_id" value="' . rawurldecode($_GET['flow_id']) . '" />' . "\n";
	
	if ( isset($_GET['tx_ids']) && is_array($_GET['tx_ids']) && count($_GET['tx_ids']) > 0 ) {
		foreach ( $_GET['tx_ids'] as $trx_id ) {
			$form .= '<input type="hidden" name="tx_ids[]" value="' . $trx_id . '" />' . "\n";
		}
	}

	$redir = ( !empty($redirect_url) ) ? urldecode($redirect_url) : '';
	if ( isset($_GET['redir']) )
		$redir = urldecode($_GET['redir']);
	
	$form .= ( !empty($redir) ) ? '<input type="hidden" name="redir" value="' . urldecode($redir) . '" />' . "\n" : '';
	$form .= '<div style="width:235px">' . "\n";
	$form .= '<p>' . "\n";
	$form .= '<label for="im_username" class="insta-login-label">' . $username_label . '</label>' . "\n";
	$form .= '<input type="text" name="im_username" id="im_username" value="" style="width:220px; border:2px solid #c0c0c0; font-size:14px; padding:8px;" autofocus />' . "\n";
	$form .= '</p>' . "\n";
	$form .= '<p>' . "\n";
	$form .= '<label for="im_username" class="insta-login-label">' . $password_label . '</label>' . "\n";
	$form .= '<input type="password" name="im_password" id="im_password" value="" style="width:220px; border:2px solid #c0c0c0; font-size:14px; padding:8px;" />' . "\n";
	$form .= '</p>' . "\n";
	
	if ( $remember == 'yes' ) :
		$form .= '<p>' . "\n";
		$form .= '<input type="checkbox" class="checkbox" id="rememberme" name="rememberme" value="1" style="display:inline !important" />' . "\n";
		$form .= '<label for="rememberme" style="display:inline !important">' . $remember_label . '</label>';
		$form .= '</p>' . "\n";
	endif;
	
	if ( stms_isset($stms['integration']['recaptcha']['public']) != '' ) {
		$transient = get_transient('stms_brute_' . $ipaddress);
		if ( stms_isset($transient['attempts']) >= 2 ) {
			require_once( STMS_SOURCES . '/recaptcha/recaptchalib.php');
			$form .= '<strong>' . $captcha_label . '</strong>' . "\n";
			$form .= recaptcha_get_html(trim($stms['integration']['recaptcha']['public']), '', is_ssl());

		}
	}
	
	$form .= '<p style="margin-top:20px"><input type="submit" name="instamem-login" id="instamem-login" class="button gray btn btn-primary btn_custom" value="' . $button_label . '" /></p>' . "\n";
	$form .= '</div>' . "\n";
	$form .= '</form>' . "\n";
	
	if ( wp_get_theme() == 'InstaSpark' )
		$form .= '</div>' . "\n";
	$form .= '</div>';	
		
	return $form;
}

add_shortcode('insta_protect', 'insta_protect_handler');
function insta_protect_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'ids' => '',
		'errormsg' => '',
		'excludes' => '',
		'requires' => '',
		'paysys' => '',
		'expired' => 0,
		'expired_mode' => 'day',
		'expired_msg' => '',
		'expired_upgrade_id' => 0
	), $atts ) );
	
	if ( empty($ids) )
		return do_shortcode($content);
	
	$member_id = stms_get_current_member_id();
	if ( strtolower($ids) == "x" ) {
		if ( is_stms_admin() && stms_isset($stms['global']['admin_unlock']) == 1 )
			return do_shortcode($content);
		
		if ( !is_stmsmember_logged_in() ) {
			if ( !empty($errormsg) )
				return wpautop(instamember_macro(html_entity_decode($errormsg)));
			else
				return '';
		}
			
		global $wpdb;
		$sql = "SELECT * FROM `{$wpdb->prefix}stms_transactions` WHERE `member_id` = {$member_id} AND `status` = 'completed'";
		if ( !$payment = $wpdb->get_results($sql) )
			return do_shortcode($content);
		else
			return '';
	}
	
	$access = stms_check_shortcode_access($ids);
	if ( $access ) {
		$show = true;
		if ( !empty($requires) ) {
			$requirement = false;
			$requires_ids = explode(",", $requires);
			for ( $i = 0; $i < count($requires_ids); $i++ ) {
				if ( stms_member_has_access($member_id, $requires_ids[$i]) ) {
					$requirement = true;
				}
			}
			$show = ( $requirement ) ? true : false;
		}
		
		if ( !empty($excludes) ) {
			$excludes_ids = explode(",", $excludes);
			for ( $i = 0; $i < count($excludes_ids); $i++ ) {
				if ( stms_member_has_access($member_id, $excludes_ids[$i]) ) {
					$show = false;
					break;
				}
			}
		}
		
		if ( !empty($paysys) ) {
			$show = false;
			$paysys = explode(",", $paysys);
			if ( $payments = stms_get_member_payments($member_id) ) {
				foreach ( $payments as $py ) {
					if ( in_array($py->paysys, $paysys) ) {
						$show = true;
						break;
					}
				}
			}
		}
		
		$stms = get_option('stms_settings');
		if ( is_stms_admin() && stms_isset($stms['global']['admin_unlock']) == 1 )
			$show = true;
	
		if ( $show ) {
			$product_ids = explode(",", $ids);
			if ( count($product_ids) == 1 && $expired > 0 ) {
				if ( $expire = stms_check_shortcode_expired($ids, $expired, $expired_mode, $expired_upgrade_id) ) {
					$color = ( $expire['status'] == 'expired' ) ? '#cc0000' : 'inherit';
					return '<table class="insta-protect-table">'
					. '<tbody>'
					. '<tr><td colspan="2">' . ( ($expire['status'] == 'expired') ?  html_entity_decode($expired_msg) : do_shortcode($content) ) . '</td></tr>'
					. '<tr><td style="width:50%"><small><strong>Date of purchase:</strong></small><br /><strong>' . date("F d, Y", $expire['purchase_date']) . '</strong></td>'
					. '<td style="width:50%"><strong>You have <span style="color:' . $color . '">' . $expire['remaining'] . ' Days</span></strong><br /><small><strong>of upgrades remaining</strong></small></td></tr>'
					. '<tr><td colspan="2"><strong> Current Status: <span style="color:' . $color . '">' . ucwords($expire['status']) . '</span></strong></td></tr>'
					. '</tbody>'
					. '</table>';
				}
			}
			return do_shortcode($content);
		} else
			return '';
	} else {
		if ( !empty($errormsg) )
			return instamember_macro(html_entity_decode($errormsg));
		else
			return '';
	}
}

add_shortcode('insta_link', 'insta_link_handler');
function insta_link_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'id' => '',
		'payment' => 'paypal',
		'target' => '_self',
	), $atts ) );
	
	$payment_url = stms_format_url(get_bloginfo('url'), 'insta_pay=' . $payment . '&id=' . $id);
	$type = 'text';
	if ( $path_info = @pathinfo($content) ) {
		$ext = stms_isset($path_info['extension'], '');
		$allowed = array('jpg', 'jpeg', 'gif', 'bmp', 'png');
		$type = ( in_array(strtolower($ext), $allowed) ) ? 'image' : 'text';
	}
	
	$label = ( $type == 'image' ) ? "<img src='{$content}' border='0' class='insta-img' />" : $content;
	$html = '';
	$html .= "<a href='{$payment_url}' target='{$target}'>{$label}</a>";
	return $html;
}

add_shortcode('insta_pay_button', 'insta_pay_button_handler');
function insta_pay_button_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'id' => '',
		'color' => 'orange_addtocart',
		'payment' => 'paypal',
		'target' => '_self',
	), $atts ) );
	
	$parts = explode("_", $color);
	$btn_color = stms_isset($parts[0], 'color');
	$btn_label = stms_isset($parts[1], 'addtocart');
	$images_url = STMS_URL . "images/buttons/big-{$btn_color}-{$btn_label}-circle.png";
	$payment_url = stms_format_url(get_bloginfo('url'), 'insta_pay=' . $payment . '&id=' . $id);
	
	$html = '';
	$html .= "<a href='{$payment_url}' target='{$target}'><img src='{$images_url}' border='0' class='insta-img' /></a>";
	return $html;
}

add_shortcode('insta_autoswitch_button', 'insta_autoswitch_button_handler');
function insta_autoswitch_button_handler( $atts, $content = null ) {
	extract( shortcode_atts( array(
		'id' => '',
		'color' => 'orange_addtocart'
	), $atts ) );
	
	$id = intval($id);
	$parts = explode("_", $color);
	$color = stms_isset($parts[0], 'color');
	$label = stms_isset($parts[1], 'addtocart');
	$js_url = stms_format_url(get_bloginfo('url'), "insta_mode=js_auto&id={$id}&text={$label}&color={$color}");
	
	$html = '';
	$html .= "<script type='text/javascript' src='{$js_url}'></script>";
	return $html;
}

if ( !function_exists('sdisc_shortcode_handler') ) add_shortcode('insta_discount', 'insta_discount_handler');
function insta_discount_handler( $atts, $content = null ) {
	return 'The "Discount Module" isn\'t available. You need to install InstaMember Developer version first.';
}

add_filter('the_content', 'instamember_macro');
function instamember_macro( $content ) {
	if ( !is_singular() )
		return $content;
	
	global $post;
	$permalink = get_permalink($post->ID);
	$login_url = stms_login_url($permalink);
	$content = str_replace('%login_url%', $login_url, $content);
	
	return $content;
}

$injs_flow = false;
$inflow_count = 0;
function instamember_flowplayer( $video ) {
	if ( !is_array($video) )
		return '';
	
	$url = stms_isset($video['video_url']);
	if ( $url == '' )
		return '';
	
	$width = ( stms_isset($video['video_width']) != '' ) ? $video['video_width'] : '640';
	$height = ( stms_isset($video['video_height']) != '' ) ? $video['video_height'] : '360';
	$autoplay = ( stms_isset($video['autoplay']) == 1 ) ? 'true' : 'false';
	$autohide = ( stms_isset($video['autohide']) == 1 ) ? 'true' : 'false';
	$disable_control = ( stms_isset($video['disable_control']) == 1 ) ? true : false;
	$splash = trim( stms_isset($video['video_scr']) );

	if ( defined('OPL_URL') ) {
		global $ezjs_flow, $ezflow_count;
		$flowjs_count = $ezjs_flow;
		$flow_count = $ezflow_count;
	} else {
		global $injs_flow, $inflow_count;
		$flowjs_count = $injs_flow;
		$flow_count = $inflow_count;
	}
	
	$flow = '';
	$flowpath = ( defined('OPL_URL') ) ? OPL_URL : STMS_URL;
	if ( !$flowjs_count ) {
		if ( !file_exists($flowpath . 'js/flowplayer/flowplayer-3.2.12.min.js') ) $flowpath = STMS_URL;
		$flow .= '<script type="text/javascript" src="' . $flowpath . 'js/flowplayer/flowplayer-3.2.12.min.js"></script>' . "\n";
		$flow .= '<script type="text/javascript" src="' . $flowpath . 'js/flowplayer/flowplayer.ipad-3.2.12.min.js"></script>' . "\n";
		
		if ( defined('OPL_URL') )
			$GLOBALS['ezjs_flow'] = true;
		else
			$GLOBALS['injs_flow'] = true;
	}
	
	$flow .= '<div id="instaflowplayer' . $flow_count . '" style="display:inline-block; width:' . $width . 'px; height:' . $height . 'px"></div>' . "\n\n";
	$flow .= '
	<script type="text/javascript">
	flowplayer("instaflowplayer' . $flow_count . '", "' . $flowpath . 'js/flowplayer/flowplayer-3.2.16.swf", {
		playlist: [
	';

	if ( $splash != '' ) {
		$flow .= '
			{
				url: "' . $splash . '", 
				scaling: "orig"
			},
		';
	}

	$flow .= '			
			{
				url: "' . $url . '",
				scaling: "scale",
				autoPlay: ' . $autoplay . ',
				autoBuffering: true,
				autoMute: false
			}
		],
	';

	if ( $disable_control ) {
		$flow .= '
		plugins: {
				controls: null
		}
		';
	} else {
		$flow .= '
		plugins: {
			controls: { 
				autoHide: ' . $autohide . '
			}
		}
		';
	}

	$flow .= '}).ipad();' . "\n" . '</script>';
	
	if ( defined('OPL_URL') )
		$GLOBALS['ezflow_count'] += 1;
	else
		$GLOBALS['inflow_count'] += 1;
	return $flow;
}

function instamember_jwplayer( $video ) {
	if ( !is_array($video) )
		return '';
	
	$url = stms_isset($video['video_url']);
	if ( $url == '' )
		return '';
	
	$width = ( stms_isset($video['video_width']) != '' ) ? $video['video_width'] : '640';
	$height = ( stms_isset($video['video_height']) != '' ) ? $video['video_height'] : '360';
	$autoplay = ( stms_isset($video['autoplay']) == 1 ) ? 'true' : 'false';
	$autohide = ( stms_isset($video['autohide']) == 1 ) ? true : false;
	$disable_control = ( stms_isset($video['disable_control']) == 1 ) ? true : false;
	$splash = trim( stms_isset($video['video_scr']) );
	
	$jw = '';
	$jwplayer = ( defined('JWPLAYER_FILES_URL') ) ? JWPLAYER_FILES_URL . '/player/player.swf' : get_option('jw_player_location');
	$jwplayerjs = str_replace('player.swf', 'jwplayer.js', $jwplayer);
	
	$is_mp4 = ( stms_isset($video['mp4'], 0) == 1 ) ? true : false;
	if ( $is_mp4 ) {
			$poster = ( $splash != '' ) ? ' poster="' . $splash . '" ' : ' ';
			$plugin_version = ( defined('JWP6') ) ? get_option(JWP6 . 'plugin_version') : 5;
			$jw .= '
			<video' . $poster . 'id="stms-jwplayer" height="' . $height . '" width="' . $width . '">
				<source src="' . $url . '" type="video/mp4">
			</video>
			
			';
			
			if ( $plugin_version >= 6 ) {
				$jw .= '
				<script type="text/javascript">
				if ( typeof(jQuery) == "function" ){ (function($){ $.fn.fitVids=function(){}})(jQuery)};
				jwplayer("stms-jwplayer").setup({ "aspectratio":null, "width":' . $width . ',"height":' . $height . ', "primary":"flash", "file":"' . $url . '",
				';
				if ( $disable_control ) {
					$jw .= '
						controlbar: "none",
					';
				} else if ( !$disable_control && $autohide ) {
					$jw .= 'controlbar: "over",';
				} else if ( !$disable_control && !$autohide ) {
					$jw .= 'controlbar: "bottom",';
				}
				$jw .= '
						autostart: ' . $autoplay . '
	    		});
				</script>
				';
			} else {
				$jw .= '
				<script type="text/javascript">
	    		jwplayer("stms-jwplayer").setup({
	        		flashplayer: "' . $jwplayer . '",		
				';
	
				if ( $disable_control ) {
					$jw .= '
						controlbar: "none",
					';
				} else if ( !$disable_control && $autohide ) {
					$jw .= 'controlbar: "over",';
				} else if ( !$disable_control && !$autohide ) {
					$jw .= 'controlbar: "bottom",';
				}
				$jw .= '
						autostart: ' . $autoplay . '
	    		});
				</script>
				';
			}
			
	} else {
			$jw .= '<div id="stms-jwplayer">Please install JW Player Plugin For WordPress.</div>';
			$jw .= "
			<script type='text/javascript'>
    		var so = new SWFObject('{$jwplayer}', 'playerID', '{$width}', '{$height}', '9');
    		so.addParam('allowfullscreen','true');
    		so.addParam('allowscriptaccess','always');
			so.addParam('wmode','transparent');
    		so.addVariable('file', '{$url}');
 			so.addVariable('autostart', {$autoplay});
 			";
			$jw .= ( $disable_control ) ? "so.addVariable('controlbar','none');" : '';
			$jw .= ( !$disable_control && $autohide ) ? "so.addVariable('controlbar','over')" : '';
			$jw .= ( !$disable_control && !$autohide ) ? "so.addVariable('controlbar','bottom')" : '';
			$jw .= ( $autoplay == 'false' && $splash != '' ) ? "so.addVariable('image','{$splash}');" : '';
			$jw .= "
			so.addVariable('stretching','fill');
    		so.write('stms-jwplayer');
			</script>
			";
	}
	
	return $jw;
}

function instamember_jwplayer_installed() {
	if ( defined('JWPLAYER_FILES_URL') ) return true;
	if ( defined('JWP6_PLUGIN_FILE') ) return true; // compatibility with JW Player 6 (2.1.0)
	if ( get_option('jw_player_location') === TRUE ) return true;
	return false;
}
