<?php
/*
Payment Name: PayPal Standard
Payment Key: paypal
One Time: yes
Recurring: yes
Flow Type: normal
*/

add_action('stms_global_payment_paypal', 'stms_paypal_global_settings');
function stms_paypal_global_settings($stms) {
?>
	<table class="form-table">
	<tr valign="top">
		<th scope="row"><label style="cursor:default"><strong>Enable PayPal Standard</strong></label></th>
		<td><div class="stms-toggle-btn"></div><input type="checkbox" name="stms_sets[is_paypal]" id="stms_pay_paypal" class="stms-toggle-checkbox stms-toggle-switch" data-blockname="paypal-paysets" value="1" <?php if ( stms_isset($stms['payments']['is_paypal']) == 1 ) echo 'checked="checked"'; ?> /></td>
	</tr>
	<tr valign="top" class="paypal-paysets">
		<th scope="row" style="text-align:right"><label for="stms_paypal_addr"><?php _e('Primary Paypal Email', 'stms'); ?></label></th>
		<td>
			<input class="widefat stms-field" type="text" name="stms_sets[paypal][addr]" id="stms_paypal_addr" value="<?php echo stms_isset($stms['payments']['paypal']['addr']); ?>" />
			<div class="stms-desc">If you have registered more than one email addresses under one PayPal account, then please enter your primary/main email address here. Otherwise, the integration won't work.</div>
		</td>
	</tr>
	<tr valign="top" class="paypal-paysets">
		<th scope="row" style="text-align:right"><label for="stms_paypal_ipn"><?php _e('IPN URL (optional)', 'stms'); ?></label></th>
		<td><input class="widefat stms-field" type="text" id="stms_paypal_ipn" value="<?php echo stms_get_ipnurl('paypal'); ?>" />
			<div class="stms-desc">Please login to your PayPal account, and navigate to "Profile -> My Selling Tools -> Instant Payment Notifications" and make sure that the PayPal IPN feature is turned on. If not, please turn on the IPN feature and enter this URL into the setting.</div>
		</td>
	</tr>
	<tr valign="top" class="paypal-paysets">
		<th scope="row" style="text-align:right"><label for="stms_sandbox_paypal_addr"><?php _e('Sandbox Paypal Email (optional)', 'stms'); ?></label></th>
		<td><input class="widefat stms-field" type="text" name="stms_sets[paypal][sandbox_addr]" id="stms_sandbox_paypal_addr" value="<?php echo stms_isset($stms['payments']['paypal']['sandbox_addr']); ?>" />
			<div class="stms-desc">For PayPal testing purpose only. You can get a PayPal Sandbox account <a href="http://developer.paypal.com/" target="_blank">here</a>.</div>
		</td>
	</tr>
	<tr valign="top" class="paypal-paysets">
		<th scope="row" style="text-align:right; padding-bottom:20px"><label for="stms_sandbox_paypal_mode"><?php _e('Enable PayPal Sandbox Mode', 'stms'); ?></label></th>
		<td style="padding-bottom:20px"><span class='stms-fancy-chkbox'></span><input type="checkbox" name="stms_sets[paypal][sandbox_mode]" id="stms_sandbox_paypal_mode" class="stms-chkbox" value="1" <?php if ( stms_isset($stms['payments']['paypal']['sandbox_mode']) == 1 ) echo 'checked="checked"'; ?> /></td>
	</tr>
	
	</table>
<?php
}

add_action('stms_ipn_paypal', 'stms_paypal_ipn_handler');
function stms_paypal_ipn_handler() {
	// STEP 1: Read POST data
	$raw_posts = explode('&', file_get_contents('php://input'));
	$posts = array();
	foreach ( $raw_posts as $keyval ) {
		$keyval = explode ('=', $keyval);
		  if ( count($keyval) == 2 )
			$posts[$keyval[0]] = urldecode($keyval[1]);
	}
	
	$get_magic_quotes_exists = false;
	$req = 'cmd=_notify-validate';
	if ( function_exists('get_magic_quotes_gpc') )
		$get_magic_quotes_exists = true;

	foreach ( $posts as $key => $value ) {
		if ( $get_magic_quotes_exists == true && get_magic_quotes_gpc() == 1 )
			$value = urlencode(stripslashes($value));
		else
			$value = urlencode($value);
		$req .= "&$key=$value";
	}

	// STEP 2: Post IPN data back to paypal to validate
	$stms = get_option('stms_settings');
	$paypal_url = ( stms_isset($stms['payments']['paypal']['sandbox_mode']) == 1 ) ? 'https://www.sandbox.paypal.com/cgi-bin/webscr' : 'https://www.paypal.com/cgi-bin/webscr';
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $paypal_url);
	curl_setopt($ch, CURLOPT_VERBOSE, 1);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_POST, 1);
	curl_setopt($ch, CURLOPT_POSTFIELDS, $req);
	$res = curl_exec($ch);

	curl_close($ch);

	// STEP 3: Inspect IPN validation result and act accordingly
 	$process = false;
	if ( strcmp($res, "VERIFIED") == 0 ) {
		$process = stms_process_paypal($_POST);
		return $process;
	}
}

function stms_process_paypal( $vars ) {
	if ( !is_array($vars) )
		return false;
	
	$stms = get_option('stms_settings');
    $paypal_email = ( stms_isset($stms['payments']['paypal']['sandbox_mode']) == 1 ) ? trim(stms_isset($stms['payments']['paypal']['sandbox_addr'])) : trim(stms_isset($stms['payments']['paypal']['addr']));
	$receiver_check = false;
	if ( strcasecmp($paypal_email, $vars['receiver_email']) == 0 )
		$receiver_check = true;
	
	if ( !$receiver_check )
		stms_error_handler("Receiver email is different. Expected: {$paypal_email}, Received: {$vars['receiver_email']}", 0, '', 'ipn');
	
	if ( in_array($vars['payment_status'], array('Reversed', 'Reversal', 'Refunded')) ) {
		stms_paypal_refund($vars);
		return true;
	}
	
	if ( $vars['txn_type'] == 'cart' )
		$process = stms_paypal_cart($vars);
	else if ( $vars['txn_type'] == 'web_accept' )
		$process = stms_paypal_onetime($vars);
	else
		$process = stms_paypal_recurring($vars);
}

function stms_paypal_cart( $vars ) {
	if ( !is_array($vars) )
		stms_error_handler('No Postdata variables from Paypal for one-time transaction. PayPal IPN ignored.', 0, '', 'ipn');
	
	if ( $vars['payment_status'] != 'Completed' )
		stms_error_handler('Payment status is "' . $vars['payment_status'] . '". PayPal IPN ignored.', 0, '', 'ipn');
	
	$stms = get_option('stms_settings');
	$currency = stms_isset($stms['global']['currency'], 'USD');
	
	$first_name = stms_isset($vars['first_name']);
	$last_name = stms_isset($vars['last_name']);
	$payer_email = stms_isset($vars['payer_email']);
	$txn_id = stms_isset($vars['txn_id']);
	$subscr_id = stms_isset($vars['subscr_id']);
	$invoice = stms_isset($vars['invoice']);
	$custom_vars = base64_decode($vars['custom']);
	parse_str($custom_vars, $custom);
	
	//$product_id = ( isset($custom['product_id']) ) ? intval($custom['product_id']) : 0;
	$member_id = ( isset($custom['member_id']) ) ? intval($custom['member_id']) : 0;
	$tx_ids = stms_isset($custom['tx_ids']);
	
	// process upsells items
	if ( is_array($tx_ids) && count($tx_ids) > 0 ) {
		$split_fee = number_format(@($payment_fee/count($tx_ids)), 2);
		foreach ( $tx_ids as $trx_id ) {
			if ( $py = stms_get_payment($trx_id) ) {
				$args = array(
					'product_id' => $py->product_id,
					'member_id' => $member_id,
					'payment_type' => $py->payment_type,
					'paysys' => 'paypal',
					'receipt' => ( $py->payment_type == 'recurr' ) ? $subscr_id : $txn_id,
					'fee' => $split_fee,
					'txn_id' => $txn_id,
					'paid' => 1,
					'is_rebill' => 0,
					'status' => 'completed',
					'expired_date' => stms_get_expired($py->product_id, true),
				);
				stms_complete_payment($trx_id, $args);
			}
		}
	}
	exit;
}

function stms_paypal_onetime( $vars ) {
	if ( !is_array($vars) )
		stms_error_handler('No Postdata variables from Paypal for one-time transaction. PayPal IPN ignored.', 0, '', 'ipn');
	
	if ( $vars['payment_status'] != 'Completed' )
		stms_error_handler('Payment status is "' . $vars['payment_status'] . '". PayPal IPN ignored.', 0, '', 'ipn');

	$stms = get_option('stms_settings');
	$currency = stms_isset($stms['global']['currency'], 'USD');
	$txn_type = stms_isset($vars['txn_type']);
	$first_name = stms_isset($vars['first_name']);
	$last_name = stms_isset($vars['last_name']);
	$item_name = stms_isset($vars['item_name']);
	$item_number = stms_isset($vars['item_number']);
	$payment_status = stms_isset($vars['payment_status']);
	$payment_amount = stms_isset($vars['mc_gross']);
	$payment_gross = stms_isset($vars['payment_gross']);
	$payment_fee = stms_isset($vars['mc_fee']);
	$payment_currency = stms_isset($vars['mc_currency']);
	$txn_id = stms_isset($vars['txn_id']);
	$receiver_email = stms_isset($vars['receiver_email']);
	$payer_email = stms_isset($vars['payer_email']);
	$payer_id = stms_isset($vars['payer_id']);
	$invoice = stms_isset($vars['invoice']);
	$custom_vars = base64_decode($vars['custom']);
	parse_str($custom_vars, $custom);
	
	$product_id = ( isset($custom['product_id']) ) ? intval($custom['product_id']) : 0;
	$member_id = ( isset($custom['member_id']) ) ? intval($custom['member_id']) : 0;
	$discount = ( isset($custom['discount']) ) ? $custom['discount'] : 0.00;
	$amount = ( $payment_gross > 0.0 ) ? $payment_gross : $payment_amount;

	$aff_id = 0;
	$aff_meta = stms_get_member_meta($member_id, 'aff_id');
	if ( !empty($aff_meta) && $member_id != $aff_meta )
		$aff_id = $aff_meta;
		
	$tx_id = (int) $invoice;
	if ( !$payment = stms_get_payment($tx_id) ) {
		$args = array(
			'product_id' => $product_id,
			'member_id' => $member_id,
			'aff_id' => $aff_id,
			'payment_type' => 'onetime',
			'paysys' => 'paypal',
			'currency' => $currency,
			'amount' => $amount,
			'fee' => $payment_fee,
			'status' => 'pending',
			'date' => stms_time()
		);
		$tx_id = stms_add_pending_payment($args);
		$payment = stms_get_payment($tx_id);
	}
	
	if ( $amount != $payment->amount )
		stms_error_handler('Received amount is ' . $amount . ', it should be ' . $payment->amount, $member_id);
	
	$product = stms_get_product($product_id);
	$check = get_transient('stms_payment_check_' . $tx_id);
	if ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) {
		if ( stms_isset($check['acc_type']) == 'temp' )
			$member_id = stms_auto_register_customer($tx_id, $payer_email, $first_name, $last_name, $payer_id);
	}
	
	// update payer id...
	$update = stms_update_member($member_id, array('paypal_id' => $payer_id));
	$args = array(
		'product_id' => $product_id,
		'member_id' => $member_id,
		'paysys' => 'paypal',
		'receipt' => $txn_id,
		'currency' => $currency,
		'amount' => $amount,
		'fee' => $payment_fee,
		'txn_id' => $txn_id,
		'paid' => 1,
		'is_rebill' => 0,
		'status' => 'completed',
		'expired_date' => 0,
	);

	stms_complete_payment($tx_id, $args);
	exit;
}

function stms_paypal_recurring($vars) {
	if ( !is_array($vars) )
		stms_error_handler('No Postdata variables from Paypal for subscription transaction. PayPal IPN ignored.', 0, '', 'ipn');

	$stms = get_option('stms_settings');
	$currency = stms_isset($stms['global']['currency'], 'USD');
	$txn_type = stms_isset($vars['txn_type']);
	$item_name = stms_isset($vars['item_name']);
	$item_number = stms_isset($vars['item_number']);
	$first_name = stms_isset($vars['first_name']);
	$last_name = stms_isset($vars['last_name']);
	$payment_status = stms_isset($vars['payment_status']);
	$payment_amount = stms_isset($vars['mc_gross']);
	$payment_gross = stms_isset($vars['payment_gross']);
	$payment_fee = stms_isset($vars['mc_fee']);
	$payment_currency = stms_isset($vars['mc_currency']);
	$txn_id = stms_isset($vars['txn_id']);
	$subscr_id = stms_isset($vars['subscr_id']);
	$receiver_email = stms_isset($vars['receiver_email']);
	$payer_email = stms_isset($vars['payer_email']);
	$payer_id = stms_isset($vars['payer_id']);
	$invoice = stms_isset($vars['invoice']);
	$custom_vars = base64_decode($vars['custom']);
	parse_str($custom_vars, $custom);
	
	$product_id = ( isset($custom['product_id']) ) ? intval($custom['product_id']) : 0;
	$member_id = ( isset($custom['member_id']) ) ? intval($custom['member_id']) : 0;
	
	$aff_id = 0;
	$aff_meta = stms_get_member_meta($member_id, 'aff_id');
	if ( !empty($aff_meta) && $member_id != $aff_meta )
		$aff_id = $aff_meta;
	
	$product = stms_get_product($product_id);
	switch ( $txn_type ) {
        case 'subscr_signup':
			$tx_id = (int) $invoice;
			if ( stms_is_payment_locked($tx_id) )
				stms_error_handler('Another instance of the same IPN is already running.', $member_id, 0, '', 'ipn');
			
			if ( isset($vars['mc_amount1']) && $vars['mc_amount1'] == 0.00 ) {
				// Free trial period
				if ( !$payment = stms_get_payment($tx_id) ) {
					$args = array(
						'product_id' => $product_id,
						'member_id' => $member_id,
						'aff_id' => $aff_id,
						'payment_type' => 'recurr',
						'paysys' => 'paypal',
						'currency' => $currency,
						'amount' => $payment_amount,
						'status' => 'pending',
						'date' => stms_time()
					);
					$tx_id = stms_add_pending_payment( $args );
					$payment = stms_get_payment($tx_id);
				}
				
				$check = get_transient('stms_payment_check_' . $tx_id);
				if ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) {
					if ( stms_isset($check['acc_type']) == 'temp' )
						$member_id = stms_auto_register_customer($tx_id, $payer_email, $first_name, $last_name, $payer_id);
				}
				
				$update = stms_update_member($member_id, array('paypal_id' => $payer_id));
				$args = array(
					'product_id' => $product_id,
					'member_id' => $member_id,
					'payment_type' => 'recurr',
					'paysys' => 'paypal',
					'receipt' => $subscr_id,
					'currency' => $currency,
					'amount' => 0.00,
					'fee' => 0.00,
					'txn_id' => $txn_id,
					'paid' => 1,
					'is_rebill' => 0,
					'status' => 'completed',
					'expired_date' => stms_get_expired($payment->product_id, true),
				);
				stms_complete_payment($tx_id, $args);
			}
			break;
			
		case 'subscr_payment':
	        if ( $payment_status != 'Completed' )
	            stms_error_handler('Payment status is "' . $vars['payment_status'] . '". PayPal IPN ignored.', $member_id, $subscr_id, 'ipn');
			
			$tx_id = stms_paypal_last_id($subscr_id, $invoice);
			stms_payment_get_lock($tx_id);
			if ( !$tx_id ) {
				$args = array(
					'product_id' => $product_id,
					'member_id' => $member_id,
					'aff_id' => $aff_id,
					'payment_type' => 'recurr',
					'paysys' => 'paypal',
					'currency' => $currency,
					'amount' => $payment_amount,
					'fee' => $payment_fee,
					'status' => 'pending',
					'date' => stms_time(),
				);
				$tx_id = stms_add_pending_payment($product_id, $args);
				$force_tx_id = $tx_id;
			}
			$payment = stms_get_payment($tx_id);
			if ( !$payment )
				stms_error_handler('Cannot find transaction record for this IPN (tx_id:' . $tx_id . ').', $member_id, $subscr_id, 'ipn');
			
			$product = stms_get_product($payment->product_id);
			$is_first = ( $tx_id == $invoice && $payment->txn_id == '' ) ? true : false;
			if ( $is_first ) { // first payment to subscription
				if ( !isset($custom['tx_ids']) ) {
					$check = get_transient('stms_payment_check_' . $tx_id);
					if ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) {
						if ( stms_isset($check['acc_type']) == 'temp' )
							$member_id = stms_auto_register_customer($tx_id, $payer_email, $first_name, $last_name);
					}
				}
				
				$split_fee = $payment_fee;
				$first_fee = $payment_fee;
				if ( isset($custom['tx_ids']) ) {
					$tx_ids = $custom['tx_ids'];
					if ( is_array($tx_ids) && count($tx_ids) > 0 ) {
						$count = count($tx_ids);
						if ( $product->is_trial == 1 && $product->t_price == 0.00 ) {
							$count = $count - 1;
							$split_fee = number_format(@($payment_fee/$count), 2);
							$first_fee = 0.00;
						} else {
							$split_fee = number_format(@($payment_fee/$count), 2);
							$first_fee = $split_fee;
						}
					}
				}
				
				$update = stms_update_member($member_id, array('paypal_id' => $payer_id));
				$args = array(
					'product_id' => $product->ID,
					'member_id' => $member_id,
					'payment_type' => 'recurr',
					'paysys' => 'paypal',
					'receipt' => $subscr_id,
					'currency' => $currency,
					'fee' => $first_fee,
					'txn_id' => $txn_id,
					'paid' => 1,
					'is_rebill' => 0,
					'status' => 'completed',
					'expired_date' => stms_get_expired($product->ID, true),
				);
				stms_complete_payment($tx_id, $args);
				
				// process upsells items
				if ( isset($custom['tx_ids']) ) {
					$tx_ids = $custom['tx_ids'];
					if ( is_array($tx_ids) && count($tx_ids) > 0 ) {
						foreach ( $tx_ids as $trx_id ) {
							if ( $trx_id != $invoice ) {
								if ( $py = stms_get_payment($trx_id) ) {
									$args = array(
										'product_id' => $py->product_id,
										'member_id' => $member_id,
										'payment_type' => $py->payment_type,
										'paysys' => 'paypal',
										'receipt' => ( $py->payment_type == 'recurr' ) ? $subscr_id : $txn_id,
										'fee' => $split_fee,
										'txn_id' => $txn_id,
										'paid' => 1,
										'is_rebill' => 0,
										'status' => 'completed',
										'expired_date' => stms_get_expired($py->product_id, true),
									);
									stms_complete_payment($trx_id, $args);
								}
							}
						}
					}
				} 
				
				stms_payment_release_lock($tx_id);
			} else { // rebill
				if ( $force_tx_id != $tx_id ) {
					$new_data = array();
					$new_data['status'] = 'expired';
					$new_data['expired_date'] = strtotime("5 minutes ago", stms_time());
					stms_update_payment($tx_id, $new_data);
					wp_cache_delete($member_id, 'stms_member_payments'); // delete cache
					stms_payment_release_lock($tx_id);
				}
				
				$args = array(
					'product_id' => $payment->product_id,
					'member_id'=> $payment->member_id,
					'aff_id' => $payment->aff_id,
					'payment_type' => 'recurr',
					'paysys'=> 'paypal',
					'receipt'=> $subscr_id,
					'currency'=> $currency,
					'amount'=> $payment_amount,
					'fee' => $payment_fee,
					'txn_id' => $txn_id,
					'paid'=> 1,
					'is_rebill' => 1,
					'status'=> 'completed',
					'date'=> stms_time(),
					'expired_date'=> stms_get_expired($payment->product_id, false)
				);
				
				if ( $force_tx_id == $tx_id )
					$tx_id = stms_complete_payment($tx_id, $args);
				else
					$tx_id = stms_add_payment($args);
			}
			break;
			
		case 'subscr_cancel':
			$tx_id = stms_paypal_last_id($subscr_id, $invoice);
			if ( !$tx_id )
				stms_error_handler('Cannot find transaction record for this IPN (tx_id:' . $tx_id . ').', $member_id, $subscr_id, 'ipn');
			
			$new_data = array();
			$new_data['status'] = 'cancelled';
			stms_update_payment($tx_id, $new_data);
			wp_cache_delete($member_id, 'stms_member_payments'); // delete cache
        	
			do_action('stms_subscr_cancelled', $tx_id);
        	break;
	}
	exit;
}

function stms_paypal_refund($vars) {
	global $wpdb;
	if ( isset($vars['custom']) ) {
		$custom_vars = base64_decode($vars['custom']);
		parse_str($custom_vars, $custom);
		
		$product_id = ( isset($custom['product_id']) ) ? intval($custom['product_id']) : 0;
		$member_id = ( isset($custom['member_id']) ) ? intval($custom['member_id']) : 0;
		if ( !$payments = stms_get_member_payments($member_id) )
			return false;
		foreach ( $payments as $payment ) {
			if ( $payment->product_id == $product_id && $payment->status == 'completed' ) {
				stms_refund_payment($payment->tx_id);
			}
		}
	} else if ( !isset($vars['custom']) && isset($vars['parent_txn_id']) ) {
		$txn_id = $vars['parent_txn_id'];
		$sql = "SELECT * FROM `{$wpdb->prefix}stms_transactions` WHERE `txn_id` = %s";
		if ( !$payments = $wpdb->get_results($wpdb->prepare($sql, $txn_id)) )
			return false;
		
		foreach ( $payments as $payment ) {
			if ( $payment->status == 'completed' ) {
				stms_refund_payment($payment->tx_id);
			}
		}
	} else
		return false;
}

function stms_paypal_last_id( $receipt, $invoice ) {
	global $wpdb;
	
	$invoice = (int) $invoice;
	$last_payment = $wpdb->get_row(
		$wpdb->prepare("SELECT tx_id FROM `{$wpdb->prefix}stms_transactions`
			WHERE `receipt` = %s OR `tx_id` = %d
			ORDER BY tx_id DESC LIMIT 1",
			$receipt, $invoice
		)
	);
	
	if ( !$last_payment )
		return false;
	
	return $last_payment->tx_id;
}

add_action('insta_flow_checkout_paypal', 'stms_paypal_upsell_checkout');
function stms_paypal_upsell_checkout( $flow_id ) {
	$stms = get_option('stms_settings');
	$sflow = get_option('stms_sales_flow');
	$flow = stms_isset($sflow[$flow_id]);
	$front_id = stms_isset($flow['front_id']);
	$currency = stms_isset($stms['global']['currency'], 'USD');
	$home_url = get_bloginfo('url');
	
	if ( !$member_id = stms_get_current_member_id() ) {
		wp_redirect($home_url);
		exit;
	}
		
	if ( empty($stms['payments']['paypal']['addr']) )
		wp_die('Configuration Error: No PayPal Email Address Provided.');
	
	if ( !$items = sflow_get_cart_items($flow_id) )
		wp_die('ERROR: No items found in the shopping cart.');
	
	$product = stms_get_product($front_id);
	$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_bloginfo('url');
	$cancel_url = ( !empty($product->info['thankuri']) ) ? $product->info['thankuri'] : $home_url;
	
	$recurring = 0;
	$total_amount = 0;
	$desc = '';
	$rebill = 0;
	$rebill_times = 0;
	$tx_ids = 'tx_fill=none';
	$mode = '';
	$cycle = '';
	$t_mode = '';
	$t_cycle = '';
	$invoice = '';
	$product_id = '';
	foreach ( $items as $item ) {
		if ( $pr = stms_get_product($item->product_id) ) {
			$tx_ids .= "&tx_ids[]={$item->tx_id}";
			$price = ( $pr->type == 'recurr' && $pr->is_trial == 1 ) ? $pr->t_price : $pr->price;
			if ( $pr->type == 'recurr' )
				$recurring += 1;
			
			$total_amount += $price;
			$desc .= stms_cleanup($pr->title) . ' - ' . $price . ' ' . $currency . '; ';
			if ( $pr->type == 'recurr' ) {
				$product_id = $item->product_id;
				$invoice = $item->tx_id;
				$rebill += $pr->price;
				if ( $pr->rebill_times > 0 )
					$rebill_times = ( $pr->is_trial == 1 ) ? $pr->rebill_times : $pr->rebill_times - 1;
				else
					$rebill_times = 0;
				$t_dur = maybe_unserialize($pr->t_duration);
				$dur = maybe_unserialize($pr->duration);
				$cycle = $dur['num'];
				$mode = $dur['mode'];
				$t_cycle = $dur['num'];
				$t_mode = $dur['mode'];
				if ( $pr->is_trial == 1 ) {
					$t_cycle = $t_dur['num'];
					$t_mode = $t_dur['mode'];
				}
			}
		}
	}
	
	if ( $recurring > 1 )
		wp_die("ERROR: Invalid Shopping Cart Items");
	
	$paypal_email = ( stms_isset($stms['payments']['paypal']['sandbox_mode']) == 1 ) ? trim(stms_isset($stms['payments']['paypal']['sandbox_addr'])) : trim(stms_isset($stms['payments']['paypal']['addr']));
	$paypal_url = ( stms_isset($stms['payments']['paypal']['sandbox_mode']) == 1 ) ? 'https://www.sandbox.paypal.com/cgi-bin/webscr' : 'https://www.paypal.com/cgi-bin/webscr';
	$return_url = stms_format_url($stms['global']['magic'], 'insta_mode=all_access&flow_id=' . $flow_id);
	$cancel_url = $cancel_url;
	$ipn_url = stms_get_ipnurl('paypal');
	
	// custom vars
	$custom_vars = base64_encode("{$tx_ids}&product_id={$product_id}&member_id={$member_id}");
	$cmd = ( !$recurring ) ? '_cart' : '_xclick-subscriptions';
	$bn  = ( !$recurring ) ? 'InstaMember_AddToCart_WPS_US' : 'InstaMember_Subscribe_WPS_US';
?>
<!DOCTYPE html>
<!--[if IE 6]>
<html id="ie6" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 7]>
<html id="ie7" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 8]>
<html id="ie8" <?php language_attributes(); ?>>
<![endif]-->
<!--[if !(IE 6) | !(IE 7) | !(IE 8)  ]><!-->
<html <?php language_attributes(); ?>>
<!--<![endif]-->
<head>
<meta charset="<?php bloginfo( 'charset' ); ?>" />
<title>Secure Payment Redirection ...</title>
<!--[if lt IE 9]>
	<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->
<link rel="profile" href="http://gmpg.org/xfn/11" />
<meta name="HandheldFriendly" content="True" />
<meta name="viewport" content="width=device-width, initial-scale=1"/>
<!-- Adding "maximum-scale=1" fixes the Mobile Safari auto-zoom bug: http://filamentgroup.com/examples/iosScaleBug/ -->
<link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Droid+Sans" type="text/css" media="all" />
<style type="text/css">
body {
	margin:0;
	padding:0;
	font-size:14px;
	font-family:'Droid Sans', sans-serif;
	background-color:#F5F5F5;
	color:#808080;
}
#redirect_msg {
	margin:25px auto 0;
	padding:15px 20px;
	width:40%;
	background-color:#FFF;
	text-align:center;
}
</style>
</head>
<body>
<div id="redirect_msg">
<p><strong>Please wait while we redirect you to a secure payment page...</strong></p>
<p><img src="<?php echo STMS_URL; ?>images/admin-loader.gif" border="0" /></p>

<form action="<?php echo $paypal_url; ?>" method="post" id="stms-paypal-form">
<input type="hidden" name="cmd" value="<?php echo $cmd; ?>">
<input type="hidden" name="business" value="<?php echo $paypal_email; ?>">
<?php if ( !$recurring ) : ?>
<input type="hidden" name="upload" value="1">
<?php
$i = 0;
foreach ( $items as $item ) {
	if ( $pr = stms_get_product($item->product_id) ) {
		$i++;
		echo '<input type="hidden" name="item_name_' . $i . '" value="' . stms_cleanup($pr->title) . '">';
  		echo '<input type="hidden" name="amount_' . $i . '" value="' . stms_cleanup($pr->price) . '">';
	}
}
else: ?>
<input type="hidden" name="invoice" value="<?php echo $invoice; ?>">
<input type="hidden" name="item_name" value="<?php echo $desc; ?>">
<input type="hidden" name="item_number" value="<?php echo $flow_id; ?>">
<?php
	echo '<input type="hidden" name="a1" value="' . $total_amount . '">';
	echo '<input type="hidden" name="p1" value="' . $t_cycle . '">';
	echo '<input type="hidden" name="t1" value="' . strtoupper($t_mode) . '">';
	echo '<input type="hidden" name="a3" value="' . $rebill . '">' . "\n";
	echo '<input type="hidden" name="p3" value="' . $cycle . '">' . "\n";
	echo '<input type="hidden" name="t3" value="' . strtoupper($mode) . '">' . "\n";
	echo '<input type="hidden" name="sra" value="1">' . "\n";
	echo '<input type="hidden" name="src" value="1">' . "\n";

	if ( $rebill_times > 0 ) {
		echo '<input type="hidden" name="srt" value="' . $rebill_times . '">' . "\n";
	}
endif;
?>
<input type="hidden" name="no_shipping" value="1">
<input type="hidden" name="return" value="<?php echo $return_url; ?>">
<input type="hidden" name="cancel_return" value="<?php echo $cancel_url; ?>">
<input type="hidden" name="no_note" value="1">
<input type="hidden" name="shipping" value="0.00">
<input type="hidden" name="currency_code" value="<?php echo $currency; ?>">
<input type="hidden" name="lc" value="US">
<input type="hidden" name="bn" value="<?php echo $bn; ?>">
<input type="hidden" name="rm" value="2">

<input type="hidden" name="notify_url" value="<?php echo $ipn_url; ?>">
<input type="hidden" name="custom" value="<?php echo $custom_vars; ?>">
</form>

<script>
	setTimeout(function redir(){
		document.getElementById('stms-paypal-form').submit();
	}, 2000);
</script>

</div>
</body>
</html>
<?php
}

add_action('stms_payment_proceed_paypal', 'stms_paypal_proceed_handler');
function stms_paypal_proceed_handler( $tx_id ) {
	if ( !$payment = stms_get_payment($tx_id) )
		return false;
	
	if ( $payment->paysys != 'paypal' )
		return false;
	
	$payment_url = stms_format_url(get_bloginfo('siteurl'), 'insta_mode=payment_redirect&trans_id=' . $tx_id);
	if ( isset($_GET['lc']) )
		$payment_url .= '&lc=' . $_GET['lc'];
	
	wp_redirect($payment_url);
	exit;
}

add_action('stms_payment_redirect_body', 'stms_paypal_form');
function stms_paypal_form( $payment ) {
	if ( !is_object($payment) || $payment->paysys != 'paypal' )
		return false;
	
	$stms = get_option('stms_settings');
	if ( empty($stms['payments']['paypal']['addr']) )
		wp_die('Configuration Error: No PayPal Email Address Provided.');
	
	if ( !$product = stms_get_product($payment->product_id) )
		wp_die('Configuration Error: Product Not Found.');
	
	$paypal_email = ( stms_isset($stms['payments']['paypal']['sandbox_mode']) == 1 ) ? trim(stms_isset($stms['payments']['paypal']['sandbox_addr'])) : trim(stms_isset($stms['payments']['paypal']['addr']));
	$paypal_url = ( stms_isset($stms['payments']['paypal']['sandbox_mode']) == 1 ) ? 'https://www.sandbox.paypal.com/cgi-bin/webscr' : 'https://www.paypal.com/cgi-bin/webscr';
	$currency = stms_isset($stms['global']['currency']);
	$return_url = stms_format_url($stms['global']['magic'], 'insta_mode=thanks&processor=paypal');
	$cancel_url = ( !empty($product->info['salesuri']) ) ? $product->info['salesuri'] : get_bloginfo('url');
	$ipn_url = stms_get_ipnurl('paypal');
	
	$check = get_transient('stms_payment_check_' . $payment->tx_id);
	
	$lc = ( isset($_GET['lc']) ) ? urldecode($_GET['lc']) : 'US';
	$custom_vars = base64_encode("product_id={$product->ID}&member_id={$payment->member_id}&discount=" . stms_isset($check['discount'], '0.00'));
	$cmd = ( $product->type == 'onetime' ) ? '_xclick' : '_xclick-subscriptions';
	$bn  = ( $product->type == 'onetime' ) ? 'InstaMember_BuyNow_WPS_' . $lc : 'InstaMember_Subscribe_WPS_' . $lc;
?>
<form action="<?php echo $paypal_url; ?>" method="post" id="stms-paypal-form">
<input type="hidden" name="cmd" value="<?php echo $cmd; ?>">
<input type="hidden" name="business" value="<?php echo $paypal_email; ?>">
<input type="hidden" name="item_name" value="<?php echo wptexturize(stms_cleanup($payment->title)); ?>">
<input type="hidden" name="item_number" value="<?php echo $payment->product_id; ?>">
<input type="hidden" name="invoice" value="<?php echo $payment->tx_id; ?>">
<?php
if ( $product->type == 'recurr' ) {
	if ( $product->is_trial == 1 ) {
		$t_dur = maybe_unserialize($product->t_duration);
		$trial_period = (int) $t_dur['num'];
		$trial_mode = $t_dur['mode'];
		if ( $trial_period > 0 ) {
			echo '<input type="hidden" name="a1" value="' . stms_isset($check['amount'], $product->t_price) . '">';
			echo '<input type="hidden" name="p1" value="' . $trial_period . '">';
			echo '<input type="hidden" name="t1" value="' . strtoupper($trial_mode) . '">';
		}
	}
	
	if ( $product->is_trial != 1 && $check['amount'] != $product->price ) {
		echo '<input type="hidden" name="a1" value="' . stms_isset($check['amount'], $product->price) . '">';
		echo '<input type="hidden" name="p1" value="' . $period . '">';
		echo '<input type="hidden" name="t1" value="' . strtoupper($mode) . '">';
	}
	
	$dur = maybe_unserialize($product->duration);
	$period = (int) $dur['num'];
	$mode = $dur['mode'];
	$subs_price = ( $product->is_trial == 1 ) ? $product->price : stms_isset($check['amount'], $product->price);
	echo '<input type="hidden" name="a3" value="' . $subs_price . '">' . "\n";
	echo '<input type="hidden" name="p3" value="' . $period . '">' . "\n";
	echo '<input type="hidden" name="t3" value="' . strtoupper($mode) . '">' . "\n";
	echo '<input type="hidden" name="sra" value="1">' . "\n";
	echo '<input type="hidden" name="src" value="1">' . "\n";

	if ( $product->rebill_times > 0 ) {
		echo '<input type="hidden" name="srt" value="' . $product->rebill_times . '">' . "\n";
	}
} else {
	echo '<input type="hidden" name="amount" value="' . stms_isset($check['amount'], $product->price) . '">' . "\n";
}
?>
<input type="hidden" name="no_shipping" value="1" />
<input type="hidden" name="return" value="<?php echo $return_url; ?>" />
<input type="hidden" name="cancel_return" value="<?php echo $cancel_url; ?>" />
<input type="hidden" name="no_note" value="1" />
<input type="hidden" name="shipping" value="0.00" />
<input type="hidden" name="currency_code" value="<?php echo $currency; ?>" />
<input type="hidden" name="lc" value="<?php echo $lc; ?>" />
<input type="hidden" name="bn" value="<?php echo $bn; ?>" />
<input type="hidden" name="rm" value="2" />

<input type="hidden" name="notify_url" value="<?php echo $ipn_url; ?>" />
<input type="hidden" name="custom" value="<?php echo $custom_vars; ?>" />
</form>

<script>
	setTimeout(function redir(){
		document.getElementById('stms-paypal-form').submit();
	}, 2000);
</script>
<?php
}

//add_action('stms_payment_thanks_paypal', 'stms_paypal_thanks_handler');
function stms_paypal_thanks_handler($tx_id) {
	if ( !$payment = stms_get_payment($tx_id) )
		wp_die('ERROR: Cannot Find Payment Data.');
	
	$stms = get_option('stms_settings');
	$support = ( filter_var(stms_isset($stms['global']['support']), FILTER_VALIDATE_EMAIL) ) ? 'mailto:' . $stms['global']['support'] : $stms['global']['support'];
	if ( $payment->status == 'completed' && $payment->paid == 1 ) {
		// Payment success... proceed to next step
		if ( !$product = stms_get_product($payment->product_id) )
			wp_die('ERROR: Payment cannot be processed. Product Unknown.');
		
		$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : '';
		$redirect_url = ( filter_var($product->info['thankuri'], FILTER_VALIDATE_URL) ) ? $product->info['thankuri'] : $home_url;
		if ( $redirect_url == '' )
			$redirect_url = ( !empty($stms['global']['home']) ) ? get_permalink($stms['global']['home']) : get_bloginfo('siteurl');
				
		delete_transient('stms_paypal_retries_' . $tx_id);
		if ( $product->info['steps'] == '2,1' ) {
			if ( is_stmsmember_logged_in() ) {
				$check = get_transient('stms_payment_check_' . $tx_id);
				if ( stms_isset($check['acc_type']) == 'real' ) {
					wp_redirect($redirect_url);
					exit;
				}
			}
			// proceed to registration
			$session = base64_encode("{$product->ID}|paypal|direct");
			$insta_token = stms_generate_secret(16, true, true );
			$token_data = array( 'token' => $insta_token, 'tx_id' => $payment->tx_id);
			set_transient('insta_token_' . $insta_token, $token_data, 60*60);
			$_reg_url = stms_get_handler_url('registration');
			$reg_url = stms_format_url($_reg_url, 'key=' . $session . '&insta_token=' . $insta_token);
			wp_redirect($reg_url);
			exit;
		} else {	
			wp_redirect($redirect_url);
			exit;
		}
	} else {
		$retries = get_transient('stms_paypal_retries_' . $tx_id);
		$thanks_url = stms_format_url($stms['global']['magic'], 'insta_mode=thanks&processor=paypal');
		if ( $retries >= 10 ) {
			$page_title = 'Payment Error';
			$refresh = '';
			$message = '
				<p style="text-align:center;"><strong>Payment Error</strong></p>
				<p>We\'re sorry, but there\'s a problem in validating your payment. Please contact our <a href="' . $support . '">support</a> to resolve this issue.</p>
			';
		} else {
			$count = $retries + 1;
			set_transient('stms_paypal_retries_' . $tx_id, $count, 60*15);
			$page_title = 'Waiting For Payment Confirmation...';
			$refresh = '<meta http-equiv="refresh" content="5;URL=\'' . $thanks_url . '\'" />';
			$message = '
				<p style="text-align:center;"><strong>Please Wait...</strong></p>
				<p style="text-align:center;"><img src="' . STMS_URL . 'images/admin-loader.gif" border="0" /></p>
				<p>This page will be auto-refreshed every 5 seconds until we receive a valid payment notification from PayPal. Please do NOT close this page.</p>
			';
		}
?>
<!DOCTYPE html>
<!--[if IE 6]>
<html id="ie6" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 7]>
<html id="ie7" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 8]>
<html id="ie8" <?php language_attributes(); ?>>
<![endif]-->
<!--[if !(IE 6) | !(IE 7) | !(IE 8)  ]><!-->
<html <?php language_attributes(); ?>>
<!--<![endif]-->
<head>
<meta charset="<?php bloginfo( 'charset' ); ?>" />
<title><?php echo $page_title; ?></title>
<?php echo $refresh; ?>
<!--[if lt IE 9]>
	<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->
<link rel="profile" href="http://gmpg.org/xfn/11" />
<meta name="HandheldFriendly" content="True" />
<meta name="viewport" content="width=device-width, initial-scale=1"/>
<!-- Adding "maximum-scale=1" fixes the Mobile Safari auto-zoom bug: http://filamentgroup.com/examples/iosScaleBug/ -->
<link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Droid+Sans" type="text/css" media="all" />
<style type="text/css">
body {
	margin:0;
	padding:0;
	font-size:14px;
	font-family:'Droid Sans', sans-serif;
	background-color:#F5F5F5;
	color:#808080;
}
#redirect_msg {
	margin:25px auto 0;
	padding:15px 20px;
	width:40%;
	background-color:#FFF;
}
</style>
</head>
<body>
<div id="redirect_msg">
	<?php echo $message; ?>
</div>
</body>
</html>
<?php
	exit;
	}
}
