<?php
/*
Payment Name: Payza
Payment Key: payza
One Time: yes
Recurring: yes
Flow Type: normal
*/

add_action('stms_global_payment_payza', 'stms_payza_global_settings');
function stms_payza_global_settings($stms) {
?>
	<table class="form-table">
	<tr valign="top">
		<th scope="row"><label style="cursor:default"><strong>Enable Payza</strong></label></th>
		<td><div class="stms-toggle-btn"></div><input type="checkbox" name="stms_sets[is_payza]" id="stms_pay_payza" class="stms-toggle-checkbox stms-toggle-switch" data-blockname="payza-paysets" value="1" <?php if ( stms_isset($stms['payments']['is_payza']) == 1 ) echo 'checked="checked"'; ?> /></td>
	</tr>
	<tr valign="top" class="payza-paysets">
		<th scope="row" style="text-align:right"><label for="stms_payza_addr"><?php _e('Primary Payza Email', 'stms'); ?></label></th>
		<td><input class="widefat stms-field" type="text" name="stms_sets[payza][addr]" id="stms_payza_addr" value="<?php echo stms_isset($stms['payments']['payza']['addr']); ?>" /></td>
	</tr>
	<tr valign="top" class="payza-paysets">
		<th scope="row" style="text-align:right"><label for="stms_sandbox_payza_addr"><?php _e('Sandbox Payza Email (optional)', 'stms'); ?></label></th>
		<td><input class="widefat stms-field" type="text" name="stms_sets[payza][sandbox_addr]" id="stms_sandbox_payza_addr" value="<?php echo stms_isset($stms['payments']['payza']['sandbox_addr']); ?>" />
			<div class="stms-desc">For Payza testing purpose only. You can get a Payza Sandbox account <a href="https://sandbox.payza.com/" target="_blank">here</a>.</div>
		</td>
	</tr>
	<tr valign="top" class="payza-paysets">
		<th scope="row" style="text-align:right"><label for="stms_sandbox_payza_mode"><?php _e('Enable Payza Sandbox Mode', 'stms'); ?></label></th>
		<td><span class='stms-fancy-chkbox'></span><input type="checkbox" name="stms_sets[payza][sandbox_mode]" id="stms_sandbox_payza_mode" class="stms-chkbox" value="1" <?php if ( stms_isset($stms['payments']['payza']['sandbox_mode']) == 1 ) echo 'checked="checked"'; ?> /></td>
	</tr>
	</table>
<?php
}

add_action('stms_ipn_payza', 'stms_payza_ipn_handler');
function stms_payza_ipn_handler() {
	$stms = get_option('stms_settings');
	$token = 'token=' . urlencode($_POST['token']);
	$payza_url = ( stms_isset($stms['payments']['payza']['sandbox_mode']) == 1 ) ? 'https://sandbox.Payza.com/sandbox/IPN2.ashx' : 'https://secure.payza.com/ipn2.ashx';
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $payza_url);
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	curl_setopt($ch, CURLOPT_HEADER, false);
	curl_setopt($ch, CURLOPT_TIMEOUT, 60);
	curl_setopt($ch, CURLOPT_POST, 1);
	curl_setopt($ch, CURLOPT_POSTFIELDS, $token);
	$response = curl_exec($ch);

	curl_close($ch);

 	$process = false;
	if ( strlen($response) > 0 ) {
		if ( urldecode($response) == "INVALID TOKEN" ) {
			stms_error_handler('Invalid Payza Token. Payza Alert Ignored.', 0, $vars['ap_referencenumber'], 'ipn');
		} else {
			$data = rawurldecode($response);
			parse_str($data, $vars);
			$process = stms_process_payza($vars);
			return $process;
		}
	}
}

function stms_process_payza( $vars ) {
	if ( !is_array($vars) )
		return false;
	
	$stms = get_option('stms_settings');
    $payza_email = stms_isset($stms['payments']['payza']);
	$receiver_check = false;
	if ( strcasecmp($payza_email, $vars['ap_merchant']) == 0 )
		$receiver_check = true;
	
	if ( !$receiver_check )
		stms_error_handler('Payza receiver email didn\'t match. Payza Alert ignored.', 0, $vars['ap_referencenumber'], 'ipn');
	
	if ( $vars['ap_purchasetype'] == 'subscription' )
		$process = stms_payza_recurring($vars);
	else if ( $vars['ap_purchasetype'] == 'item-goods' )
		$process = stms_payza_onetime($vars);
	else if ( $vars['ap_purchasetype'] == 'item' )
		$process = stms_payza_cart($vars);
}

function stms_payza_cart( $vars ) {
	if ( !is_array($vars) )
		stms_error_handler('No Postdata variables from Payza for one-time transaction. Payza IPN ignored.', 0, '', 'ipn');
	
	if ( $vars['ap_status'] != 'Success' )
		stms_error_handler('Payment status is "' . $vars['ap_status'] . '". Payza IPN ignored.', 0, $vars['ap_referencenumber'], 'ipn');
	
	$stms = get_option('stms_settings');
	$currency = stms_isset($stms['global']['currency'], 'USD');
	
	$custom_vars = base64_decode($vars['apc_1']);
	parse_str($custom_vars, $custom);
	
	$member_id = ( isset($custom['member_id']) ) ? intval($custom['member_id']) : 0;
	$tx_ids = stms_isset($custom['tx_ids']);
	
	// process upsells items
	if ( is_array($tx_ids) && count($tx_ids) > 0 ) {
		$split_fee = number_format(@($vars['ap_feeamount']/count($tx_ids)), 2);
		foreach ( $tx_ids as $trx_id ) {
			if ( $py = stms_get_payment($trx_id) ) {
				$args = array(
					'product_id' => $py->product_id,
					'member_id' => $member_id,
					'payment_type' => $py->payment_type,
					'paysys' => 'payza',
					'receipt' => ( $py->payment_type == 'recurr' ) ? stms_isset($vars['ap_subscriptionreferencenumber']) : $vars['ap_referencenumber'],
					'fee' => $split_fee,
					'txn_id' => $vars['ap_referencenumber'],
					'paid' => 1,
					'is_rebill' => 0,
					'status' => 'completed',
					'expired_date' => stms_get_expired($py->product_id, true),
				);
				stms_complete_payment($trx_id, $args);
			}
		}
	}
	exit;
}

function stms_payza_onetime( $vars ) {
	if ( !is_array($vars) )
		stms_error_handler('No Postdata variables from payza for one-time transaction. Payza IPN ignored.', 0, '', 'ipn');
	
	if ( $vars['ap_status'] != 'Success' )
		stms_error_handler('Payment status is "' . $vars['ap_status'] . '". Payza IPN ignored.', 0, $vars['ap_referencenumber'], 'ipn');

	$stms = get_option('stms_settings');
	$currency = stms_isset($stms['global']['currency'], 'USD');
	
	$invoice = stms_isset($vars['apc_1']);
	$custom_vars = base64_decode($vars['apc_2']);
	parse_str($custom_vars, $custom);
	
	$product_id = ( isset($custom['product_id']) ) ? intval($custom['product_id']) : 0;
	$member_id = ( isset($custom['member_id']) ) ? intval($custom['member_id']) : 0;
	
	$aff_id = 0;
	$aff_meta = stms_get_member_meta($member_id, 'aff_id');
	if ( !empty($aff_meta) && $member_id != $aff_meta )
		$aff_id = $aff_meta;
	
	$tx_id = (int) $invoice;
	if ( !$payment = stms_get_payment($tx_id) ) {
		$args = array(
			'product_id' => $product_id,
			'member_id' => $member_id,
			'aff_id' => $aff_id,
			'payment_type' => 'onetime',
			'paysys' => 'payza',
			'currency' => $currency,
			'amount' => $vars['ap_amount'],
			'fee' => $vars['ap_feeamount'],
			'status' => 'pending',
			'date' => stms_time()
		);
		$tx_id = stms_add_pending_payment($product_id, $args);
		$payment = stms_get_payment($tx_id);
	}
	
	if ( $vars['ap_amount'] != $payment->amount )
		stms_error_handler('Received amount is ' . $vars['ap_totalamount'] . ', it should be ' . $payment->amount, $member_id);
	
	$product = stms_get_product($product_id);
	$check = get_transient('stms_payment_check_' . $tx_id);
	if ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) {
		if ( stms_isset($check['acc_type']) == 'temp' )
			$member_id = stms_auto_register_customer($tx_id, $vars['ap_custemailaddress'], $vars['ap_custfirstname'], $vars['ap_custlastname']);
	}
	
	$args = array(
		'product_id' => $product_id,
		'member_id' => $member_id,
		'payment_type' => 'onetime',
		'paysys' => 'payza',
		'receipt' => $vars['ap_referencenumber'],
		'currency' => $currency,
		'amount' => $vars['ap_amount'],
		'fee' => $vars['ap_feeamount'],
		'txn_id' => $vars['ap_referencenumber'],
		'paid' => 1,
		'is_rebill' => 0,
		'status' => 'completed',
		'expired_date' => 0
	);

	stms_complete_payment($tx_id, $args);
	exit;
}

function stms_payza_recurring($vars) {
	if ( !is_array($vars) )
		stms_error_handler('No Postdata variables from Payza for subscription transaction. Payza IPN ignored.', 0, '', 'ipn');

	$stms = get_option('stms_settings');
	$currency = stms_isset($stms['global']['currency'], 'USD');
	$invoice = stms_isset($vars['apc_1']);
	$custom_vars = base64_decode($vars['apc_2']);
	parse_str($custom_vars, $custom);
	
	$product_id = ( isset($custom['product_id']) ) ? intval($custom['product_id']) : 0;
	$member_id = ( isset($custom['member_id']) ) ? intval($custom['member_id']) : 0;
	
	$aff_id = 0;
	$aff_meta = stms_get_member_meta($member_id, 'aff_id');
	if ( !empty($aff_meta) && $member_id != $aff_meta )
		$aff_id = $aff_meta;
	
	switch ( $vars['ap_status'] ) {
        case 'Subscription-Payment-Success':
			$tx_id = stms_payza_last_id($vars['ap_subscriptionreferencenumber'], $invoice);
			if ( !$payment = stms_get_payment($tx_id) ) {
				$args = array(
					'product_id' => $product_id,
					'member_id' => $member_id,
					'aff_id' => $aff_id,
					'payment_type' => 'recurr',
					'paysys' => 'payza',
					'currency' => $currency,
					'amount' => $vars['ap_totalamount'],
					'discount' => $discount,
					'status' => 'pending',
					'date' => stms_time(),
				);
				$tx_id = stms_add_pending_payment( $product_id, $args );
				$payment = stms_get_payment($tx_id);
				$force_tx_id = $tx_id;
			}
			
			if ( isset($vars['ap_trialamount']) && $vars['ap_trialamount'] == 0 ) {
				// Free trial period
				$check = get_transient('stms_payment_check_' . $tx_id);
				if ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) {
					if ( stms_isset($check['acc_type']) == 'temp' )
						$member_id = stms_auto_register_customer($tx_id, $vars['ap_custemailaddress'], $vars['ap_custfirstname'], $vars['ap_custlastname']);
				}
				$args = array(
					'product_id' => $payment->product_id,
					'member_id' => $member_id,
					'payment_type' => 'recurr',
					'paysys' => 'payza',
					'receipt' => stms_isset($vars['ap_subscriptionreferencenumber']),
					'currency' => $currency,
					'amount' => 0.00,
					'fee' => 0.00,
					'discount' => 0.00,
					'txn_id' => $vars['ap_referencenumber'],
					'paid' => 1,
					'is_rebill' => 0,
					'status' => 'completed',
					'expired_date' => stms_get_expired($payment->product_id, true)
				);
				stms_complete_payment($tx_id, $args);
				return true;
			}
            
			$product = stms_get_product($payment->product_id);
			if ( $invoice == $tx_id && $payment->txn_id == '' ) { // New payment...
				if ( !isset($custom['tx_ids']) ) {
					$check = get_transient('stms_payment_check_' . $tx_id);
					if ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) {
						if ( stms_isset($check['acc_type']) == 'temp' )
							$member_id = stms_auto_register_customer($tx_id, $vars['ap_custemailaddress'], $vars['ap_custfirstname'], $vars['ap_custlastname']);
					}
				}

				$split_fee = $vars['ap_feeamount'];
				$first_fee = $vars['ap_feeamount'];
				if ( isset($custom['tx_ids']) ) {
					$tx_ids = $custom['tx_ids'];
					if ( is_array($tx_ids) && count($tx_ids) > 0 ) {
						$count = count($tx_ids);
						if ( $product->is_trial == 1 && $product->t_price == 0.00 ) {
							$count = $count - 1;
							$split_fee = number_format(@($vars['ap_feeamount']/$count), 2);
							$first_fee = 0.00;
						} else {
							$split_fee = number_format(@($vars['ap_feeamount']/$count), 2);
							$first_fee = $split_fee;
						}
					}
				}
				
				$args = array(
					'product_id' => $product->ID,
					'member_id' => $member_id,
					'payment_type' => 'recurr',
					'paysys' => 'payza',
					'receipt' => stms_isset($vars['ap_subscriptionreferencenumber']),
					'currency' => $currency,
					'amount' => ( $product->is_trial == 1 ) ? $product->t_price : $product->price,
					'fee' => $first_fee,
					'txn_id' => $vars['ap_referencenumber'],
					'paid' => 1,
					'is_rebill' => 0,
					'status' => 'completed',
					'expired_date' => stms_get_expired($product->ID, true),

				);
				stms_complete_payment($tx_id, $args);
				
				// process upsells items
				if ( isset($custom['tx_ids']) ) {
					$tx_ids = $custom['tx_ids'];
					if ( is_array($tx_ids) && count($tx_ids) > 0 ) {
						foreach ( $tx_ids as $trx_id ) {
							if ( $trx_id != $invoice ) {
								if ( $py = stms_get_payment($trx_id) ) {
									$args = array(
										'product_id' => $py->product_id,
										'member_id' => $member_id,
										'payment_type' => $py->payment_type,
										'paysys' => 'payza',
										'receipt' => ( $py->payment_type == 'recurr' ) ? stms_isset($vars['ap_subscriptionreferencenumber']) : $vars['ap_referencenumber'],
										'fee' => $split_fee,
										'txn_id' => $vars['ap_referencenumber'],
										'paid' => 1,
										'is_rebill' => 0,
										'status' => 'completed',
										'expired_date' => stms_get_expired($py->product_id, true),
									);
									stms_complete_payment($trx_id, $args);
								}
							}
						}
					}
				}
			} else { // Rebill
				if ( $force_tx_id != $tx_id ) {
					$new_data = array();
					$new_data['status'] = 'expired';
					$new_data['expired_date'] = strtotime("5 minutes ago", stms_time());
					stms_update_payment($tx_id, $new_data);
					wp_cache_delete($member_id, 'stms_member_payments'); // delete cache
				}
				
				$args = array(
						'product_id' => $payment->product_id,
						'member_id' => $payment->member_id,
						'payment_type' => 'recurr',
						'paysys' => 'payza',
						'receipt' => stms_isset($vars['ap_subscriptionreferencenumber']),
						'currency' => $currency,
						'amount' => $vars['ap_totalamount'],
						'fee' => $vars['ap_feeamount'],
						'txn_id' => $vars['ap_referencenumber'],
						'paid'=> 1,
						'is_rebill' => 1,
						'status'=> 'completed',
						'date'=> stms_time(),
						'expired_date'=> stms_get_expired($payment->product_id, false)
					);
				$new_tx_id = stms_add_payment($args);
			}
			break;
			
		case 'Subscription-Canceled':
			$tx_id = stms_payza_last_id($vars['ap_subscriptionreferencenumber'], $invoice);
			if ( !$tx_id )
				stms_error_handler('Cannot find transaction record for this IPN (tx_id:' . $tx_id . ').', 0, '', 'ipn');
			
			$payment = stms_get_payment($tx_id);
			$new_data = array();
			$new_data['status'] = 'cancelled';
			stms_update_payment($tx_id, $new_data );
        	wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
        	
			do_action('stms_subscr_cancelled', $tx_id);
        	break;
	}
	exit;
}

function stms_payza_last_id($receipt, $invoice) {
	global $wpdb;
	
	$invoice = (int) $invoice;
	$last_payment = $wpdb->get_row(
		$wpdb->prepare("SELECT tx_id FROM `{$wpdb->prefix}stms_transactions`
			WHERE `receipt` = %s OR `tx_id` = %d
			ORDER BY tx_id, date DESC",
			$receipt,
			$invoice
		)
	);
	
	if ( !$last_payment )
		return false;
	
	return $last_payment->tx_id;
}

add_action('insta_flow_checkout_payza', 'stms_payza_upsell_checkout');
function stms_payza_upsell_checkout( $flow_id ) {
	if ( !function_exists('sflow_get_cart_items') )
		wp_die('ERROR: Invalid Operation.');
	
	$stms = get_option('stms_settings');
	$sflow = get_option('stms_sales_flow');
	$flow = stms_isset($sflow[$flow_id]);
	$front_id = stms_isset($flow['front_id']);
	$currency = stms_isset($stms['global']['currency'], 'USD');
	$home_url = get_bloginfo('url');
	
	if ( !$member_id = stms_get_current_member_id() ) {
		wp_redirect($home_url);
		exit;
	}
		
	if ( empty($stms['payments']['payza']['addr']) )
		wp_die('Configuration Error: No Payza Email Address Provided.');
	
	if ( !$items = sflow_get_cart_items($flow_id) )
		wp_die('ERROR: No items found in the shopping cart.');
	
	$product = stms_get_product($front_id);
	$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_bloginfo('url');
	$cancel_url = ( !empty($product->info['thankuri']) ) ? $product->info['thankuri'] : $home_url;
	
	$recurring = 0;
	$total_amount = 0;
	$desc = '';
	$rebill = 0;
	$rebill_times = 0;
	$tx_ids = 'tx_fill=none';
	$mode = '';
	$cycle = '';
	$t_mode = '';
	$t_cycle = '';
	$invoice = '';
	$product_id = '';
	foreach ( $items as $item ) {
		if ( $pr = stms_get_product($item->product_id) ) {
			$tx_ids .= "&tx_ids[]={$item->tx_id}";
			$price = ( $pr->type == 'recurr' && $pr->is_trial == 1 ) ? $pr->t_price : $pr->price;
			if ( $pr->type == 'recurr' )
				$recurring += 1;
			
			$total_amount += $price;
			$desc .= stms_cleanup($pr->title) . ' - ' . $price . ' ' . $currency . '; ';
			if ( $pr->type == 'recurr' ) {
				$product_id = $item->product_id;
				$invoice = $item->tx_id;
				$rebill += $pr->price;
				if ( $pr->rebill_times > 0 )
					$rebill_times = ( $pr->is_trial == 1 ) ? $pr->rebill_times : $pr->rebill_times - 1;
				else
					$rebill_times = 0;
				
				$t_dur = maybe_unserialize($pr->t_duration);
				$dur = maybe_unserialize($pr->duration);
				$cycle = $dur['num'];
				$mode = $dur['mode'];
				$t_cycle = $dur['num'];
				$t_mode = $dur['mode'];
				if ( $pr->is_trial == 1 ) {
					$t_cycle = $t_dur['num'];
					$t_mode = $t_dur['mode'];
				}
			}
		}
	}
	
	if ( $recurring > 1 )
		wp_die("ERROR: Invalid Shopping Cart Items");
	
	$payza_email = ( stms_isset($stms['payments']['payza']['sandbox_mode']) == 1 ) ? trim(stms_isset($stms['payments']['payza']['sandbox_addr'])) : trim(stms_isset($stms['payments']['payza']['addr']));
	$payza_url = ( stms_isset($stms['payments']['payza']['sandbox_mode']) == 1 ) ? 'https://sandbox.payza.com/sandbox/payprocess.aspx ' : 'https://secure.payza.com/checkout';
	$return_url = stms_format_url($stms['global']['magic'], 'insta_mode=all_access&flow_id=' . $flow_id);
	$cancel_url = $cancel_url;
	$alert_url = stms_get_ipnurl('payza');
	
	// custom vars
	$custom_vars = base64_encode("{$tx_ids}&product_id={$product_id}&member_id={$member_id}");
?>
<!DOCTYPE html>
<!--[if IE 6]>
<html id="ie6" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 7]>
<html id="ie7" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 8]>
<html id="ie8" <?php language_attributes(); ?>>
<![endif]-->
<!--[if !(IE 6) | !(IE 7) | !(IE 8)  ]><!-->
<html <?php language_attributes(); ?>>
<!--<![endif]-->
<head>
<meta charset="<?php bloginfo( 'charset' ); ?>" />
<title>Secure Payment Redirection ...</title>
<!--[if lt IE 9]>
	<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->
<link rel="profile" href="http://gmpg.org/xfn/11" />
<meta name="HandheldFriendly" content="True" />
<meta name="viewport" content="width=device-width, initial-scale=1"/>
<!-- Adding "maximum-scale=1" fixes the Mobile Safari auto-zoom bug: http://filamentgroup.com/examples/iosScaleBug/ -->
<link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Droid+Sans" type="text/css" media="all" />
<style type="text/css">
body {
	margin:0;
	padding:0;
	font-size:14px;
	font-family:'Droid Sans', sans-serif;
	background-color:#F5F5F5;
	color:#808080;
}
#redirect_msg {
	margin:25px auto 0;
	padding:15px 20px;
	width:40%;
	background-color:#FFF;
	text-align:center;
}
</style>
</head>
<body>
<div id="redirect_msg">
<p><strong>Please wait while we redirect you to a secure payment page...</strong></p>
<p><img src="<?php echo STMS_URL; ?>images/admin-loader.gif" border="0" /></p>

<form action="<?php echo $payza_url; ?>" method="post" id="stms-payza-form">
<input type="hidden" name="ap_merchant" value="<?php echo $payza_email; ?>" />
<input type="hidden" name="ap_currency" value="<?php echo $currency; ?>">
<?php if ( !$recurring ) : ?>
<input type="hidden" name="ap_purchasetype" value="item" />
<?php
echo '<input type="hidden" name="apc_1" value="' . $custom_vars . '" />' . "\n";
$i = 0;
foreach ( $items as $item ) {
	if ( $pr = stms_get_product($item->product_id) ) {
		$item_name = ( $i == 0 ) ? 'ap_itemname' : 'ap_itemname_' . $i;
		$item_amount = ( $i == 0 ) ? 'ap_amount' : 'ap_amount_' . $i;
		$item_code = ( $i == 0 ) ? 'ap_itemcode' : 'ap_itemcode_' . $i;
		$quantity = ( $i == 0 ) ? 'ap_quantity' : 'ap_quantity_' . $i;
		echo '<input type="hidden" name="' . $item_name . '" value="' . stms_cleanup($pr->title) . '">';
  		echo '<input type="hidden" name="' . $item_amount . '" value="' . stms_cleanup($pr->price) . '">';
		echo '<input type="hidden" name="' . $item_code . '" value="' . stms_cleanup($pr->ID) . '">';
		echo '<input type="hidden" name="' . $quantity . '" value="1">';
		$i++;
	}
}
else: ?>
<input type="hidden" name="ap_purchasetype" value="subscription" />
<input type="hidden" name="ap_itemname" value="Multiple Item(s)">
<input type="hidden" name="ap_itemcode" value="<?php echo $flow_id; ?>">
<input type="hidden" name="ap_description" value="<?php echo $desc; ?>">
<?php
	$trial_unit = ucwords(str_replace('s', '', stms_duration_str($t_cycle, $t_mode, false)));
	echo '<input type="hidden" name="ap_trialamount" value="' . $total_amount . '">';
	echo '<input type="hidden" name="ap_trialperiodlength" value="' . $t_cycle . '">';
	echo '<input type="hidden" name="ap_trialtimeunit" value="' . $trial_unit . '">';
	
	$unit = ucwords(str_replace('s', '', stms_duration_str($cycle, $mode, false)));
	echo '<input type="hidden" name="ap_periodlength" value="' . $cycle . '">' . "\n";
	echo '<input type="hidden" name="ap_timeunit" value="' . $unit . '">' . "\n";
	echo '<input type="hidden" name="ap_periodcount" value="' . $rebill_times . '">' . "\n";
	echo '<input type="hidden" name="apc_1" value="' . $invoice . '" />' . "\n";
	echo '<input type="hidden" name="apc_2" value="' . $custom_vars . '" />' . "\n";
endif;
?>
<input type="hidden" name="ap_returnurl" value="<?php echo $return_url; ?>">
<input type="hidden" name="ap_cancelurl" value="<?php echo $cancel_url; ?>">
<input type="hidden" name="ap_ipnversion" value="2">
<input type="hidden" name="ap_alerturl" value="<?php echo $alert_url; ?>">
</form>

<script>
	setTimeout(function redir(){
		document.getElementById('stms-payza-form').submit();
	}, 2000);
</script>

</div>
</body>
</html>
<?php
}

add_action('stms_payment_proceed_payza', 'stms_payza_proceed_handler');
function stms_payza_proceed_handler($tx_id) {
	if ( !$payment = stms_get_payment($tx_id) )
		return false;
	
	if ( $payment->paysys != 'payza' )
		return false;
	
	$payment_url = stms_format_url(get_bloginfo('url'), 'insta_mode=payment_redirect&trans_id=' . $tx_id);
	wp_redirect($payment_url);
	exit;
}

add_action('stms_payment_redirect_body', 'stms_payza_form');
function stms_payza_form($payment) {
	if ( !is_object($payment) || $payment->paysys != 'payza' )
		return false;
	
	$stms = get_option('stms_settings');
	if ( empty($stms['payments']['payza']['addr']) )
		wp_die('Configuration Error: No Payza Email Address Provided.');
	
	if ( !$product = stms_get_product($payment->product_id) )
		wp_die('Configuration Error: Product Not Found.');
	
	$payza_email = ( stms_isset($stms['payments']['payza']['sandbox_mode']) == 1 ) ? trim(stms_isset($stms['payments']['payza']['sandbox_addr'])) : trim(stms_isset($stms['payments']['payza']['addr']));
	$payza_url = ( stms_isset($stms['payments']['payza']['sandbox_mode']) == 1 ) ? 'https://sandbox.Payza.com/sandbox/payprocess.aspx' : 'https://secure.payza.com/checkout';

	$currency = stms_isset($stms['global']['currency']);
	$purchase_type = ( $product->type == 'onetime' ) ? 'item-goods' : 'subscription';
	$return_url = stms_format_url($stms['global']['magic'], 'insta_mode=thanks&processor=payza');
	$cancel_url = ( !empty($product->info['canceluri']) && filter_var($product->info['canceluri'], FILTER_VALIDATE_URL) ) ? $product->info['canceluri'] : get_bloginfo('url');
	$alert_url = stms_get_ipnurl('payza');
	
	$check = get_transient('stms_payment_check_' . $payment->tx_id);
	$price = ( $product->type == 'recurr' && $product->is_trial == 1 ) ? $product->price : stms_isset($check['amount'], $product->price);
	if ( $product->type == 'recurr' && $product->is_trial != 1 && $price != $product->price ) {
		$discounted_price = stms_isset($check['amount'], $product->price);	
		$price = $product->price;
	}
	
	// Custom vars
	$custom_vars = base64_encode("product_id={$product->ID}&member_id={$payment->member_id}&discount=" . stms_isset($check['discount'], '0.00'));
?>
<form action="<?php echo $payza_url; ?>" method="post" id="stms-payza-form">
<input type="hidden" name="ap_merchant" value="<?php echo $payza_email; ?>" />
<input type="hidden" name="ap_purchasetype" value="<?php echo $purchase_type; ?>" />
<input type="hidden" name="ap_itemname" value="<?php echo stms_cleanup($payment->title); ?>" />
<input type="hidden" name="ap_itemcode" value="<?php echo $payment->product_id; ?>" />

<input type="hidden" name="ap_currency" value="<?php echo $currency; ?>">
<?php
if ( $product->type == 'recurr' ) {
	if ( $product->is_trial == 1 ) {
		$t_dur = maybe_unserialize($product->t_duration);
		$trial_period = (int) $t_dur['num'];
		$trial_mode = ucwords(str_replace('s', '', stms_duration_str($t_dur['num'], $t_dur['mode'], false)));
		if ( $trial_period > 0 ) {
			echo '<input type="hidden" name="ap_trialamount" value="' . stms_isset($check['amount'], $product->t_price) . '">';
			echo '<input type="hidden" name="ap_trialperiodlength" value="' . $trial_period . '">';
			echo '<input type="hidden" name="ap_trialtimeunit" value="' . $trial_mode . '">';
		}
		
		if ( isset($discounted_price) ) {
			echo '<input type="hidden" name="ap_trialamount" value="' . $discounted_price . '">';
			echo '<input type="hidden" name="ap_trialperiodlength" value="' . $period . '">';
			echo '<input type="hidden" name="ap_trialtimeunit" value="' . $mode . '">';
		}
	}
	$dur = maybe_unserialize($product->duration);
	$period = (int) $dur['num'];
	$mode = ucwords(str_replace('s', '', stms_duration_str($dur['num'], $dur['mode'], false)));
	echo '<input type="hidden" name="ap_periodlength" value="' . $period . '">' . "\n";
	echo '<input type="hidden" name="ap_timeunit" value="' . $mode . '">' . "\n";
	echo '<input type="hidden" name="ap_periodcount" value="' . $product->rebill_times . '">' . "\n";
}
?>
<input type="hidden" name="ap_amount" value="<?php echo $price; ?>" />
<input type="hidden" name="ap_returnurl" value="<?php echo $return_url; ?>">
<input type="hidden" name="ap_cancelurl" value="<?php echo $cancel_url; ?>">
<input type="hidden" name="apc_1" value="<?php echo $payment->tx_id; ?>" />
<input type="hidden" name="apc_2" value="<?php echo $custom_vars; ?>" />
<input type="hidden" name="ap_ipnversion" value="2">
<input type="hidden" name="ap_alerturl" value="<?php echo $alert_url; ?>">
</form>

<script>
	setTimeout(function redir(){
		document.getElementById('stms-payza-form').submit();
	}, 2000);
</script>
<?php
}