<?php
/*
Payment Name: ClickBank
Payment Key: clickbank
One Time: yes
Recurring: yes
Flow Type: normal
*/

add_action('stms_thirdpay_settings_clickbank', 'stms_clickbank_product_settings', 10, 2);
function stms_clickbank_product_settings($product, $pay_settings) {
?>
	<tr valign="top" class="stms-clickbank-level">
		<th scope="row" colspan="2" style="border-top:1px solid #E5E5E5"><h3 style="color:#808080"><?php _e('Clickbank Product Integration Settings', 'stms'); ?></h3></th>
	</tr>
	<tr valign="top" class="stms-clickbank-level">
		<th scope="row" style="text-align:right"><label for="stms_cb_vendor"><strong><span style="color:#cc0000">*</span> <?php _e('Vendor ID/Username', 'stms'); ?></strong></label></th>
		<td>
			<input class="stms-field" type="text" name="stms_level[pay_settings][clickbank][vendor]" id="stms_cb_vendor" value="<?php echo stms_cleanup(stms_isset($pay_settings['clickbank']['vendor'])); ?>" style="width:140px" />
			<div class="stms-desc">It's the same with your Clickbank username</div>
		</td>
	</tr>
	<tr valign="top" class="stms-clickbank-level">
		<th scope="row" style="text-align:right"><label for="stms_cb_item"><strong><span style="color:#cc0000">*</span> <?php _e('Item ID#', 'stms'); ?></strong></label></th>
		<td>
			<input class="stms-field" type="text" name="stms_level[pay_settings][clickbank][item]" id="stms_cb_item" value="<?php echo stms_cleanup(stms_isset($pay_settings['clickbank']['item'])); ?>" style="width:80px" />
			<div class="stms-desc">The product item number in your Clickbank account (<code>Account Settings -> My Products</code>)</div>
		</td>
	</tr>
	<tr valign="top" class="stms-clickbank-level">
		<th scope="row" style="text-align:right"><label><strong><span style="color:#cc0000">*</span> <?php _e('Thank You Page (URL)', 'stms'); ?></strong></label></th>
		<td>
			<input class="widefat stms-field" type="text" value="<?php echo stms_get_returnurl('clickbank', false, true, $product->ID); ?>" readonly />
			<div class="stms-desc">You MUST set the thank you page URL of the product you created in Clickbank to this URL.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-clickbank-level">
		<th scope="row" style="text-align:right"><label for="stms_clickbank_payurl"><?php _e('ClickBank Payment URL', 'stms'); ?></label></th>
		<td>
			<input class="stms-field widefat" type="text" name="stms_level[pay_settings][clickbank][payurl]" id="stms_clickbank_payurl" value="<?php echo stms_sanitize($pay_settings['clickbank']['payurl']); ?>" />
			<div class="stms-desc">This is required only if you use the "Auto-Switch" payment button feature. If your ClickBank payment URL have some additional paramaters, such as tracking id or pitch-plus parameters, then you must include them in the URL.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-clickbank-level stms-last-integrate">
		<th scope="row" colspan="2"></th>
	</tr>
<?php
}

add_action('stms_global_payment_clickbank', 'stms_clickbank_global_settings');
function stms_clickbank_global_settings($stms) {
	$cb_secret = ( stms_isset($stms['payments']['clickbank']['secret']) != '' ) ? trim($stms['payments']['clickbank']['secret']) : strtoupper(stms_generate_secret(16, true, false));
?>
	<table class="form-table">
	<tr valign="top">
		<th scope="row"><label style="cursor:default"><strong>Enable ClickBank</strong></label></th>
		<td><div class="stms-toggle-btn"></div><input type="checkbox" name="stms_sets[is_clickbank]" id="stms_pay_clickbank" class="stms-toggle-checkbox stms-toggle-switch" data-blockname="cb-paysets" value="1" <?php if ( stms_isset($stms['payments']['is_clickbank']) == 1 ) echo 'checked="checked"'; ?> /></td>
	</tr>
	<tr valign="top" class="cb-paysets">
		<th scope="row" style="text-align:right"><label for="stms_cb_secret"><?php _e('Secret Key', 'stms'); ?></label></th>
		<td>
			<input class="widefat stms-field" type="text" name="stms_sets[clickbank][secret]" id="stms_cb_secret" value="<?php echo $cb_secret; ?>" />
			<div class="stms-desc">Please enter this secret key in your Clickbank account (<code>My Site -> Advanced Tools -> edit -> Secret Key</code>). <strong>Important:</strong> The secret key value MUST be the same as in your ClickBank account.</div>
		</td>
	</tr>
	<tr valign="top" class="cb-paysets">
		<th scope="row" style="text-align:right"><label><?php _e('Instant Notification URL', 'stms'); ?></label></th>
		<td>
			<input class="widefat stms-field" type="text" value="<?php echo stms_get_ipnurl('clickbank'); ?>" readonly />
			<div class="stms-desc">Please enter this URL in your Clickbank account (<code>Account Settings -> My Site -> Advanced Tools -> Edit</code>) and you MUST set the version to <code>2.1</code></div>
		</td>
	</tr>
	</table>
<?php
}

add_action('stms_ipn_clickbank', 'stms_clickbank_ipn_handler');
function stms_clickbank_ipn_handler() {
	$stms = get_option('stms_settings');
	$secret_key = stms_sanitize($stms['payments']['clickbank']['secret'], '', true);
	$pop = '';

	if ( !isset($_POST['cverify']) ) exit;
	
	$ipnFields = array();
    foreach ( $_POST as $key => $value ) {
        if ( $key == "cverify" ) {
            continue;
        }
        $ipnFields[] = $key;
    }
	
	sort($ipnFields);
    foreach ($ipnFields as $field) {
        // if Magic Quotes are enabled $_POST[$field] will need to be
        // un-escaped before being appended to $pop
        if ( get_magic_quotes_gpc() )
			$pop = $pop . stripslashes_deep($_POST[$field]) . "|";
		else
        	$pop = $pop . $_POST[$field] . "|";
    }
	$pop = $pop . $secret_key;
	
	if ( function_exists('mb_convert_encoding') )
    	$calcedVerify = sha1(mb_convert_encoding($pop, "UTF-8"));
	else
		$calcedVerify = sha1($pop);
	
    $calcedVerify = strtoupper(substr($calcedVerify,0,8));
	
	if ( $calcedVerify != $_POST['cverify'] )
		stms_error_handler("ClickBank INS Validation Failed. ClickBank INS Ignored.", 0, $_POST['cverify'], 'ipn');
	
	$process = stms_process_clickbank($_POST);
}

function stms_process_clickbank( $vars ) {
	global $wpdb;
	
	$stms = get_option('stms_settings');
	$ctransreceipt = explode("-", $vars['ctransreceipt']);
	$old_receipt = stms_isset($ctransreceipt[0], $vars['ctransreceipt']);
	$old_invoice = stms_clickbank_last_id($old_receipt);
	
	$custom_vars = $vars['cvendthru'];
	parse_str($custom_vars, $custom);
	
	$vendor = ( isset($vars['ctranspublisher']) && $vars['ctranspublisher'] != '' ) ? $vars['ctranspublisher'] : $vars['ctransvendor'];
	$vendor = stms_sanitize($vendor);
	$item = stms_sanitize($vars['cproditem']);
	$title = stms_sanitize($vars['cprodtitle']);
	
	// customer data...
	$first_name = strtolower(stms_sanitize($vars['ccustfirstname']));
	$last_name = strtolower(stms_sanitize($vars['ccustlastname']));
	$email = sanitize_email($vars['ccustemail']);
		
	if ( !$product = stms_find_product_by_externalid('clickbank', $item, $vendor) ) {
		if ( !$product = stms_get_product_bytitle($title) )
			stms_error_handler("Product not found. ClickBank INS Ignored.", 0, $vars['ctransreceipt'], 'ipn');
	}
	
	// Vendor check...
	$pay_settings = stms_get_paysettings($product->ID);
	$clickbank = stms_isset($pay_settings['clickbank']);
	$cb_vendor = stms_sanitize($clickbank['vendor']);
	
	if ( strtolower($cb_vendor) != strtolower($vendor) )
		stms_error_handler("Clickbank vendor not match. Expected value: {$cb_vendor}; Received: {$vendor}", 0, $vars['ctransreceipt'], 'ipn');
			
	switch ( $vars['ctransaction'] ) {
		case 'SALE':
        case 'TEST_SALE':
			if ( $lock = stms_clickbank_get_lock($product->ID, $vars['ctransreceipt']) )
				return false;
			
			// Create pending transaction first...
			$args = array(
				'product_id' => $product->ID,
				'member_id' => 0,
				'payment_type' => $product->type,
				'paysys' => 'clickbank',
				'currency' => stms_isset($stms['global']['currency'], 'USD'),
				'amount' => $product->price
			);
			$tx_id = stms_add_pending_payment($args);
			
			if ( !$member_id = stms_email_exists($email) ) {
				// check who's login on the other side... He/she is using a different email 
				$temp = stms_generate_temporary();
				
				$userdata = array();
				$userdata['member_id'] = 0;
				$userdata['username'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? stms_generate_username($vars['ccustemail'], $first_name, $last_name) : $temp['username'];
				$userdata['password'] = wp_generate_password(8, false);
				$userdata['email'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? $vars['ccustemail'] : $temp['email'];
				$userdata['first_name'] = ucwords($first_name);
				$userdata['last_name'] = ucwords($last_name);
				$userdata['zipcode'] = stms_sanitize($vars['ccustzip']);
				$userdata['country'] = stms_isset($vars['ccustcc']);
				$userdata['status'] = ( stms_isset($product->info['auto_register']) == 1 ) ? 1 : 4;
				$userdata['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
				$userdata['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
				
				$member_id = stms_save_member($userdata);
			}
			
			// log the ipn for further usage...
			$ipn = stms_log_ipn($vars, $tx_id, 'clickbank', 0, $member_id, $vars['ctransreceipt']);
			
			// let's complete the payment...
			$args = array(
				'member_id' => $member_id,
				'paysys' => 'clickbank',
				'receipt' => $vars['ctransreceipt'],
				'currency' => stms_isset($stms['global']['currency'], 'USD'),
				'amount' => $product->price,
				'paid' => 1,
				'status' => 'completed',
				'date' => stms_time(),
				'expired_date' => stms_get_expired($product->ID, true)
			);
			
			if ( $product->type == 'recurr' && $product->is_trial == 1 )
				$args['amount'] = $product->t_price;
			
			stms_complete_payment($tx_id, $args);
			break;
			
		case 'BILL':
            if ( $payment = stms_get_payment($old_invoice) ) {
            	// set the status of previous payment to expire...
				$new_data = array();
				$new_data['status'] = 'expired';
				$new_data['expired_date'] = strtotime("5 minutes ago", stms_time());
				stms_update_payment($payment->tx_id, $new_data);
				wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
				
				// get the product data...
				$product = stms_get_product($payment->product_id);
				$member_id = $payment->member_id;
            }

			if ( !$member_id ) {
				$member_id = stms_email_exists($email);
				if ( !$member_id )
					stms_error_handler("Cannot find member associated with this recurring transaction. ClickBank INS Ignored.", 0, $vars['ctransreceipt'], 'ipn');
			}
				
			// calculate expire date...
			$expired_date = stms_get_expired($product->ID, false);
			if ( is_numeric($product->rebill_times) && $product->rebill_times > 0 ) {
				if ( isset($vars['crebillstatus']) && $vars['crebillstatus'] == 'COMPLETED' ) {
					$expired_date = strtotime("19 Jan 2038", stms_time());
				} else {
					$rebills = $wpdb->get_results(
						"SELECT * FROM {$wpdb->prefix}stms_transactions
						WHERE paysys = 'clickbank' AND `member_id` = {$member_id}
						AND is_rebill = 1 AND product_id = {$product->ID}"
					);
					
					if ( $rebills ) {
						$rebilled = count($rebills);
						if ( $product->is_trial != 1 )
							$rebilled += 1;
							
						if ( $rebilled >= $product->rebill_times ) {
							$expired_date = strtotime("19 Jan 2038", stms_time());
						}
					}
				}
			}
			
			// Add new payment for rebill
			$args = array(
				'product_id' => $product->ID,
				'member_id' => $member_id,
				'payment_type' => 'recurr',
				'paysys' => 'clickbank',
				'receipt' => $vars['ctransreceipt'],
				'currency' => stms_isset($stms['global']['currency'], 'USD'),
				'amount' => $product->price,
				'txn_id' => '',
				'paid' => 1,
				'is_rebill' => 1,
				'status' => 'completed',
				'expired_date' => $expired_date
			);
			$new_tx_id = stms_add_payment($args);
            break;
			
		case 'RFND':
		case 'CGBK':
		case 'INSF':
			if ( !$payment = stms_get_payment($old_invoice) )
				stms_error_handler("Cannot find transaction record for this ClickBank IPN (tx_id:{$tx_id})", 0, $vars['ctransreceipt'], 'ipn');

			$payments = stms_get_member_payments($payment->member_id);
			if ( $payments ) {
		        foreach ( $payments as $pay ) {
		            if ( $pay->product_id  == $payment->product_id && $pay->status == 'completed' ) {
		            	if ( isset($vars['ctransreceipt']) && $vars['ctransreceipt'] != $pay->receipt )
							continue;
							
						$refund_id = $pay->tx_id;
						stms_refund_payment($refund_id, 'third');
		            }
		        }
			}
			
			break;
			
		case 'CANCEL-REBILL':
	 	case 'CANCEL-TEST-REBILL':
			if ( !$payment = stms_get_payment($old_invoice) )
				stms_error_handler("Cannot find transaction record for this DigiResults IPN (tx_id:{$tx_id})", 0, $vars['ctransreceipt'], 'ipn');
			
			$new_data = array();
			$new_data['status'] = 'cancelled';
			stms_update_payment($payment->tx_id, $new_data );
        	wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
        	
			do_action('stms_subscr_cancelled', $payment->tx_id);
			break;
			
		case 'UNCANCEL-REBILL': //Reversing the cancellation of a recurring billing product.
		case 'UNCANCEL-TEST-REBILL':
           if ( !$payment = stms_get_payment($old_invoice) )
				stms_error_handler("Cannot find transaction record for this IPN (tx_id:{$tx_id})", 0, $vars['ctransreceipt'], 'ipn');
			
			$new_data = array();
			$new_data['status'] = 'completed';
			stms_update_payment($payment->tx_id, $new_data );
			wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
			
			break;
	}
	exit;
}

// add_action('stms_validate_access_clickbank', 'stms_clickbank_thanks_handler');
function stms_clickbank_thanks_handler() {
	@set_time_limit(0);

	global $wpdb;

	$product_id = intval($_GET['insta_id']);
	$stms = get_option('stms_settings');
	$secret_key = stms_sanitize($stms['payments']['clickbank']['secret'], '', true);
	$rcpt = ( isset($_GET['cbreceipt']) ) ? stms_sanitize($_GET['cbreceipt']) : stms_sanitize($_GET['insta_receipt']);
	
	/*
	$time = stms_sanitize($_GET['time']);
	$item = stms_sanitize($_GET['item']);
	$cbpop = stms_sanitize($_GET['cbpop']);
	
	$xxpop = sha1("$secret_key|$rcpt|$time|$item");
	$xxpop = strtoupper(substr($xxpop,0,8));
	
	if ( $cbpop != $xxpop )
		wp_die("ERROR: Page not found.");
	*/
	if ( empty($rcpt) ) {
		if ( $product = stms_get_product($product_id) ) {
			$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_permalink($stms['global']['home']);
			wp_redirect($home_url);
			exit;
		} else {
			wp_die("ERROR: Page Not Found.");
		}
	}
	
	$support = stms_get_handler_url('support');
	if ( $ipn = stms_find_clickbank_ipn($rcpt) ) {
		$vars = maybe_unserialize($ipn->postdata);
		if ( !$payment = stms_get_payment($ipn->tx_id) )
			wp_die('ERROR: Invalid Payment');
		
		$product = stms_get_product($payment->product_id);
		$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_permalink($stms['global']['home']);
		$redirect_url = ( !empty($product->info['thankuri']) ) ? $product->info['thankuri'] : $home_url;
	
		if ( $ipn->status == 1 ) { // check if IPN has been processed...
			// IPN has been processed... then redirect to members download page...
			wp_redirect($home_url);
			exit;
		}
		
		$member_id = ( is_stmsmember_logged_in() ) ? stms_get_current_member_id() : $payment->member_id;
		$is_logged_in = ( is_stmsmember_logged_in() ) ? true : false;
		$is_existing = false;
		
		$temp_member = stms_member($payment->member_id);
		if ( $is_logged_in ) {
			if ( $member_id != $payment->member_id ) {
				// fix license key data ...
				$settings = stms_get_product_meta($payment->product_id, 'license');
				if ( $settings['enable'] == 1 )
					$wpdb->update("{$wpdb->prefix}stms_licenses", array('member_id' => $member_id), array('tx_id' => $payment->tx_id));
				
				// Update member id trx
				$new_data = array();
				$new_data['member_id'] = $member_id;
				stms_update_payment($payment->tx_id, $new_data);
				
				do_action('stms_payment_completed', $payment->tx_id); // so the receipt can be resend...
				
				if ( $temp_member->status == 4 )
					stms_delete_member($temp_member->ID, false);
			}
			
			$is_existing = true;
		}
			
		// Recall payment data to make sure that we use latest data...
		$payment = stms_get_payment($ipn->tx_id);
		// let's update ipn log...
		$update = $wpdb->update("{$wpdb->prefix}stms_ipn_log", array('status' => 1, 'member_id' => $payment->member_id), array('log_id' => $ipn->log_id));
		// remove retries data...
		delete_transient('stms_thanks_retries_' . $rcpt);
		
		do_action('stms_custom_autoresponder', $payment->tx_id); // for custom autoresponder...
		
		if ( $is_existing ) {
			// check and complete other payments (if any)
			stms_other_payments($member_id, $vars['ccustemail'], 'clickbank');
			
			// logging out member first...
			stms_clear_auth_cookie();
		
			// re-logging in...
			stms_set_auth_cookie($member_id);
			
			// now redirecting...
			wp_redirect($redirect_url);
			exit;
		}
		
		$first_name = strtolower(stms_sanitize($vars['ccustfirstname']));
		$last_name = strtolower(stms_sanitize($vars['ccustlastname']));
			
		if ( stms_isset($product->info['auto_register']) == 1 ) {
			// update member data...
			$userdata = array();
			$userdata['member_id'] = $payment->member_id;
			$userdata['email'] = stms_isset($vars['ccustemail']);
			$userdata['status'] = 1;
			$userdata['fix_username'] = stms_generate_username($vars['ccustemail'], ucwords($first_name), ucwords($last_name));
			$userdata['pwd1'] = wp_generate_password(8, false);
			$userdata['first_name'] = ucwords($first_name);
			$userdata['last_name'] = ucwords($last_name);
			$userdata['zipcode'] = stms_sanitize($vars['ccustzip']);
			$userdata['country'] = stms_isset($vars['ccustcc']);
			$userdata['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
			$userdata['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
			
			stms_save_member($userdata, false, false, $payment->tx_id);
			
			// check and complete other payments (if any)
			stms_other_payments($member_id, $vars['ccustemail'], 'clickbank');
			
			// clean any login cookies first...
			stms_clear_auth_cookie();
			
			// logging in and then redirect...
			stms_set_auth_cookie($payment->member_id);
			wp_redirect($redirect_url);
			exit;
		} else {
			// Proceed to registration
			$session = base64_encode("{$product->ID}|clickbank|third");
			$insta_token = stms_generate_secret(16, true, true );
			$token_data = array( 'token' => $insta_token, 'tx_id' => $payment->tx_id);
			set_transient('insta_token_' . $insta_token, $token_data, 86400);
			
			$reg_url = stms_format_url($stms['global']['magic'], 'imhandler=registration&key=' . $session . '&insta_token=' . $insta_token);
			// build reg vars...
			$reg_vars = array();
			$reg_vars['im_reg_email'] = stms_isset($vars['ccustemail']);
			$reg_vars['im_reg_first_name'] = ucwords($first_name);
			$reg_vars['im_reg_last_name'] = ucwords($last_name);
			$reg_vars['im_reg_zipcode'] = stms_sanitize($vars['ccustzip']);
			$reg_vars['im_reg_country'] = stms_isset($vars['ccustcc']);
			$auth_vars = base64_encode(json_encode($reg_vars));
			
			wp_redirect($reg_url . '&auth_vars=' . $auth_vars);
			exit;
		}
	} else {
		$retries = get_transient('stms_thanks_retries_' . $rcpt);
		$thanks_url = stms_format_url($stms['global']['magic'], http_build_query($_GET));
		if ( $retries >= 15 ) {
			$page_title = 'Payment Error';
			$refresh = '';
			$message = '
				<p style="text-align:center;"><strong>Payment Error</strong></p>
				<p>We\'re sorry, but there\'s a problem in validating your payment. Please contact our <a href="' . $support . '">support</a> to resolve this issue.</p>
			';
		} else {
			$count = $retries + 1;
			set_transient('stms_thanks_retries_' . $rcpt, $count, 60*15);
			$page_title = 'Waiting For Payment Confirmation...';
			$refresh = '<meta http-equiv="refresh" content="5;URL=\'' . $thanks_url . '\'" />';
			$message = '
				<h2 style="text-align:center;"><strong>Waiting For Payment Confirmation...</strong></h2>
				<p style="text-align:center;"><img src="' . STMS_URL . 'images/admin-loader.gif" border="0" /></p>
				<p style="text-align:center;">This page will be auto-refresh every 5 seconds until your payment confirmed.</p>
				<p style="text-align:center;"><strong>Please DO NOT close this page.</strong></p>
			';
		}
?>
<!DOCTYPE html>
<!--[if IE 6]>
<html id="ie6" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 7]>
<html id="ie7" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 8]>
<html id="ie8" <?php language_attributes(); ?>>
<![endif]-->
<!--[if !(IE 6) | !(IE 7) | !(IE 8)  ]><!-->
<html <?php language_attributes(); ?>>
<!--<![endif]-->
<head>
<meta charset="<?php bloginfo( 'charset' ); ?>" />
<title><?php echo $page_title; ?></title>
<?php echo $refresh; ?>
<!--[if lt IE 9]>
	<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->
<link rel="profile" href="http://gmpg.org/xfn/11" />
<meta name="HandheldFriendly" content="True" />
<meta name="viewport" content="width=device-width, initial-scale=1"/>
<!-- Adding "maximum-scale=1" fixes the Mobile Safari auto-zoom bug: http://filamentgroup.com/examples/iosScaleBug/ -->
<link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Droid+Sans" type="text/css" media="all" />
<style type="text/css">
body {
	margin:0;
	padding:0;
	font-size:14px;
	font-family:'Droid Sans', sans-serif;
	background-color:#F5F5F5;
	color:#808080;
}
#redirect_msg {
	margin:25px auto 0;
	padding:15px 20px;
	width:40%;
	background-color:#FFF;
}
</style>
</head>
<body>
<div id="redirect_msg">
	<?php echo $message; ?>
</div>
</body>
</html>
<?php
	}
	exit;
}

function stms_clickbank_get_lock( $product_id, $receipt ) {
	global $wpdb;
	
	$sql = $wpdb->prepare(
				"SELECT COUNT(*) FROM `{$wpdb->prefix}stms_transactions`
				WHERE `product_id` = %d AND `receipt` = %s
				AND `is_rebill` = 0",
				$product_id, $receipt
			);
	$count = $wpdb->get_var($sql);
	if ( $count > 0 ) return true;
	return false;
}

function stms_clickbank_get_member_id( $email ) {
	global $wpdb;
	
	$email = sanitize_email($email);
	if ( $member = stms_get_data_by('email', $email) )
		return $member->ID;

	return false;
}

function stms_clickbank_last_id( $receipt ) {
	global $wpdb;
	
	$last_payment = $wpdb->get_row("SELECT `tx_id` FROM `{$wpdb->prefix}stms_transactions`
			WHERE `receipt` <> '' AND (`receipt` LIKE '{$receipt}-B%' OR `receipt` = '{$receipt}')
			ORDER BY tx_id DESC LIMIT 1");
	
	if ( !$last_payment )
		return false;
	
	return $last_payment->tx_id;
}

function stms_find_clickbank_ipn( $receipt ) {
	global $wpdb;
	$ipn = $wpdb->get_row(
		"SELECT * FROM `{$wpdb->prefix}stms_ipn_log`
		WHERE `paysys` = 'clickbank' AND `receipt` = '{$receipt}'"
	);
	if ( !$ipn ) return false;
	return $ipn;
}

add_filter('stms_thanks_setup_clickbank', 'stms_clickbank_thanks_setup');
function stms_clickbank_thanks_setup( $settings ) {
	$settings['rcpt'] = 'cbreceipt';
	$settings['flow'] = 'normal';
	$settings['email_check'] = false;
	$settings['email_var'] = 'ccustemail';
	$settings['firstname_var'] = 'ccustfirstname';
	$settings['lastname_var'] = 'ccustlastname';
	return $settings;
}
