<?php
/*
Payment Name: DealGuardian
Payment Key: dealguardian
One Time: yes
Recurring: yes
Flow Type: normal
*/

add_action('stms_thirdpay_settings_dealguardian', 'stms_dealguardian_product_settings', 10, 2);
function stms_dealguardian_product_settings( $product, $pay_settings ) {
	$stms = get_option('stms_settings');
	$dealguardian_secret = ( stms_isset($stms['payments']['dealguardian']['secret']) != '' ) ? trim($stms['payments']['dealguardian']['secret']) : stms_generate_secret(8, true, false, true);
?>
	<tr valign="top" class="stms-dealguardian-level">
		<th scope="row" colspan="2" style="border-top:1px solid #E5E5E5"><h3 style="color:#808080"><?php _e('DealGuardian Product Integration Settings', 'stms'); ?></h3></th>
	</tr>
	<tr valign="top" class="stms-dealguardian-level">
		<th scope="row" style="text-align:right"><label for="stms_dealguardian_item"><strong><span style="color:#cc0000">*</span> <?php _e('Product ID#', 'stms'); ?></strong></label></th>
		<td>
			<input class="stms-field" type="text" name="stms_level[pay_settings][dealguardian][item]" id="stms_dealguardian_item" value="<?php echo stms_cleanup(stms_isset($pay_settings['dealguardian']['item'])); ?>" style="width:80px" />
			<div class="stms-desc">Please enter the Product ID of your DealGuardian Product.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-dealguardian-level">
		<th scope="row" style="text-align:right"><label for="stms_dealguardian_url"><strong><span style="color:#cc0000">*</span> <?php _e('Download URL', 'stms'); ?></strong></label></th>
		<td>
			<input class="stms-field widefat" type="text" id="stms_dealguardian_url" value="<?php echo stms_get_returnurl('dealguardian', false, true, $product->ID); ?>" readonly />
			<div class="stms-desc">Put this download URL in your DealGuardian's product access page. New customers MUST click this link so InstaMember can give access to the product.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-dealguardian-level">
		<th scope="row" style="text-align:right"><label><strong><span style="color:#cc0000">*</span> <?php _e('IPN URL', 'stms'); ?></strong></label></th>
		<td>
			<input class="widefat stms-field" type="text" value="<?php echo stms_get_ipnurl('dealguardian') . '&insta_id=' . $product->ID; ?>&key=<?php echo $dealguardian_secret; ?>" readonly />
			<div class="stms-desc">You MUST create a new IPN integration in your DealGuardian's account (under <code>Authors -> 3rd Party Integrations</code>), and enter this URL.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-dealguardian-level">
		<th scope="row" style="text-align:right"><label for="stms_dealguardian_code"><?php _e('Order Button Code', 'stms'); ?></label></th>
		<td>
			<textarea class="widefat stms-field" name="stms_level[pay_settings][dealguardian][code]" id="stms_dealguardian_code" style="height:80px"><?php echo esc_html(stms_sanitize($pay_settings['dealguardian']['code'])); ?></textarea>
			<div class="stms-desc">This is only required if you use the "Auto-Switch" payment button feature. Since DealGuardian has its own payment button, then you have to enter the order button code here so that InstaMember can display it correctly. If you're not using the "Auto-Switch" payment feature, then you can leave this empty.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-dealguardian-level stms-last-integrate">
		<th scope="row" colspan="2"></th>
	</tr>
<?php
}

add_action('stms_global_payment_dealguardian', 'stms_dealguardian_global_settings');
function stms_dealguardian_global_settings( $stms ) {
	$dealguardian_secret = ( stms_isset($stms['payments']['dealguardian']['secret']) != '' ) ? trim($stms['payments']['dealguardian']['secret']) : stms_generate_secret(8, true, false, true);
?>
	<table class="form-table">
	<tr valign="top">
		<th scope="row"><label style="cursor:default"><strong>Enable DealGuardian</strong></label></th>
		<td><div class="stms-toggle-btn"></div><input type="checkbox" name="stms_sets[is_dealguardian]" id="stms_pay_dealguardian" class="stms-toggle-checkbox stms-toggle-switch" data-blockname="dealguardian-paysets" value="1" <?php if ( stms_isset($stms['payments']['is_dealguardian']) == 1 ) echo 'checked="checked"'; ?> /></td>
	</tr>
	</table>
	<input type="hidden" name="stms_sets[dealguardian][secret]" value="<?php echo $dealguardian_secret; ?>" />
<?php
}

add_action('stms_ipn_dealguardian', 'stms_dealguardian_ipn_handler');
function stms_dealguardian_ipn_handler() {
	$stms = get_option('stms_settings');
	$secret = stms_isset($stms['payments']['dealguardian']['secret']);
	$key = trim($_REQUEST['key']);
	if ( $secret != $key )
		stms_error_handler("Secret key is different. Expected: {$secret}, Received: {$key}. DealGuardian IPN ignored.", 0, '', 'ipn');
	
	$process = stms_process_dealguardian($_REQUEST);
}

function stms_process_dealguardian( $vars ) {
	global $wpdb;
	
	$stms = get_option('stms_settings');
	$trans_type = strtolower($vars['transaction_type']);
	$receipt = stms_isset($vars['transaction_id'], '');
	$subscr_id = stms_isset($vars['transaction_subscription_id'], '');
	
	$item = stms_sanitize($vars['product_id']);
	$title = stms_sanitize($vars['product_name']);
	$product_id = stms_isset($vars['insta_id']);
	 
	// customer data...
	$first_name = stms_sanitize($vars['buyer_first_name']);
	$last_name = stms_sanitize($vars['buyer_last_name']);
	$email = sanitize_email($vars['buyer_email']);
	
	if ( !$product = stms_get_product($product_id) ) {
		if ( !$product = stms_find_product_by_externalid('dealguardian', $item) ) {
			if ( !$product = stms_get_product_bytitle($title) )
				stms_error_handler("Product not found. DealGuardian IPN Ignored.", 0, $receipt, 'ipn');
		}
	}
	
	$receipt = ( $product->type == 'recurr' ) ? $subscr_id : $receipt;
	switch ( $trans_type ) {
		case 'subscr_signup':
			if ( $lock = stms_dealguardian_get_lock($product->ID, $receipt) )
				return false;

			$invoice = stms_dealguardian_last_id($subscr_id);
			$is_rebill = ( !$invoice ) ? false : true;
			$is_freetrial = ( $product->type == 'recurr' && $product->is_trial == 1 && $product->t_price <= 0.00 ) ? true : false;
			if ( $is_freetrial && !$is_rebill ) { // FREE TRIAL
				// Create pending transaction first...
				$args = array(
					'product_id' => $product->ID,
					'member_id' => 0,
					'payment_type' => $product->type,
					'paysys' => 'dealguardian',
					'currency' => stms_isset($stms['global']['currency'], 'USD'),
					'amount' => $product->price,
					'ipaddr' => ''
				);
				$tx_id = stms_add_pending_payment($args);
				if ( !$member_id = stms_email_exists($email) ) {
					$temp = stms_generate_temporary();
					
					$userdata = array();
					$userdata['member_id'] = 0;
					$userdata['username'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? stms_generate_username($email, $first_name, $last_name) : $temp['username'];
					$userdata['password'] = wp_generate_password(8, false);
					$userdata['email'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? $email : $temp['email'];
					$userdata['first_name'] = ucwords($first_name);
					$userdata['last_name'] = ucwords($last_name);
					$userdata['status'] = ( stms_isset($product->info['auto_register']) == 1 ) ? 1 : 4;
					$userdata['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
					$userdata['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
					$member_id = stms_save_member($userdata);
				}
				
				// log the ipn for further usage...
				$ipn = stms_log_ipn($vars, $tx_id, 'dealguardian', 0, $member_id, $receipt);
				
				//  let's complete the payment...
				$args = array(
					'member_id' => $member_id,
					'paysys' => 'dealguardian',
					'receipt' => $receipt,
					'currency' => stms_isset($stms['global']['currency'], 'USD'),
					'amount' => 0.00,
					'paid' => 1,
					'status' => 'completed',
					'date' => stms_time(),
					'expired_date' => stms_get_expired($product->ID, true)
				);
				
				stms_complete_payment($tx_id, $args);
			}
			break;
			
        case 'sale':
			if ( $lock = stms_dealguardian_get_lock($product->ID, $receipt) )
				return false;

			$invoice = stms_dealguardian_last_id($subscr_id);
			$is_rebill = ( !$invoice ) ? false : true;
			if ( !$is_rebill ) { // INITIAL PAYMENT
				// Create pending transaction first...
				$args = array(
					'product_id' => $product->ID,
					'member_id' => 0,
					'payment_type' => $product->type,
					'paysys' => 'dealguardian',
					'currency' => stms_isset($stms['global']['currency'], 'USD'),
					'amount' => $product->price,
					'ipaddr' => ''
				);
				
				$tx_id = stms_add_pending_payment($args);
				if ( !$member_id = stms_dealguardian_get_member_id($email) ) {
					$temp = stms_generate_temporary();
					$userdata = array();
					$userdata['member_id'] = 0;
					$userdata['username'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? stms_generate_username($email, $first_name, $last_name) : $temp['username'];
					$userdata['password'] = wp_generate_password(8, false);
					$userdata['email'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? $email : $temp['email'];
					$userdata['first_name'] = ucwords($first_name);
					$userdata['last_name'] = ucwords($last_name);
					$userdata['status'] = 4; //( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? 1 : 4;
					$userdata['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
					$userdata['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
					$member_id = stms_save_member($userdata);	
				}
				
				// log the ipn for further usage...
				$ipn = stms_log_ipn($vars, $tx_id, 'dealguardian', 0, $member_id, $receipt);
				
				$price = $product->price;
				if ( $product->type == 'recurr' && $product->is_trial == 1 )
					$price = $product->t_price;
					
				$total_amount = ( isset($vars['product_price_point']) ) ? number_format($vars['product_price_point'], 2, '.', '') : $price;
				//  let's complete the payment...
				$args = array(
					'member_id' => $member_id,
					'paysys' => 'dealguardian',
					'receipt' => $receipt,
					'currency' => stms_isset($stms['global']['currency'], 'USD'),
					'amount' => $total_amount,
					'paid' => 1,
					'status' => 'completed',
					'date' => stms_time(),
					'expired_date' => stms_get_expired($product->ID, true)
				);
				
				stms_complete_payment($tx_id, $args);
			} else { // REBILL
				if ( $payment = stms_get_payment($invoice) ) {
	            	// set the status of previous payment to expire...
					$new_data = array();
					$new_data['status'] = 'expired';
					$new_data['expired_date'] = strtotime("5 minutes ago", stms_time());
					stms_update_payment($payment->tx_id, $new_data);
					wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
					
					// get the product data...
					$product = stms_get_product($payment->product_id);
					$member_id = $payment->member_id;
            	}

				if ( !$member_id ) {
					$member_id = stms_email_exists($email);
					if ( !$member_id )
						stms_error_handler("Cannot find member associated with this recurring transaction. DealGuardian IPN Ignored.", 0, $receipt, 'ipn');
				}
				
				// calculate expire date...
				$expired_date = stms_get_expired($product->ID, false);
				if ( is_numeric($product->rebill_times) && $product->rebill_times > 0 ) {
					$rebills = $wpdb->get_results(
						"SELECT * FROM {$wpdb->prefix}stms_transactions
						WHERE paysys = 'dealguardian' AND `member_id` = {$member_id}
						AND is_rebill = 1 AND product_id = {$product->ID}"
					);
					
					if ( $rebills ) {
						$rebilled = count($rebills);
						if ( $product->is_trial != 1 )
							$rebilled += 1;
							
						if ( $rebilled >= $product->rebill_times ) {
							$expired_date = strtotime("19 Jan 2038", stms_time());
						}
					}
				}
				
				$total_amount = ( isset($vars['product_price_point']) ) ? number_format($vars['product_price_point'], 2, '.', '') : $product->price;
				// Add new payment for rebill
				$args = array(
					'product_id' => $product->ID,
					'member_id' => $member_id,
					'payment_type' => 'recurr',
					'paysys' => 'dealguardian',
					'receipt' => $receipt,
					'currency' => stms_isset($stms['global']['currency'], 'USD'),
					'amount' => $total_amount,
					'txn_id' => '',
					'paid' => 1,
					'is_rebill' => 1,
					'status' => 'completed',
					'expired_date' => $expired_date
				);
				$new_tx_id = stms_add_payment($args);
			}
			break;
			
		case 'refund':
			$receipt = ( $product->type == 'recurr' ) ? $subscr_id : $vars['transaction_parent_id'];
			$invoice = stms_dealguardian_last_id($receipt);
			if ( !$payment = stms_get_payment($invoice) )
				stms_error_handler("Cannot find transaction record for this DealGuardian IPN", 0, $receipt, 'ipn');

			$payments = stms_get_member_payments($payment->member_id);
			if ( $payments ) {
		        foreach ( $payments as $pay ) {
		            if ( $receipt == $pay->receipt && $pay->status == 'completed' ) {		
						$refund_id = $pay->tx_id;
						stms_refund_payment($refund_id, 'third');
					}
		        }
			}
			break;
			
		case 'subscr_cancel':
			$invoice = stms_dealguardian_last_id($subscr_id);
			if ( !$payment = stms_get_payment($invoice) )
				stms_error_handler("Cannot find transaction record for this DealGuardian IPN", 0, $receipt, 'ipn');
			
			$new_data = array();
			$new_data['status'] = 'cancelled';
			stms_update_payment($payment->tx_id, $new_data );
        	wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
        	
			do_action('stms_subscr_cancelled', $payment->tx_id);
			break;
	}
	exit;
}

function stms_dealguardian_get_lock( $product_id, $receipt ) {
	global $wpdb;
	
	$sql = $wpdb->prepare(
				"SELECT COUNT(*) FROM `{$wpdb->prefix}stms_transactions`
				WHERE `paysys` = %s AND `product_id` = %d AND `receipt` = %s
				AND `is_rebill` = 0",
				'dealguardian', $product_id, $receipt
			);
	$count = $wpdb->get_var($sql);
	if ( $count > 0 ) return true;
	return false;
}

function stms_dealguardian_get_member_id( $email ) {
	global $wpdb;
	if ( $member = stms_get_data_by('email', $email) ) return $member->ID;
	return false;
}

function stms_dealguardian_last_id( $receipt ) {
	global $wpdb;
	
	$last_payment = $wpdb->get_row(
		$wpdb->prepare("SELECT `tx_id` FROM `{$wpdb->prefix}stms_transactions`
			WHERE `receipt` = %s
			ORDER BY `tx_id` DESC LIMIT 1",
			$receipt
		)
	);
	
	if ( !$last_payment )
		return false;
	
	return $last_payment->tx_id;
}

//add_action('stms_validate_access_dealguardian', 'stms_dealguardian_thanks_handler');
function stms_dealguardian_thanks_handler() {
	@set_time_limit(0);

	global $wpdb;

	$product_id = intval($_GET['insta_id']);
	$stms = get_option('stms_settings');
	$secret_key = stms_sanitize($stms['payments']['clickbank']['secret'], '', true);
	$rcpt = stms_sanitize($_GET['insta_receipt']);
	
	if ( empty($rcpt) ) {
		if ( $product = stms_get_product($product_id) ) {
			$check_url = stms_format_url($stms['global']['magic'], 'imhandler=dealguardian&product_id=' . $product->ID);
			wp_redirect($check_url);
			exit;
		} else {
			wp_die("ERROR: Page Not Found.");
		}
	}
	
	$support = stms_get_handler_url('support');
	if ( $ipn = stms_find_dealguardian_ipn($rcpt) ) {
		$vars = maybe_unserialize($ipn->postdata);
		if ( !$payment = stms_get_payment($ipn->tx_id) )
			wp_die('ERROR: Invalid Payment');
		
		$product = stms_get_product($payment->product_id);
		$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_permalink($stms['global']['home']);
		$redirect_url = ( !empty($product->info['thankuri']) ) ? $product->info['thankuri'] : $home_url;
	
		if ( $ipn->status == 1 ) { // check if IPN has been processed...
			// IPN has been processed... then redirect to members download page...
			wp_redirect($home_url);
			exit;
		}
		
		$member_id = ( is_stmsmember_logged_in() ) ? stms_get_current_member_id() : $payment->member_id;
		$is_logged_in = ( is_stmsmember_logged_in() ) ? true : false;
		$is_existing = false;
		
		$temp_member = stms_member($payment->member_id);
		if ( $is_logged_in ) {
			if ( $member_id != $payment->member_id ) {
				// fix license key data ...
				$settings = stms_get_product_meta($payment->product_id, 'license');
				if ( $settings['enable'] == 1 )
					$wpdb->update("{$wpdb->prefix}stms_licenses", array('member_id' => $member_id), array('tx_id' => $payment->tx_id));
				
				// Update member id trx
				$new_data = array();
				$new_data['member_id'] = $member_id;
				stms_update_payment($payment->tx_id, $new_data);
				wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
				
				do_action('stms_payment_completed', $payment->tx_id); // so the receipt can be resend...
				
				if ( $temp_member->status == 4 )
					stms_delete_member($temp_member->ID, false);
			}
			
			$is_existing = true;
		}
			
		// Recall payment data to make sure that we use latest data...
		$payment = stms_get_payment($ipn->tx_id);
		// let's update ipn log...
		$update = $wpdb->update("{$wpdb->prefix}stms_ipn_log", array('status' => 1, 'member_id' => $payment->member_id), array('log_id' => $ipn->log_id));
		// remove retries data...
		delete_transient('stms_thanks_retries_' . $rcpt);
		
		do_action('stms_custom_autoresponder', $payment->tx_id); // for custom autoresponder...
		
		if ( $is_existing ) {
			// check and complete other payments (if any)
			stms_other_payments($member_id, $vars['buyer_email'], 'dealguardian');
			
			// logging out member first...
			stms_clear_auth_cookie();
		
			// re-logging in...
			stms_set_auth_cookie($member_id);
			
			// now redirecting...
			wp_redirect($redirect_url);
			exit;
		}
		
		$first_name = stms_sanitize($vars['buyer_first_name']);
		$last_name = stms_sanitize($vars['buyer_last_name']);
			
		if ( stms_isset($product->info['auto_register']) == 1 ) {
			// update member data...
			$userdata = array();
			$userdata['member_id'] = $payment->member_id;
			$userdata['email'] = stms_isset($vars['buyer_email']);
			$userdata['status'] = 1;
			$userdata['fix_username'] = stms_generate_username($vars['buyer_email'], $first_name, $last_name);
			$userdata['pwd1'] = wp_generate_password(8, false);
			$userdata['first_name'] = $first_name;
			$userdata['last_name'] = $last_name;
			$userdata['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
			$userdata['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
			
			stms_save_member($userdata, false, false, $payment->tx_id);
			
			// check and complete other payments (if any)
			stms_other_payments($member_id, $vars['buyer_email'], 'dealguardian');
			
			// clean any login cookies first...
			stms_clear_auth_cookie();
			
			// logging in and then redirect...
			stms_set_auth_cookie($payment->member_id);
			wp_redirect($redirect_url);
			exit;
		} else {
			// Proceed to registration
			$session = base64_encode("{$product->ID}|clickbank|third");
			$insta_token = stms_generate_secret(16, true, true );
			$token_data = array( 'token' => $insta_token, 'tx_id' => $payment->tx_id);
			set_transient('insta_token_' . $insta_token, $token_data, 86400);
			
			$reg_url = stms_format_url($stms['global']['magic'], 'imhandler=registration&key=' . $session . '&insta_token=' . $insta_token);
			// build reg vars...
			$reg_vars = array();
			$reg_vars['im_reg_email'] = stms_isset($vars['ccustemail']);
			$reg_vars['im_reg_first_name'] = ucwords($first_name);
			$reg_vars['im_reg_last_name'] = ucwords($last_name);
			$reg_vars['im_reg_zipcode'] = stms_sanitize($vars['ccustzip']);
			$reg_vars['im_reg_country'] = stms_isset($vars['ccustcc']);
			$auth_vars = base64_encode(json_encode($reg_vars));
			
			wp_redirect($reg_url . '&auth_vars=' . $auth_vars);
			exit;
		}
	} else {
		$retries = get_transient('stms_thanks_retries_' . $rcpt);
		$thanks_url = stms_format_url($stms['global']['magic'], http_build_query($_GET));
		if ( $retries >= 15 ) {
			$page_title = 'Payment Error';
			$refresh = '';
			$message = '
				<p style="text-align:center;"><strong>Payment Error</strong></p>
				<p>We\'re sorry, but there\'s a problem in validating your payment. Please contact our <a href="' . $support . '">support</a> to resolve this issue.</p>
			';
		} else {
			$count = $retries + 1;
			set_transient('stms_thanks_retries_' . $rcpt, $count, 60*15);
			$page_title = 'Waiting For Payment Confirmation...';
			$refresh = '<meta http-equiv="refresh" content="5;URL=\'' . $thanks_url . '\'" />';
			$message = '
				<h2 style="text-align:center;"><strong>Waiting For Payment Confirmation...</strong></h2>
				<p style="text-align:center;"><img src="' . STMS_URL . 'images/admin-loader.gif" border="0" /></p>
				<p style="text-align:center;">This page will be auto-refresh every 5 seconds until your payment confirmed.</p>
				<p style="text-align:center;"><strong>Please DO NOT close this page.</strong></p>
			';
		}
?>
<!DOCTYPE html>
<!--[if IE 6]>
<html id="ie6" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 7]>
<html id="ie7" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 8]>
<html id="ie8" <?php language_attributes(); ?>>
<![endif]-->
<!--[if !(IE 6) | !(IE 7) | !(IE 8)  ]><!-->
<html <?php language_attributes(); ?>>
<!--<![endif]-->
<head>
<meta charset="<?php bloginfo( 'charset' ); ?>" />
<title><?php echo $page_title; ?></title>
<?php echo $refresh; ?>
<!--[if lt IE 9]>
	<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->
<link rel="profile" href="http://gmpg.org/xfn/11" />
<meta name="HandheldFriendly" content="True" />
<meta name="viewport" content="width=device-width, initial-scale=1"/>
<!-- Adding "maximum-scale=1" fixes the Mobile Safari auto-zoom bug: http://filamentgroup.com/examples/iosScaleBug/ -->
<link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Droid+Sans" type="text/css" media="all" />
<style type="text/css">
body {
	margin:0;
	padding:0;
	font-size:14px;
	font-family:'Droid Sans', sans-serif;
	background-color:#F5F5F5;
	color:#808080;
}
#redirect_msg {
	margin:25px auto 0;
	padding:15px 20px;
	width:40%;
	background-color:#FFF;
}
</style>
</head>
<body>
<div id="redirect_msg">
	<?php echo $message; ?>
</div>
</body>
</html>
<?php
	}
	exit;
}

function stms_find_dealguardian_ipn( $receipt ) {
	global $wpdb;
	$ipn = $wpdb->get_row(
		"SELECT * FROM `{$wpdb->prefix}stms_ipn_log`
		WHERE `paysys` = 'dealguardian' AND `receipt` = '{$receipt}'"
	);
	if ( !$ipn ) return false;
	return $ipn;
}

add_filter('stms_thanks_setup_dealguardian', 'stms_dealguardian_thanks_setup');
function stms_dealguardian_thanks_setup( $settings ) {
	$settings['rcpt'] = 'insta_receipt';
	$settings['flow'] = 'normal';
	$settings['email_check'] = true;
	$settings['email_var'] = 'buyer_email';
	$settings['firstname_var'] = 'buyer_first_name';
	$settings['lastname_var'] = 'buyer_last_name';
	return $settings;
}