<?php
/*
Payment Name: JVZoo
Payment Key: jvzoo
One Time: yes
Recurring: yes
Flow Type: normal
*/

add_action('stms_thirdpay_settings_jvzoo', 'stms_jvzoo_product_settings', 10, 2);
function stms_jvzoo_product_settings($product, $pay_settings) {
?>
	<tr valign="top" class="stms-jvzoo-level">
		<th scope="row" colspan="2" style="border-top:1px solid #E5E5E5"><h3 style="color:#808080"><?php _e('JVZoo Product Integration Settings', 'stms'); ?></h3></th>
	</tr>
	<tr valign="top" class="stms-jvzoo-level">
		<th scope="row" style="text-align:right"><label for="stms_jvzoo_item"><strong><span style="color:#cc0000">*</span> <?php _e('Product ID#', 'stms'); ?></strong></label></th>
		<td>
			<input class="stms-field" type="text" name="stms_level[pay_settings][jvzoo][item]" id="stms_jvzoo_item" value="<?php echo stms_cleanup(stms_isset($pay_settings['jvzoo']['item'])); ?>" style="width:80px" />
			<div class="stms-desc">Please enter the Product ID of your JVZoo Product. To obtain the product ID, simply edit the product in your JVZoo account, and look at the URL in your address bar. The numbers at the end of the URL (after the slash) is the product ID.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-jvzoo-level">
		<th scope="row" style="text-align:right"><label><strong><span style="color:#cc0000">*</span> <?php _e('Thank You Page (URL)', 'stms'); ?></strong></label></th>
		<td>
			<input class="widefat stms-field" type="text" value="<?php echo stms_get_returnurl('jvzoo', false, true, $product->ID); ?>" readonly />
			<div class="stms-desc">You MUST set the thank you page URL of the product you created in JVZoo to this URL, and also CHECK the <code>Pass parameters to Download Page</code> option.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-jvzoo-level">
		<th scope="row" style="text-align:right"><label><strong><span style="color:#cc0000">*</span> <?php _e('JVZIPN (URL)', 'stms'); ?></strong></label></th>
		<td>
			<input class="widefat stms-field" type="text" value="<?php echo stms_get_ipnurl('jvzoo') . '&insta_id=' . $product->ID; ?>" readonly />
			<div class="stms-desc">You MUST set the JVZIPN URL in the product settings in your JVZoo account to this URL.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-jvzoo-level">
		<th scope="row" style="text-align:right"><label for="stms_jvzoo_code"><?php _e('Order Button Code', 'stms'); ?></label></th>
		<td>
			<textarea class="widefat stms-field" name="stms_level[pay_settings][jvzoo][code]" id="stms_jvzoo_code" style="height:80px"><?php echo esc_html(stms_sanitize($pay_settings['jvzoo']['code'])); ?></textarea>
			<div class="stms-desc">This is only required if you use the "Auto-Switch" payment button feature. Since JVZoo has its own payment button, then you have to enter the order button code here so that InstaMember can display it correctly. If you're not using the "Auto-Switch" payment feature, then you can leave this empty.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-jvzoo-level stms-last-integrate">
		<th scope="row" colspan="2"></th>
	</tr>
<?php
}

add_action('stms_global_payment_jvzoo', 'stms_jvzoo_global_settings');
function stms_jvzoo_global_settings($stms) {
	$jvzoo_secret = ( stms_isset($stms['payments']['jvzoo']['secret']) != '' ) ? trim($stms['payments']['jvzoo']['secret']) : strtoupper(stms_generate_secret(16, true, false));
?>
	<table class="form-table">
	<tr valign="top">
		<th scope="row"><label style="cursor:default"><strong>Enable JVZoo</strong></label></th>
		<td><div class="stms-toggle-btn"></div><input type="checkbox" name="stms_sets[is_jvzoo]" id="stms_pay_jvzoo" class="stms-toggle-checkbox stms-toggle-switch" data-blockname="jvzoo-paysets" value="1" <?php if ( stms_isset($stms['payments']['is_jvzoo']) == 1 ) echo 'checked="checked"'; ?> /></td>
	</tr>
	<tr valign="top" class="jvzoo-paysets">
		<th scope="row" style="text-align:right"><label for="stms_jvz_secret"><?php _e('JVZIPN Secret Key', 'stms'); ?></label></th>
		<td>
			<input class="widefat stms-field" type="text" name="stms_sets[jvzoo][secret]" id="stms_jvz_secret" value="<?php echo $jvzoo_secret; ?>" />
			<div class="stms-desc">Please enter this secret key in your JVZoo account (<code>My Account -> JVZIPN Secret Key</code>). <strong>Important:</strong> The secret key value MUST be the same as in your JVZoo account.</div>
		</td>
	</tr>
	</table>
<?php
}

add_action('stms_ipn_jvzoo', 'stms_jvzoo_ipn_handler');
function stms_jvzoo_ipn_handler() {
	$stms = get_option('stms_settings');
	$secret_key = stms_sanitize($stms['payments']['jvzoo']['secret'], '', true);
	$pop = '';

	if ( !isset($_POST['cverify']) ) exit;
	
	$ipnFields = array();
	foreach ( $_POST as $key => $value ) {
		if ( $key == "cverify" )
			continue;
		$ipnFields[] = $key;
	}
	sort( $ipnFields );
	foreach ( $ipnFields as $field ) {
    	$pop = $pop . $_POST[$field] . "|";
	}
	$pop = $pop . $secret_key;
	if ( function_exists('mb_convert_encoding') )
		$xxpop = sha1(mb_convert_encoding($pop, "UTF-8"));
	else
		$xxpop = sha1($pop);
	
	$xxpop = strtoupper(substr($xxpop,0,8));
	
	$process = false;
	if ( $_POST['cverify'] != $xxpop ) {
		stms_log_ipn($vars, 0, 'jvzoo', 1, 0, $_POST['ctransreceipt']);
		stms_error_handler("JVZoo IPN Validation Failed. JVZoo IPN Ignored. Received key: {$_POST['cverify']} - IM Key: {$xxpop}", 0, $_POST['ctransreceipt'], 'ipn');
	}
	
	$process = stms_process_jvzoo($_POST);
}

function stms_process_jvzoo($vars) {
	global $wpdb;
	
	$stms = get_option('stms_settings');
	$ctransreceipt = explode("-", $vars['ctransreceipt']);
	$old_receipt = stms_isset($ctransreceipt[0], $vars['ctransreceipt']);
	$old_invoice = stms_jvzoo_last_id($old_receipt);
	
	$custom_vars = $vars['cvendthru'];
	parse_str($custom_vars, $custom);
	
	$vendor = ( isset($vars['ctranspublisher']) && $vars['ctranspublisher'] != '' ) ? $vars['ctranspublisher'] : $vars['ctransvendor'];
	$item = stms_sanitize($vars['cproditem']);
	$title = stms_sanitize($vars['cprodtitle']);
	
	// customer data...
	$full_name = stms_sanitize($vars['ccustname']);
	$names = explode(" ", $full_name, 2);
	$first_name = strtolower(stms_sanitize($names[0]));
	$last_name = strtolower(stms_sanitize($names[1]));
	$email = sanitize_email($vars['ccustemail']);
	
	if ( !$product = stms_find_product_by_externalid('jvzoo', $item) ) {
		if ( !$product = stms_get_product_bytitle($title) )
			stms_error_handler("Product for JVZoo item '{$item}' not found. JVZoo IPN Ignored.", 0, $vars['ctransreceipt'], 'ipn');
	}

	switch ( $vars['ctransaction'] ) {
        case 'SALE': 
        case 'TEST_SALE':
			if ( $lock = stms_jvzoo_get_lock($product->ID, $vars['ctransreceipt']) )
				return false;

			// Create pending transaction first...
			$args = array(
				'product_id' => $product->ID,
				'member_id' => 0,
				'payment_type' => $product->type,
				'paysys' => 'jvzoo',
				'currency' => stms_isset($stms['global']['currency'], 'USD'),
				'amount' => $product->price,
				'ipaddr' => ''
			);
			$tx_id = stms_add_pending_payment($args);
			
			if ( !$member_id = stms_email_exists($email) ) {
				$temp = stms_generate_temporary();
				
				$userdata = array();
				$userdata['member_id'] = 0;
				$userdata['username'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? stms_generate_username($vars['ccustemail'], $first_name, $last_name) : $temp['username'];
				$userdata['password'] = wp_generate_password(8, false);
				$userdata['email'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? $vars['ccustemail'] : $temp['email'];
				$userdata['first_name'] = ucwords($first_name);
				$userdata['last_name'] = ucwords($last_name);
				$userdata['country'] = stms_isset($vars['ccustcc']);
				$userdata['status'] = ( stms_isset($product->info['auto_register']) == 1 ) ? 1 : 4;
				$userdata['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
				$userdata['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
				$userdata['paypal_email'] = $vars['ccustemail'];
				
				$member_id = stms_save_member($userdata);	
			}
			
			// log the ipn for further usage...
			$ipn = stms_log_ipn($vars, $tx_id, 'jvzoo', 0, $member_id, $vars['ctransreceipt']);
			
			$price = $product->price;
			if ( $product->type == 'recurr' && $product->is_trial == 1 )
				$price = $product->t_price;
				
			$total_amount = ( isset($vars['ctransamount']) ) ? number_format($vars['ctransamount'], 2, '.', '') : $price;
			// let's complete the payment...
			$args = array(
				'member_id' => $member_id,
				'paysys' => 'jvzoo',
				'receipt' => $vars['ctransreceipt'],
				'currency' => stms_isset($stms['global']['currency'], 'USD'),
				'amount' => $total_amount,
				'paid' => 1,
				'status' => 'completed',
				'date' => stms_time(),
				'expired_date' => stms_get_expired($product->ID, true)
			);
			
			stms_complete_payment($tx_id, $args);
			break;
			
		case 'BILL':
		  	if ( $payment = stms_get_payment($old_invoice) ) {
            	// set the status of previous payment to expire...
				$new_data = array();
				$new_data['status'] = 'expired';
				$new_data['expired_date'] = strtotime("5 minutes ago", stms_time());
				stms_update_payment($payment->tx_id, $new_data);
				wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
				
				// get the product data...
				$product = stms_get_product($payment->product_id);
				$member_id = $payment->member_id;
            }
			
			if ( !$member_id ) {
				$member_id = stms_jvzoo_get_member_id($email);
				if ( !$member_id )
					stms_error_handler("Cannot find member associated with this recurring transaction. JVZoo INS Ignored.", 0, $vars['ctransreceipt'], 'ipn');
			}
			
			// calculate expire date...
			$expired_date = stms_get_expired($product->ID, false);
			if ( is_numeric($product->rebill_times) && $product->rebill_times > 0 ) {
				$rebills = $wpdb->get_results(
					"SELECT * FROM {$wpdb->prefix}stms_transactions
					WHERE paysys = 'jvzoo' AND `member_id` = {$member_id}
					AND is_rebill = 1 AND product_id = {$product->ID}"
				);
				
				if ( $rebills ) {
					$rebilled = count($rebills);
					if ( $product->is_trial != 1 )
						$rebilled += 1;
						
					if ( $rebilled >= $product->rebill_times ) {
						$expired_date = strtotime("19 Jan 2038", stms_time());
					}
				}
			}
			
			$total_amount = ( isset($vars['ctransamount']) ) ? number_format($vars['ctransamount'], 2, '.', '') : $product->price;	
			// Add new payment for rebill
			$args = array(
				'product_id' => $product->ID,
				'member_id' => $member_id,
				'payment_type' => 'recurr',
				'paysys' => 'jvzoo',
				'receipt' => $vars['ctransreceipt'],
				'currency' => stms_isset($stms['global']['currency'], 'USD'),
				'amount' => $total_amount,
				'txn_id' => '',
				'paid' => 1,
				'is_rebill' => 1,
				'status' => 'completed',
				'expired_date' => $expired_date
			);
			$new_tx_id = stms_add_payment($args);
            break;
			
		case 'RFND':
		case 'CGBK':
		case 'INSF':
			if ( !$payment = stms_get_payment($old_invoice) )
				stms_error_handler("Cannot find transaction record for this JVZoo IPN (tx_id:{$tx_id})", 0, $vars['ctransreceipt'], 'ipn');

			$payments = stms_get_member_payments($payment->member_id);
			if ( $payments ) {
		        foreach ( $payments as $pay ) {
		            if ( $pay->product_id  == $payment->product_id && $pay->status == 'completed' ) {
		            	if ( isset($vars['ctransreceipt']) && $vars['ctransreceipt'] != $pay->receipt )
							continue;
							
						$refund_id = $pay->tx_id;
						stms_refund_payment($refund_id, 'third');
		            }
		        }
			}
			break;
			
		case 'CANCEL-REBILL':
	 	case 'CANCEL-TEST-REBILL':
			if ( !$payment = stms_get_payment($old_invoice) )
				stms_error_handler("Cannot find transaction record for this JVZoo IPN (tx_id:{$tx_id})", 0, $vars['ctransreceipt'], 'ipn');
			
			$new_data = array();
			$new_data['status'] = 'cancelled';
			stms_update_payment($payment->tx_id, $new_data );
        	wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
        	
			do_action('stms_subscr_cancelled', $payment->tx_id);
			break;
			
		case 'UNCANCEL-REBILL': //Reversing the cancellation of a recurring billing product.
		case 'UNCANCEL-TEST-REBILL':
           if ( !$payment = stms_get_payment($old_invoice) )
				stms_error_handler("Cannot find transaction record for this JVZoo IPN (tx_id:{$tx_id})", 0, $vars['ctransreceipt'], 'ipn');
			
			$new_data = array();
			$new_data['status'] = 'completed';
			stms_update_payment($payment->tx_id, $new_data );
        	wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
        	
			do_action('stms_subscr_cancelled', $payment->tx_id);
			break;
	 }
	 exit;
}

function stms_jvzoo_get_member_id( $email ) {
	global $wpdb;
	if ( $member = stms_get_data_by('email', $email) )
		return $member->ID;

	// let's try to check on IPN log...
	$sql = "SELECT * FROM `{$wpdb->prefix}stms_ipn_log` WHERE `pay_email` = '{$email}' AND `tx_id` <> 0 ORDER BY `log_id` DESC LIMIT 1";
	$ipn = $wpdb->get_row($sql);
	if ( isset($ipn->tx_id) ) {
		$p = stms_get_payment($ipn->tx_id);
		if ( isset($p->member_id) && $p->member_id > 0 )
			return $p->member_id;
	}
	
	return false;
}

function stms_find_jvzoo_ipn( $receipt ) {
	global $wpdb;
	$ipn = $wpdb->get_row(
		"SELECT * FROM `{$wpdb->prefix}stms_ipn_log`
		WHERE `paysys` = 'jvzoo' AND `receipt` = '{$receipt}'"
	);
	if ( !$ipn ) return false;
	return $ipn;
}

function stms_jvzoo_get_lock( $product_id, $receipt ) {
	global $wpdb;
	
	$sql = $wpdb->prepare(
				"SELECT COUNT(*) FROM `{$wpdb->prefix}stms_transactions`
				WHERE `product_id` = %d AND `receipt` = %s
				AND `is_rebill` = 0",
				$product_id, $receipt
			);
	$count = $wpdb->get_var($sql);
	if ( $count > 0 ) return true;
	return false;
}

function stms_jvzoo_last_id( $receipt ) {
	global $wpdb;
	
	$last_payment = $wpdb->get_row("SELECT `tx_id` FROM `{$wpdb->prefix}stms_transactions`
			WHERE (`receipt` = '{$receipt}' OR `receipt` LIKE '{$receipt}-B%')
			ORDER BY tx_id DESC LIMIT 1");
	
	if ( !$last_payment )
		return false;
	
	return $last_payment->tx_id;
}

add_filter('stms_thanks_setup_jvzoo', 'stms_jvzoo_thanks_setup');
function stms_jvzoo_thanks_setup( $settings ) {
	$settings['rcpt'] = 'cbreceipt';
	$settings['flow'] = 'normal';
	$settings['email_check'] = true;
	$settings['email_var'] = 'ccustemail';
	$settings['fullname_var'] = 'ccustname';
	return $settings;
}