<?php
/*
Payment Name: ProductPay
Payment Key: productpay
One Time: yes
Recurring: yes
Flow Type: normal
*/

add_action('stms_thirdpay_settings_productpay', 'stms_productpay_product_settings', 10, 2);
function stms_productpay_product_settings($product, $pay_settings) {
?>
	<tr valign="top" class="stms-productpay-level">
		<th scope="row" colspan="2" style="border-top:1px solid #E5E5E5"><h3 style="color:#808080"><?php _e('ProductPay Product Integration', 'stms'); ?></h3></th>
	</tr>
	<tr valign="top" class="stms-productpay-level">
		<th scope="row" style="text-align:right"><label for="stms_productpay_item"><strong><span style="color:#cc0000">*</span> <?php _e('Product ID#', 'stms'); ?></strong></label></th>
		<td>
			<input class="stms-field" type="text" name="stms_level[pay_settings][productpay][item]" id="stms_productpay_item" value="<?php echo stms_cleanup(stms_isset($pay_settings['productpay']['item'])); ?>" style="width:80px" />
			<div class="stms-desc">Please enter the Unique ID of the product that you create in your ProductPay account.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-productpay-level">
		<th scope="row" style="text-align:right"><label for="stms_productpay_url"><strong><span style="color:#cc0000">*</span> <?php _e('Download Page URL', 'stms'); ?></strong></label></th>
		<td>
			<input class="stms-field widefat" type="text" id="stms_productpay_url" value="<?php echo stms_get_returnurl('productpay', false, true, $product->ID); ?>" readonly />
			<div class="stms-desc">You MUST set the download page URL of the product you created in ProductPay to this URL.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-productpay-level">
		<th scope="row" style="text-align:right"><label><strong><span style="color:#cc0000">*</span> <?php _e('Payment Notification URL', 'stms'); ?></strong></label></th>
		<td>
			<input class="widefat stms-field" type="text" value="<?php echo stms_get_ipnurl('productpay') . '&insta_id=' . $product->ID; ?>" readonly />
			<div class="stms-desc">You MUST set the Destination URL in the product settings (under <code>Payment Notification -> Destination URL</code>) in your ProductPay account to this URL.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-productpay-level">
		<th scope="row" style="text-align:right"><label for="stms_productpay_code"><?php _e('Order Button Code', 'stms'); ?></label></th>
		<td>
			<textarea class="widefat stms-field" name="stms_level[pay_settings][productpay][code]" id="stms_productpay_code" style="height:80px"><?php echo esc_html(stms_sanitize($pay_settings['productpay']['code'])); ?></textarea>
			<div class="stms-desc">This is only required if you use the "Auto-Switch" payment button feature. Since ProductPay has its own payment button, then you have to enter the order button code here so that InstaMember can display it correctly. If you're not using the "Auto-Switch" payment feature, then you can leave this empty.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-productpay-level stms-last-integrate">
		<th scope="row" colspan="2"></th>
	</tr>
<?php
}

add_action('stms_global_payment_productpay', 'stms_productpay_global_settings');
function stms_productpay_global_settings($stms) {
?>
	<table class="form-table">
	<tr valign="top">
		<th scope="row"><label style="cursor:default"><strong>Enable ProductPay</strong></label></th>
		<td><div class="stms-toggle-btn"></div><input type="checkbox" name="stms_sets[is_productpay]" id="stms_pay_productpay" class="stms-toggle-checkbox stms-toggle-switch" data-blockname="productpay-paysets" value="1" <?php if ( stms_isset($stms['payments']['is_productpay']) == 1 ) echo 'checked="checked"'; ?> /></td>
	</tr>
	<tr valign="top" class="productpay-paysets">
		<th scope="row" style="text-align:right"><label for="stms_productpay_secret"><?php _e('Secret Key', 'stms'); ?></label></th>
		<td>
			<input class="widefat stms-field" type="text" name="stms_sets[productpay][secret]" id="stms_productpay_secret" value="<?php echo stms_isset($stms['payments']['productpay']['secret']); ?>" />
			<div class="stms-desc">Please enter your ProductPay secret key. You can find this when you create/edit a product in your ProductPay account.</div>
		</td>
	</tr>
	</table>
<?php
}

add_action('stms_ipn_productpay', 'stms_productpay_ipn_handler');
function stms_productpay_ipn_handler() {
	$stms = get_option('stms_settings');
	$secret = stms_sanitize($stms['payments']['productpay']['secret'], '', true);
	
	$pop = '';
	$ipnFields = array();
	foreach ( $_POST as $key => $value ) {
		if ( $key == "cverify" )
			continue;
		$ipnFields[] = $key;
	}
	sort( $ipnFields );
	foreach ( $ipnFields as $field ) {
    	$pop = $pop . $_POST[$field] . "|";
	}
	$pop = $pop . $secret_key;
	if ( function_exists('mb_convert_encoding') )
    	$xxpop = sha1(mb_convert_encoding($pop, "UTF-8"));
	else
		$xxpop = sha1($pop);
	
    $xxpop = strtoupper(substr($xxpop,0,8));
	if ( $xxpop != $_POST['cverify'] )
		stms_error_handler("ProductPay IPN Validation Failed. ProductPay INS Ignored.", 0, $_POST['cverify'], 'ipn');
	
	$process = stms_process_productpay($_POST);
}

function stms_process_productpay( $vars ) {
	if ( !is_array($vars) )
		stms_error_handler('No Postdata variables from ProductPay for one-time transaction. ProductPay IPN ignored.', 0, '', 'ipn');
	
	$stms = get_option('stms_settings');
	$email = stms_isset($vars['user_email']);
	$receipt = stms_isset($vars['sale_id']);
	$item = stms_sanitize($vars['product_id']);
	$title = stms_sanitize($vars['product_name']);
	
	$names = explode(" ", $email, 2);
	$first_name = stms_sanitize($names[0]);
	$last_name = '';
	
	$product_id = ( isset($vars['insta_id']) ) ? $vars['insta_id'] : 0;
	if ( !$product = stms_get_product($product_id) ) {
		if ( !$product = stms_find_product_by_externalid('productpay', $item) ) {
			if ( !$product = stms_get_product_bytitle($title) )
				stms_error_handler("Product not found. ProductPay IPN Ignored.", 0, $receipt, 'ipn');
		}
	}
	
	switch ( $vars['status'] ) {
		case 'PREAPPROVED':
			if ( $lock = stms_productpay_get_lock($product->ID, $receipt) )
				return false;

			$invoice = stms_productpay_last_id($receipt);
			$is_rebill = ( !$invoice ) ? false : true;
			$is_freetrial = ( $product->type == 'recurr' && $product->is_trial == 1 && $product->t_price <= 0.00 ) ? true : false;
			if ( $is_freetrial && !$is_rebill ) { // FREE TRIAL
				// Create pending transaction first...
				$args = array(
					'product_id' => $product->ID,
					'member_id' => 0,
					'payment_type' => $product->type,
					'paysys' => 'productpay',
					'currency' => stms_isset($stms['global']['currency'], 'USD'),
					'amount' => $product->price,
					'ipaddr' => ''
				);
				$tx_id = stms_add_pending_payment($args);
				if ( !$member_id = stms_email_exists($email) ) {
					$temp = stms_generate_temporary();
					
					$userdata = array();
					$userdata['member_id'] = 0;
					$userdata['username'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? stms_generate_username($vars['user_email'], $first_name, $last_name) : $temp['username'];
					$userdata['password'] = wp_generate_password(8, false);
					$userdata['email'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? $vars['user_email'] : $temp['email'];
					$userdata['first_name'] = $first_name;
					$userdata['last_name'] = $last_name;
					$userdata['status'] = ( stms_isset($product->info['auto_register']) == 1 ) ? 1 : 4;
					$userdata['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
					$userdata['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
					$member_id = stms_save_member($userdata);
				}
				
				// log the ipn for further usage...
				$ipn = stms_log_ipn($vars, $tx_id, 'productpay', 0, $member_id, $receipt);
				
				// Found matching member... let's complete the payment...
				$args = array(
					'member_id' => $member_id,
					'paysys' => 'productpay',
					'receipt' => $receipt,
					'currency' => stms_isset($stms['global']['currency'], 'USD'),
					'amount' => 0.00,
					'paid' => 1,
					'status' => 'completed',
					'date' => stms_time(),
					'expired_date' => stms_get_expired($product->ID, true)
				);
				
				stms_complete_payment($tx_id, $args);
			}
			break;
			
		case 'PURCHASED':
			if ( $lock = stms_productpay_get_lock($product->ID, $receipt) )
				return false;

			$invoice = stms_productpay_last_id($receipt);
			$is_rebill = ( !$invoice ) ? false : true;
			if ( !$is_rebill ) { // FIRST PAYMENT
				// Create pending transaction first...
				$args = array(
					'product_id' => $product->ID,
					'member_id' => 0,
					'payment_type' => $product->type,
					'paysys' => 'productpay',
					'currency' => stms_isset($stms['global']['currency'], 'USD'),
					'amount' => $amount,
					'ipaddr' => ''
				);
				$tx_id = stms_add_pending_payment($args);
				if ( !$member_id = stms_email_exists($email) ) {
					$temp = stms_generate_temporary();
					
					$userdata = array();
					$userdata['member_id'] = 0;
					$userdata['username'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? stms_generate_username($vars['user_email'], $first_name, $last_name) : $temp['username'];
					$userdata['password'] = wp_generate_password(8, false);
					$userdata['email'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? $vars['user_email'] : $temp['email'];
					$userdata['first_name'] = $first_name;
					$userdata['last_name'] = $last_name;
					$userdata['status'] = 4; //( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? 1 : 4;
					$userdata['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
					$userdata['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
					$member_id = stms_save_member($userdata);
				}
				
				// log the ipn for further usage...
				$ipn = stms_log_ipn($vars, $tx_id, 'productpay', 0, $member_id, $receipt);
				
				$price = $product->price;
				if ( $product->type == 'recurr' && $product->is_trial == 1 )
					$price = $product->t_price;
					
				$total_amount = ( isset($vars['sale_total']) ) ? number_format($vars['sale_total'], 2, '.', '') : $price;
				// let's complete the payment...
				$args = array(
					'member_id' => $member->ID,
					'paysys' => 'productpay',
					'receipt' => $receipt,
					'currency' => stms_isset($stms['global']['currency'], 'USD'),
					'amount' => $total_amount,
					'paid' => 1,
					'status' => 'completed',
					'date' => stms_time(),
					'expired_date' => stms_get_expired($product->ID, true)
				);
				
				stms_complete_payment($tx_id, $args);
			} else { // REBILL
				if ( $payment = stms_get_payment($invoice) ) {
	            	// set the status of previous payment to expire...
					$new_data = array();
					$new_data['status'] = 'expired';
					$new_data['expired_date'] = strtotime("5 minutes ago", stms_time());
					stms_update_payment($payment->tx_id, $new_data);
					wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
					
					// get the product data...
					$product = stms_get_product($payment->product_id);
					$member_id = $payment->member_id;
            	}

				if ( !$member_id ) {
					$member_id = stms_email_exists($email);
					if ( !$member_id )
						stms_error_handler("Cannot find member associated with this recurring transaction. ProductPay IPN Ignored.", 0, $receipt, 'ipn');
				}
				
				// calculate expire date...
				$expired_date = stms_get_expired($product->ID, false);
				if ( is_numeric($product->rebill_times) && $product->rebill_times > 0 ) {
					$rebills = $wpdb->get_results(
						"SELECT * FROM {$wpdb->prefix}stms_transactions
						WHERE paysys = 'productpay' AND `member_id` = {$member_id}
						AND is_rebill = 1 AND product_id = {$product->ID}"
					);
					
					if ( $rebills ) {
						$rebilled = count($rebills);
						if ( $product->is_trial != 1 )
							$rebilled += 1;
							
						if ( $rebilled >= $product->rebill_times ) {
							$expired_date = strtotime("19 Jan 2038", stms_time());
						}
					}
				}
				
				$total_amount = ( isset($vars['sale_total']) ) ? number_format($vars['sale_total'], 2, '.', '') : $product->price;
				// Add new payment for rebill
				$args = array(
					'product_id' => $product->ID,
					'member_id' => $member_id,
					'payment_type' => 'recurr',
					'paysys' => 'productpay',
					'receipt' => $receipt,
					'currency' => stms_isset($stms['global']['currency'], 'USD'),
					'amount' => $total_amount,
					'txn_id' => '',
					'paid' => 1,
					'is_rebill' => 1,
					'status' => 'completed',
					'expired_date' => $expired_date
				);
				$new_tx_id = stms_add_payment($args);
			} 
			break;
			
		case 'REFUNDED':
			$invoice = stms_productpay_last_id($receipt);
			if ( !$payment = stms_get_payment($invoice) )
				stms_error_handler("Cannot find transaction record for this ProductPay IPN", 0, $receipt, 'ipn');

			$payments = stms_get_member_payments($payment->member_id);
			if ( $payments ) {
		        foreach ( $payments as $pay ) {
		            if ( $receipt == $pay->receipt && $pay->status == 'completed' ) {		
						$refund_id = $pay->tx_id;
						stms_refund_payment($refund_id, 'third');
					}
		        }
			}
			break;
	}
	exit;
}

function stms_productpay_get_lock( $product_id, $receipt ) {
	global $wpdb;
	
	$sql = $wpdb->prepare(
				"SELECT COUNT(*) FROM `{$wpdb->prefix}stms_transactions`
				WHERE `paysys` = %s AND `product_id` = %d AND `receipt` = %s",
				'productpay', $product_id, $receipt
			);
	$count = $wpdb->get_var($sql);
	if ( $count > 0 ) return true;
	return false;
}

function stms_find_productpay_ipn( $receipt ) {
	global $wpdb;
	$ipn = $wpdb->get_row(
		"SELECT * FROM `{$wpdb->prefix}stms_ipn_log`
		WHERE `paysys` = 'productpay' AND `receipt` = '{$receipt}'"
	);
	if ( !$ipn ) return false;
	return $ipn;
}

function stms_productpay_get_member_id( $email ) {
	global $wpdb;
	if ( $member = stms_get_data_by('email', $email) ) return $member->ID;
	return false;
}

function stms_productpay_last_id( $receipt ) {
	global $wpdb;
	
	$last_payment = $wpdb->get_row(
		$wpdb->prepare("SELECT `tx_id` FROM `{$wpdb->prefix}stms_transactions`
			WHERE `receipt` = %s
			ORDER BY `tx_id` DESC LIMIT 1",
			$receipt
		)
	);
	
	if ( !$last_payment )
		return false;
	
	return $last_payment->tx_id;
}

//add_action('stms_validate_access_productpay', 'stms_productpay_thanks_handler');
function stms_productpay_thanks_handler($tx_id, $product_id) {
	@set_time_limit(0);
	
	global $wpdb;
	
	$product_id = intval($_GET['insta_id']);
	$stms = get_option('stms_settings');
	$secret_key = stms_sanitize($stms['payments']['productpay']['secret'], '', true);
	
	$rcpt = '';
	if ( isset($_GET['cbreceipt']) ) {
		$rcpt = rawurldecode(trim($_GET['cbreceipt']));
	} else if ( isset($_GET['insta_receipt']) ) {
		$rcpt = rawurldecode(trim($_GET['insta_receipt']));
	} else {
		$find_receipt = '';
		foreach ( $_GET as $k => $v ) {
			if ( stristr($_GET[$k], '?cbreceipt') ) {
				$find_receipt = rawurldecode(trim($_GET[$k]));
				break;
			}
		}
		
		$parts = explode("?", $find_receipt);
		$ppreceipt = explode("=", stms_isset($parts[1], ''));
		$rcpt = stms_isset($ppreceipt[1], '');
	}
	
	if ( empty($rcpt) ) {
		if ( $product = stms_get_product($product_id) ) {
			$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_permalink($stms['global']['home']);
			wp_redirect($home_url);
			exit;
		} else {
			wp_die("ERROR: Page Not Found.");
		}
	}
	
	$support = stms_get_handler_url('support');
	if ( $ipn = stms_find_productpay_ipn($rcpt) ) {
		$vars = maybe_unserialize($ipn->postdata);
		if ( !$payment = stms_get_payment($ipn->tx_id) )
			wp_die('ERROR: Invalid Payment');
		
		$product = stms_get_product($payment->product_id);
		$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_permalink($stms['global']['home']);
		$redirect_url = ( !empty($product->info['thankuri']) ) ? $product->info['thankuri'] : $home_url;
	
		if ( $ipn->status == 1 ) { // check if IPN has been processed...
			// IPN has been processed... then redirect to members download page...
			wp_redirect($home_url);
			exit;
		}
		
		$member_id = ( is_stmsmember_logged_in() ) ? stms_get_current_member_id() : $payment->member_id;
		$is_logged_in = ( is_stmsmember_logged_in() ) ? true : false;
		$is_existing = false;
		
		$temp_member = stms_member($payment->member_id);
		if ( $is_logged_in ) {
			if ( $member_id != $payment->member_id ) {
				// fix license key data ...
				$settings = stms_get_product_meta($payment->product_id, 'license');
				if ( $settings['enable'] == 1 )
					$wpdb->update("{$wpdb->prefix}stms_licenses", array('member_id' => $member_id), array('tx_id' => $payment->tx_id));
				
				// Update member id trx
				$new_data = array();
				$new_data['member_id'] = $member_id;
				stms_update_payment($payment->tx_id, $new_data);
				
				do_action('stms_payment_completed', $payment->tx_id); // so the receipt can be resend...
				
				if ( $temp_member->status == 4 )
					stms_delete_member($temp_member->ID, false);
			}
			
			$is_existing = true;
		}

		// Recall payment data to make sure that we use latest data...
		$payment = stms_get_payment($ipn->tx_id);
		// let's update ipn log...
		$update = $wpdb->update("{$wpdb->prefix}stms_ipn_log", array('status' => 1, 'member_id' => $payment->member_id), array('log_id' => $ipn->log_id));
		// remove retries data...
		delete_transient('stms_thanks_retries_' . $rcpt);
		
		do_action('stms_custom_autoresponder', $payment->tx_id); // for custom autoresponder...
		
		if ( $is_existing ) {
			// check and complete other payments (if any) ...
			stms_other_payments($member_id, $vars['user_email'], 'productpay');
			
			// logging out member first...
			stms_clear_auth_cookie();
		
			// re-logging in...
			stms_set_auth_cookie($member_id);
			
			// now redirecting...
			wp_redirect($redirect_url);
			exit;
		}
		
		$names = explode("@", $vars['user_email'], 2);
		$first_name = stms_sanitize($names[0]);
		$last_name = '';
			
		if ( stms_isset($product->info['auto_register']) == 1 ) {
			// update member data...
			$userdata = array();
			$userdata['member_id'] = $payment->member_id;
			$userdata['email'] = stms_isset($vars['user_email']);
			$userdata['status'] = 1;
			$userdata['fix_username'] = stms_generate_username($vars['user_email'], $first_name, $last_name);
			$userdata['pwd1'] = wp_generate_password(8, false);
			$userdata['first_name'] = $first_name;
			$userdata['last_name'] = $last_name;
			$userdata['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
			$userdata['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
			
			stms_save_member($userdata, false, false, $payment->tx_id);
			
			// check and complete other payments (if any) ...
			stms_other_payments($member_id, $vars['user_email'], 'productpay');
			
			// clean any login cookies first...
			stms_clear_auth_cookie();
			
			// logging in and then redirect...
			stms_set_auth_cookie($payment->member_id);
			wp_redirect($redirect_url);
			exit;
		} else {
			// Proceed to registration
			$session = base64_encode("{$product->ID}|productpay|third");
			$insta_token = stms_generate_secret(16, true, true );
			$token_data = array( 'token' => $insta_token, 'tx_id' => $payment->tx_id);
			set_transient('insta_token_' . $insta_token, $token_data, 21600); // lasted for six hours
			
			$reg_url = stms_format_url($stms['global']['magic'], 'imhandler=registration&key=' . $session . '&insta_token=' . $insta_token);
			// build reg vars...
			$reg_vars = array();
			$reg_vars['im_reg_email'] = $vars['user_email'];
			$auth_vars = base64_encode(json_encode($reg_vars));
			
			wp_redirect($reg_url . '&auth_vars=' . $auth_vars);
			exit;
		}
	} else {
		// still not receiving IPN...
		$retries = get_transient('stms_thanks_retries_' . $rcpt);
		$thanks_url = stms_format_url($stms['global']['magic'], "insta_mode=product_access&processor=productpay&insta_id={$product->ID}&cbreceipt={$rcpt}");
		if ( $retries >= 10 ) {
			$page_title = 'Payment Error';
			$refresh = '';
			$message = '
				<p style="text-align:center;"><strong>Payment Error</strong></p>
				<p>We\'re sorry, but there\'s a problem in validating your payment. Please contact our <a href="' . $support . '">support</a> to resolve this issue.</p>
			';
		} else {
			$count = $retries + 1;
			set_transient('stms_thanks_retries_' . $rcpt, $count, 60*15);
			$page_title = 'Waiting For Payment Confirmation...';
			$refresh = '<meta http-equiv="refresh" content="5;URL=\'' . $thanks_url . '\'" />';
			$message = '
				<h2 style="text-align:center;"><strong>Waiting For Payment Confirmation...</strong></h2>
				<p style="text-align:center;"><img src="' . STMS_URL . 'images/admin-loader.gif" border="0" /></p>
				<p style="text-align:center;">This page will be auto-refresh every 5 seconds until your payment confirmed.</p>
				<p style="text-align:center;"><strong>Please DO NOT close this page.</strong></p>
			';
		}
?>
<!DOCTYPE html>
<!--[if IE 6]>
<html id="ie6" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 7]>
<html id="ie7" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 8]>
<html id="ie8" <?php language_attributes(); ?>>
<![endif]-->
<!--[if !(IE 6) | !(IE 7) | !(IE 8)  ]><!-->
<html <?php language_attributes(); ?>>
<!--<![endif]-->
<head>
<meta charset="<?php bloginfo( 'charset' ); ?>" />
<title><?php echo $page_title; ?></title>
<?php echo $refresh; ?>
<!--[if lt IE 9]>
	<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->
<link rel="profile" href="http://gmpg.org/xfn/11" />
<meta name="HandheldFriendly" content="True" />
<meta name="viewport" content="width=device-width, initial-scale=1"/>
<!-- Adding "maximum-scale=1" fixes the Mobile Safari auto-zoom bug: http://filamentgroup.com/examples/iosScaleBug/ -->
<link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Droid+Sans" type="text/css" media="all" />
<style type="text/css">
body {
	margin:0;
	padding:0;
	font-size:14px;
	font-family:'Droid Sans', sans-serif;
	background-color:#F5F5F5;
	color:#808080;
}
#redirect_msg {
	margin:25px auto 0;
	padding:15px 20px;
	width:40%;
	background-color:#FFF;
}
</style>
</head>
<body>
<div id="redirect_msg">
	<?php echo $message; ?>
</div>
</body>
</html>
<?php
	}
	exit;
}

add_filter('stms_thanks_setup_productpay', 'stms_productpay_thanks_setup');
function stms_productpay_thanks_setup( $settings ) {
	$settings['rcpt'] = 'cbreceipt';
	$settings['flow'] = 'normal';
	$settings['email_check'] = false;
	$settings['email_var'] = 'user_email';
	$settings['fullname_var'] = 'user_email';
	return $settings;
}
