<?php
/*
Payment Name: WSO Pro Standard
Payment Key: wsopro
One Time: yes
Recurring: yes
Flow Type: normal
*/

add_action('stms_thirdpay_settings_wsopro', 'stms_wsopro_product_settings', 10, 2);
function stms_wsopro_product_settings($product, $pay_settings) {
	$stms = get_option('stms_settings');
	$wsopro_secret = ( stms_isset($stms['payments']['wsopro']['secret']) != '' ) ? trim($stms['payments']['wsopro']['secret']) : stms_generate_secret(8, true, false, true);
?>
	<tr valign="top" class="stms-wsopro-level">
		<th scope="row" colspan="2" style="border-top:1px solid #E5E5E5"><h3 style="color:#808080"><?php _e('WSO Pro Standard Product Integration Settings', 'stms'); ?></h3></th>
	</tr>
	<tr valign="top" class="stms-wsopro-level">
		<th scope="row" colspan="2"><div class="stms-help-msg"><p><strong>Important: </strong>To successfully integrate InstaMember and WSOPro Standard, the product name in your WSOPro Standard account must be EXACTLY the same as in InstaMember. Make sure it has no colons, forward slashes and backslashes or the integration will break.</p></div></th>
	</tr>
	<tr valign="top" class="stms-wsopro-level">
		<th scope="row" style="text-align:right"><label for="stms_wsopro_url"><strong><span style="color:#cc0000">*</span> <?php _e('Delivery URL', 'stms'); ?></strong></label></th>
		<td>
			<input class="stms-field widefat" type="text" id="stms_wsopro_url" value="<?php echo stms_get_returnurl('wsopro', false, true, $product->ID); ?>" readonly />
			<div class="stms-desc">You MUST set the product delivery page URL of the product you created in WSO Standard to this URL.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-wsopro-level">
		<th scope="row" style="text-align:right"><label><strong><span style="color:#cc0000">*</span> <?php _e('Key Generation URL', 'stms'); ?></strong></label></th>
		<td>
			<input class="widefat stms-field" type="text" value="<?php echo stms_get_ipnurl('wsoprokeygen') . '&insta_id=' . $product->ID; ?>&key=<?php echo $wsopro_secret; ?>" readonly />
			<div class="stms-desc">You MUST set the Key Generation URL in the WSO Standard product settings to this URL.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-wsopro-level">
		<th scope="row" style="text-align:right"><label><strong><span style="color:#cc0000">*</span> <?php _e('IPN Forwarding URL', 'stms'); ?></strong></label></th>
		<td>
			<input class="widefat stms-field" type="text" value="<?php echo stms_get_ipnurl('wsopro') . '&insta_id=' . $product->ID; ?>&key=<?php echo $wsopro_secret; ?>" readonly />
			<div class="stms-desc">You MUST set the IPN Forwarding URL in the WSO Standard product settings to this URL.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-wsopro-level">
		<th scope="row" style="text-align:right"><label for="stms_wsopro_code"><?php _e('Order Button Code', 'stms'); ?></label></th>
		<td>
			<textarea class="widefat stms-field" name="stms_level[pay_settings][wsopro][code]" id="stms_wsopro_code" style="height:80px"><?php echo esc_html(stms_sanitize($pay_settings['wsopro']['code'])); ?></textarea>
			<div class="stms-desc">This is only required if you use the "Auto-Switch" payment button feature. Since WSOPro Standard has its own payment button, then you have to enter the order button code here so that InstaMember can display it correctly. If you're not using the "Auto-Switch" payment feature, then you can leave this empty.</div>
		</td>
	</tr>
	<tr valign="top" class="stms-wsopro-level stms-last-integrate">
		<th scope="row" colspan="2"></th>
	</tr>
<?php
}

add_action('stms_global_payment_wsopro', 'stms_wsopro_global_settings');
function stms_wsopro_global_settings( $stms ) {
	$wsopro_secret = ( stms_isset($stms['payments']['wsopro']['secret']) != '' ) ? trim($stms['payments']['wsopro']['secret']) : stms_generate_secret(8, true, false, true);
?>
	<table class="form-table">
	<tr valign="top">
		<th scope="row"><label style="cursor:default"><strong>Enable WSO Pro Standard</strong></label></th>
		<td><div class="stms-toggle-btn"></div><input type="checkbox" name="stms_sets[is_wsopro]" id="stms_pay_wsopro" class="stms-toggle-checkbox stms-toggle-switch" data-blockname="wsopro-paysets" value="1" <?php if ( stms_isset($stms['payments']['is_wsopro']) == 1 ) echo 'checked="checked"'; ?> /></td>
	</tr>
	<tr valign="top" class="wsopro-paysets">
		<th scope="row" style="text-align:right"><label for="stms_wsopro_paypal"><?php _e('PayPal Email', 'stms'); ?></label></th>
		<td>
			<input class="widefat stms-field" type="text" name="stms_sets[wsopro][paypal]" id="stms_wsopro_paypal" value="<?php echo stms_isset($stms['payments']['wsopro']['paypal']); ?>" />
			<div class="stms-desc">Your PayPal email address that is registered under your WSO Pro account</div>	
		</td>
	</tr>
	</table>
	<input type="hidden" name="stms_sets[wsopro][secret]" value="<?php echo $wsopro_secret; ?>" />
<?php
}

add_action('stms_ipn_wsoprokeygen', 'stms_wsopro_keygen_handler');
function stms_wsopro_keygen_handler() {
	
	$stms = get_option('stms_settings');
	$secret = stms_isset($stms['payments']['wsopro']['secret']);
	$key = trim($_REQUEST['key']);
	if ( $secret != $key )
		stms_error_handler("Secret key is different. Expected: {$secret}, Received: {$key}. WSO Pro KeyGen ignored.", 0, $_POST['txn_id'], 'ipn');

	$posts = $_REQUEST;
	
	$txn_id = stms_isset($posts['txn_id']);
	$subscr_id = stms_sanitize($posts['subscr_id']);
	$first_name = stms_isset($posts['first_name']);
	$payer_email = sanitize_email($posts['payer_email']);
	$product_id = stms_isset($posts['product_id'], 0);
	$product_id = stms_isset($posts['insta_id'], $product_id);
	
	$receipt = ( !empty($subscr_id) ) ? $subscr_id : $txn_id;
	$access_url = stms_get_returnurl('wsopro', false, true) . "&insta_id={$product_id}&insta_receipt={$receipt}";
	
	echo "<p><strong>Congratulations on Your Purchase!</strong></p>";
	echo "<p><strong>In order to access the product, you MUST visit the special URL below:</strong><br />";
	echo "<a href='{$access_url}'><strong>&gt;&gt; Click Here To Access Your Purchase NOW! &lt;&lt;</strong></a></p>";
	exit;
}

add_action('stms_ipn_wsopro', 'stms_wsopro_ipn_handler');
function stms_wsopro_ipn_handler() {
	$stms = get_option('stms_settings');
	$secret = stms_isset($stms['payments']['wsopro']['secret']);
	
	$key = trim($_REQUEST['key']);
	if ( $secret != $key )
		stms_error_handler("Secret key is different. Expected: {$secret}, Received: {$key}. WSO Pro KeyGen ignored.", 0, $_POST['txn_id'], 'ipn');
	
	$posts = $_REQUEST;
    $paypal_email = stms_isset($stms['payments']['wsopro']['paypal']);
	
	if ( in_array(stms_isset($posts['payment_status']), array('Reversed', 'Reversal', 'Refunded')) ) {
		stms_wsopro_refund($posts);
		return true;
	}
	
	$process = stms_process_wsopro($posts);
}

function stms_process_wsopro( $vars ) {
	global $wpdb;
	
	$stms = get_option('stms_settings');
	$currency = stms_isset($stms['global']['currency'], 'USD');
	
	$txn_type = $vars['txn_type'];
	$txn_id = stms_sanitize($vars['txn_id']);
	$subscr_id = stms_sanitize($vars['subscr_id']);
	$payer_id = stms_sanitize($vars['payer_id']);
	
	$receipt = ( !empty($subscr_id) ) ? $subscr_id : $txn_id;
	$title = stms_sanitize($vars['item_name']);
	$product_id = ( isset($vars['insta_id']) ) ? $vars['insta_id'] : 0;
	
	// customer data...
	$first_name = stms_sanitize($vars['first_name']);
	$last_name = stms_sanitize($vars['last_name']);
	$email = sanitize_email($vars['payer_email']);
	
	if ( !$product = stms_get_product($product_id) ) {
		if ( !$product = stms_get_product_bytitle($title) )
			stms_error_handler("Product not found. WSO Pro IPN Ignored.", 0, $txn_id, 'ipn');
	}
	
	$trans_type = '';
	switch ( $txn_type ) {
		case 'web_accept':
			if ( $vars['payment_status'] == 'Completed' )
				$trans_type = 'SALE';
			break;
		case 'subscr_signup':
			if ( isset($vars['mc_amount1']) && $vars['mc_amount1'] == 0.00 )
				$trans_type = 'SALE';
			break;
		case 'subscr_payment':
			if ( $vars['payment_status'] == 'Completed' ) {
				if ( $invoice = stms_wsopro_last_id($subscr_id) )
					$trans_type = 'BILL';
				else
					$trans_type = 'SALE';
			}
			break;
	}
	
	
	switch ( $trans_type ) {
		case 'SALE':
			$args = array(
				'product_id' => $product->ID,
				'member_id' => 0,
				'payment_type' => $product->type,
				'paysys' => 'wsopro',
				'currency' => stms_isset($stms['global']['currency'], 'USD'),
				'amount' => $product->price,
				'ipaddr' => ''
			);
			$tx_id = stms_add_pending_payment($args);
			
			if ( !$member_id = stms_wsopro_get_member_id($email, $payer_id) ) {
				$temp = stms_generate_temporary();
				
				$userdata = array();
				$userdata['member_id'] = 0;
				$userdata['username'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? stms_generate_username($vars['payer_email'], $first_name, $last_name) : $temp['username'];
				$userdata['password'] = wp_generate_password(8, false);
				$userdata['email'] = ( stms_isset($product->info['steps']) == '2,1' && stms_isset($product->info['auto_register']) == 1 ) ? $vars['payer_email'] : $temp['email'];
				$userdata['first_name'] = $first_name;
				$userdata['last_name'] = $last_name;
				$userdata['status'] = ( stms_isset($product->info['auto_register']) == 1 ) ? 1 : 4;
				$userdata['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
				$userdata['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
				$userdata['paypal_email'] = stms_isset($vars['payer_email']);
				
				if ( !empty($vars['payer_id']) )
					$userdata['paypal_id'] = stms_isset($vars['payer_id']);
				
				$member_id = stms_save_member($userdata);
			}
			
			// log the ipn for further usage...
			$ipn = stms_log_ipn($vars, $tx_id, 'wsopro', 0, $member_id, $receipt);
			
			$price = $product->price;
			if ( $product->type == 'recurr' && $product->is_trial == 1 )
				$price = $product->t_price;
				
			$total_amount = ( isset($vars['mc_gross']) ) ? number_format($vars['mc_gross'], 2, '.', '') : $price;
			// let's complete the payment...
			$args = array(
				'member_id' => $member_id,
				'paysys' => 'wsopro',
				'receipt' => $receipt,
				'currency' => stms_isset($stms['global']['currency'], 'USD'),
				'amount' => $total_amount,
				'paid' => 1,
				'status' => 'completed',
				'date' => stms_time(),
				'expired_date' => stms_get_expired($product->ID, true)
			);
			
			stms_complete_payment($tx_id, $args);
			break;
			
		case 'BILL':
			if ( !$tx_id = stms_wsopro_last_id($subscr_id) )
				stms_error_handler('Cannot find transaction record for this IPN (tx_id:' . $tx_id . '). WSO Pro IPN Ignored.', 0, $subscr_id, 'ipn');
	
			$payment = stms_get_payment($tx_id);
			$new_data = array();
			$new_data['status'] = 'expired';
			$new_data['expired_date'] = strtotime("5 minutes ago", stms_time());
			stms_update_payment($tx_id, $new_data);
			wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
			
			stms_payment_release_lock($tx_id);
				
			// calculate expire date...
			$expired_date = stms_get_expired($product->ID, false);
			if ( is_numeric($product->rebill_times) && $product->rebill_times > 0 ) {
				$rebills = $wpdb->get_results(
					"SELECT * FROM {$wpdb->prefix}stms_transactions
					WHERE paysys = 'wsopro' AND `member_id` = {$member_id}
					AND is_rebill = 1 AND product_id = {$product->ID}"
				);
				
				if ( $rebills ) {
					$rebilled = count($rebills);
					if ( $product->is_trial != 1 )
						$rebilled += 1;
						
					if ( $rebilled >= $product->rebill_times ) {
						$expired_date = strtotime("19 Jan 2038", stms_time());
					}
				}
			}
			
			$total_amount = ( isset($vars['mc_gross']) ) ? number_format($vars['mc_gross'], 2, '.', '') : $product->price;
			$args = array(
				'product_id' => $payment->product_id,
				'member_id'=> $payment->member_id,
				'payment_type' => 'recurr',
				'paysys'=> 'wsopro',
				'receipt'=> $subscr_id,
				'currency'=> $currency,
				'amount'=> $total_amount,
				'txn_id' => $txn_id,
				'paid'=> 1,
				'is_rebill' => 1,
				'status'=> 'completed',
				'date'=> stms_time(),
				'expired_date'=> $expired_date
			);
				
			$tx_id = stms_complete_payment($tx_id, $args);
			break;
				
		case 'subscr_cancel':
			$tx_id = stms_wsopro_last_id($subscr_id);
			if ( !$tx_id )
				stms_error_handler('Cannot find transaction record for this IPN (tx_id:' . $tx_id . '). WSO Pro IPN Ignored.', 0, $subscr_id, 'ipn');
			
			$payment = stms_get_payment($tx_id);
			$new_data = array();
			$new_data['status'] = 'cancelled';
			stms_update_payment($tx_id, $new_data);
        	wp_cache_delete($payment->member_id, 'stms_member_payments'); // delete cache
        	
			do_action('stms_subscr_cancelled', $tx_id);
        	break;
			
		default:
			if ( $txn_type != 'subscr_signup' )
				stms_error_handler("Transaction type is empty; trans_type: {$trans_type} - txn_type: {$txn_type}", 0, $txn_id, 'ipn');
	}
	exit;
}

//add_action('stms_validate_access_wsopro', 'stms_wsopro_thanks_handler');
function stms_wsopro_thanks_handler() {
	@set_time_limit(0);
	
	global $wpdb;
	$product_id = intval($_GET['insta_id']);
	$stms = get_option('stms_settings');
	$rcpt = ( isset($_GET['access_key']) ) ? stms_sanitize($_GET['access_key']) : stms_sanitize($_GET['insta_receipt']);
	if ( empty($rcpt) ) {
		if ( $product = stms_get_product($product_id) ) {
			$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_permalink($stms['global']['home']);
			wp_redirect($home_url);
			exit;
		} else {
			wp_die("ERROR: Page Not Found.");
		}
	}
	
	$support = stms_get_handler_url('support');
	if ( $ipn = stms_find_wsopro_ipn($rcpt) ) {
		$vars = maybe_unserialize($ipn->postdata);
		$vars = maybe_unserialize($ipn->postdata);
		if ( !$payment = stms_get_payment($ipn->tx_id) )
			wp_die('ERROR: Invalid Payment');
		
		$product = stms_get_product($payment->product_id);
		$home_url = ( !empty($product->home_id) ) ? get_permalink($product->home_id) : get_permalink($stms['global']['home']);
		$redirect_url = ( !empty($product->info['thankuri']) ) ? $product->info['thankuri'] : $home_url;
	
		if ( $ipn->status == 1 ) { // check if IPN has been processed...
			// IPN has been processed... then redirect to members download page...
			wp_redirect($home_url);
			exit;
		}
			
		$member_id = ( is_stmsmember_logged_in() ) ? stms_get_current_member_id() : $payment->member_id;
		$is_logged_in = ( is_stmsmember_logged_in() ) ? true : false;
		$is_existing = false;
		
		$temp_member = stms_member($payment->member_id);
		if ( $is_logged_in ) {
			if ( $member_id != $payment->member_id ) {
				// fix license key data ...
				$settings = stms_get_product_meta($payment->product_id, 'license');
				if ( $settings['enable'] == 1 )
					$wpdb->update("{$wpdb->prefix}stms_licenses", array('member_id' => $member_id), array('tx_id' => $payment->tx_id));
				
				// Update member id trx
				$new_data = array();
				$new_data['member_id'] = $member_id;
				stms_update_payment($payment->tx_id, $new_data);
				
				do_action('stms_payment_completed', $payment->tx_id); // so the receipt can be resend...
				
				if ( $temp_member->status == 4 )
					stms_delete_member($temp_member->ID, false);
			}
			
			$is_existing = true;
		}

		// Recall payment data to make sure that we use latest data...
		$payment = stms_get_payment($ipn->tx_id);
		// let's update ipn log...
		$update = $wpdb->update("{$wpdb->prefix}stms_ipn_log", array('status' => 1, 'member_id' => $payment->member_id), array('log_id' => $ipn->log_id));
		// remove retries data...
		delete_transient('stms_thanks_retries_' . $rcpt);
		
		do_action('stms_custom_autoresponder', $payment->tx_id); // for custom autoresponder...
		
		if ( $is_existing ) {
			// check and complete other payments (if any) ...
			stms_other_payments($member_id, $vars['payer_email'], 'wsopro');
			
			// logging out member first...
			stms_clear_auth_cookie();
		
			// re-logging in...
			stms_set_auth_cookie($member_id);
			
			// now redirecting...
			wp_redirect($redirect_url);
			exit;
		}
		
		$first_name = strtolower(stms_sanitize($vars['first_name']));
		$last_name = strtolower(stms_sanitize($vars['last_name']));
			
		if ( stms_isset($product->info['auto_register']) == 1 ) {
			// update member data...
			$userdata = array();
			$userdata['member_id'] = $payment->member_id;
			$userdata['email'] = stms_isset($vars['payer_email']);
			$userdata['status'] = 1;
			$userdata['fix_username'] = stms_generate_username($vars['payer_email'], ucwords($first_name), ucwords($last_name));
			$userdata['pwd1'] = wp_generate_password(8, false);
			$userdata['first_name'] = ucwords($first_name);
			$userdata['last_name'] = ucwords($last_name);
			$userdata['is_aff'] = ( stms_isset($stms['global']['manual_aff'], 0) == 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
			$userdata['is_aff_pending'] = ( stms_isset($stms['global']['manual_aff'], 0) != 1 ) ? 0 : (( stms_isset($stms['global']['click_aff'], 1) == 1 ) ? 0 : 1);
			
			stms_save_member($userdata, false, false, $payment->tx_id);
			
			// check and complete other payments (if any) ...
			stms_other_payments($member_id, $vars['payer_email'], 'wsopro');
			
			// clean any login cookies first...
			stms_clear_auth_cookie();
			
			// logging in and then redirect...
			stms_set_auth_cookie($payment->member_id);
			wp_redirect($redirect_url);
			exit;
		} else {
			// Proceed to registration
			$session = base64_encode("{$product->ID}|wsopro|third");
			$insta_token = stms_generate_secret(16, true, true );
			$token_data = array( 'token' => $insta_token, 'tx_id' => $payment->tx_id);
			set_transient('insta_token_' . $insta_token, $token_data, 21600); // lasted for six hours
			
			$reg_url = stms_format_url($stms['global']['magic'], 'imhandler=registration&key=' . $session . '&insta_token=' . $insta_token);
			// build reg vars...
			$reg_vars = array();
			$reg_vars['im_reg_email'] = $vars['payer_email'];
			$reg_vars['im_reg_first_name'] = ucwords($first_name);
			$reg_vars['im_reg_last_name'] = ucwords($last_name);
			$auth_vars = base64_encode(json_encode($reg_vars));
			
			wp_redirect($reg_url . '&auth_vars=' . $auth_vars);
			exit;
		}
	} else {
		$retries = get_transient('stms_thanks_retries_' . $rcpt);
		$thanks_url = stms_format_url($stms['global']['magic'], http_build_query($_GET));
		if ( $retries >= 15 ) {
			$page_title = 'Payment Error';
			$refresh = '';
			$message = '
				<p style="text-align:center;"><strong>Payment Error</strong></p>
				<p>We\'re sorry, but there\'s a problem in validating your payment. Please contact our <a href="' . $support . '">support</a> to resolve this issue.</p>
			';
		} else {
			$count = $retries + 1;
			set_transient('stms_thanks_retries_' . $rcpt, $count, 60*15);
			$page_title = 'Waiting For Payment Confirmation...';
			$refresh = '<meta http-equiv="refresh" content="5;URL=\'' . $thanks_url . '\'" />';
			$message = '
				<h2 style="text-align:center;"><strong>Waiting For Payment Confirmation...</strong></h2>
				<p style="text-align:center;"><img src="' . STMS_URL . 'images/admin-loader.gif" border="0" /></p>
				<p style="text-align:center;">This page will be auto-refresh every 5 seconds until your payment confirmed.</p>
				<p style="text-align:center;"><strong>Please DO NOT close this page.</strong></p>
			';
		}
?>
<!DOCTYPE html>
<!--[if IE 6]>
<html id="ie6" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 7]>
<html id="ie7" <?php language_attributes(); ?>>
<![endif]-->
<!--[if IE 8]>
<html id="ie8" <?php language_attributes(); ?>>
<![endif]-->
<!--[if !(IE 6) | !(IE 7) | !(IE 8)  ]><!-->
<html <?php language_attributes(); ?>>
<!--<![endif]-->
<head>
<meta charset="<?php bloginfo( 'charset' ); ?>" />
<title><?php echo $page_title; ?></title>
<?php echo $refresh; ?>
<!--[if lt IE 9]>
	<script src="http://html5shim.googlecode.com/svn/trunk/html5.js"></script>
<![endif]-->
<link rel="profile" href="http://gmpg.org/xfn/11" />
<meta name="HandheldFriendly" content="True" />
<meta name="viewport" content="width=device-width, initial-scale=1"/>
<!-- Adding "maximum-scale=1" fixes the Mobile Safari auto-zoom bug: http://filamentgroup.com/examples/iosScaleBug/ -->
<link rel="stylesheet" href="http://fonts.googleapis.com/css?family=Droid+Sans" type="text/css" media="all" />
<style type="text/css">
body {
	margin:0;
	padding:0;
	font-size:14px;
	font-family:'Droid Sans', sans-serif;
	background-color:#F5F5F5;
	color:#808080;
}
#redirect_msg {
	margin:25px auto 0;
	padding:15px 20px;
	width:40%;
	background-color:#FFF;
}
</style>
</head>
<body>
<div id="redirect_msg">
	<?php echo $message; ?>
</div>
</body>
</html>
<?php
	}
	exit;
}

function stms_wsopro_refund( $vars ) {
	if ( !isset($vars['parent_txn_id']) )
		return false;
	
	global $wpdb;
	$txn_id = $vars['parent_txn_id'];
	$payments = $wpdb->get_results($wpdb->prepare("SELECT * FROM `{$wpdb->prefix}stms_transactions` WHERE `txn_id` = %s AND `status` = %s", $txn_id, 'completed'));
	if ( !payments )
		return false;
	
	foreach ( $payments as $payment ) {
		$tx_id = $payment->tx_id;
		stms_refund_payment($tx_id);
	}
	return true;
}

function stms_wsopro_get_member_id( $email, $paypal_id ) {
	global $wpdb;
	
	$email = sanitize_email($email);
	if ( $member = stms_get_data_by('email', $email) )
		return $member->ID;

	$table = "{$wpdb->prefix}stms_members";
	if ( $member = $wpdb->get_row("SELECT * FROM `$table` WHERE `paypal_id` = {$paypal_id}") )
		return $member->ID;
	
	return false;
}

function stms_wsopro_last_id( $receipt ) {
	global $wpdb;
	
	$invoice = (int) $invoice;
	$last_payment = $wpdb->get_row(
		$wpdb->prepare("SELECT tx_id FROM `{$wpdb->prefix}stms_transactions`
			WHERE `receipt` = %s ORDER BY tx_id DESC LIMIT 1",
			$receipt
		)
	);
	
	if ( !$last_payment )
		return false;
	
	return $last_payment->tx_id;
}

function stms_find_wsopro_ipn( $receipt ) {
	global $wpdb;
	$ipn = $wpdb->get_row(
		"SELECT * FROM `{$wpdb->prefix}stms_ipn_log`
		WHERE `paysys` = 'wsopro' AND `receipt` = '{$receipt}'"
	);
	if ( !$ipn ) return false;
	return $ipn;
}

add_filter('stms_thanks_setup_wsopro', 'stms_wsopro_thanks_setup');
function stms_wsopro_thanks_setup( $settings ) {
	$settings['rcpt'] = 'insta_receipt';
	$settings['flow'] = 'normal';
	$settings['email_check'] = true;
	$settings['email_var'] = 'payer_email';
	$settings['firstname_var'] = 'first_name';
	$settings['lastname_var'] = 'last_name';
	return $settings;
}
